<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Language\Text;

//use LartsSubscriptions\TemporaryTransaction;
use LartsSubscriptions\PSP\SubsPayment;
use LartsSubscriptions\Message;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Template;


class paylaterHelperResponse
{
	private $params;
	private $config;
	var $process_order;
	var $create_invoice;
	var $send_invoice;
	var $completed_template;

	
	public function __construct($plugin)
	{
		$this->params = $plugin->params;
		$this->config = new stdClass;

		$this->process_order      = $this->params->def('process_orders', 0);
		$this->create_invoice     = $this->params->def('create_invoice', 0);
		$this->send_invoice       = $this->params->def('send_invoice', 0);
		//$this->completed_template = $this->params->def('completed_template', 'thanks-for-ordering-page');

    $template_id = $this->params->def('completed_template', null);
    $this->completed_template = Template::lookupAlias($template_id);
		if (empty($this->completed_template)) {
      $this->completed_template = 'thanks-for-ordering-page';
		}
}

	// Response in the browser, perform some actions first.

  public function response()
	{
		$payment = new SubsPayment;

    // Get the ordercode if there is one.
		$ordercode = intval(Factory::getSession()->get('ordercode'));

    $transaction_data = Order::lookup($ordercode);
		if(empty($transaction_data)) {
      Factory::getApplication()->enqueueMessage('Fatal error while processing the order', 'error');
      return;
    }

    Order::setStatus($transaction_data->ordercode, OrderStatus::PENDING, true);

    // do post checkout...
    $response = array(
			'transaction_token'  => strtoupper(uniqid()), //md5($transaction_data->ordercode),
			'transaction_id'  => md5($transaction_data->ordercode),
			'ordercode'       => $transaction_data->ordercode,
			'userid'          => $transaction_data->userid,
			'payment_type'    => 'PayLater',
			'payment_details' => '',
			'plugin'          => 'paylater',
		);
    $cm = new CheckoutManager($transaction_data->ordercode);
    $cm->doPostCheckout($response);

    $cart = new Cart((int) $transaction_data->ordercode);

    // show checkout final message
    $amounts = new OrderAmounts();
    $amounts->setJson($transaction_data->amounts);

    $variables = [
			'brutto_price'       => $amounts->grossprice,
			'gross_price'        => $amounts->grossprice,
			'net_price'          => $amounts->netprice,
			'disco_price'        => $amounts->discount,
			'vat_total'          => $amounts->vat_total,
			'vat_percents'       => $amounts->getVatPercentsAppliedStr(),
			'transaction_id'   => '', //$transaction_data->transaction_id,
			'ordercode'        => $transaction_data->ordercode,
      'orderlist'        => $cart->getOrderedItemsHTML(false),
      'orderlist_simple' => $cart->getOrderedItemsHTML(true),
			'date'             => 'NOW',
			'userid'           => $transaction_data->userid,
			//'download_id'      => md5($transaction_data->userid),
		];
    
		$message = new Message;
		$body = $message->alias($this->completed_template)
		                ->user($transaction_data->userid)
		                ->variables($variables)
		                ->getBody();

		PluginManager::execPlugins('lartssubscriptions', 'onShowPaymentMessage', [$variables]);

		$layout = new FileLayout('payment_response', JPATH_SITE . '/components/com_lartssubscriptions/layouts');

		echo $layout->render([
			'header' => $message->getSubject(),
			'body'   => $body,
		]);
	}
  
  public function ipn($response)
	{
    // nothing to do
  }
}