<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

require_once __DIR__ . '/upgrade_base.php';


class LartsUpgrade extends LartsUpgradeBase
{
  var $ctx = null;

	public function __construct($ctx)
	{
		$this->ctx = $ctx;

    parent::__construct();
	}

  /*
	 * Check is being used to trigger all functionalities after updates
	 */
	function doPostInstall()
	{
		if (!$this->updateDatabase()) {
			Factory::getApplication()->enqueueMessage(Text::_('Could not update the database, please try again.'), 'error');
			return false;
		}

		if (!$this->updateTemplates()) {
			return false;
		}

		if (!$this->removeFilesAndFolders()) {
			return false;
		}

		return true;
	}

	private function updateDatabase()
	{
    if($this->ctx->installedVersion == null) {
      return true;
    }

    
    return true;
	}

	private function updateTemplates()
	{
		$templates = $this->templateUpdates();

		foreach ($templates as $alias => $data)
		{
			$template = $this->loadTemplateByAlias($alias);

			if (!empty($template))
			{
				continue;
			}

			if (!$this->insertNewTemplate($alias, $data))
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * Inserting the new template as it is not yet available in the template center
	 */
	private function insertNewTemplate($alias = null, $template = [])
	{
		if (empty($template) || empty($alias))
		{
			return false;
		}

		$tmpl              = new stdClass;
		$tmpl->alias       = $alias;
		$tmpl->type        = $template['type'];
		$tmpl->secured     = $template['secured'];
		$tmpl->subject     = $template['subject'];
		$tmpl->description = $template['description'];

		if (!Factory::getDbo()->insertObject('#__larts_subs_templates', $tmpl))
		{
			return false;
		}

		return true;
	}

	/**
	 * Loading a template by alias.
	 */
	private function loadTemplateByAlias($alias)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['id'])
		            ->from($db->quoteName('#__larts_subs_templates'))
		            ->where($db->quoteName('alias') . ' = ' . $db->quote($alias));

		$db->setQuery($query, 0, 1);

		return $db->loadObject();
	}

	private function templateUpdates()
	{
		return [];
	}

	/**
	 * Array filled with folder which can be deleted in the next install.
	 */
	public function removeFolders()
	{
		return [];
	}

	/**
	 * Array filled with files which can be deleted in the next install.
	 */
	public function removeFiles()
	{
		return [];
	}
}
