<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

class LartsUpgradeBase
{
	public function __construct()
	{
	}

	public function getExtensionData($element)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['e.extension_id', 'e.type', 'e.name', 'e.manifest_cache', 'us.update_site_id', 'us.enabled', 'us.extra_query'])
		            ->from($db->quoteName('#__extensions', 'e'))
		            ->join('LEFT OUTER', $db->quoteName('#__update_sites_extensions', 'use') . ' ON (' . $db->quoteName('e.extension_id') . ' = ' .
			            $db->quoteName('use.extension_id') . ')')
		            ->join('LEFT OUTER', $db->quoteName('#__update_sites', 'us') . ' ON (' . $db->quoteName('us.update_site_id') . ' = ' .
			            $db->quoteName('use.update_site_id') . ')')
		            ->where($db->quoteName('element') . ' = ' . $db->quote($element));

		$db->setQuery($query);

		return $db->loadObject();
	}


	// Removing files and folders if they exsists
	public function removeFilesAndFolders()
	{
		$folders = $this->removeFolders();

		foreach ($folders as $folder)
		{
			$this->recursive_remove_directory($folder);
		}

		$files = $this->removeFiles();

		foreach ($files as $file)
		{
			if (file_exists($file))
			{
				unlink($file);
			}
		}

		return true;
	}

	// Removing a folder recursively
	public function recursive_remove_directory($directory)
	{
		// if the path has a slash at the end we remove it here
		if (substr($directory, -1) == '/')
		{
			$directory = substr($directory, 0, -1);
		}

		// if the path is not valid or is not a directory ...
		if (!file_exists($directory) || ! is_dir($directory))
		{
			return false;
			// ... if the path is not readable
		}
		elseif (!is_readable($directory))
		{
			// ... we return false and exit the function
			return false;
			// ... else if the path is readable
		}
		else
		{
			// we open the directory
			$handle = opendir($directory);

			// and scan through the items inside
			while (false !== ($item = readdir($handle)))
			{
				// if the filepointer is not the current directory
				// or the parent directory
				if ($item != '.' && $item != '..')
				{
					// we build the new path to delete
					$path = $directory . '/' . $item;

					// if the new path is a directory
					if (is_dir($path))
					{
						// we call this function with the new path
						$this->recursive_remove_directory($path);
						// if the new path is a file
					}
					else
					{
						// we remove the file
						unlink($path);
					}
				}
			}

			closedir($handle);

			if (!rmdir($directory))
			{
				return false;
			}

			return true;
		}
	}
  
 
}
