<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Statistics;
use LartsSubscriptions\Order;
use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Price;


HTMLHelper::script('com_lartssubscriptions/chart.js', array('version' => 'auto', 'relative' => true));

// calc orders/month
$data = Statistics::getOrdersPerMonth();
$labels = array();
$d = new Date;
foreach($data as $item) {
  $labels[] = $d->monthToString($item->m, true). ' ' .$item->y;
}
$labels = implode('\',\'', $labels);
$labels = '\'' .$labels. '\'';
$ydata = array_column($data, 'num');

// get last pendigns
$pendings = Order::getLastPendings(4);
?>

<div style="height: 30px"></div>
<h2>Dashboard</h2>
<ul class="nav nav-tabs" id="myTabTabs" style="width: fit-content">
  <li class="active"><a href="#tab_orders" data-toggle="tab">Orders</a></li>
  <li class=""><a href="#tab_categories" data-toggle="tab">Pending Orders</a></li>
</ul>
<div class="tab-content" id="myTabContent" style="width: fit-content">
  <div id="tab_orders" class="tab-pane active">
    <canvas id="chartOrders" style="width: 750px; height: 250px; border: 1px dotted black; display: block; box-sizing: border-box;" width="750" height="250"></canvas>
  </div>
  <div id="tab_categories" class="tab-pane">
    <table class="table table-striped">
      <thead>
        <tr>
          <th width="80px"><?php echo Text::_('COM_LARTSSUBS_ORDERCODE'); ?></th>
          <th width="110px"><?php echo Text::_('COM_LARTSSUBS_CREATED'); ?></th>
          <th width="250px"><?php echo Text::_('COM_LARTSSUBS_NAME'); ?></th>
          <th class="right" width="110px"><?php echo Text::_('COM_LARTSSUBS_AMOUNT'); ?></th>
        </tr>
      </thead>
      <tbody>
        <?php foreach($pendings as $item): ?>
          <?php $totals = new OrderAmounts;
            $totals->setJson($item->amounts);
          ?>
          <tr style="cursor: pointer; cursor: hand;" onclick="window.location='http://localhost/joomla-base/administrator/index.php?option=com_lartssubscriptions&view=pendings';">
            <td><?php echo $item->ordercode; ?></td>
            <td><?php echo DateEx::_($item->created); ?></td>
            <td><?php echo $item->firstname, ', '. $item->lastname; ?></td>
            <td class="right"><?php echo Price::_($totals->netprice); ?></td>
          </tr>
        <?php endforeach; ?>
      </tbody>
    </table>
  </div>
</div>

 
<script>
var ctx = document.getElementById('chartOrders').getContext('2d');
var chartOrders = new Chart(ctx, {
    type: 'bar',
    data: {
        labels: [<?php echo $labels; ?>],
        datasets: [{
            label: 'Orders per month',
            data: [<?php echo implode(',', $ydata); ?>],
            backgroundColor: [
                'rgba(54, 162, 235, 0.2)',
            ],
            borderColor: [
                'rgba(54, 162, 235, 1)',
            ],
            borderWidth: 1
        }]
    },
    options: {
        maintainAspectRatio: false,
        responsive: false,
        scales: {
            y: {
                beginAtZero: true,
                suggestedMin: 0,
                suggestedMax: 20
            }
        }
    }
});
</script>


 

 