<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Lartssubscriptions\Pdf;
defined('_JEXEC') or die();

// Reference the Dompdf namespace 
use Dompdf\Dompdf; 

class PdfPrinter
{
  var $m_contentHeader;
  var $m_contentFooter;
  var $m_file;              // output file
  var $m_css = array();     // css path
  
	function __construct()
  {
    $this->addCSS(__DIR__ . '/css/docbase.css');
	}

  public function addCSS($filename) {
    $this->m_css[] = $filename;
  }
  public function setHeader($content) {
    $this->m_contentHeader = $content;
  }
  public function setFooter($content) {
    $this->m_contentFooter = $content;
  }

  public function genHTMLDoc($content)
  {
    $html = '<html>'.PHP_EOL;
    // head
    $html .= '<head>' .PHP_EOL;
    // css includes...
    foreach ($this->m_css as $css) {
      $css = '<link rel="stylesheet" href="'.$css.'">' .PHP_EOL;
      $html .= $css;
    }
    $html .= '</head>' .PHP_EOL;
    // body
    $html .= '<body>' .PHP_EOL;
    if(!empty($this->m_contentHeader)) {
      $html .= '<header>' .$this->m_contentHeader. '</header>' .PHP_EOL;
    }
    if(!empty($this->m_contentFooter)) {
      $html .= '<footer>' .$this->m_contentFooter. '</footer>' .PHP_EOL;
    }

    $html .= '<main>' .$content. '</main>' .PHP_EOL;
    $html .= '</body>' .PHP_EOL;
    $html .= '</html>' .PHP_EOL;
    return $html;
  }

  public function doPrint($content)
  {
    $doc = $this->genHTMLDoc($content);
    
    
    // Instantiate and use the dompdf class 
    $dompdf = new Dompdf();
    //$pdfRoot = JPATH_LIBRARIES .'/logical-arts/pdf';
    $pdfRoot = JPATH_ROOT;
    //echo 'pdfRoot: '. $pdfRoot;
    $dompdf->getOptions()->setChroot($pdfRoot);
    $dompdf->getOptions()->set('isHtml5ParserEnabled', true);
    $dompdf->getOptions()->set('isRemoteEnabled', true);
    $dompdf->setBasePath($pdfRoot);
      
    // Load HTML content 
    $dompdf->loadHtml($doc); 

    // (Optional) Setup the paper size and orientation 
    $dompdf->setPaper('A4', 'portrait'); 

    // Render the HTML as PDF 
    $dompdf->render(); 
    $output = $dompdf->output();
    // Output the generated PDF to Browser 
    //$dompdf->stream();

    // save file
    if(empty($this->m_file)) {
      $this->m_file = static::createTmpFile();
    }
    file_put_contents($this->m_file, $output);
	}

  public function getFile() {
    return $this->m_file;
  }
  
  private static function createTmpFile()
  {
    $tmpfile = tempnam(JPATH_ROOT. '/tmp/', '~');
    $file = JPATH_ROOT. '/tmp/' .basename($tmpfile). '.pdf';
    rename($tmpfile, $file);
		return $file;
  }
}





