<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Filesystem\File;

class CleanTemporaryFiles
{
	/**
	 * Removing old zip files from the temporary folder.
	 * We don't want to mess up the temporary folder.
	 *
	 * @since 2.5.0
	 */
	public function removeTemporaryPDFFiles()
  {
		$this->removeDownloadsOlderThan(0.5, '*.pdf');
	}

	/**
	 * Removing old zip files from the temporary folder.
	 * We don't want to mess up the temporary folder.
	 *
	 * @since 2.5.0
	 */
	public function removeTemporaryZipFiles()
	{
		$this->removeDownloadsOlderThan(0.5, '*.zip');
	}

	/**
	 * Removing temporary downloads older then 3 hours.
	 * A download can become temporary when it is being downloaded from an external URL first.
	 *
	 * @param float  $hours
	 * @param string $type
	 *
	 *
	 * @since 2.5.0
	 */
	private function removeDownloadsOlderThan($hours = 0.5, $type = '*.zip')
	{
		// define the amount of seconds.
		$seconds = ($hours * 60) * 60;

		// Directory which needs to be cleaned.
		$dir = Factory::getConfig()->get('tmp_path');

		// Cycle through all files in the directory
		foreach (glob($dir .'/'. $type) as $file)
		{
			if (time() - filectime($file) > $seconds)
			{
				unlink($file);
			}
		}
	}
}