<?php
/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\Database;

class CronBase
{
  var $table = '';
  var $db = null;
	var $cronpass = null;       // the secret pass key
	var $ipaddress = null;      // current IP running this cron
	var $whitelisted_ips = [];  // allowed IPs
  
  public function __construct()
  {
		// IP Address of the requestor.
		$this->ipaddress = $_SERVER['REMOTE_ADDR'];

		$this->db = Factory::getDbo();
  }

 	public function createCronInfo($ident)
	{
		$query = $this->db->getQuery(true)
                    ->insert($this->db->quoteName($this->table))
                    ->columns($this->db->quoteName('name'))
                    ->values($this->db->quote($ident));
    $this->db->setQuery($query);
    if(!$this->db->execute())
      return false;
    return $this->db->insertid();
	}

  public function getCronInfo($ident, $select = '*')
	{
		$query = $this->db->getQuery(true)
		                  ->select($select)
		                  ->from($this->table)
		                  ->where($this->db->quoteName('name') . ' = ' .$this->db->quote($ident));
		$this->db->setQuery($query);
		return $this->db->loadObject();
	}

 	public function setCronInfo($record)
	{
    if(!isset($record['id']) || $record['id'] == 0) {
      return Database::insert($this->table, $record);
    }

    return Database::update($this->table, 'id', $record);
	}

 	public function checkCronInfo($task)
	{
    $croninfo = $this->getCronInfo($task);

    // not in here? -> create new entry
    if(empty($croninfo)) {
      $this->createCronInfo($task);
      $croninfo = $this->getCronInfo($task);
    }
    
    $now = new Date;
    if($croninfo->lastrun != null) {
      // should we run again?
      $last = new Date($croninfo->lastrun);
      $diff = $last->diff($now);
      $minutes = ($diff->days * 24 * 60) + ($diff->h * 60) + $diff->i;
      if($minutes < $croninfo->period) {
        return false;   // period running, nothing to do
      }
    }

    return true;
  }
  
 	public function updateCronInfo($task)
	{
    $croninfo = $this->getCronInfo($task);

    // not in here? -> create new entry
    if(empty($croninfo)) {
      $this->createCronInfo($task);
      $croninfo = $this->getCronInfo($task);
    }
    
    $now = new Date;
    $record = [ 'id' => $croninfo->id,
                'lastrun' => $now,
              ];
    $this->setCronInfo($record);
    return true;
  }
  
  public function checkExec($pass)
  {
    // Check for the key or if the IP is whitelisted
    if (strlen($this->cronpass) > 0 && $this->cronpass == $pass) {
			return true;
		}
    if(!empty($this->whitelisted_ips)) {
      if (in_array($this->ipaddress, $this->whitelisted_ips)) {
        return true;
      }
    }
    
  	return false;
  }
  
}
