<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;

// class Database 

class Database
{
  public static function loadObject($table, $id, $key = 'id', $select = '*')
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select($select)
      ->from($db->quoteName($table))
      ->where($db->quoteName($key) .'='. $db->quote($id));

    $db->setQuery($query);
    $db->loadObject();
    $result = $db->loadObject();
    return $result;
  }

  public static function loadObjectList($table, $cids = [], $select = '*')
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select($select)
      ->from($db->quoteName($table));
      if(!empty($cids)) {
        $query->where($db->quoteName('id') . ' IN (' . implode(',', $cids) . ')');
      }

    $db->setQuery($query);
    //$sql = $query->__toString();
    //$db->loadObject();
    $result = $db->loadObjectList();
    return $result;
  }

  public static function insert($table, $data)
	{
    $db = Factory::getDbo();
    $strValues = '';
  	foreach ($data as $val) {
			if(!empty($strValues)) {
				$strValues .= ',';
			}
			if($val === null) {
	      $strValues .= 'null';
			}
			else {
	      $strValues .= $db->quote($val);
			}
    }
    
		$query = $db->getQuery(true)
			->insert($db->quoteName($table))
			->columns($db->quoteName(array_keys($data)))
			->values($strValues);
		$db->setQuery($query);
		if(!$db->execute())
      return false;
    return $db->insertid();
	}

  public static function update($table, $key, $columns)
  {
    $db = Factory::getDbo();

    $fields = array();
  	foreach ($columns as $column => $val) {
      if($column != $key) {
				if($val === null) {
	  			$fields[] = $db->quoteName($column) . '=NULL';
				}
				else {
	  			$fields[] = $db->quoteName($column) . '=' . $db->quote($val);
				}
      }
		}

		$query = $db->getQuery(true)
			->update($db->quoteName($table))
			->set($fields)
      ->where($db->quoteName($key) .'='. $db->quote($columns[$key]));
		$db->setQuery($query);
    //$sql = $query->__toString();
		return $db->execute();
 }
  
  public static function deleteObject($table, $key, $value)
  {
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete()
		            ->from($db->quoteName($table))
		            ->where($db->quoteName($key) .'='. $db->quote($value));
		$db->setQuery($query);
		return $db->execute();
  }

  public static function deleteObjects($table, $cids = [], $key = 'id')
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
           ->delete()
           ->from($db->quoteName($table))
           ->where($db->quoteName($key) . ' IN (' . implode(',', $cids) . ')');

    $db->setQuery($query);
 		return $db->execute();
  }

	public static function validateFields($table, $values)
	{
    $db = Factory::getDbo();
		$tableColumns = $db->getTableColumns($table);
		
		foreach (array_keys($values) as $k) {
				// Skip columns that don't exist in the table.
				if (!\array_key_exists($k, $tableColumns)) {
					unset($values[$k]);
					continue;
				}

				// Ignore any internal fields.
				if ($k[0] === '_') {
					unset($values[$k]);
				}
		}
		
		return $values;
	}

  public static function truncate($table)
  {
    $db = Factory::getDbo();
    $query = 'TRUNCATE ' .$db->quoteName($table);
		$db->setQuery($query);
		return $db->execute();
  }

  private static function getColumns($table)
	{
		$db = Factory::getDbo();

		$query = 'SHOW COLUMNS FROM ' . $db->quoteName($table);
		$db->setQuery($query);
		$results = $db->loadAssocList();

		$columns = [];
		foreach ($results as $value) {
			$columns[] = $value['Field'];
		}

		return $columns;
	}
  
	// ALTER TABLE `$table` DROP `$column`
  public static function dropColumn($table, $column)
	{
		$columns = self::getColumns($table);

		// col not in table?
    if(!in_array($column, $columns)) {
			return;
		}

		$db = Factory::getDbo();
		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' DROP ' . $db->quoteName($column) . '';

    $db->setQuery($query);
		return $db->execute();
	}

	// ALTER TABLE `$table` TRUNCATE `$column`
  public static function truncatepColumn($table, $column)
	{
		$columns = self::getColumns($table);

		// col not in table?
    if(!in_array($column, $columns)) {
			return;
		}

		$db = Factory::getDbo();
		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' TRUNCATE ' . $db->quoteName($column) . '';

    $db->setQuery($query);
		return $db->execute();
	}

	// ALTER TABLE `$table` ADD `$column` $datatype AFTER `$after`,
	public static function addColumn($table, $column, $datatype, $after = '')
	{
		// Check the categories table:
		$columns = self::getColumns($table);

		// col already in table?
		if (in_array($column, $columns)) {
			return true;
		}

		$db = Factory::getDbo();
		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' ADD COLUMN ' . $db->quoteName($column) . ' ' . $datatype;

		if(!empty($after)) {
			$query .= ' AFTER ' . $db->quoteName($after);
		}

    $db->setQuery($query);
		return $db->execute();
	}

	// ALTER TABLE `$table` CHANGE COLUMN `$column` `$new_name` $datatype AFTER `$after`,
	public static function renameColumn($table, $column, $new_name, $datatype, $after = '')
	{
		$db = Factory::getDbo();

    // Check the categories table:
		$columns = $db->getColumns($table);

		if(!in_array($column, $columns) || in_array($new_name, $columns)) {
			return true;
		}

		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' CHANGE COLUMN  ' . $db->quoteName($column) . ' ' . $db->quoteName($new_name) . ' ' . $datatype;

		if(!empty($after) && in_array($after, $columns)) {
			$query .= ' AFTER ' . $db->quoteName($after);
		}

    $db->setQuery($query);
		return $db->execute();
	}

	public static function setColumn($table, $column, $key, $cids = [], $value = 1)
	{
		if (empty($cids)) {
			return true;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName($table))
		            ->set($db->quoteName($column) . ' = ' . (int) $value)
		            ->where($db->quoteName($key) . ' IN ' . '( ' . implode(',', $cids) . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

  public static function mysql_escape_string($inp) {
      if(is_array($inp))
          return array_map(__METHOD__, $inp);

      if(!empty($inp) && is_string($inp)) {
          return str_replace(array('\\', "\0", "\n", "\r", "'", '"', "\x1a"), array('\\\\', '\\0', '\\n', '\\r', "\\'", '\\"', '\\Z'), $inp);
      }

      return $inp;
  }   
}

