<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use LartsSubscriptions\Config as LartsConfig;


class DateEx
{
	public static function _($date = null, $format = '')
	{
		if ($date == '0000-00-00') {
			return '';
		}

		if(!$date || $date == 'NOW' || $date == 'now') {
			$date = new Date;
			$date = $date->toSql();
		}

		$format = $format ?: self::getDateFormat();

		$config = Factory::getConfig();
		$user   = Factory::getApplication()->getIdentity();

		// Get a date object based on the correct timezone.
		$date = Factory::getDate($date, 'UTC');
		$date->setTimezone(new \DateTimeZone($user->getParam('timezone', $config->get('offset'))));

		return $date->format($format, true, false);
	}

	public static function getDateFormat()
	{
		return LartsConfig::get()->date_format;
	}

  // timestamp $date
  // return timestamp
  public static function AddMonthWithoutOvershoot($date, $nCount)
  {
    // using "normal' 
    // $end_to = date('Y-m-d', strtotime($start_to . " + " . $period . " months"));
    // can result of overshooting to next month.
    // e.g. 31.01.200 -> 02.03.2000 but we want 31.01.200 -> 28.02.2000
    $dtDate = new \DateTime;
    $dtDate->setTimestamp($date);
    $oldDay = $dtDate->format("d");
    $dtDate->add(new \DateInterval("P".$nCount."M")); // 2016-03-02
    $newDay = $dtDate->format("d");

    if($oldDay != $newDay) {
      // Check if the day is changed, if so we skipped to the next month.
      // Substract days to go back to the last day of previous month.
      $dtDate->sub(new \DateInterval("P".$newDay."D"));
    }
    return $dtDate->getTimestamp();
  }

  public static function getQuater($dateStr)
  {
    $month = date("n", strtotime($dateStr));
    return ceil($month / 3);
  }
  
 	public static function getDateRangeForQuarter($quarter, $year)
	{
		$dStart = new \DateTime();
    $dEnd = new \DateTime;

    switch($quarter) {
    case 1:
      $dStart->setDate($year, 1, 1);
      $dEnd->setDate($year, 3, 31);
      break;
    case 2:
      $dStart->setDate($year, 4, 1);
      $dEnd->setDate($year, 6, 30);
      break;
    case 3:
      $dStart->setDate($year, 7, 1);
      $dEnd->setDate($year, 9, 30);
      break;
    defaut:
      $dStart->setDate($year, 10, 1);
      $dEnd->setDate($year, 12, 31);
      break;
    }
    return ['begin' => $dStart->format('Y-m-d H:i:s'), 'end' => $dEnd->format('Y-m-d H:i:s')];
	}


  
}

