<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filesystem\File;

class FileBase
{
		  
 	/**
	 * This method will crawl a starting directory and get all the valid files
	 * that will be analyzed by __construct. Then it organizes them into an
	 * associative array.
	 *
	 * @param   string $path          Folder where we should start looking
	 * @param   array  $ignoreFolders Folder ignore list
	 * @param   array  $ignoreFiles   File ignore list
	 *
	 * @return  array   Associative array, where the `fullpath` key contains the path to the file,
	 *                  and the `classname` key contains the name of the class
	 */
	static function getFiles($path, array $ignoreFolders = array(), array $ignoreFiles = array())
	{
		$return = array();

		$files = self::scanDirectory($path, $ignoreFolders, $ignoreFiles);

		// Ok, I got the files, now I have to organize them
		foreach ($files as $file)
		{
			$clean = str_replace($path, '', $file);
			$clean = trim(str_replace('\\', '/', $clean), '/');

			$parts = explode('/', $clean);

			$return[] = array(
				'fullpath'  => $file,
				'file' => $parts[0]
			);
		}

		return $return;
	}

	/**
	 * Recursive function that will scan every directory unless it's in the
	 * ignore list. Files that aren't in the ignore list are returned.
	 *
	 * @param   string $path          Folder where we should start looking
	 * @param   array  $ignoreFolders Folder ignore list
	 * @param   array  $ignoreFiles   File ignore list
	 *
	 * @return  array   List of all the files
	 */
	protected static function scanDirectory($path, array $ignoreFolders = array(), array $ignoreFiles = array())
	{
		$return = array();

		$handle = @opendir($path);

		if ( !$handle)
		{
			return $return;
		}

		while (($file = readdir($handle)) !== false)
		{
			if ($file == '.' || $file == '..')
			{
				continue;
			}

			$fullpath = $path . '/' . $file;

			if ((is_dir($fullpath) && in_array($file, $ignoreFolders)) || (is_file($fullpath) && in_array($file, $ignoreFiles)))
			{
				continue;
			}

			if (is_dir($fullpath))
			{
				$return = array_merge(self::scanDirectory($fullpath, $ignoreFolders, $ignoreFiles), $return);
			}
			else
			{
				$return[] = $path . '/' . $file;
			}
		}

		return $return;
	}

  public static function getFormattedFilesize($value, $string_only = true)
  {
    global $user;

    $available_units = array(
      array(
        'min' 		=> 1099511627776, // pow(2, 40)
        'div'		  => 4,
        'si_unit'	=> 'TB',
        'unit'	  => 'TB',
      ),
      array(
        'min' 	=> 1073741824, // pow(2, 30)
        'div'		=> 3,
        'unit'	=> 'GB',
      ),
      array(
        'min'		=> 1048576, // pow(2, 20)
        'div'		=> 2,
        'unit'	=> 'MB',
      ),
      array(
        'min'		=> 1024, // pow(2, 10)
        'div'		=> 1,
        'unit'	=> 'KB',
      ),
      array(
        'min'		=> 0,
        'div'	  => 0,
        'unit'	=> 'BYTES', // Language index
      ),
    );
    
    foreach ($available_units as $idx => $unit_info)
    {
      if ($value >= $unit_info['min']) {
        break;
      }
    }
    $unit_info = $available_units[$idx];
    for ($i = 0; $i < $unit_info['div']; $i++) {
      $value /= 1024;
    }
    $value = round($value, 2);

    if (!$string_only) {
      $unit_info['value'] = $value;
      return $unit_info;
    }

    return $value  . ' ' . $unit_info['unit'];
  }
}