<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;

use LartsSubscriptions\Config as LartsConfig;

jimport('joomla.filesystem.file');

class Image
{
  var $acceptTypes;
  var $imageDir;
  var $uploadDir;
  
  public function __construct()
	{
    $this->acceptTypes = array('jpg', 'png', 'bmp', 'gif');
    $dir = LartsConfig::get('upload_dir');
    $this->imageDir = JPATH_ADMINISTRATOR. '/components/com_lartssubscriptions/images';
	}
  
  public function getUploadDir() {
     return $this->uploadDir;
  }
  public function getFilePath($filename, $ext = '') {
     $fn = $this->uploadDir. '/' .$filename;
     if($ext != '') {
       $fn .= '.' .$ext;
     }
     return $fn;
   }
  
  public static function validateFile($file, $accept)
  {
    $fileext = strtolower(File::getExt($file));
    if (!in_array($fileext, $accept)) {
      return false;
    }
    return true;
  }

	private static function getImagePath()
	{
		return JPATH_SITE . '/components/com_lartssubscriptions/images/';
	}

 	public static function getNoImage()
	{
		return Uri::root() . '/media/com_lartssubscriptions/images/no-image.png';;
	}

  private static function getAllowedTypes()
	{
		return [
			'image/pjpeg',
			'image/gif',
			'image/png',
			'image/jpeg',
			'image/JPG',
			'image/jpg',
		];
	}

	public static function getUrl($filename)
	{
		$folder   = 'components/com_lartssubscriptions/images/';
    
    if(empty($filename)) {
      return self::getNoImage();
    }
    
		$filename = Uri::root() . $folder . $filename;

		return $filename;
	}

	public static function remove($filename)
	{
		// Destination path
		$folder = self::getImagePath();

		if ($filename === 'no-image.png') {
			return true;
		}

		if(!File::delete($folder . $filename)) {
			return false;
		}

		return true;
	}



	/**
	 * Uploading an image image to a file or product.
	 */
	public static function upload( $id, $file = [], $type = 'product')
	{
		// Determin a filename for this upload.
		$filename = self::getUrl($id, 'product', false);

		// Remove file if it is present
		if ($filename && file_exists($filename))
		{
			File::delete($filename);
		}

		// Check filesize
		if ($file['size'] > 5000000)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_IMAGE_TOO_BIG'), 'error');

			return false;
		}

		// Is the uploaded file allowed and an image
		if(!in_array($file['type'], self::getAllowedTypes()))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_IMAGE_MIME_TYPE_NOT_VALID'), 'error');

			return false;
		}

		$af_size = getimagesize($file['tmp_name']);

		switch ($af_size[2])
		{
			case 1 :
			{
				$thispicext = 'gif';
				break;
			}
			case 2 :
			{
				$thispicext = 'jpg';
				break;
			}
			case 3 :
			{
				$thispicext = 'png';
				break;
			}
		}

		chmod($file['tmp_name'], 0644);

		if(!File::upload($file['tmp_name'], self::getImagePath() . $type . '-' . $id . '.' . $thispicext))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_MOVE_IMAGE'), 'error');

			return false;
		}

		return true;
	}
}