<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts-Extensions
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         {licence}
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

class JoomlaHelper
{
	public static function getCurrentUser()
	{
		return Factory::getApplication()->getIdentity() ?? (new User());
	}
	
	public static function renderFieldsetWithTooltips($form, $name, $options = [])
	{
			$fields = $form->getFieldset($name);
			$html   = [];

			foreach ($fields as $field) {
				$desc = $field->description;
				if(!empty($desc)) {
					// embed with tooltip
					$label = $field->getAttribute('label', '');
					$name = $field->getAttribute('name', '');
					$class = $field->getAttribute('class', '');
					
					$id = 'tip-'.$name;
					$class = $field->__get('labelclass');
					$field->__set('labelclass', $class .' hasTooltip" aria-labelledby="'.$id.'"');

					$descnew = '<div role="tooltip" id="'.$id.'">' .Text::_($desc). '</div>';
					$html[] = $descnew;
				}
				$html[] = $field->renderField($options);
			}

			return implode('', $html);
	}

	public static Function enqueueErrorMessages($errors, $nMax = 3)
  {
		$app = Factory::getApplication();

		// Push up to three validation messages out to the user.
		for ($i = 0, $n = \count($errors); $i < $n && $i < $nMax; $i++) {
			if ($errors[$i] instanceof \Exception) {
				$app->enqueueMessage($errors[$i]->getMessage(), 'error');
			} else {
				$app->enqueueMessage($errors[$i], 'error');
			}
		}
  }

	public static Function IsLocalhost()
  {
    $base = Uri::getInstance()->base();
    $res = strstr($base, 'localhost', false);
    return !empty($res);
  }

  public static function getRedirectBack($defaultView)
  {
    $link = Route::_('index.php?option=com_lartssubscriptions&view='. $defaultView, false);
    
		$jinput = Factory::getApplication()->getInput();
    $aid = $jinput->get('aid');
    if($aid != 0) {
      $link = Route::_('index.php?option=com_content&catid=2&view=article&id=' .$aid, false);
    }
    return $link;
  }

  public static Function getRedirectSame($option, $controller = '')
  {
 		$jinput = Factory::getApplication()->getInput();
		$view = $jinput->get('view', $controller);
    $task = $view .'.edit';
    $path = 'index.php?option=' .$option;
    if(!empty($view)) {
      $path .= '&view=' .$view;
    }
    else if(!empty($task)) {
      $path .= '&task=' .$task;
    }
    return $path;
  }
  
  
}