<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Http\HttpFactory;

class Maxmind
{
	/**
	 * Inserting states into the the database.
	 *
	 * @return bool
	 *
	 * @since 2.5.0
	 */
	public function importStates($table)
	{
		if(empty($table))
		{
			return false;
		}

		// Exlode on new lines
		$states  = explode("\n", $this->getStatesList());

		if(empty($states))
		{
			return false;
		}

		// Truncate the current database.
		if(!$this->emptyStateDatabase($table))
		{
			return false;
		}

		// Loop through lines
		foreach ($states as $state)
		{
			// List the items from the line into a variable.
			list($country, $code, $state_name) = explode(",", $state);

			if(empty($state_name))
				continue;

			// preparing object
			$item = (object) [
				'country'   => $country,
				'code'      => $code,
				'state'     => trim($state_name,'"'),
				'published' => 1,
			];

			// Insert the object into the states table
			Factory::getDbo()->insertObject($table, $item);
		}

		return true;
	}

	/**
	 * Cleaning database table for the states.
	 * As we're saving states by name in the user table this table can be truncated all the time.
	 *
	 * @param $table
	 *
	 * @return mixed
	 *
	 * @since 2.5.0
	 */
	private function emptyStateDatabase($table)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->delete($db->quoteName($table));

		$db->setQuery($query);

		return $db->execute();
	}

	/**
	 * Loading the states from Maxmind
	 *
	 * @return bool|string
	 *
	 * @since 2.5.0
	 */
	private function getStatesList()
	{
		$states = HttpFactory::getHttp()->get('http://www.maxmind.com/download/geoip/misc/region_codes.csv', null, 30);

		if(!$states || $states->code != 200 || empty($states->body))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_DOWNLOAD_MAXMIND_STATES_FAILED'), 'error');

			return false;
		}

		return $states->body;
	}
}