<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Event\GenericEvent;
use Joomla\CMS\Event\Model\PrepareDataEvent;

use LartsSubscriptions\Base\GenericReturnsEvent;

/**
 * Description of PluginTrigger
 * @author Christian
 */
class PluginManager
{
  var $plugins;

  public static function dispatchPrepareDataEvent($group, $plugin, $data = array(), $bUseArrayValues = false)
	{
		$eventName = 'onContentPrepareData';
		
    if(empty($group)) {
      if(PluginHelper::importPlugin($plugin) == null)
        return null;
    }
    else {
      if(PluginHelper::importPlugin($group, $plugin) == null)
        return null;
    }

		$dispatcher = Factory::getApplication()->getDispatcher();
		$event = new PrepareDataEvent($eventName, $bUseArrayValues ? array_values($data) : [$data]);
		$dispatcher->dispatch($eventName, $event);
		//$result = $event->getResult();
		$result = [];
    
		//return !isset($result['result']) || \is_null($result['result']) ? [] : $result['result'];
		return $result;
	}
	
  public static function execPlugin($group, $plugin, $callback, $data = array(), $bUseArrayValues = false)
	{
    if(empty($group)) {
      if(PluginHelper::importPlugin($plugin) == null)
        return null;
    }
    else {
      if(PluginHelper::importPlugin($group, $plugin) == null)
        return null;
    }

		$dispatcher = Factory::getApplication()->getDispatcher();
		$event = new GenericReturnsEvent($callback, $bUseArrayValues ? array_values($data) : [$data]);
		$dispatcher->dispatch($callback, $event);
		$result = $event->getResult();
		//$ret = !isset($result['result']) || \is_null($result['result']) ? [] : $result['result'];

		return $result;
  }
  
  public static function execPlugins($group, $callback, $data = array(), $bUseArrayValues = false, $bCheckResult = true)
	{
    if(!PluginHelper::importPlugin($group)) {
      return false;
		}
		
		//$ret = Factory::getApplication()->triggerEvent($callback, $bUseArrayValues ? array_values($data) : [$data]);

		$dispatcher = Factory::getApplication()->getDispatcher();
		$event = new GenericReturnsEvent($callback, $bUseArrayValues ? array_values($data) : [$data]);
		$dispatcher->dispatch($callback, $event);
		$result = $event->getResult();
		//$ret = !isset($result['result']) || \is_null($result['result']) ? [] : $result['result'];

    if(!$bCheckResult) {
      return $result;
		}
    
    // check result on false
    if(!empty($result)) {
      foreach($result as $return) {
        if($return == false)
          return false;
      }
    }
    return true;
  }

  public static function checkBool($result)
  {
    if(!empty($result)) {
      foreach($result as $ret) {
        if($ret === false) {
          return false;
        }
      }
    }
    return true;
  }
  
  public static function concatStrings($result, $delim = ',')
  {
    $string = '';
    if(!empty($result)) {
      foreach($result as $ret) {
        if(!empty($ret)) {
          if(strlen($string) > 0)
            $string += $delim;
          $string += $ret;
        }
      }
    }
    return $string;
  }
 
}



