<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class Recurring
{
	private $ordercode;
	private $userid;

	public function __construct()
	{
		$this->userid    = Factory::getApplication()->getIdentity()->id;
		$this->ordercode = intval(Factory::getSession()->get('ordercode'));
	}

	public function getRecordFromRecurringDb($name = null)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['*'])
		            ->from($db->quoteName('#__larts_subs_recurring'))
		            ->where($db->quoteName('userid') . ' = ' . $this->userid)
		            ->where($db->quoteName('name') . ' = ' . $db->quote($name));

		$db->setQuery($query, 0, 1);

		$data = $db->loadObject();

		return ! empty($data) ? $data->value : null;
	}

	public function addRecordToRecurringDb($name = null, $value = null, $product = null)
	{
		$recurring             = new \stdClass;
		$recurring->userid     = $this->userid;
		$recurring->name       = $name;
		$recurring->value      = $value;
		$recurring->product_id = $product;
		$recurring->added      = time();

		return Factory::getDbo()->insertObject('#__larts_subs_recurring', $recurring);
	}

	/**
	 * Getting all products in cart as these products needs to be inserted as recurring subscription at the PSP.
	 * We only need the recurring items, all others will be processed after the first purchase. The recurring items will be inserted as recurring subscription.
	 *
	 */
	public function getProductsInCart($ordercode)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['p.id', 'o.id', 'p.period', 'p.period_type', 'p.price', 'p.name', 'p.recurring'])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p.id') . ' = ' . $db->quoteName('o.product_id') . ')')
		            ->where($db->quoteName('o.ordercode') . ' = ' . $ordercode)
		            ->where($db->quoteName('o.product_id') . ' > 0')
		            ->group($db->quoteName('o.product_id'));

		$db->setQuery($query);

		return $db->loadObjectList();
	}
}