<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Router\Route;

class Redirect
{
	public $tblJoomlaContent;
	public $cid;
	public $itemid;
	public $thiscid;
	public $nextc;

	public function __construct()
  {
    if(!class_exists('ContentHelperRoute')) {
      require_once JPATH_SITE.'/components/com_content/helpers/route.php';
    }

		$app = Factory::getApplication();

		$this->cid = $app->input->get('cid', null, 'ARRAY');
		$this->itemid = $app->input->getUint('itemid');
		$this->thiscid = $app->input->getString("thiscid", null);
		$this->nextc = $app->input->getString("nextc", null);

		if($this->thiscid == null) {
			$menu = $app->getMenu();
			$item = $menu->getActive();
			$this->thiscid = @$item->query["id"];
		}
	} 

  public static function getIdFromAlias($alias)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
          ->select($db->quoteName('id'))
          ->from($db->quoteName('#__content'))
          ->where($db->quoteName('alias') . ' = ' . $db->quote($alias));
     $id = $db->loadResult(); 
     return $id;
  }

	function getTableJoomlaContent()
  {
		if(!isset($this->tblJoomlaContent)) {
			$this->tblJoomlaContent = Table::getInstance('Content', 'Table');
		}
		return $this->tblJoomlaContent;
	}

	private function jumpToId($nextcid, $msg = null)
  {
		$tcon = $this->getTableJoomlaContent();
		$article = $tcon->load($nextcid);
		$url = Uri::root() . \ContentHelperRoute::getArticleRoute($article->id . ":" . $article->alias, $article->catid);
		if(!empty($msg)) {
			Factory::getApplication()->enqueueMessage($msg);
		}
		$url = $this->SetPageItemId($url);
		Factory::getApplication()->redirect($url);
	}

	public static function toView($view, $task = null, $msg = null)
  {
 		$url = 'index.php?option=com_lartssubscriptions&view=' .$view;

		if(!empty($msg)) {
			Factory::getApplication()->enqueueMessage($msg);
		}
    if(!empty($task)) {
      $url .= '&task=' .$task;
    }
		Factory::getApplication()->redirect(Route::_($url));
	}

 	public static function getLinkFromMenuId($id)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['link'])
		            ->from($db->quoteName('#__menu'))
		            ->where($db->quoteName('id') .' = '.(int)$id);

		$db->setQuery($query);
		$item = $db->loadObject();
    
		$url = empty($item) ? '#' : Route::_($item->link);
    return $url;
	}

  function RedirectTo($url, $msg = null) {
		if(!empty($msg)) {
			Factory::getApplication()->enqueueMessage($msg);
		}
		$url = $this->SetPageItemId($url);
		Factory::getApplication()->redirect($url);
	}

	function RedirectToThis($msg = null) {
		if(!empty($msg)) {
			Factory::getApplication()->enqueueMessage($msg);
		}
		$this->jumpToId($this->thiscid);
	}

	function RedirectToNext($msg = null) {
		if(!empty($msg)) {
			Factory::getApplication()->enqueueMessage($msg);
		}
		$url = $this->GetURLToNext();
		$url = $this->SetPageItemId($url);
		Factory::getApplication()->redirect($url);
	}

	function GetURLToNext() {
		if(strlen($this->nextc) == 0) {
			$this->jumpToId($this->thiscid, 'Missing next id! (nextc=???)');
		}
		$con = $this->GetTableJoomlaContent();
		$nextcid = $con->getPlgContentId($this->nextc);
		if($nextcid == null) {
			$this->jumpToId($this->thiscid, 'Unknown next id (nextc='.$this->nextc.')');
		}
		$con->load($nextcid);
		$url = Uri::root() . \ContentHelperRoute::getArticleRoute($con->id . ":" . $con->alias, $con->catid, 140);

		return $url;
	}

	function GetURLToThis() {
		$con = $this->GetTableJoomlaContent();
		$con->load($this->thiscid);
		$url = Uri::root() . \ContentHelperRoute::getArticleRoute($con->id . ":" . $con->alias, $con->catid);
		return $url;
	}

	function AppendAllData($url, $data) {
		$link = new Uri($url);
		foreach ($data as $key => $value) {
  		$link->setVar($key, $value);
		}
		$url = $link->toString();
		return $url;
  }

	function AppendAllData_POST($url) {
		$data = Factory::getApplication()->GetInput()->post->getArray();
    return $this->AppendAllData($url, $data);
  }
	function AppendAllData_GET($url) {
		$data = Factory::getApplication()->GetInput()->get->getArray();
    return $this->AppendAllData($url, $data);
  }

  function SetPageItemId($url) {
		$link = new Uri($url);
		$link->setVar('Itemid', $this->itemid);
		$url = $link->toString();
		return $url;
	}	
  
  
}

