<?php
/**
 * @package         Logical-Arts
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Layout\FileLayout;

class UpdateKey
{
	public function __construct()
	{
	}

	/** 
   * Showing an overview of installed products/extensions and their update state.
	 */
	public function preCheckUpdateKey($element = null, $name = null)
	{
		if(!$element)
		{
			return false;
		}

		$component = $this->getExtensionData($element);
		$extension = empty($name) ? $component->name : $name;
		$manifest  = json_decode($component->manifest_cache);

		$layout = new FileLayout('download_key_check');

		echo $layout->render([
			'extension'  => $extension,
			'type'       => $component->type,
			'version'    => $manifest->version,
			'state'      => ! empty($component->enabled) ? $component->enabled : null,
			'downloadid' => ! empty($component->extra_query) ? $component->extra_query : null,
			'available'  => ! empty($component->new_version) ? $component->new_version : null,
		]);
	}

	/**
	 * Getting the update server from the XML manifest file.
	 */
	public function getUpdaterURLFromXMLManifest($path = null)
	{
		$object = simplexml_load_file($path);
		$result = json_decode(json_encode($object));

		return ! empty($result->updateservers->server) ? $result->updateservers->server : null;
	}

  public function loadUpdateSiteElement($id)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select('*')
      ->from($db->quoteName('#__update_sites'))
      ->where($db->quoteName('update_site_id') .'='. $id);

    $db->setQuery($query);
    return $db->loadObject();
  }

  public function getPackageInfo($extension)
  {
    $extension = $this->getExtensionByElement($extension);
    $extension_id = $extension->extension_id;
    if($extension->package_id <= 0) {
      return $extension;
    }
    // part of a package...
    $extension = $this->getExtensionById($extension->package_id);
    return $extension;
  }

  public function getExtensionInfo($extension)
  {
    $extension = $this->getExtensionByElement($extension);
    return $extension;
  }
  
  public function updateOrAddDownloadId($element = null, $key = null, $url = null, $type = 'extension')
	{
		$db = Factory::getDbo();

    // Getting the extension data.
    $extension = $this->getExtensionByElement($element);
    $extension_id = $extension->extension_id;
    if($extension->package_id > 0) {
      // part of a package...
      $extension_id = $extension->package_id;
    }
    
		$component = $this->getExtensionData($extension_id);
		if($component == null || empty($component->update_site_id)) {
			return false;
		}

    $extension = $this->loadUpdateSiteElement($component->update_site_id);
    
    if($url != null) {
  		$extension->location = $url;
      $extension->enabled = 1;
    }
    $extension->last_check_timestamp = 0;
		$extension->extra_query = empty($key) ? '' : 'key=' . $key;

		return $db->updateObject('#__update_sites', $extension, 'update_site_id');
	}

	/** Get extension information from the update tables.
	 */
	public function getExtensionData($extension_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->select([
			                  'e.extension_id', 'e.type', 'e.name', 'e.manifest_cache', 'us.update_site_id', 'us.enabled', 'us.extra_query',
			                  'u.version as new_version',
		                  ])
		                  ->from($db->quoteName('#__extensions', 'e'))
		                  ->join('LEFT OUTER', $db->quoteName('#__update_sites_extensions', 'use') . ' ON (' . $db->quoteName('e.extension_id') . ' = ' . $db->quoteName('use.extension_id') . ')')
		                  ->join('LEFT OUTER', $db->quoteName('#__update_sites', 'us') . ' ON (' . $db->quoteName('us.update_site_id') . ' = ' . $db->quoteName('use.update_site_id') . ')')
		                  ->join('LEFT OUTER', $db->quoteName('#__updates', 'u') . ' ON (' . $db->quoteName('u.update_site_id') . ' = ' . $db->quoteName('use.update_site_id') . ')')
		                  ->where($db->quoteName('e.extension_id') . ' = ' . $db->quote($extension_id));

		$db->setQuery($query);

		return $db->loadObject();
	}

 	public function getExtensionByElement($element, $type = 'component')
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->select('*')
		                  ->from($db->quoteName('#__extensions'))
		                  ->where($db->quoteName('type') . ' = ' . $db->quote($type))
		                  ->andWhere($db->quoteName('element') . ' = ' . $db->quote($element));

		$db->setQuery($query);

		return $db->loadObject();
	}

 	public function getExtensionById($extension_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->select('*')
		                  ->from($db->quoteName('#__extensions'))
		                  ->where($db->quoteName('extension_id') . ' = ' . $db->quote($extension_id));

		$db->setQuery($query);

		return $db->loadObject();
	}

}