<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Base\Database;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Coupon;
use LartsSubscriptions\Discount;


class Cart
{
	public $ordercode = 0;
  public $products = null;
  public $coupon_code = '';
  public $hasLifetime = false;
  public $hasOnetime = false;
  public $config = null;
  public $orderedItems = null;
  public $orderedItemsSimple = null;
 

	public function __construct($ordercode = 0)
	{
		$this->config = LartsConfig::get();

    // Getting ordercode from the session
		$this->ordercode = $ordercode ?: intval(Factory::getSession()->get('ordercode'));
  }

  public function isEmpty() {
    return empty($this->products);
  }

  function getHasLifetime() {
		return $this->hasLifetime;
	}

  function getHasOnetime() {
		return $this->hasOnetime;
	}

  public function getProducts($bIncludeCoupons = false)
  {
    if($this->products == null) {
      $this->products = $this->getProductList($bIncludeCoupons);
    }
    return $this->products;
  }
  
  public function addChartItem($record)
  {
		$db = Factory::getDbo();
    $obj = (object)$record;
    return $db->insertObject('#__larts_subs_cart_items', $obj);
  }
  
  public function updateCartItem($record)
  {
    return Database::update('#__larts_subs_cart_items', 'id', $record);
  }
  
  public static function setLicence($id, $licence) {
    $record = [ 'id' => $id, 'licence' => $licence];
    return Database::update('#__larts_subs_cart_items', 'id', $record);
  }
          
 	public function updateCartItemProductLic($product_id, $lic)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->update($db->quoteName('#__larts_subs_cart_items'))
                      ->set($db->quoteName('licence') . ' = ' . (int) $lic)
		                  ->where([
			                  $db->quoteName('product_id') . ' = ' . (int) $product_id,
			                  $db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
		                  ]);
    if($lic == 0) {
       $query->set($db->quoteName('licence') . ' = NULL');
    }

		$db->setQuery($query);

		return $db->execute();
  }
  
	/**
	 * Updating the quantity in the cart when a client clicks the update button/dropdown.
	 * @return bool
	 */
	public function updateOrderQuantity($product_id, $quantity)
	{
		//$orders = $this->getItemsForProductId($ordercode, $product_id);
		//$total  = count($orders);
    //		$total = $this->getItemsInCartForProductId($ordercode, $product_id);
    //		if ($total == $quantity) {
    //			return true;
    //		}

    if($quantity == 0) {
      return false;
    }
    
		return $this->updateProductQuantity($product_id, $quantity);
    
    //		if ($total > $quantity)
    //		{
    //			return $this->removeOrdersByCount($orders, $total - $quantity);
    //		}
    //
    //		return $this->addOrdersByCount($orders, $quantity - $total);
	}

  public function updateProductQuantity($product_id, $quantity)
	{
     if($quantity == 0) {
      return $this->removeProduct($product_id);
    }
    
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->update($db->quoteName('#__larts_subs_cart_items'))
		                  ->set($db->quoteName('quantity') . ' = ' . (int)$quantity)
		                  ->where([
			                  $db->quoteName('product_id') . ' = ' . (int) $product_id,
			                  $db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
		                  ]);

		$db->setQuery($query);
		return $db->execute();
	}

	public function remoeAll($ordercode)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		                  ->delete($db->quoteName('#__larts_subs_cart_items'))
		                  ->where($db->quoteName('ordercode') . ' = ' . $ordercode);
		$db->setQuery($query);

		return $db->execute();
	}

	// Removing specific ordered products from a cart.
	public function removeProduct($product_id, $ordercode = null)
	{
    if($ordercode == null) {
      $ordercode = $this->ordercode;
    }
    
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->delete($db->quoteName('#__larts_subs_cart_items'))
		                  ->where($db->quoteName('product_id') . ' = ' . (int) $product_id)
			                ->andWhere($db->quoteName('ordercode') . ' = ' . (int) $this->ordercode);

		$db->setQuery($query);

		return $db->execute();
	}

	// Getting the amount of orders for
	public function getProduct($product_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		                  ->select('*')
		                  ->from($db->quoteName('#__larts_subs_cart_items'))
		                  ->where([
			                  $db->quoteName('product_id') . ' = ' . (int) $product_id,
			                  $db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
		                  ]);

		$db->setQuery($query);
    return $db->loadObject();
	}

 	// Getting all cart items
	public function getItems($selects = ['*'])
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select($selects)
			->from($db->quoteName('#__larts_subs_cart_items'))
			->where($db->quoteName('ordercode') .' = '. (int)$this->ordercode)
			;
		$db->setQuery($query);

    return $db->loadObjectList();
	}

  // Checking the amount of ordered items in cart.
	public function getTotals()
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['COUNT(id) as items', 'SUM(quantity) as total_qty', 'SUM(price * quantity) AS totalprice', 'sum(discount) AS discount', 'SUM(vat) as vat_total'])
		            ->from($db->quoteName('#__larts_subs_cart_items'))
		            ->where($db->quoteName('ordercode') . ' = ' . $this->ordercode);

		$db->setQuery($query);
		$totals = $db->loadObject();
    if($totals->items == 0) {
      $totals->total_qty = 0;
      $totals->totalprice = 0;
      $totals->discount = 0;
      $totals->vat_total = 0;
    }
    return $totals;
	}

  // Checking the amount of ordered items in cart.
	public function getItemCount()
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['COUNT(id) as total'])
		            ->from($db->quoteName('#__larts_subs_cart_items'))
		            ->where($db->quoteName('ordercode') . ' = ' . $this->ordercode);

		$db->setQuery($query);
		$result = $db->loadObject();

		return ($result->total > 0) ? $result->total : 0;
	}

	public function checkForDiscount()
	{
		$data = $this->getItems(['SUM(price) AS price', 'SUM(discount) AS discount']);

		if (empty($data) || $data[0]->price == 0 && $data[0]->discount == 0) {
			return false;
		}

		return true;
	}
  
//  public function getProducts()
//	{
//		$query = $db->getQuery(true)
//        ->select(['cart.quantity', 'p.name', 'p.price', 'p.vat_class'])
//        ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
//        ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p.id') . ' = ' . $db->quoteName('cart.product_id') . ')')
//        ->where($db->quoteName('cart.ordercode') . ' = ' . (int)$this->ordercode)
//        ->where($db->quoteName('cart.product_id') . ' > 0')
//        ->group($db->quoteName('cart.product_id'))
//        ->order($db->quoteName('p.name') . ' ASC');
//
//		$db->setQuery($query);
//		return $db->loadObjectList();
//	}
	public function getProductList($bIncludeCoupons = false)
	{
		if (empty($this->ordercode)) {
			return [];
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select([
			            'cart.*',
			            'p.name as product_name', 'p.alias', 'p.type', 'p.price', 'p.period', 'p.period_type', 'p.vat_class', 'p.has_licence'
		            ])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('cart.product_id') . ' = ' . $db->quoteName('p.id') . ')')
		            ->where($db->quoteName('cart.ordercode') . ' = ' . (int)$this->ordercode)
		            ->order('cart.id ASC');
    if(!$bIncludeCoupons) {
      $query->andWhere($db->quoteName('cart.product_id') . ' > 0');
    }
		$db->setQuery($query);

		return $db->loadObjectList();
	}

	// prepare cart items in front of getting the list to show at cart.
  // this will ensure bundled products will be removed and
  // discounts calculated properly
  public function prepareCartItems()
	{
		if (empty($this->ordercode)) {
			return [];
		}

		$db = Factory::getDbo();

		$has_children = $db->getQuery(true)
		                   ->select(['COUNT(bundled_product_id)'])
		                   ->from($db->quoteName('#__larts_subs_products_bundled'))
		                   ->where($db->quoteName('product_id') . ' = ' . $db->quoteName('cart.product_id'));

		$query = $db->getQuery(true)
		            ->select([
			            'cart.*',
			            'p.name', 'p.price', 'p.period', 'p.period_type', 
		            ])
		            ->select('(' . $has_children . ') AS has_children')
		            ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('cart.product_id') . ' = ' . $db->quoteName('p.id') . ')')
		            ->where($db->quoteName('cart.ordercode') . ' = ' . $this->ordercode)
		            ->where($db->quoteName('cart.product_id') . ' > 0')
		            ->order($db->quoteName('has_children') . ' DESC');

		$db->setQuery($query);
		$cartitems = $db->loadObjectList();

		$this->removeChildProductsOfGroups($cartitems);
		$this->removeLimitedOneTimeProducts($cartitems);

		$discount = new Discount($this->ordercode);
		$discount->apply($cartitems);

	}
  
  public function removeChildProductsOfGroups(&$cartitems)
	{
		foreach ($cartitems as $i => &$item)
		{
			// Product has no children, so can not be an upgrade
			if (!$item->has_children)
			{
				continue;
			}

			$children = BundledProducts::getChildrenById($item->product_id);

			$this->removeChildProductsFromList($cartitems, $item, $children);
		}
	}

  /**
	 * Checking if a customer has already bought this onetime offer before.
	 * If you can buy it only once, it needs to be removed from the cart.
	 */
	private function removeLimitedOneTimeProducts(&$cartitems)
	{
		if(!Factory::getApplication()->getIdentity()->id) {
			return;
		}

		foreach ($cartitems as $i => &$product)
		{
			// Only one time products needs to be checked
			if ($product->period_type != 'onetime') {
				continue;
			}

			// look for previous subscription
      $userid = Factory::getApplication()->getIdentity()->id;
      // Customer is not logged in, so we need to check it later on.
      if ($userid > 0) {
        $result = Periods::lookupProduct($userid, $product->product_id);

        if(!empty($result))
        {
          $this->removeProduct($product->product_id);
          unset($cartitems[$i]);

          Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_ORDER_ITEM_REMOVED_NOT_ALLOWED', $product->name), 'notice');
        }
      }
    }

		return;
	}

	public function removeChildProductsFromList(&$products, $parent, $children)
	{
		foreach ($products as $i => &$product)
		{
			if(!isset($children[$product->product_id])) {
				continue;
			}

			$this->removeProduct($product->product_id);
			unset($products[$i]);

			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_ORDER_ITEM_REMOVED_UPGRADE', $product->name, $parent->name), 'notice');
		}
	}

  
	public function getCoupons()
	{
		if (empty($this->ordercode)) {
			return [];
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['cart.*'])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
		            ->where($db->quoteName('cart.ordercode') . ' = ' . (int)$this->ordercode)
                ->andWhere($db->quoteName('cart.product_id') . ' = 0')
		            ->order('cart.id ASC');
		$db->setQuery($query);

		return $db->loadObjectList();
	}

  public function applyCoupon($coupon)
	{
  }
  
	public function getProductCount()
	{
    return count($this->getProducts());
//		$query = $db->getQuery(true)
//			->select($db->quoteName('id'))
//			->from($db->quoteName('#__larts_subs_cart_items'))
//			->where($db->quoteName('ordercode') . ' = ' . (int) $this->ordercode)
//			->where($db->quoteName('product_id') . ' > 0');
//
//		$db->setQuery($query, 0, 1);
//		return $db->loadResult();
	}

	public function getProductsListStr()
	{
		$products = self::getProducts();

		$products_desc = [];
		foreach ($products as $product) {
			$products_desc[] = $product->product_name . ($product->quantity > 1 ? ' (' . $product->quantity . 'x)' : '');
		}
		return implode(', ', $products_desc);
	}

	public function getFullOrderCoupon()
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
      ->select($db->quoteName('coupon_code'))
			->from($db->quoteName('#__larts_subs_cart_items'))
			->where([
				$db->quoteName('product_id') . ' = 0',
				$db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
			]);

		$db->setQuery($query);
		return $db->loadResult();
	}
  
  public function getOrderedItemsHTML($simple = false)
  {
    if(empty($this->orderedItemsSimple))
    {
  		$db = Factory::getDbo();
      $query = $db->getQuery(true)
                  ->select(['o.id', 'p.name as productname', 'p.period_type'])
                  ->from($db->quoteName('#__larts_subs_cart_items', 'o'))
                  ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p.id') . ' = ' . $db->quoteName('o.product_id') . ')')
                  ->where($db->quoteName('o.ordercode') . ' = ' . $this->ordercode);

      $db->setQuery($query);
      $data = $db->loadObjectList();

      $this->getOrderedItemsListHTML($data);
    
    }
		return $simple ? $this->orderedItemsSimple : $this->orderedItems;
  }

  public function getOrderedItemsListHTML($data)
	{
		$this->orderedItemsSimple = '<ul>';
		$this->orderedItems       = '<ul>';

		foreach ($data as $row)
		{
			if ($row->productname == '') {
				continue;
			}

			if ($row->period_type == 'lifetime') {
				$this->hasLifetime = true;
			}

			if ($row->period_type == 'onetime') {
				$this->hasOnetime = true;
			}

			$this->orderedItemsSimple .= '<li>' . $row->productname . '</li>';
			$this->orderedItems       .= '<li>[' . $row->id . '] [' . $this->ordercode . '] ' . $row->productname . '</li>';
		}

		$this->orderedItemsSimple .= '</ul>';
		$this->orderedItems       .= '</ul>';
	}

 	public function removeCouponsFromOrder()
	{
		$this->removeFullOrderDiscount();
		$this->updateCouponCode('');
	}

  private function removeFullOrderDiscount()
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->delete($db->quoteName('#__larts_subs_cart_items'))
			->where([
				$db->quoteName('product_id') . ' = 0',
				$db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
			]);

		$db->setQuery($query);
		return $db->execute();
	}

 	public function updateCouponCode($coupon_code = '')
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__larts_subs_cart_items'))
			->set($db->quoteName('coupon_code') . ' = ' . $db->quote($coupon_code))
			->where($db->quoteName('ordercode') . ' = ' . (int) $this->ordercode);

		$db->setQuery($query);
		$db->execute();
	}

  public function setExtraInfo($item_id, $extra)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__larts_subs_cart_items'))
			->set($db->quoteName('extra_info') . ' = ' . $db->quote($extra))
			->where($db->quoteName('id') . ' = ' . (int) $item_id);

		$db->setQuery($query);
		$db->execute();
  }
}



  