<?php
/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;

use LartsSubscriptions\Product;


class Categories {
  const TYPE_PRODUCT = 0;
  const TYPE_SUBSCRIPTION = 1;
  
  public function __construct() {
  }

  public static function load($id)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_categories'))
		            ->where($db->quoteName('id') .'='. $db->quote($id));
		$db->setQuery($query);
		return $db->loadObject();
  }

  public static function removeImage($id)
  {
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_categories'))
		            ->set($db->quoteName('image') . ' = null')
		            ->where($db->quoteName('id') . ' = ' .$id);
		$db->setQuery($query);
    //$sql = $query->__toString();
    
		return $db->execute();
  }

  public static function getChildCategories($parentCat = 0, $type = self::TYPE_PRODUCT)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('*')
			->from($db->quoteName('#__larts_subs_categories'))
			->where($db->quoteName('parentid') . '=' .$db->quote($parentCat))
			->where($db->quoteName('published') . ' = 1')
			->andWhere($db->quoteName('cattype') . '=' .$type)
			->order('ordering ASC');
   
    
		$db->setQuery($query);

		return $db->loadObjectList();
  }

 	public static function getProducts($category)
	{
    if($category == 0) {
      $products = Product::selectProductsNotCategoriesed();
      Product::prepare($products);

      return $products;
    }

    return Product::selectProductsOfCat($category);
	}

  public static function getHash($id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('hash')
		            ->from($db->quoteName('#__larts_subs_categories'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);
		return $db->loadObject();
	}

  public static function fromHash($hash)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_categories'))
		            ->where($db->quoteName('hash') . ' = ' . $db->quote($hash));

		$db->setQuery($query);
		return $db->loadObject();
	}

  public static function getFiles($category)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('f.*')
			->from($db->quoteName('#__larts_subs_file_categories', 'c'))
      ->rightJoin($db->quoteName('#__larts_subs_files', 'f') . ' ON (' . $db->quoteName('c.file_id') . ' = ' . $db->quoteName('f.id') . ')')
      ->where($db->quoteName('c.category_id') . '=' .$db->quote($category))
			//->order('file.name ASC')
      ;

		$db->setQuery($query);

		return $db->loadObjectList();
	}

  public static function getPathway($id)
  {
    $db = Factory::getDbo();
    
    $path = array();
    $path[] = ['name' => 'Home',
               'id' => 0,
               'parentid' => 0,
              ];

    $reverse = array();
    while($id != 0)
    {
      $query = $db->getQuery(true)
                  ->select(['id', 'name', 'parentid'])
                  ->from($db->quoteName('#__larts_subs_categories'))
                  ->where($db->quoteName('id') . ' = ' . (int) $id);
      $db->setQuery($query);
      $cat = $db->loadObject();
      if(empty($cat)) {
        break;
      }
      $reverse[] = [ 'name' => $cat->name,
                     'id' => $cat->id,
                     'parentid' => $cat->parentid,
                    ];
      $id = $cat->parentid;
    }
    
    $path = array_merge($path, array_reverse($reverse));
    
    return $path;
  }
  
  /**
   *  Get all cats, this file is member of
   */
  public static function getCategoires($file_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('c.id')
		            ->from($db->quoteName('#__larts_subs_file_categories', 'fc'))
                ->rightJoin($db->quoteName('#__larts_subs_categories', 'c') . ' ON (' . $db->quoteName('fc.category_id') . ' = ' . $db->quoteName('c.id') . ')')
		            ->where($db->quoteName('fc.file_id') . ' = ' .$db->quote($file_id))
		            ->andWhere($db->quoteName('c.cattype') . ' = ' .self::TYPE_SUBSCRIPTION)
                ;

		$db->setQuery($query);
		return $db->loadObjectList();
	}

  public static function getMaxItemId()
	{
 		$db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select('MAX(ordering)')
      ->from('#__larts_subs_categories');
    $db->setQuery($query);
    return $db->loadResult();
	}

}


