<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

/*
 * Transactions_Id
 * Transaktion-Token = md5(ordercode)
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\Session\Session;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\Redirect;
use LartsSubscriptions\Base\Country;
use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Base\PluginManager;

use LartsSubscriptions\UserInfo;
use LartsSubscriptions\Message;
use LartsSubscriptions\Cart;
use LartsSubscriptions\LartsSubsApp;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Logger;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\Ticket;
use LartsSubscriptions\Base\UserRegistration;
use LartsSubscriptions\UserRegState;

/**
 * After ordering processing
 * At this stage, order is pending
 * Next steps depend on Payment-Processor
 * - Offline payment
 * - Paypal
 * - Other
 * 1. Create an invoice
 * 2. Send Invoice
 * 3. Call Payment-Processor 
 *    NewOrderPending($ordercode)
 *    - Offline: do nothing (customer has invoice)
 *    - Paypal: process payment
 * 4. Call Payment-Processor (Payment processed)
 *    FinalizeOrder($ordercode)
 */


class CheckoutManager
{
  var $userid;
  var $ordercode;
  var $input;
  
	public function __construct($ordercode = 0)
	{
		//$this->userid    = Factory::getApplication()->getIdentity()->id;
		$this->ordercode = $ordercode ?: intval(Factory::getSession()->get('ordercode'));
 		$this->input = Factory::getApplication()->GetInput();

    // get the user of the order
    $this->userid = Order::getUserId($this->ordercode);

	}
  
  public function doCheckout()
  {
		// Check for request forgeries.
    Factory::getSession()->checkToken() or jexit(Text::_('JINVALID_TOKEN'));

		$app = Factory::getApplication();
    LartsSubsApp::loadLanguage('com_lartssubscriptions', true);

		$config = LartsConfig::get();
    if($config->use_demouser) {
      if(Factory::getApplication()->getIdentity()->id == 0) {
        $name = $this->input->Get('username');
        $this->input->set('password', $name);
        $this->input->set('password2', $name);
      }
    }
    
    // save form data into a registry entry
    $post = $this->input->post->getArray();
		UserRegState::setStateData($post);

    $this->userid = Factory::getApplication()->getIdentity()->id;
    
    $formdata = new \stdClass;
    $formdata->username = $this->input->getString('username', '');
    $formdata->password = $this->input->get('password', '', 'raw');
    $formdata->password2 = $this->input->get('password2', '', 'raw');
    $formdata->accepted_tos = $this->input->getBool('accepted_tos', false);
    $formdata->firstname = $this->input->getString('firstname', '');
    $formdata->lastname = $this->input->getString('lastname', '');
    $formdata->address1 = $this->input->getString('address1', '');
    $formdata->address2 = $this->input->getString('address2', '');
    $formdata->zipcode = $this->input->getString('zipcode', '');
    $formdata->city = $this->input->getString('city', '');
    $formdata->phone = $this->input->getString('phone', '');
    $formdata->emailaddress = $this->input->getString('emailaddress', '');
    $formdata->country_code = $this->input->get('country_code', '');
    $formdata->isbusiness = $this->input->get('isbusiness', 0);
    $formdata->companyname = $this->input->getString('companyname', '');
    $formdata->vatnumber = $this->input->getString('vatnumber', '');
    $formdata->vatnumber2 = $this->input->getString('vatnumber2', '');
    $formdata->processor = $this->input->getString('payment_method', 'paylater');

    if(!$this->prepareCheckout($formdata)) {
      $app->redirect(Route::_('index.php?option=com_lartssubscriptions&view=checkout'));
      return false;
    }

    if(empty($formdata->processor)) {
      // payment method needed...
      $app->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_PAYMENT_METHOD'), 'error');
      $app->redirect(Route::_('index.php?option=com_lartssubscriptions&view=checkout'));
      return false;
    }

 		// order
    if($this->createOrder($formdata) == 0) {
      // something went wrong...
      $app->enqueueMessage(Text::_('COM_LARTSSUBS_CREATE_ORDER_FAILED'), 'error');
      $app->redirect(Route::_('index.php?option=com_lartssubscriptions&view=checkout'));
      return false;
    }

    // handle coupon
    $cart = new Cart;
    $coupons = $cart->getCoupons($this->ordercode, 'ordercode');
    if(!empty($coupons))
    {
      foreach($coupons as $coupon) {
         $couponcode = $coupon->coupon_code;
         (new Coupon)->updateCouponUsage($couponcode);
       }
    }
    
    // payment processing...
    $this->StartPayment(true, $formdata->processor);

  }

  public function prepareCheckout($formdata)
  {
		$app = Factory::getApplication();
    $bNeedRegister = empty($this->userid);
    
    if(!$this->validateUserInput($formdata, $bNeedRegister)) {
			$app->redirect(Route::_('index.php?option=com_lartssubscriptions&view=checkout'));
			return false;
		}
    
    // new user?
    if($bNeedRegister)
    {
      $data = [
        'username' => $formdata->username,
        'password' => $formdata->password,
        'name'     => $formdata->firstname . ' ' . $formdata->lastname,
        'email'    => $formdata->emailaddress,
      ];

      if(!$this->RegisterNewUser($data))
			{
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_REGISTER_FAILED'), 'error');
   			return false;
			}

      // Reset the userstate - Registration is completed.
			UserRegState::clearStateData();

			// We store the new userid at the regstate, so the user can be removed 
			// (and the username/eMail adress freed) if something is going wrong.
 			$regdata = ['userid' => $this->userid,
									'ordercode' => $this->ordercode,
								 ];
			UserRegState::setStateData($regdata);
			
		}
    else {
      // logged-in user...
      if(!$this->checkOneTimeItems($this->userid)) {
        return false;
      }
    }

		// Note: at this point $this->userid is valid

    // update user informations
    $this->saveUserDetails($this->userid, $formdata);



    return true;
  }

  public function checkOneTimeItems($userid)
  {
		$app = Factory::getApplication();
    $cart = new Cart;
    $items = $cart->getProductList(false);
    
    foreach ($items as $item)
    {
      // Check if this is a one time product and the customer hasn't subscribed before.
      if ($item->period_type == 'onetime')
      {
        $result = Order::lookupProductInOrders($userid, $item->product_id);

        if (!empty($result)) {
          //Redirect the customer back. No product to add as he subscribed before.
          $app->enqueueMessage(Text::sprintf('COM_LARTSSUBS_CANT_SUBSRIBE_MORE_THAN_ONCE', $item->product_name), 'error');
          return false;
        }
      }
    }
    return true;
  }
  
  public function createOrder($formdata, $bAsOffer = false)
  {
  	// Create new order
    $vatnumber = $this->getVATNumber($formdata);

    $countrycode = $formdata->country_code;
    if (empty($countrycode) && LartsConfig::get('use_default_country')) {
			$countrycode = LartsConfig::get('default_country');
		}
    
    $id = Order::create($this->ordercode, $this->userid, $vatnumber, $countrycode, $bAsOffer);
    return $id;
  }

  public function getVATNumber($formdata)
  {
    $vatnumIn = $formdata->vatnumber;
    if(!empty($vatnumIn)) {
      $vatnumber = $formdata->country_code . $vatnumIn;
    }
    else {
      $vatnumber = $formdata->vatnumber2;
    }
    $vatnumber = $vatnumber ?: '';
    return $vatnumber;
  }
 
  public function createTicket($data, $bSend = false)
  {
    $ticket = new Ticket;
    $ticket_id = $ticket->store($data);
    if($ticket_id == 0) {
			return 0;
		}
    $ticket->createTicketPdf($ticket_id);
    
    if($bSend == true) {
      $ticket->SendTicket($ticket_id);
    }

    return $ticket_id;
  }
  
  public function createInvoice($data, $bSend = false)
  {
		$invoice = new Invoice;

    // store invoice record
    $invoice_id = $invoice->store($data, $data['paid']);
    if($invoice_id == 0) {
			return 0;
		}

    // create pdf
    $invoice->createInvoicePdf($invoice_id);
    
    if($bSend == true) {
      $this->sendInvoice($invoice_id);
    }

    return $invoice_id;
  }

  public function createOffer($data, $bSend = false)
  {
		$offer = new Offer;

    // store invoice record
    $invoice_id = $offer->store($data, $data['paid']);
    if($invoice_id == 0) {
			return 0;
		}

    // create pdf
    $offer->createOfferPdf($invoice_id);
    
    if($bSend == true) {
      $offer->sendOffer($invoice_id);
    }

    return $invoice_id;
  }

  public function sendInvoice($invoice_id, $template = 'thanks-for-ordering')
  {
    $invoice = new Invoice;
    return $invoice->sendInvoice($invoice_id, $template);
  }

  public function sendOffer($invoice_id, $template = 'send-offer')
  {
    $offer = new Offer;
    return $offer->sendOffer($invoice_id, $template);
  }

  public function StartPayment($bRedirect, $processor)
  {
    $amount = new OrderAmounts($this->ordercode, false);

		// If the amount to pay is 0.00 force to free checkout.
		if ($amount->netprice() == 0) {
			$processor = 'freecheckout';
		}
    
		// Triggers plugin onBeforePaymentRedirect
		$data = [
      'ordercode' => $this->ordercode,
			'userid'    => $this->userid,
			'method'    => $processor,
			'amount'    => $amount,
		];

    // Trigger plugins
		PluginManager::execPlugins('lartssubscriptions', 'onBeforePaymentRedirect', $data);

		if($bRedirect) {
      Factory::getApplication()->redirect(Route::_('index.php?option=com_lartssubscriptions&view=payment&processor=' .$processor));
    }
  }
  
  public function doPostCheckout($response)
  {
		// called from the plugins for post processing
    Logger::_('CheckoutManager', 'doPostCheckout: ' . var_export($response, true));

    $ordercode = $response['ordercode'];


    $config = LartsConfig::get();

    $order_state = Order::getStatus($ordercode);
    $order_paid = $order_state & OrderStatus::PAID;
    Logger::_('CheckoutManager', 'order_state: ' .$order_state);
    
    if($order_paid)
    {
      // process if pending...
      if($order_state & OrderStatus::PENDING)
      {
        Logger::_('CheckoutManager', 'processOrderedItems...');
        $this->processOrderedItems($response['transaction_id']);
      }
      
			
			// Activate the user if it is or was blocked during registration
	    //$userreg = new UserRegistration;
		  //$userreg->ActivateUserByUserId($userid);

    }

		// send the user registration mail or remove user (if new user)
		UserRegState::postProcessCheckout($order_state);

 
    $order = Order::lookup($ordercode);
    $amounts = json_decode($order->amounts, true);

    $orderstateAppend = 0;
  	if ($config->use_invoicing) {

      // create and send invoice
      $data = [
        'userid'           => $response['userid'],
        'ordercode'        => $ordercode,
        'provider'         => $response['payment_type'],
        'brutto_price'     => $amounts['gross_price'],
        'transaction_id'   => $response['transaction_id'],
        'vat_number'       => $order->vat_number,
        'vat_number'       => $order->vat_number,
        'paid'             => $order_paid ? Invoice::PAID : Invoice::UNPAID,
        'invoice_type'     => Invoice::TYPE_DEBIT,
      ];
      if($response['payment_type'] == 'PayLater') {
        $data['invoice_type'] = Invoice::TYPE_CREDIT;
      }
      
      $invoice_id = $this->createInvoice($data, false);
      $orderstateAppend |= OrderStatus::INVOICE_CREATED;
      
      switch ($response['payment_type']) {
      case 'PayLater':
        $template = 'order-banktransfer';
        break;
      case 'PayPalExpress':
      default:
        $template = 'thanks-for-ordering';
        break;
      }

      if($this->sendInvoice($invoice_id, $template)) {
        $orderstateAppend |= OrderStatus::INVOICE_SENT;
      }
    }
		else
		{
			// don't use invoicing)
      // -> send a thank you eMail
			
      $cart = new Cart($ordercode);
      $cart->getOrderedItemsHTML();
      
      // Setting some variables for the messages:
      $variables = [
        'transaction_id'   => $response['transaction_id'],
        'ordercode'        => $ordercode,
        'date'             => 'NOW',
        'price'            => $amounts['net_price'],
        'orderlist'        => $cart->getOrderedItemsHTML(false),
        'orderlist_simple' => $cart->getOrderedItemsHTML(true),
        'has_lifetime'     => $cart->getHasLifetime(),
        'has_onetime'      => $cart->getHasOnetime(),
      ];
      
			$message = new Message;
			$message->alias('thanks-for-ordering')
			        ->user($response['userid'])
			        ->variables($variables)
			        ->send();
		}
  
    Order::setStatus($ordercode, $orderstateAppend, true);
   
		$this->clearSession();
  }

  public function saveUserDetails($userid, $formdata)
  {
    $data = [
      'userid'        => $userid,
      'firstname'     => $formdata->firstname,
      'lastname'      => $formdata->lastname,
      'address1'      => $formdata->address1,
      'address2'      => $formdata->address2,
      'zipcode'       => $formdata->zipcode,
      'city'          => $formdata->city,
      'country_code'  => $formdata->country_code,
      'phone'         => $formdata->phone,
      'is_business'   => $formdata->isbusiness,
      'companyname'   => $formdata->companyname,
      'vat_number'    => $this->getVATNumber($formdata),
      //'newsletter'  => $formdata->newsletter,
      'published'     => 1,
    ];

    // Trigger plugins
		PluginManager::execPlugins('lartssubscriptions', 'onBeforeSaveUserDetails', $data);
    
 		// store user info to joomla.users
    UserInfo::saveOrUpdate($data);

		// Extending data array with the ordercode for plugin
		$data['ordercode'] = $this->ordercode;

		PluginManager::execPlugins('lartssubscriptions', 'onAfterSaveUserDetails', $data);
    
  }

  public function RegisterNewUser($data)
  {
    $app = Factory::getApplication();

    if(!$this->validateUserRegistration($data)) {
      return false;
    }
    
    // Trigger plugins
		if(!PluginManager::execPlugins('lartssubscriptions', 'onStartRegister', $data)) {
			$app->enqueueMessage(Text::sprintf('COM_LARTSSUBS_REGISTRATION_FAILED', 'Plugin'), 'error');
			return false;
    }

    $registration = new UserRegistration;
		$registration->setBlockUser(0);
 		$registration->setNewUserByArray($data, false);
    $this->userid = $registration->getuserId();
    
		if ($this->userid == 0) {
			$app->enqueueMessage(Text::sprintf('COM_LARTSSUBS_REGISTRATION_FAILED', $registration->getError()), 'error');
			return false;
		}

    // Trigger plugins
		PluginManager::execPlugins('lartssubscriptions', 'onPostRegister', ['userid' => $this->userid]);

    return true;
  }
  
 	public function validateUserInput($formdata, $bNeedRegistration = false)
	{
		$app = Factory::getApplication();

		if ($bNeedRegistration) {
      if(!$this->validateUserRegData($formdata)) {
        return false;
      }
		}

    if(!$this->validateUserAdress($formdata)) {
      return false;
    }

    if(LartsConfig::get('require_tos_accept') == 1) {
        if(!$formdata->accepted_tos) {
        $app->enqueueMessage(Text::_('COM_LARTSSUBS_PLEASE_ACCEPT_OUT_TERMS_AND_CONDITIONS'), 'error');
        return false;
      }
    }

		return true;
	}

  public function validateUserRegData($formdata)
	{
		$app = Factory::getApplication();

    if ($formdata->username == '') {
      $app->enqueueMessage(Text::_('COM_LARTSSUBS_FILL_USERNAME'), 'error');
      return false;
    }

    if(!preg_match('/^[\w\.\-\_]+$/', $formdata->username)) {
      $app->enqueueMessage(Text::_('COM_LARTSSUBS_USERNAME_INCORRECT_ALLOWED_CHARS'), 'error');
      return false;
    }

    $pass = $this->input->get('password', '', 'raw');
    $pass2 = $this->input->get('password2', '', 'raw');
    
    if (empty($formdata->password) || empty($formdata->password2)) {
      $app->enqueueMessage(Text::_('COM_LARTSSUBS_PLEASE_FILL_PASSWORD'), 'error');
      return false;
    }

    if ($formdata->password != $formdata->password2) {
      $app->enqueueMessage(Text::_('COM_LARTSSUBS_PASSWORD_DO_NOT_MATCH'), 'error');
      return false;
    }
    return true;
  }
  
 	public function validateUserAdress($formdata)
	{
		$app = Factory::getApplication();

    if(!filter_var($formdata->emailaddress, FILTER_VALIDATE_EMAIL)) {
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_NOT_VALID'), 'error');
			return false;
		}

		if ($formdata->emailaddress == '') {
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_FILL_EMAIL_ADDRESS'), 'error');
			return false;
		}

		if ($formdata->firstname == '') {
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_ENTER_YOUR_FIRSTNAME'), 'error');
			return false;
		}

		if ($formdata->lastname == '') {
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_ENTER_YOUR_LASTNAME'), 'error');
			return false;
		}

		if ($formdata->address1 == '' && LartsConfig::get('show_1st_addressline') == 1) {
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_ENTER_YOUR_ADDRESS'), 'error');
			return false;
		}

		if ($formdata->city == '' && LartsConfig::get('show_city') == 1) {
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_CHOOSE_A_CITY'), 'error');
			return false;
		}

    if(LartsConfig::get('show_country') == 1) {
      if(!preg_match('^[A-Z]{2}$^', $formdata->country_code)) {
        $app->enqueueMessage(Text::_('COM_LARTSSUBS_CHOOSE_A_COUNTRY'), 'error');
        return false;
      }
    }
  
		return true;
  }

	private function validateUserRegistration($data)
	{
    $userreg = new UserRegistration;
    
    if ($userreg->isExsistingUser('email', $data['email'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_IN_DB'), 'error');
      return $this->userid;
		}
    if ($userreg->isExsistingUser('name', $data['username'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USERNAME_IN_DB'), 'error');
			return $this->userid;
		}
		return true;
	}
  
	public function processOrderedItems($payment_id)
	{
    $ordercode = $this->ordercode;
    $userid = $this->userid;
    $config = LartsConfig::get();

		Logger::_('process_ordered_items', ['begin']);

    $db = Factory::getDbo();
    // get all cart items of this order...
    $query = $db->getQuery(true)
		            ->select([
			            'cart.id', 'cart.product_id', 'cart.licence', 'cart.quantity',
			            'prod.id as product_id, prod.type as product_type, prod.period', 'prod.period_type', 'prod.price', 'prod.name as productname', 'prod.product_ref','prod.pkgmanager_id','prod.has_licence','prod.source_id',
                  'ord.created',
		            ])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('prod.id') . ' = ' . $db->quoteName('cart.product_id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_orders', 'ord') . ' ON (' . $db->quoteName('ord.ordercode') . ' = ' . $db->quoteName('cart.ordercode') . ')')
		            ->where($db->quoteName('cart.ordercode') . ' = ' . $ordercode)
		            ->where($db->quoteName('cart.product_id') . ' > 0')
		            ->group($db->quoteName('cart.product_id'));

		$db->setQuery($query);
		$items = $db->loadObjectList();

		foreach ($items as $item)
		{
      $item->product_ref = !empty($item->product_ref) ? $item->product_ref : $item->product_id;
      $item->access_group = 0;
      
      $subscription = true;
      switch($item->product_type) {
      case Product::TICKET:
      case Product::ANALOG:
        $subscription = false;
        break;
      case Product::GROUP_ACCESS:
        $item->access_group = $item->source_id;
        break;
      }

      if($subscription && $config->LicMngrInstalled) {
        if($item->has_licence && $item->licence == 0) {
          // Prepare data for the plugin:
          $plg_data = [
            'productname'  => $item->productname,
            'product_id'   => (int) $item->product_id,
            'product_ref'  => (int) $item->product_ref,
            'userid'       => (int) $userid,
            'period'       => (int) $item->period,
            'period_type'  => (int) $item->period_type,
          ];

          // Trigger plugins
          $plgReturns = PluginManager::execPlugins('lartssubscriptions', 'onCreateNewLicence', $plg_data, false, false);
          if(!empty($plgReturns)) {
            // walk back to front and grab the first licence string found
            foreach(array_reverse($plgReturns) as $plgRet) {
              if(!empty($plgRet)) {
                $item->licence = $plgRet;
                break;
              }
            }
          }

          // if none of the plpugins gave us a licence string, we create one...
          if(empty($item->licence)) {
            $item->licence = strtoupper(uniqid());
          }

          Logger::_('create_new_licence', ['licence' => $item->licence]);
        }
      }
      
      $newPeriod = [ 'id'         => 0,
                     'valid_from' => '',
                     'valid_to'   => ''
                   ];
      if($subscription) {
        $periods = new Periods;
        $nCount = $item->quantity * $item->period;
        $newPeriod = $periods->addPeriod($item->created, $ordercode, $item->product_id, $item->product_ref, $userid, $item->licence, 
                                         $item->period_type, $nCount);
        Logger::_('process_ordered_items', ['period' => $newPeriod]);

        // update chart-item
        $cart = new Cart;
        $record = [
          'id'         => (int)$item->id,
          'period_id' => (int)$newPeriod['id'],
        ];
        $cart->updateCartItem($record);
      }
      else if($item->product_type == Product::TICKET) {
        // create ticket
        $data = ['userid'     => $userid,
                 'ordercode'  => $ordercode,
        				 'product_id'   => (int) $item->product_id,
                ];
        for($i = 0; $i < $item->quantity; $i++) {
          $this->createTicket($data);
        }
      }
      
			// Prepare data for the plugin:
			$plg_data = [
        'cart_item'     => $item,
				'product_id'    => (int) $item->product_id,
				'product_type'  => (int) $item->product_type,
				'product_ref'   => (int) $item->product_ref,
        'pkgmanager_id' => $item->pkgmanager_id,
        'has_licence'   => $item->has_licence,
				'userid'        => (int) $userid,
				'ordercode'     => (int) $ordercode,
				'licence'       => $item->licence,
        'valid_from'    => $newPeriod['valid_from'],
				'valid_to'      => $newPeriod['valid_to'],
				'payment_id'    => $payment_id,
				//'download_id'   => strtoupper(uniqid()),
        'period_id'     => (int)$newPeriod['id'],
				'order_id'      => (int) $item->id,
        'access_group'  => $item->access_group,
			];
			// Trigger plugins
			PluginManager::execPlugins('lartssubscriptions', 'onAfterProcessOrderedItem', $plg_data);

			unset($item);
		}

    // mark order as processed
    Order::setStatus($ordercode, OrderStatus::PROCESSED, true);

    
		$plg_data = [
			'ordercode'  => (int) $ordercode,
			'userid'     => (int) $userid,
			'payment_id' => $payment_id,
		];

    // Trigger plugins
		PluginManager::execPlugins('lartssubscriptions', 'onAfterProcessedAllItems', $plg_data);
    
		return true;
	}

	public function clearSession()
	{
		$session = Factory::getSession();
		$session->clear('coupon');
		$session->clear('ordercode');

		return true;
	}

  public static function getOrderData($ordercode)
  {
    return Order::getOrderData($ordercode);
  }

	//	Processing Free Checkout Order.
	public function processFreeCheckout($ordercode, $userid = null)
	{
    $config = LartsConfig::get();

    $attachment = '';

		if(!$userid) {
			// Getting the user id as it is empty.
			$userid = Order::getUserId($ordercode);
		}

		// Stop if there are no orders in cart.
    $cart = new Cart($ordercode);
    if($cart->getItemCount() == 0) {
			return ['subject' => Text::_('COM_LARTSSUBS_NO_ORDERS_IN_CART'), 'message' => Text::_('COM_LARTSSUBS_NO_ORDERS_IN_CART_DESC')];
		}

    $orderstateAppend = OrderStatus::PAID;
        
		// Process the ordered products first;
    $this->processOrderedItems('-free checkout-');

		UserRegState::postProcessCheckout(OrderStatus::PAID);
		
    // Create an invoice if this is a 100% discount order?
		$check_discount = $cart->checkForDiscount();

		if ($check_discount === true && $config->use_invoicing)
		{
      $data = [
        'userid'           => $userid,
        'ordercode'        => $ordercode,
        'provider'         => 'Free Checkout',
        'brutto_price'     => 0,
        'transaction_id'   => '',
        'paid'             => Invoice::PAID,
        'invoice_type'     => Invoice::TYPE_DEBIT,
        'vat_number'       => '',
      ];

      $ivoice_id = $this->createInvoice($data, false);
      $orderstateAppend |= OrderStatus::INVOICE_CREATED;

			$invoice = new Invoice;
      $invoicedata = $invoice->load($ivoice_id);

      $file = $invoice->getPath() . $invoicedata->invoice_number . '.pdf';
			if (file_exists($file)) {
				$attachment = $file;
        $orderstateAppend |= OrderStatus::INVOICE_SENT;
			}
		}

    Order::setStatus($ordercode, $orderstateAppend, true);

    // Variables for the message:
    $cart = new Cart($ordercode);
		$variables = [
      'transaction_id'   => 0,
			'ordercode'        => $ordercode,
			'date'             => 'NOW',
      'price'            => 0,
      'orderlist'        => $cart->getOrderedItemsHTML(false),
      'orderlist_simple' => $cart->getOrderedItemsHTML(true),
			'has_lifetime'     => $cart->getHasLifetime(),
			'has_onetime'      => $cart->getHasOnetime(),
		];

		// Create the message & subject and replace all variables:
		$message = new Message;
		$message->alias('free-product-activation')
		        ->user($userid)
		        ->variables($variables);
		if ($attachment) {
			// Add the attachment if there is one.
			$message->attachment($attachment);
		}
		$message->send();

		// Upgrade: Maybe this should be changed to an extra Page template: thanks-for-ordering-page-free
		$message = new Message;
		$message = $message->alias('thanks-for-ordering-page')
		                   ->user($userid)
		                   ->variables($variables);

    // Clear session
		$this->clearSession();

		return [
			'subject' => $message->getSubject(),
			'message' => $message->getBody(),
		];
	}

  public function saveCheckoutFormData()
  {
    // save form data into a registry entry
    $app = Factory::getApplication();
    $data = $app->getInput()->post->getArray();
    if(isset($data['coupon_code'])) {
      unset($data['coupon_code']);
    }
    if(isset($data['button'])) {
      unset($data['button']);
    }
    if(isset($data['option'])) {
      unset($data['option']);
    }
    if(isset($data['task'])) {
      unset($data['task']);
    }
    $app->setUserState('com_lartssubscriptions.registration', $data);
  }

  public function getCheckoutFormData()
  {
    $app = Factory::getApplication();
 		$post = $app->getUserState('com_lartssubscriptions.registration');
  }

  public function clearCheckoutForm()
  {
 		$app->setUserState('com_lartssubscriptions.registration', null);
  }

}





 
 
 
 
