<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;

use LartsSubscriptions\ThirdParty\LartsLicManager;
use LartsSubscriptions\ThirdParty\LartsPkgManager;


class Config
{
	static $config = null;

	public static function getJCfg($key) {
    $jcfg = new \JConfig;
		return $jcfg->$key;
  }
  
	/*
	 * The ComponentHelper::getParams() only collects the saved params
	 * This method also grabs the default values from the config.xml
	 * And returns a simple object ($key/value) for easy usage
	 */
	public static function get($key = '')
	{
		if ($key) {
			$config = self::get();
			return isset($config->{$key}) ? $config->{$key} : '';
		}

		if (!is_null(self::$config)) {
			return self::$config;
		}

		$params = ComponentHelper::getParams('com_lartssubscriptions');

		$xml = self::getXml(JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/config.xml');

		$config = new \stdClass;

		// Walk through all settings found in xml
    // array_walk(array, callback userdata)
		array_walk($xml,
			function ($value, $key, $data) {
				// Set entry in $config using the value from $params with the $xml default value as fallback
				$data['config']->{$key} = $data['params']->get($key, $value);
			},
			['config' => &$config, 'params' => $params]
		);

    // get info about installed packages related
    $config->LicMngrInstalled = LartsLicManager::IsInstalled();
    $config->PkgMngrInstalled = LartsPkgManager::IsInstalled();
    
		self::$config = $config;

		return self::$config;
	}

	public static function save($key, $value)
	{
		$config = ComponentHelper::getParams('com_lartssubscriptions');
		$config->set($key, $value);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__extensions'))
		            ->set($db->quoteName('params') . ' = ' . $db->quote(json_encode($config)))
		            ->where($db->quoteName('name') . ' = ' . $db->quote('com_lartssubscriptions'));

		$db->setQuery($query);

		return $db->execute();
	}

	private static function getXml($path = '')
	{

		$xml_parser = xml_parser_create();
		xml_parse_into_struct($xml_parser, file_get_contents($path), $fields);
		xml_parser_free($xml_parser);

		$xml = [];

		foreach ($fields as $field)
		{
			if ($field['tag'] != 'FIELD'
				|| ! isset($field['attributes'])
				|| ! isset($field['attributes']['NAME'])
				|| $field['attributes']['NAME'] == ''
				|| $field['attributes']['NAME']['0'] == '@'
				|| ! isset($field['attributes']['TYPE'])
				|| $field['attributes']['TYPE'] == 'spacer'
			)
			{
				continue;
			}

			$xml[$field['attributes']['NAME']] = isset($field['attributes']['DEFAULT']) ? $field['attributes']['DEFAULT'] : '';
		}

		return $xml;
	}

}