<?php
/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Base\CronBase;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Coupon;
use LartsSubscriptions\SubscriptionManager;
use LartsSubscriptions\Statistics;


class Cronjob extends CronBase
{
  const TASK_PROCESS_QMAILS = 'processQMails';
  const TASK_CHECK_REMINDERS = 'checkReminders';
  const TASK_AUTODELETE_COUPONS = 'autodeleteCoupons';
  const TASK_CALC_STATISTICS = 'calcStatistics';
  
	private $current_key = null;
  public $config = null;
  public $cronpass = null;
  

  public function __construct()
  {
    parent::__construct();

    $this->table = '#__larts_subs_croninfo';

    $this->current_key = Factory::getApplication()->GetInput()->get('key', null, 'string');
		$this->config = LartsConfig::get();
		$this->cronpass = $this->config->cron_pass;

		// White listed IP addresses.
    if(!empty($this->whitelisted_ips)) {
  		$this->whitelisted_ips = explode(',', str_replace(' ', '', $this->config->whitelisted_ips));
    }
  }

  public function runPseudo()
  {
    // check doRunPseudo result and run only one task per call

    if($this->config->pseudo_cron_check_reminders && $this->doRunPseudo(self::TASK_CHECK_REMINDERS)) {
      return;
    }
    if($this->config->pseudo_cron_send_qmails && $this->doRunPseudo(self::TASK_PROCESS_QMAILS)) {
      return;
    }
    if($this->config->pseudo_cron_autodelete_coupons && $this->doRunPseudo(self::TASK_AUTODELETE_COUPONS)) {
      return;
    }
    if($this->config->pseudo_cron_stats && $this->doRunPseudo(self::TASK_CALC_STATISTICS)) {
      return;
    }
  }

  public function doRunPseudo($task)
  {
    if(!defined('_DEBUG')) {
      if(!$this->checkCronInfo($task)) {
        return false; // nothing to do...
      }
    }

    $this->delegateCron($task);
    return true;
  }
  
  public function run()
	{
    if(empty($this->cronpass)) {
      Logger::_('cronjob', Text::_('COM_LARTSSUBS_CRON_PASSWORD_REQUIRED'));
      exit('Subs-Cron-Error 1' .PHP_EOL);
    }

    // Check for the key or if the IP is whitelisted
		if (!$this->checkExec($this->current_key)) {
      exit('Subs-Cron-Error 2'.PHP_EOL);
		}
    
    $task = Factory::getApplication()->GetInput()->get('cron');
   
    if(!defined('_DEBUG')) {
      if(!$this->checkCronInfo($task))
        die;  // nothing to do...
    }
    
    $this->delegateCron($task);
    exit('END'.PHP_EOL);
  }
  
 	public function delegateCron($task)
	{
    $bRet = false;
    
    switch($task) {
    case self::TASK_PROCESS_QMAILS:
      $bRet = $this->processQMails();
      break;
    case self::TASK_CHECK_REMINDERS:
      $bRet = $this->checkReminders();
      break;
    case self::TASK_AUTODELETE_COUPONS:
      $bRet = $this->removeCoupons();
      break;
    case self::TASK_CALC_STATISTICS:
      if($this->config->calculate_statistics) {
        $stats = new Statistics;
        $bRet = $stats->calculateStatistics();
      }
      break;
    default:
      echo 'Unknown task: '. $task .PHP_EOL;
      Logger::_('cronjob', Text::_('COM_LARTSSUBS_CRON_UNKNOWN_TASK').': ' .$task);
    }

    if($bRet) {
      $cron = new Cronjob;
      $cron->updateCronInfo($task);
    }
    
	}

  public function removeCoupons()
	{
    if(!$this->config->autodelete_coupons) {
      Logger::_('cronjob', Text::_('COM_LARTSSUBS_MAILER_DISABLED'));
      return false;
    }
    $coupon = new Coupon;
    $coupon->removeOldAutoCoupons();
    $coupon->removeExpiredCoupons();
    return true;
  }
  
  public function processQMails()
	{
		$subs = new SubscriptionManager;
		if(!$subs->sendQMail()) {
      return false;
    }
    return true;
  }
  
  private function checkReminders()
	{
    echo 'Checking reminders...' .PHP_EOL;
		$subscription = new SubscriptionManager;

		// Adding/Sending Reminders
		if ($this->config->send_first_reminder)
		{
			$from = $this->config->send_second_reminder && $this->config->second_expiration_days
				? $this->config->second_expiration_days + 5
				: 1;
			$from = min($from, $this->config->first_expiration_days);

      echo ' - First reminder...' .PHP_EOL;
			$subscription->sendReminder('first', $this->config->first_expiration_discount, 
                                  $this->config->first_expiration_days, $from);
		}

		if ($this->config->send_second_reminder)
		{
			$from = $this->config->send_second_reminder && $this->config->third_expiration_days
				? $this->config->third_expiration_days + 5
				: 1;
			$from = min($from, $this->config->second_expiration_days);

      echo ' - Second reminder...' .PHP_EOL;
			$subscription->sendReminder('second', $this->config->second_expiration_discount, 
                                  $this->config->second_expiration_days, $from);
		}

		if ($this->config->send_third_reminder)
		{
      echo ' - Third reminder...' .PHP_EOL;
			$subscription->sendReminder('third', $this->config->third_expiration_discount, 
                                  $this->config->third_expiration_days, 1);
		}

    return true;
	}


}
