<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Http\HttpFactory;
use Joomla\Filesystem\File;
//use Joomla\Filesystem\Folder;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\GeoIP\GeoIP;
use LartsSubscriptions\Periods;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Offer;
use LartsSubscriptions\Ticket;


//jimport('joomla.filesystem.file');

use LartsSubscriptions\Base\CleanTemporaryFiles;

class Download
{
  const DOWNLOAD_FILE = 0;
  const DOWNLOAD_INVOICE = 1;
  const DOWNLOAD_TICKET = 2;
  const DOWNLOAD_STREAM = 3;
  
  private $filepath = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/downloads/';
  public $md5_filename = '';

	function check($userid = null, $downloadid = null)
	{
		$products    = [];
		$access_list = [];


    return $access_list;
	}

	/**
	 * Logging the download in the database.
	 */
	function logging($uid, $remote_address, $filename, $type, $request_type)
	{
		// Getting the country code from GeoIP database.
    $country_code = (new GeoIP)->getCountryByIP();

		// Create and populate an object.
		$download_log               = new \stdClass;
		$download_log->userid       = $uid;
		$download_log->ip_address   = $remote_address;
		$download_log->action       = $filename;
		$download_log->type         = $type;
		$download_log->country_code = ! empty($country_code) ? $country_code : null;
		$download_log->request_type = $request_type;

		Factory::getDbo()->insertObject('#__larts_subs_logging', $download_log);

		return true;
	}

	/**
	 * Updating the log downloads
	 * @param $file_id
	 */
	private function updateCounter($file_id)
	{
		$db = Factory::getDbo();

		$fields     = [$db->quoteName('downloads') . ' = downloads+1'];
		$conditions = [$db->quoteName('id') . ' = ' . (int) $file_id];

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_files'))
		            ->set($fields)
		            ->where($conditions);

		$db->setQuery($query);
		$db->execute();

		return true;
	}

	/**
	 * Returning the proper file path based on the download type.
	 */
	public function getFilePath()
	{
		return $this->filepath;
	}

	/**
	 * Loading the file information and perform the needed actions based on the file info.
	 */
	public function fileInformation($file_id, $incDlCounter = true)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            'filename_physical', 'filename', 'filename_orig', 'alias',
			            'ispublic', 'extension',
			            'download_link', 'download_url', 'source_file_type', 
                  'use_filename','use_own_filename', 'keep_original_filename',
		            ])
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('id') . ' = ' . (int) $file_id)
		            ->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query);

		$result = $db->loadObject();

		if (empty($result)) {
			return [];
		}

		// Update download counter
    if($incDlCounter) {
      $this->updateCounter($file_id);
    }

		// Check if this is a local file, if yes.. return it.
		if(!in_array($result->source_file_type, ['link']))
		{
			return $result;
		}

		// Download the file from the external URL. (direct link)
		if ($result->source_file_type === 'link')
		{
			// Download the file from the given URL
			if(!$this->downloadFromURL($result->download_url, $result->alias)) {
				return false;
			}
		}

		// Setting the tmpfilename
		$result->filename_physical = $result->alias . '.zip';

		return $result;
	}


	// Preparing the download of an invoice.
	public function downloadInvoice($invoice_id)
	{
    $invoice = new Invoice;

    // Removing old downloads first. This waty we can keep a clean tmp folder.
		$cleanup = new CleanTemporaryFiles;
		$cleanup->removeTemporaryPDFFiles();

		$invoice_info = $invoice->getInvoiceById($invoice_id);
		$invoice_info->invoice_path = $invoice->getPath();

    $invoice_file = $invoice->getPath() . $invoice_info->invoice_number . '.pdf';

		// The file has been stored locally
		if (file_exists($invoice_file)) {
			return $invoice_info;
		}

		// Fallback to obtain the invoice.
		if(!file_exists($invoice_file)) {
      if($invoice_info->invoice_type == Invoice::TYPE_OFFER) {
        // Create the offer:
        $offer = new Offer;
        $offer->createOfferPdf($invoice_id);
      }
      else {
        // Create the invoice:
        $invoice->createInvoicePdf($invoice_id);
      }
		}

		return $invoice_info;
	}

  public function prepareDownloadTicketById($ticket_id)
	{
    $ticket = Ticket::load($ticket_id);
    return $this->downloadTicket($ticket);
  }
  
	public function prepareDownloadTicket($ticket)
	{
		// Removing old downloads first. This waty we can keep a clean tmp folder.
		$cleanup = new CleanTemporaryFiles;
		$cleanup->removeTemporaryPDFFiles();

    $ticket_file = Ticket::getFilepath($ticket->id);

		// The file has been stored locally
		if (file_exists($ticket_file)) {
			return $ticket_file;
		}

		// Fallback to obtain the invoice.
		if(!file_exists($ticket_file)) {
			// Create the ticket:
			$t = new Ticket;
			$t->createTicketPdf($ticket->id);
		}

		return $ticket_file;
	}

  // Download a file by URL, file is teporary saved in the tmp folder.
	private function downloadFromURL($url, $name)
	{
		$cleanup = new CleanTemporaryFiles;

		// Removing old downloads first. This waty we can keep a clean tmp folder.
		$cleanup->removeTemporaryZipFiles();

		// Setting the new download path.
		$this->filepath = Factory::getConfig()->get('tmp_path') . '/';

		// Download the package from the external location.
		$package = HttpFactory::getHttp()->get($url, null, 30);

		if(!$package || $package->code != 200 || empty($package->body))
		{
			return false;
		}

		// Getting the temporary path from the config.
		$download = Factory::getConfig()->get('tmp_path') . '/' . $name;
		$this->md5_filename = $name;

		// Writing the new file to this tmp location.
		if(!File::write($download, $package->body))
		{
			return false;
		}

		return true;
	}


 	function getTicketData($invoice_id)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_tickets'))
		            ->where($db->quoteName('id') . ' = ' . (int) $invoice_id);

		$db->setQuery($query);

		return $db->loadObject();
	}

	/**
	 * Preparing the download of an ticket.
	 */
	public function downloadTicket($ticket_id)
	{
    $ticket = new Ticket;

    // Removing old downloads first. This waty we can keep a clean tmp folder.
		$cleanup = new CleanTemporaryFiles;
		$cleanup->removeTemporaryPDFFiles();

		$ticket_info = $this->getTicketData($ticket_id);
    
    if(empty($ticket_info)) {
      return false;
    }

    $ticket_info->ticket_path = $ticket->getPath();
    $ticket_info->ticket_file = $ticket->getFilepath($ticket_id);

    // The file has been stored locally
    if (file_exists($ticket_info->ticket_file)) {
      return $ticket_info;
    }

    // Fallback to obtain the ticket.
    // Create the ticket:
    if(!$ticket->createTicketPdf($ticket_id)) {
      return false;
    }

		return $ticket_info;
	}

}

