<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Export;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use LartsSubscriptions\Base\PluginManager;
use Joomla\Filesystem\Folder;


class Export
{
	protected $db;
	protected $headers = [];
	protected $columns = [];
	protected $filename;
	protected $data;
	protected $tabname;
  protected $xlsx;

	public function __construct()
	{
		$this->db = Factory::getDbo();

		$this->xlsx = new XLSXWriter;
	}

	/**
	 * Write data to the Excel file which will hold all data later on
	 */
	public function writeToExcelFile($sheets)
	{
		if(!$this->performFolderCheck()) {
			return false;
		}

		$this->removeXlSxFiles();

		$this->filename = JPATH_ROOT . '/tmp/com_lartssubscriptions/' . $this->filename . '.xlsx';

		foreach ($sheets as $sheet)
		{
			if (empty($sheet['rows']))
				continue;

			$headers = !empty($sheet['headers']) ? $sheet['headers'] : $this->headers;
			$columns = !empty($sheet['columns']) ? $sheet['columns'] : $this->columns;

			$this->xlsx->writeSheetHeader($sheet['name'], $headers, $this->getHeaderStyling($columns));

			foreach ($sheet['rows'] as $row)
			{
				$this->xlsx->writeSheetRow($sheet['name'], (array) $row, $this->getRowStyling());
			}

			$this->xlsx->writeToFile($this->filename);
		}

		return true;
	}

	/**
	 * Checking if the folder is present on the server.
	 */
	private function performFolderCheck()
	{
		if(!Folder::exists(JPATH_ROOT . '/tmp/com_lartssubscriptions/'))
		{
			if(!Folder::create(JPATH_ROOT . '/tmp/com_lartssubscriptions/'))
			{
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_CREATE_TMP_FOLDER'), 'danger');

				return false;
			}
		}

		return true;
	}

	/**
	 * Removing old XLSX files as we don't want to make a mess on the client hosting :)
	 *
	 * @since 2.9.2
	 */
	private function removeXlSxFiles()
	{
		// cycle through all files in the directory and remove older than 24hrs.
		foreach (glob(JPATH_ROOT . '/tmp/com_lartssubscriptions/' . '*.xlsx') as $file)
		{
			if (time() - filectime($file) > 86400)
			{
				unlink($file);
			}
		}
	}

	/**
	 * Getting the heade rstyling for the document.
	 */
	private function getHeaderStyling($columns = [])
	{
		return [
			'font-style' => 'bold',
			'font-size'  => 9.5,
			'font'       => 'Tahoma',
			'widths'     => $columns,
			'border'     => 'bottom',
			'fill'       => '#CCC',
			'color'      => '#000',
		];
	}

	/**
	 * Getting the row styling for the document.
	 *
	 * @return array
	 *
	 * @since 2.9.2
	 */
	private function getRowStyling()
	{
		return [
			'font-size' => 9.5,
			'font'      => 'Tahoma',
			'border'    => 'top,bottom',
		];
	}

	/**
	 * Prepare headers for the export.
	 */
	public function prepareHeaders($columns = [])
	{
		$exceptions = $this->headerExceptions();

		foreach ($columns as $column => $type)
		{
			if (array_key_exists($column, $exceptions))
			{
				$this->headers[$column] = $exceptions[$column];
				$this->columns[]        = 15;

				continue;
			}

			$this->headers[$column] = 'string';
			$this->columns[]        = 15;
		}
	}

	public function headerExceptions()
	{
		return [];
	}
  
 	/**
	 * Returnfing the filename for the created file.
	 */
	public function getFileNameForDownload()
	{
		return $this->filename;
	}

	public function download($filename = null)
  {
    if($filename == null) {
      $filename = $this->getFileNameForDownload();
    }

		if(!file_exists($filename)) {
      return false;
    }
    
		PluginManager::execPlugins('lartssubscriptions', 'onBeforeMossExport', $filename);

		header('Pragma: public');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($filename)) . ' GMT');
		header('Cache-Control: private', false);
		header('Content-Type: application/vnd.ms-excel');
		header('Content-Disposition: attachment; filename="' . basename($filename) . '"');
		header('Content-Transfer-Encoding: binary');
		header('Content-Length: ' . filesize($filename));
		header('Connection: close');
		readfile($filename);
    
 		exit();
  }
}