<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Export;
defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

class InvoiceExport extends Export
{
	public $context = 'com_lartssubs.invoices';
  public $message = '';
        
	public function __construct()
	{
		parent::__construct();

		$this->filename = 'export-invoices-' . date('Ymd');
	}


	/**
	 * Returning the filename for downloads.
	 */
	public function getFilename()
	{
		return $this->filename;
	}

	/**
	 * Export the invoice list as per configuration
	 */
	public function export()
	{
		$invoices = $this->getInvoiceList();

		$sheets[] = [
			'name' => Text::_('COM_LARTSSUBS_INVOICES'),
			'rows' => $invoices,
		];

		$this->writeToExcelFile($sheets);
	}

	/**
	 * Reporting the invoice list as per configuration
	 */
	public function report()
	{
		$input = Factory::getApplication()->GetInput();

		// Resetting the userstate from the search bar.
		Factory::getApplication()->setUserState($this->context . '.filter.provider', null);
		Factory::getApplication()->setUserState($this->context . '.filter.payment', null);
		Factory::getApplication()->setUserState($this->context . '.filter.country', null);

		// Setting the userstate which is being used in the query.
		Factory::getApplication()->setUserState($this->context . '.filter.year', $input->get('year', date('Y'), 'int'));
		Factory::getApplication()->setUserState($this->context . '.filter.quarter', $input->get('quarter', 1, 'int'));

		if ($input->get('all_invoices_tab', 0, 'int') == 1)
		{
			$sheets[] = [
				'name'    => Text::_('COM_LARTSSUBS_INVOICES'),
				'rows'    => $this->getInvoiceList(),
				'headers' => $this->headers,
				'columns' => $this->columns,
			];
		}

		if ($input->get('forced_vat_tab', 0, 'int') == 1)
		{
			$sheets[] = [
				'name'    => Text::_('COM_LARTSSUBS_FORCED_VAT'),
				'rows'    => $this->getInvoiceList(false, 1, '>'),
				'headers' => $this->headers,
				'columns' => $this->columns,
			];
		}

		if ($input->get('intra_community_tab', 0, 'int') == 1)
		{
			$sheets[] = [
				'name'    => Text::_('COM_LARTSSUBS_INTRA_COMMUNITY_XLS'),
				'rows'    => $this->getInvoiceList(1, 0, '='),
				'headers' => $this->headers,
				'columns' => $this->columns,
			];
		}

		if ($input->get('moss_invoices_tab', 0, 'int') == 1)
		{
			$sheets[] = [
				'name'    => Text::_('COM_LARTSSUBS_MOSS_INVOICES_XLS'),
				'rows'    => $this->getInvoiceList(1, 0, '>'),
				'headers' => $this->headers,
				'columns' => $this->columns,
			];
		}

		if ($input->get('moss_per_country_tab', 0, 'int') == 1)
		{
			$sheets[] = [
				'name'    => Text::_('COM_LARTSSUBS_MOSS_INVOICES_COUNTRY_XLS'),
				'rows'    => $this->getInvoicesGroupdByMossCountry(),
				'headers' => $this->headers,
				'columns' => $this->columns,
			];
		}

		if ($input->get('no_vat_invoices_tab', 0, 'int') == 1)
		{
			$sheets[] = [
				'name'    => Text::_('COM_LARTSSUBS_INVOICES_WITHOUT_VAT_XLS'),
				'rows'    => $this->getInvoiceList(0),
				'headers' => $this->headers,
				'columns' => $this->columns,
			];
		}

    if(empty($sheets)) {
      $this->message = Text::_('COM_LARTSSUBS_NOTHING_SELECTED');
      return false;
    }
    
		return $this->writeToExcelFile($sheets);
	}

	private function getInvoicesGroupdByMossCountry()
	{
		$app = Factory::getApplication();

		$year    = $app->getUserStateFromRequest($this->context . '.filter.year', 'filter_year', null);
		$quarter = $app->getUserStateFromRequest($this->context . '.filter.quarter', 'filter_quarter', null);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            'c.country', 'u.country_code', 'i.vat_percentage', 'SUM(net_price) AS netto', 'SUM(gross_price) AS brutto', 'SUM(vat_total) AS vat',
			            'COUNT(i.id) AS total',
		            ])
		            ->from($db->quoteName('#__larts_subs_invoices', 'i'))
		            ->join('LEFT OUTER', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('i.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('u.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')');

		if (is_numeric($year))
		{
			$query->where('YEAR( ' . $db->quoteName('i.invoicedate') . ' ) = ' . $year);
		}

		if (is_numeric($quarter))
		{
			if ($quarter == 1)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  1 AND 3');

			if ($quarter == 2)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  4 AND 6');

			if ($quarter == 3)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  7 AND 9');

			if ($quarter == 4)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  10 AND 12');
		}

		$query->where($db->quoteName('c.requires_vat') . ' = 1');
		$query->where($db->quoteName('c.force_vat_payment') . ' = 0');
		$query->where($db->quoteName('i.vat_total') . ' > 0');
		$query->group('u.country_code');

		$this->db->setQuery($query);

		$results = $this->db->loadObjectList();

		$this->prepareHeaders($results[0]);

		return $results;
	}

	/**
	 * Getting all the information based on the selection for writing it to Excel.
	 * Exra params are possible for the VAT report and given by another request.
	 */
	private function getInvoiceList($requires_vat = null, $forced_vat = null, $vat_condition = null)
	{
		$app = Factory::getApplication();

		$provider = $app->getUserStateFromRequest($this->context . '.filter.provider', 'filter_provider', null);
		$payment  = $app->getUserStateFromRequest($this->context . '.filter.payment', 'filter_payment', null);
		$country  = $app->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null);
		$year     = $app->getUserStateFromRequest($this->context . '.filter.year', 'filter_year', null);
		$quarter  = $app->getUserStateFromRequest($this->context . '.filter.quarter', 'filter_quarter', null);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select($this->getSelectForQuery())
		            ->from($db->quoteName('#__larts_subs_invoices', 'i'))
		            ->join('LEFT OUTER', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('i.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT OUTER', $db->quoteName('#__larts_subs_transactions', 't') . ' ON (' . $db->quoteName('t.transaction_id') . ' = ' . $db->quoteName('i.transaction_id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('u.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')');

		if(!empty($provider)) {
			$query->where($db->quoteName('t.type') . ' = ' . $db->quote($provider));
		}

		if (is_numeric($payment)) {
			if ($payment == 3) {
				$query->where($db->quoteName('i.refund_invoice') . ' > 0');
			}
			else {
				$query->where($db->quoteName('i.published') . ' = ' . (int) $payment);
			}
		}

		if (is_numeric($country)) {
			$query->where($db->quoteName('c.id') . ' = ' . $country);
		}

		if (is_numeric($year)) {

			$query->where('YEAR( ' . $db->quoteName('i.invoicedate') . ' ) = ' . $year);
		}

		if (is_numeric($quarter))
		{
			if ($quarter == 1) {
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  1 AND 3');
      }
			if ($quarter == 2) {
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  4 AND 6');
      }
			if ($quarter == 3) {
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  7 AND 9');
      }
			if ($quarter == 4) {
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  10 AND 12');
      }
		}

		if (is_numeric($requires_vat)) {
			$query->where($db->quoteName('c.requires_vat') . ' = ' . $requires_vat);
		}

		if (is_numeric($forced_vat)) {
			$query->where($db->quoteName('c.force_vat_payment') . ' = ' . $forced_vat);
		}

		if(!empty($vat_condition)) {
			$query->where($db->quoteName('i.vat_total') . ' ' . $vat_condition . ' 0');
		}

		$this->db->setQuery($query);

		$results = $this->db->loadObjectList();

		$this->prepareHeaders($results[0]);

		return $results;
	}

	/**
	 * Prepare headers for the export.
	 */
	public function prepareHeaders($columns = [])
	{
		$this->headers = [];
		$this->columns = [];

		$exceptions = $this->headerExceptions();

		foreach ($columns as $column => $type)
		{
			$this->columns[] = 16;

			if (array_key_exists($column, $exceptions))
			{
				$this->headers[$column] = $exceptions[$column];

				continue;
			}

			$this->headers[$column] = 'string';
		}
	}

	/**
	 * Getting the columns we want to download from the database.
	 */
	private function getSelectForQuery()
	{
		return [
			'i.id,
			i.invoice_number, 
			i.ordercode, 
			DATE(i.invoicedate) as date,
			CONCAT(u.firstname, " ", u.lastname) AS name, 
			u.address1, 
			u.zipcode, 
			u.city, 
			u.vat_number, 
			u.companyname as company,
			CONCAT(c.country, " (", c.country_2_code, ")") AS country, 
			i.vat_percentage, 
			i.net_price as netto_price, 
			i.gross_price as brutto_price, 
			i.vat_total, 
			i.disco_price as discount, 
			i.vat_percentage',
		];
	}

	/**
	 * Getting header exceptions.
	 */
	public function headerExceptions()
	{
		return [
			'userid'      => 'int',
			'items'       => 'int',
			'date'        => 'date',
			'netto_price' => 'price',
			'brutto_price' => 'price',
			'vat_total'  => 'price',
			'netto'       => 'price',
			'brutto'       => 'price',
			'vat'         => 'price',
			'discount'    => 'price',
		];
	}
}