<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Export;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\OrderStatus;


class OrdersExport extends Export
{
  var $data = null;
  
	public function __construct()
	{
		parent::__construct();

		$this->filename = 'export-orders'.date('Ymd');
	}

	public function queryData()
	{
		$db = Factory::getDbo();

    $select = [
      $db->quoteName('o.status') . ' as OrderStatus',
      $db->quoteName('o.ordercode') . ' as Ordercode',
      $db->quoteName('o.vat_number') . ' as VATNumber',
      $db->quoteName('o.country_code') . ' as CountryCode',
      $db->quoteName('o.ip_address') . ' as IPAddress',
      $db->quoteName('o.created') . ' as Created',
      $db->quoteName('t.status') . ' as TransactionStatus',
      $db->quoteName('t.transaction_id') . ' as TransactionID',
      $db->quoteName('t.details') . ' as Details',
      $db->quoteName('t.amount') . ' as Amount',
      $db->quoteName('t.date') . ' as Date',
      $db->quoteName('t.type') . ' as Type',
      $db->quoteName('t.email_paypal') . ' as eMailPayPal',
      $db->quoteName('t.plugin') . ' as Plugin',
      $db->quoteName('t.comment') . ' as Comment',
      $db->quoteName('t.deleted') . ' as Deleted',
      $db->quoteName('cust.userid') . ' as UserID',
      $db->quoteName('cust.firstname') . ' as Firstname',
      $db->quoteName('cust.lastname') . ' as Lastname',
      $db->quoteName('cust.address1') . ' as Address1',
      $db->quoteName('cust.address2') . ' as Address2',
      $db->quoteName('cust.zipcode') . ' as ZipCode',
      $db->quoteName('cust.city') . ' as City',
      $db->quoteName('cust.country_code') . ' as CountryCode',
      $db->quoteName('cust.is_business') . ' as IsBusiness',
      $db->quoteName('cust.companyname') . ' as Companyname',
      $db->quoteName('cust.vat_number') . ' as VATNumber',
      $db->quoteName('c.country') . ' as Country',
		];

		$query = $db->getQuery(true)
		                  ->select($select)
		                  ->from($db->quoteName('#__larts_subs_transactions', 't'))
		                  ->rightJoin($db->quoteName('#__larts_subs_orders', 'o'))
		                  ->leftJoin($db->quoteName('#__larts_subs_customers', 'cust') . ' ON (' . $db->quoteName('o.userid') . ' = ' . $db->quoteName('cust.userid') . ')'
                              . ' ON (' . $db->quoteName('t.ordercode') . ' = ' . $db->quoteName('o.ordercode') . ')')
      		            ->leftJoin($db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('cust.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')');
                      ;

		$db->setQuery($query);
    //$sql = $query->__toString();

		$this->data = $db->loadObjectList();

		$this->prepareHeaders($this->data[0]);
		$this->convertOrderState();

		$sheets[] = [
			'name' => 'Orders',
			'rows' => $this->data,
		];

		return $this->writeToExcelFile($sheets);
	}

	private function convertOrderState()
	{
		foreach ($this->data as &$item)
		{
      $status = OrderStatus::Str($item->OrderStatus);
			$item->OrderStatus = $status;
		}
	}

	/**
	 * Getting header exceptions.
	 */
	public function headerExceptions()
	{
		return [
			'userid'     => 'int',
			'items'      => 'int',
			'payment_id' => 'int',
			'created'    => 'date',
			'expire'     => 'date',
		];
	}
}