<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Export;
defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;

use LartsSubscriptions\OrderStatus;

class SubscriberExport extends Export
{
	public function __construct()
	{
		parent::__construct();

		$this->filename = 'export-subscribers'.date('Ymd');
	}

	/**
	 * Query the database for the selected subscriptions.
	 */
	public function queryData()
	{
    // SQL Access2PHP (1899.12.30 00-00-00)
    // SELECT j01vi_larts_subs_periods.valid_from, j01vi_larts_subs_periods.valid_to, j01vi_larts_subs_orders.userid, j01vi_larts_subs_orders.vat_number, j01vi_larts_subs_orders.ip_address, j01vi_larts_subs_orders.created, j01vi_larts_subs_orders.status, j01vi_larts_subs_customers.userid, j01vi_larts_subs_customers.firstname, j01vi_larts_subs_customers.lastname, j01vi_larts_subs_customers.address1, j01vi_larts_subs_customers.address2, j01vi_larts_subs_customers.zipcode, j01vi_larts_subs_customers.city, j01vi_larts_subs_customers.country
    // FROM ((j01vi_larts_subs_periods LEFT JOIN j01vi_larts_subs_products ON j01vi_larts_subs_periods.product_id = j01vi_larts_subs_products.id) LEFT JOIN j01vi_larts_subs_orders ON j01vi_larts_subs_periods.ordercode = j01vi_larts_subs_orders.ordercode) LEFT JOIN j01vi_larts_subs_customers ON j01vi_larts_subs_periods.userid = j01vi_larts_subs_customers.userid

    $query = ' SELECT j01vi_larts_subs_periods.valid_from, j01vi_larts_subs_periods.valid_to, j01vi_larts_subs_orders.userid, j01vi_larts_subs_orders.vat_number, j01vi_larts_subs_orders.ip_address, j01vi_larts_subs_orders.created, j01vi_larts_subs_orders.status, j01vi_larts_subs_customers.userid, j01vi_larts_subs_customers.firstname, j01vi_larts_subs_customers.lastname, j01vi_larts_subs_customers.address1, j01vi_larts_subs_customers.address2, j01vi_larts_subs_customers.zipcode, j01vi_larts_subs_customers.city, j01vi_larts_subs_customers.country'
            .' FROM ((j01vi_larts_subs_periods '
            . 'LEFT JOIN j01vi_larts_subs_products ON j01vi_larts_subs_periods.product_id = j01vi_larts_subs_products.id) '
            . 'LEFT JOIN j01vi_larts_subs_orders ON j01vi_larts_subs_periods.ordercode = j01vi_larts_subs_orders.ordercode) '
            . 'LEFT JOIN j01vi_larts_subs_customers ON j01vi_larts_subs_periods.userid = j01vi_larts_subs_customers.userid';


    
		$select = [
			$this->db->quoteName('per.userid') . ' as UserID',
			$this->db->quoteName('cust.firstname') . ' as Firstname',
			$this->db->quoteName('cust.lastname') . ' as Lastname',
			$this->db->quoteName('cust.address1') . ' as Address1',
			$this->db->quoteName('cust.address2') . ' as Address1',
			$this->db->quoteName('cust.zipcode') . ' as Zipcode',
			$this->db->quoteName('cust.city') . ' as City',
			$this->db->quoteName('cust.companyname') . ' as Company',
			$this->db->quoteName('cust.vat_number') . ' as VatNumber',
			//$this->db->quoteName('cc.country') . ' as Country',
			//$this->db->quoteName('cc.country_2_code') . ' as CountryCode',
			$this->db->quoteName('o.ip_address') . ' as IPAddress',
			//$this->db->quoteName('per.payment_id') . ' as PaymentID',
			$this->db->quoteName('per.ordercode') . ' as OrderCode',
			//$this->db->quoteName('per.download_id') . ' as DownloadID',
			$this->db->quoteName('per.created') . ' as StartDate',
			$this->db->quoteName('per.valid_from') . ' as PeriodStart',
			$this->db->quoteName('per.valid_to') . ' as PeriodEnd',
			$this->db->quoteName('o.status') . ' as OrderStatus',
			$this->db->quoteName('p.name') . ' as Product',
			//'CONCAT(' . $this->db->quoteName('p.period') . '  , ' . $this->db->quoteName('p.period_type') . ') as Period',
			//$this->db->quoteName('o.price') . ' as Paid',
			//$this->db->quoteName('o.discount') . ' as Discount',
			//$this->db->quoteName('o.discount_type') . ' as DiscountType',
			//$this->db->quoteName('o.discount_amount') . ' as DiscountAmount',
			//$this->db->quoteName('t.type') . ' as PaymentProvider',
		];

		$query = $this->db->getQuery(true)
		                  ->select($select)
		                  ->from($this->db->quoteName('#__larts_subs_periods', 'per'))
		                  ->join('LEFT', $this->db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $this->db->quoteName('per.product_id') . ' = ' . $this->db->quoteName('p.id') . ')')
		                  ->join('LEFT', $this->db->quoteName('#__larts_subs_orders', 'o') . ' ON (' . $this->db->quoteName('per.ordercode') . ' = ' . $this->db->quoteName('o.ordercode') . ')')
		                  ->join('LEFT', $this->db->quoteName('#__larts_subs_customers', 'cust') . ' ON (' . $this->db->quoteName('per.userid') . ' = ' . $this->db->quoteName('cust.userid') . ')');

		$this->db->setQuery($query);
    //$sql = $query->__toString();

		$this->data = $this->db->loadObjectList();

		$this->prepareHeaders($this->data[0]);
		$this->convertOrderState();

		$sheets[] = [
			'name' => 'Subscribers',
			'rows' => $this->data,
		];

		return $this->writeToExcelFile($sheets);
	}


	/**
	 * Convert the status value to a human readable text
	 */
	private function convertOrderState()
	{
		foreach ($this->data as &$item)
		{
      $status = OrderStatus::Str($item->OrderStatus);
			$item->OrderStatus = $status;
		}
	}

  /**
	 * Getting header exceptions.
	 */
	public function headerExceptions()
	{
		return [
			'userid'     => 'int',
			'items'      => 'int',
			'payment_id' => 'int',
			'created'    => 'date',
			'expire'     => 'date',
		];
	}
}