<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filesystem\File;

use LartsSubscriptions\Base\FileBase;

class Files extends FileBase
{
  const SOURCE_TYPE_LOCAL = 0;
  const SOURCE_TYPE_EXTERN = 1;
  const SOURCE_TYPE_UPLOAD = 2;

	private $path = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/downloads';

	public function setBasePath($path)
	{
		if(!empty($path)) {
			$this->path = $path;
		}
	}
  public function getBasePath() {
    return $this->path;
  }
  
	// Using the direct link download for this file.
	public function setDirectLinkDownload($file_id, $data = [])
	{
		// Getting the current file name on the server.
		$current_file = $this->lookup($file_id);

		// If there is a file already, remove it.
		if(!empty($current_file->filename_physical))
		{
			// Removing the current file from the server.
			if(!$this->removeFile($current_file->filename_physical))
			{
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_REMOVE_CURRENT_FILE'), 'error');

				return false;
			}
		}

		// Save the file name to the database.
		if(!$this->saveFilenameToDatabase($file_id, null)) {
			return false;
		}

		return true;
	}

	// The user has chosen to use a direct link for downloads.
	public function setServerFile($file_id, $filename)
	{
		if (empty($filename)) {
			return true;
		}

		// Getting the current file name on the server.
		$current_file  = $this->lookup($file_id);
    //$this->removeFile($file)

		// Save the file name to the database.
		if(!$this->saveFilenameToDatabase($file_id, $filename))
		{
			return false;
		}

		return true;
	}

	// Uploads a downloadable zip file for a file.
	public function uploadFile($file_id, $file = [], $otherFilename = [], $use_orig_filename = false)
	{
		if ($file['error']) {
			return false;
		}

		// Getting the current file name on the server.
		$current_file = $this->lookup($file_id);

		// Getting some file information from the uploaded file.
		$filename  = $file['name'];
		$source    = $file['tmp_name'];

		// If there is a file already, remove it.
		if(!empty($current_file->filename_physical))
		{
      if($current_file->source_file_type != Files::SOURCE_TYPE_EXTERN) {
        // Removing the current file from the server.
        if(!$this->removeFile($current_file->filename_physical))
        {
          Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_REMOVE_CURRENT_FILE'), 'error');

          return false;
        }
      }
		}

		// We want a different name
		if($use_orig_filename)
		{
      $server_filename = $filename;
    }
    else {
      if(strlen($otherFilename)) {
        $server_filename = $otherFilename;
      }
      else {
        // create filename 
        $userid = Factory::getApplication()->getIdentity()->id;
        $server_filename = $userid .'_'. time();
      }
		}
   

		// Upload the file to the downloads section.
    $fullquali = $this->path. '/' .$server_filename;
		if(!File::upload($source, $fullquali, false, true))
    {
			Factory::getApplication()->enqueueMessage($file['name'] . ' ' . Text::_('COM_LARTSSUBS_COULD_NOT_MOVE_FILE'), 'error');
			return false;
		}

		// Save the file name to the database.
		if(!$this->saveFilenameToDatabase($file_id, $server_filename)) {
			return false;
		}

		return true;
	}

	// Saving the new file to the database.
	public function saveFilenameToDatabase($id, $filename)
	{
		$extension = File::getExt($filename);

    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_files'))
		            ->set([
			            $db->quoteName('filename_physical') . ' = ' . $db->quote($filename),
			            $db->quoteName('extension') . ' = ' . $db->quote($extension),
			            $db->quoteName('sha256_checksum') . ' = ' . $db->quote($this->getFileChecksum('sha256', $filename)),
		            ])
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);

		return $db->execute();
	}

	// Removing a file by path. If no path give, use the global path for downloads.
	public function removeFile($file, $path = null)
	{
		$path = ! empty($path) ? $path : $this->path;

    $fullpath = $path . '/' . $file;
		if (file_exists($fullpath))
		{
			if(!File::delete($fullpath)) {
				return false;
			}
		}

		return true;
	}

	// Getting the current filename for the file which is active.
	public static function lookup($id = null)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['hash', 'filename_physical', 'source_file_type', 'version'])
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);
		return $db->loadObject();
	}


  // Getting the current filename for the file which is active.
	public static function lookupByHash($hash)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('hash') . ' = ' .$db->quote($hash));

		$db->setQuery($query);
		return $db->loadObject();
	}

  // Getting the Checksum for a file.
	public function getFileChecksum($algo, $file)
	{
    $fullquali = $this->path. '/' .$file;
    
		if(empty($file)) {
			return null;
		}
		if(!file_exists($fullquali)) {
			return null;
		}

		return hash_file($algo, $fullquali);
	}

  public static function getCategories($file_id, $asIdList = true)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_file_categories'))
		            ->where($db->quoteName('file_id') . ' = ' . (int) $file_id);

		$db->setQuery($query);
		$result = $db->loadObjectList();
    
    if(!$asIdList) {
  		return $result;
    }
 
    $list = [];
    foreach ($result as $item) {
      $list[] = $item->category_id;
    }

		return $list;
	}

  public static function deleteCategories($file_id)
  {
  	// Delete all assoc categories
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_file_categories'))
		            ->where($db->quoteName('file_id') . ' = ' . (int) $file_id);
		$db->setQuery($query);

		return $db->execute();
  }
 
	public static function saveCategories($file_id, $categories)
  {
		// Assoc categories
		foreach ($categories as $category)
		{
			$item              = new \stdClass;
			$item->file_id     = $file_id;
			$item->category_id = $category;

      if(!Factory::getDbo()->insertObject('#__larts_subs_file_categories', $item)) {
        return false;
  		}
		}
    return true;
  }
  
}

