<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Object\CMSObject;

use LartsSubscriptions\Discount;
use LartsSubscriptions\Price;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Base\BSWrapper;
use LartsSubscriptions\Periods;

class Helper
{
	// Returns permissions - Gets a list of the actions that can be performed.
	public static function getActions()
	{
		$user   = Factory::getApplication()->getIdentity();
		$result = new CMSObject;

		$actions = [
			'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.own', 'core.edit.state', 'core.delete',
		];

		foreach ($actions as $action)
		{
			$result->set($action, $user->authorise($action, 'com_lartssubscriptions'));
		}

		return $result;
	}

  public static function getProductItemSubtext($item)
	{
    $txt = '';
    
    switch($item->type) {
    case Product::ANALOG:
      break;
    case Product::TICKET:
      $txt = 'Ticket';
      break;
    default:
      $txt = Helper::getPeriodText($item->period, $item->period_type) .' '. Text::_('COM_LARTSSUBS_SUBSCRIPTION');
    }
    return $txt;
 }
  
	public static function getPeriodText($period, $period_type)
	{
		switch ($period_type)
		{
			case 'days':
				return $period . ' ' . ($period == 1 ? Text::_('COM_LARTSSUBS_DAY') : Text::_('COM_LARTSSUBS_DAYS'));
			case 'months':
				return $period . ' ' . ($period == 1 ? Text::_('COM_LARTSSUBS_MONTH') : Text::_('COM_LARTSSUBS_MONTHS'));
			case 'years':
				return $period . ' ' . ($period == 1 ? Text::_('COM_LARTSSUBS_YEAR') : Text::_('COM_LARTSSUBS_YEARS'));
			case 'lifetime':
				return Text::_('COM_LARTSSUBS_LIFETIME');
			case 'onetime':
				return Text::_('COM_LARTSSUBS_ONE_TIME_FEE');
			default:
				return Text::_('COM_LARTSSUBS_UNKNOWN');
		}
	}

	public static function getStabilityLevel($level)
	{
		switch ($level)
		{
			case 1:
				return '<span class="badge bg-success">' . Text::_('COM_LARTSSUBS_STABILITY_STABLE') . '</span>';
			case 2:
				return '<span class="badge bg-info">' . Text::_('COM_LARTSSUBS_STABILITY_RC') . '</span>';
			case 3:
				return '<span class="badge bg-warning">' . Text::_('COM_LARTSSUBS_STABILITY_BETA') . '</span>';
			case 4:
				return '<span class="badge bg-danger">' . Text::_('COM_LARTSSUBS_STABILITY_ALPHA') . '</span>';
			case 5:
				return '<span class="badge bg-inverse">' . Text::_('COM_LARTSSUBS_STABILITY_DEV') . '</span>';
			default:
				return '<span class="label">' . Text::_('COM_LARTSSUBS_UNKNOWN') . '</span>';
		}
	}

	public static function getPaymentState($state)
	{
		switch ($state) {
    case Invoice::PAID:
      $text = Text::_('COM_LARTSSUBS_COMPLETED');
      $type = 'success';
      break;
    default:
      $text = Text::_('COM_LARTSSUBS_UNPAID');
      $type = 'danger';
      break;
		}
    return BSWrapper::label($text, $type);
	}

	public static function getSubscriptionState($bExpired, $periodStatus)
	{
    $text = $bExpired ? Text::_('COM_LARTSSUBS_EXPIRED') : Text::_('COM_LARTSSUBS_ACTIVE');
    $type = $bExpired ? 'danger' : 'success';
    switch($periodStatus) {
    case Periods::STATE_DISABLED:
      $text = Text::_('COM_LARTSSUBS_DISABLED');
      $type = 'default';
      break;
    case Periods::STATE_CANCELED:
      $text = Text::_('COM_LARTSSUBS_CANCELED');
      $type = 'default';
      break;
    }
    return BSWrapper::label($text, $type);
	}

	public static function extractUnit($string, $start, $end)
	{
		$pos        = stripos($string, $start);
		$str        = substr($string, $pos);
		$str_two    = substr($str, strlen($start));
		$second_pos = stripos($str_two, $end);
		$str_three  = substr($str_two, 0, $second_pos);
		$unit       = trim($str_three); // remove whitespaces

		return $unit;
	}

	public static function getRenewSubscriptionButtonBoxed($downloadid, $userid, $reminder = null)
	{
		$btn_text = Text::_('COM_LARTSSUBS_RENEW_SUBSCRIPTION');

		if ($reminder)
		{
			$config = LartsConfig::get();

			switch ($reminder)
			{
				case 'first_expiration_days':
					$percent = $config->first_expiration_discount;
					break;
				case 'second_expiration_days':
					$percent = $config->second_expiration_discount;
					break;
				case 'third_expiration_days':
					$percent = $config->third_expiration_discount;
					break;
				default:
					$percent = $config->renewal_discount;
					break;
			}

			$db = Factory::getDbo();

			$query = $db->getQuery(true)
				->select(['p.price', 'p.published'])
				->from($db->quoteName('#__larts_subs_periods', 'p2u'))
				->join('LEFT OUTER', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p.id') . ' = ' . $db->quoteName('p2u.product_id') . ')')
				->where($db->quoteName('p2u.download_id') . ' = ' . $db->quote($downloadid));

			$db->setQuery($query);
			$product = $db->loadObject();

			$disabled = ! $product->published ? ' disabled' : '';

			$renewal  = new Discount;
			$discount = $renewal->getDiscount($product->price, $percent, 'percent');

			$btn_text = Text::_('COM_LARTSSUBS_RENEW_FOR_ONLY') . ' ' . Price::_($product->price - $discount);
		}

		$html = '<form action="index.php" method="post" class="form-inline control-group">'

			. '<input type="hidden" name="option" value="com_lartssubscriptions" />'
			. '<input type="hidden" name="view" value="checkout" />'
			. '<input type="hidden" name="task" value="renew" />'
			. '<input type="hidden" name="downloadid" value="' . $downloadid . '" />'
			. '<input type="hidden" name="userid" value="' . $userid . '" />'
			. '<input type="hidden" name="reminder" value="' . $reminder . '" />'

			. '<button type="submit" class="btn btn-success pull-left' . $disabled . '">
					<i class="icon-cart icon-white"></i> ' . $btn_text . '
			   </button>'

			. '</form>';
    return $html;
	}

	public static function getRenewSubscriptionButton($product_id, $userid, $butClassExtra = '')
	{
		$html = '<form action="index.php" method="post" class="form-inline control-group" style="width:100%;margin:0px">'

			. '<input type="hidden" name="option" value="com_lartssubscriptions" />'
			. '<input type="hidden" name="view" value="checkout" />'
			. '<input type="hidden" name="task" value="checkout.renew" />'
			. '<input type="hidden" name="productid" value="' . $product_id . '" />'
			. '<input type="hidden" name="userid" value="' . $userid . '" />'
			. '<button type="submit" class="btn btn-sm btn-success '.$butClassExtra.'">' . Text::_('COM_LARTSSUBS_RENEW') . '</button>'

			. '</form>';
    return $html;
	}

  public static function getBuyButton($product_id, $butClassExtra = '')
	{
		$html = '<form action="index.php" method="post" class="form-inline control-group" style="width:100%;margin:0px">'

			. '<input type="hidden" name="option" value="com_lartssubscriptions" />'
			. '<input type="hidden" name="task" value="order.addCartitem" />'
			. '<input type="hidden" name="id" value="' . $product_id . '" />'
			. '<button type="submit" class="btn btn-sm btn-success '.$butClassExtra.'">' . Text::_('COM_LARTSSUBS_BUY') . '</button>'

			. '</form>';
    return $html;
	}

	public static function getUpgradeSubscriptionButton($productid, $discount = 25)
	{
		if(!$productid || ! $discount)
		{
			return;
		}

		$upgrade = new Discount;

		$html = '';

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['p.name', 'p.price', 'p.id'])
			->from($db->quoteName('#__larts_subs_products_bundled', 'p2c'))
			->join('LEFT OUTER', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p.id') . ' = ' . $db->quoteName('p2c.product_id') . ')')
			->where($db->quoteName('p2c.bundled_product_id') . ' = ' . $db->quote($productid))
			->order($db->quoteName('p.price') . ' ASC');

		$db->setQuery($query);
		$products = $db->loadObjectList();

		foreach ($products as $product)
		{
			// Get the uphgrade price for this product
			$price = $product->price - $upgrade->getDiscount($product->price, $discount, 'percent');

			// Button information
			$text = Text::sprintf('COM_LARTSSUBS_UPGRADE_TO_PRODUCT', $product->name, Price::_($price));
			$link = Route::_('index.php?option=com_lartssubscriptions&view=order&task=product&id=' . $product->id);

			// Generate button look-a-like link :)
			$html .= '<a href="' . $link . '" class="btn btn-primary pull-left boxed-upgrade-button"><i class="icon-star icon-white"></i> ' . $text . '</a>';
			$html .= '<div class="clearfix"></div>';
		}

		return $html;
	}


  
  public static function getShowProductButton($productid, $butClassExtra = '')
  {
    $link = 'index.php?option=com_lartssubscriptions&view=product&id=' .$productid;
    $html = '<a href="' . Route::_($link) . '" class="btn '.$butClassExtra.'">'
              . '<i class="fas fa-star" aria-hidden="true"></i> ' . Text::_('COM_LARTSSUBS_CONTENT_PRODUCT_PAGE') 
              . '</a>';
    echo $html;
  }

  public static function getExtraUrlButton($url, $butClassExtra = '')
  {
    $local = true;
    $arr = parse_url($url);
    if(isset($arr["scheme"]) || isset($arr["host"])) {
      $local = false;
    }
    $link = $local ? Route::_($url) : $url;

    $html = '<a href="' . Route::_($link) . '" class="btn '.$butClassExtra.'">'
              . '<i class="fas fa-star" aria-hidden="true"></i> ' . Text::_('COM_LARTSSUBS_CONTENT_ADDITIONALS') 
              . '</a>';
    echo $html;
  }

  public static function getNoneButton()
  {
    echo '<div class="larts-btn-mysubs-card" style="height: 38px"></div>';
  }
  
}


