<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Price;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Logger;

class Message
{
	private $template     = null;
	private $attachment   = null;
	private $user         = null;
	private $variables    = [];
	private $replacements = [];
	private $templates    = null;
	private $language     = '';
	public  $show_empty   = false;

	public function __construct($alias = '')
	{
		$this->setDefaultVariables();

		if ($alias) {
			$this->alias($alias);
		}

		$this->language = Factory::getLanguage()->getTag();
	}

	public static function getInstance($alias = '')
	{
		$instance = new Message;

		if ($alias) {
			$instance->alias($alias);
		}

		return $instance;
	}

	/*
	 * Sets the message based on the id
	 */
	public function alias($alias)
	{
		$this->setTemplate($alias);

		// Set variables regarding the message
		$variables = [
			'from_name'      => ! empty($this->template->from_name) ? $this->template->from_name : '',
			'from_email'     => ! empty($this->template->from_email) ? $this->template->from_email : '',
			'reply_to_name'  => ! empty($this->template->reply_to_name) ? $this->template->reply_to_name : '',
			'reply_to_email' => ! empty($this->template->reply_to_email) ? $this->template->reply_to_email : '',
		];

		$this->variables = array_merge($this->variables, $variables);

		return $this;
	}

	// Sets the user based on the id and set the user variables
	public function user($id)
	{
		if(!$id) {
			$id = isset($this->variables['userid']) ? (int) $this->variables['userid'] : 0;
		}

		$this->user = $this->getUserObject($id);
		$this->user->password = '***';

		// Set variables regarding the user
		$this->variables = array_merge($this->variables, (array) $this->user);

		if (isset($this->template->type) && $this->template->type == 'email')
		{
			$this->language = $this->user->country_code;
			// Reset message
			$this->alias($this->template->alias);
		}

		return $this;
	}

	private function getUserObject($id)
	{
		$db = Factory::getDbo();

		$name = $db->getQuery(true)->concatenate([$db->quoteName('cust.firstname'), $db->quote(' '), $db->quoteName('cust.lastname')]);

		$query = $db->getQuery(true)
		            ->select([
			            'u.name as uname, u.username', 'u.email', 'cust.*', $name . 'AS name', 'cust.country_code',
                  'c.country'
		            ])
		            ->from($db->quoteName('#__users', 'u'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'cust') . ' ON (' . $db->quoteName('cust.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('cust.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')')
                ;
    
    
		// Only limit by user id when a userid is given (logged in)
		if(!empty($id)) {
			$query->where($db->quoteName('u.id') . ' = ' . (int) $id);
		}

    $db->setQuery($query, 0, 1);
		$user = $db->loadAssoc();

		// Empty the values if no user id is set (when visitor is a guest)
		// As the returned values will be from the first user found in the database
		if (empty($id)) {
			$user = array_fill_keys(array_keys($user), '');
  		return (object) $user;
		}

    
		if (empty($user)) {
			return $this->getUserObject(0);
		}

		// if no name given (customer 
    if (empty($user['name'])) {
			$user['name'] = $user['uname'];
		}

		return (object) $user;
	}

	// Sets the user based on the userid
	public function customer($id)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('userid')
		            ->from($db->quoteName('#__larts_subs_customers'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);
		$userid = $db->loadResult();

		if (empty($userid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_NOT_FOUND'), 'error');
		}

		$this->user($userid);

		return $this;
	}

	// Sets the invoice variables
	public function invoice($id = 0)
	{
		if(!$id) {
			if (empty($variables['invoice_id'])) {
				return $this;
			}

			$id = $variables['invoice_id'];
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
                  // 'invoice_number as invoice_id', 
                  'invoice_number', 'userid', 'ordercode', 'invoicedate as date',
			            'vat_number', 'vat_percentage',
			            'net_price as price', 'net_price', 'gross_price', 'gross_price as brutto_price', 'vat_total', 'disco_price',
			            'paid', 'provider', 'transaction_id',
			            'invoice_type', 'refund_invoice', 'refund_reason', 'refund_date',
		            ])
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);
		$db->setQuery($query);

		$variables = $db->loadAssoc();

		if (empty($variables)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVOICE_NOT_FOUND'), 'error');
		}

		// Also set the user if it isn't set yet
		if (is_null($this->user)) {
			$this->user($variables['userid']);
		}

		$this->variables = array_merge($this->variables, $variables);

		return $this;
	}

	// Adds user-defined variables to the variables list
	public function variables($variables = [])
	{
		if (is_object($variables))
		{
			$variables = (array) $variables;
		}

		$this->variables = array_merge($this->variables, $variables);

		if (is_null($this->user) && ! empty($variables['userid']))
		{
			$this->user($variables['userid']);
		}

		$this->setVariablesAliases();

		return $this;
	}

	// Adds extra variables aliases to the variables list
	public function variable_aliases($aliases = [])
	{
		if (is_object($aliases))
		{
			$aliases = (array) $aliases;
		}

		$this->setVariablesAliases($aliases);

		return $this;
	}

	public function attachment($attachment = null)
	{
		$this->attachment = $attachment;

		return $this;
	}

	public function send()
	{
		if (empty($this->template)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MESSAGE_NOT_FOUND'), 'error');
			return false;
		}

		if (empty($this->user)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_NOT_FOUND'), 'error');
			return false;
		}

		if ($this->template->published != 1) {
			return false;
		}

		// Wrap the body in sans-serif styling
		$body = '<div style="font-family:sans-serif">' . $this->getBody() . '</div>';
		if (empty($body)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMPTY_TEMPLATE_RESULT'), 'error');
			return false;
		}

		$config = LartsConfig::get();

		// Should the email be sent to the user or the shop owner?
		$send_to_email = $this->variables['email'];
		$send_to_name  = empty($this->variables['name']) ? $this->variables['username'] : $this->variables['name'];

		if (isset($this->template->send_to) && $this->template->send_to == 'owner')
		{
			$send_to_email = $config->company_email;
			$send_to_name  = $config->company_name;
		}

 		if (empty($send_to_email) || empty($send_to_name)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_RECIPIENT'), 'error');
			return false;
		}

		$from_name      = ! empty($this->variables['from_name']) ? $this->variables['from_name'] : ($config->emails_sent_from_name ?: $config->company_name);
		$from_email     = ! empty($this->variables['from_email']) ? $this->variables['from_email'] : ($config->emails_sent_from ?: $config->company_email);
		$reply_to_name  = ! empty($this->variables['reply_to_name']) ? $this->variables['reply_to_name'] : $config->emails_reply_to_name;
		$reply_to_email = ! empty($this->variables['reply_to_email']) ? $this->variables['reply_to_email'] : $config->emails_reply_to;

 		if (empty($from_email) || empty($from_name)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_SENDER_MAIL'), 'error');
			return false;
		}

		$mailer = Factory::getMailer();
		$mailer->setSubject($this->getSubject());
		$mailer->setBody($body);
		$mailer->isHtml(true);

		$mailer->setSender([$from_email, $from_name]);
		$mailer->addRecipient($send_to_email, $send_to_name);

		if(!empty($reply_to_email))
		{
			$mailer->addReplyTo($reply_to_email, $reply_to_name);
		}

		if ($this->template->receive_bcc)
		{
			$mailer->addBcc($from_email, $from_name);

			// Adding an extra BCC to this email if the BCC items has been filled
			if(!empty($this->template->bcc1_name) && ! empty($this->template->bcc1_email))
			{
				$mailer->addBcc($this->template->bcc1_email, $this->template->bcc1_name);
			}

			// Adding an extra BCC to this email if the BCC items has been filled
			if(!empty($this->template->bcc2_name) && ! empty($this->template->bcc2_email))
			{
				$mailer->addBcc($this->template->bcc2_email, $this->template->bcc2_name);
			}
      
		}

		if(!empty($this->attachment))
		{
			$mailer->addAttachment($this->attachment);
		}

		// Logging
		Logger::_('message_emails', [
			'template'       => $this->template->alias,
			'send_to_name'   => $send_to_name,
			'send_to_email'  => $send_to_email,
			'from_name'      => $from_name,
			'from_email'     => $from_email,
			'reply_to_name'  => $reply_to_name,
			'reply_to_email' => $reply_to_email,
		]);

    if($config->mailer_enabled == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MAILER_DISABLED'), 'info');
      return false;
    }
    else if(!$mailer->Send()) {
      return false;
    }
    
    return true;
	}

	public function getBody()
	{
		if(!isset($this->template->body))
		{
			return '';
		}

		$string = $this->template->body;

		$this->replaceVariables($string);

		return $string;
	}

	/*
	 * Returns the subject of the message with replaced vaiables
	 */
	public function getSubject()
	{
		if(!isset($this->template->subject))
		{
			return '';
		}

		$string = $this->template->subject;

		$this->replaceVariables($string);

		return $string;
	}

	public function getVariables()
	{
		return $this->variables;
	}

	// Returns the template object based on the given id
	public function getTemplate($alias = '')
	{
		if(!$alias && ! is_null($this->template))
		{
			return $this->template;
		}
		$this->setTemplate($alias = '');

		return $this->template;
	}

	// Returns the message object based on the given id
	public function setTemplate($alias = '')
	{
		if (!$alias && !is_null($this->template)) {
			return $this;
		}

		// Strip off the 'core' or 'user' prefix
		$alias = preg_replace('#^(core|user)\.#', '', $alias);

		$db = Factory::getDbo();

		$alias_condition = $db->quoteName('alias') . ' = ' . $db->quote($alias);

		$language_condition = '('
			. $db->quoteName('language') . ' = ' . $db->quote($this->language)
			. ' OR ' . $db->quoteName('language') . ' LIKE ' . $db->quote('%-' . $this->language)
			. ')';

		// Try to find template based on set language
		$query = $db->getQuery(true);
    $query->select('*')
              ->from($db->quoteName('#__larts_subs_templates'))
              ->where($alias_condition)
              // VisualWaves -> prevent null result on searching template
              //->where($language_condition)
              ->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query, 0, 1);

    $template = $db->loadObject();

		if(!empty($template))
		{
      $this->template = $db->loadObject();
			return $this;
		}

		// Try to find template based on set language
		if ($this->language != Factory::getLanguage()->getTag())
		{
			$query->clear('where')
			      ->where($alias_condition)
			      ->where($db->quoteName('language') . ' = ' . $db->quote(Factory::getLanguage()->getTag()))
			      ->where($db->quoteName('published') . ' = 1');

			$db->setQuery($query, 0, 1);
			$template = $db->loadObject();

			if(!empty($template))
			{
				$this->template = $db->loadObject();

				return $this;
			}
		}

		// No matching language dependant template found, return the main one
		$query->clear('where')
		      ->where($alias_condition)
		      ->where($db->quoteName('language') . ' = ' . $db->quote(''))
		      ->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query, 0, 1);

		$this->template = $db->loadObject();

		return $this;
	}

	// Adds a set of default variables to the variables list
	private function setDefaultVariables()
	{
		$config = LartsConfig::get();

		$this->variables = [
			'@owner'                => 'COM_LARTSSUBS_OWNER',
			'logo'                  => '',
			'company_name'          => $config->company_name,
			'company_address'       => $config->company_address,
			'company_zip_city'      => $config->company_zip_city,
			'company_country'       => $config->company_country,
			'company_email'         => $config->company_email,
			'company_website'       => $config->company_website,
			'company_chamber'       => $config->company_chamber,
			'company_vatnumber'     => $config->company_vatnumber,
			'company_bank_name'     => $config->company_bank_name,
			'company_bank_account'  => $config->company_bank_account,
			'company_bank_bic'      => $config->company_bank_bic,

			'@customer'    => 'COM_LARTSSUBS_CUSTOMER',
			'username'     => '',
			'password'     => '',
			'email'        => '',
			'userid'       => '',
			'name'         => '',
			'firstname'    => '',
			'lastname'     => '',
			'address1'     => '',
			'address2'     => '',
			'zipcode'      => '',
			'city'         => '',
			'country'      => '',
			'country_code' => '',
			'phone'        => '',
			'companyname'  => '',
			'vatnumber'    => '',
			'ip_address'   => '',
			//'watchful_key' => '',

			'@others'        => 'COM_LARTSSUBS_MISCELLANEOUS',
			'sitename'       => Factory::getConfig()->get('sitename'),
			'siteurl'        => Uri::root(),
			'currency'       => $config->currency,
			'from_name'      => '',
			'from_email'     => '',
			'reply_to_name'  => '',
			'reply_to_email' => '',

			'@subscription'       => 'COM_LARTSSUBS_SUBSCRIPTION',
			'product'             => '',
			'product_alias'       => '',
			'description'         => '',
			'price'               => '',
			'discount'            => '',
			'date'                => '',
			'date_short'          => '',
			'expiry_date'         => '',
			'expiry_date_short'   => '',
			'subscription_period' => '',
			'ordercode'           => '',
			'transaction_id'      => '',
			//'download_id'         => '',
			'action'              => '',
			'coupon'              => '',
			'orderlist'           => '',
			'orderlist_simple'    => '',
			'has_lifetime'        => '',
			'has_onetime'         => '',

			'@invoice'              => 'COM_LARTSSUBS_INVOICE',
			'invoice_id'            => '',
			'invoice_brutto'        => '',
			'invoice_netto'         => '',
			'invoice_netto_pre_vat' => '',
			'invoice_vat'           => '',
			'invoice_vat_pct'       => '',
			'invoice_discount'      => '',
			'invoice_items'         => '',
			'invoice_refund'        => '',
			'payment_provider'      => '',

			'@invoice_item'         => 'COM_LARTSSUBS_INVOICE_ITEM',
			'item_amount'           => '',
			'item_name'             => '',
			'item_description'      => '',
			'item_price'            => '',
			'item_start_date'       => '',
			'item_start_date_short' => '',
			'item_end_date'         => '',
			'item_end_date_short'   => '',
			'item_period'           => '',
			'item_period_type'      => '',
      'item_has_licence'      => '',
      'item_licence'          => '',

			'@ticket'               => 'COM_LARTSSUBS_TICKET',
			'ticket_id'             => '',
			'ticket_number'         => '',
			'ticket_brutto'         => '',
			'ticket_netto'          => '',
			'ticket_vat'            => '',
			'ticket_vat_pct'        => '',
      
			'@end' => 'Ignore any other variables in the visual output of the Template Variable list',
		];
	}

	// Adds variable aliases to the variables list
	private function setVariablesAliases($aliases = [])
	{
		$variables = [];

		foreach ($aliases as $variable => $key)
		{
			$variables[$variable] = isset($variables[$key]) ? $variables[$key] : '';
		}

		$this->variables = array_merge($this->variables, $variables);

		if (empty($this->variables['password']))
		{
			$this->variables['password'] = '***';
		}
	}

	// Replaces all variables in the string: templates, if structures and normal variable tags
	private function replaceVariables(&$string)
	{
		// Set/Init variables if not already done so
		$this->variables();

		$this->replaceTemplates($string);
		$this->replaceIfStructures($string);
		$this->replaceLogo($string);
		$this->replaceMainVariables($string);

		if (in_array($this->template->type, ['email', 'invoice']))
		{
			$this->makeUrlsAbsolute($string);
		}
	}

	// Replaces the {logo} or {logo:xxx,xxx} variables in the string with an image tag
	private function replaceLogo(&$string)
	{
		$regex = '#\{logo(?:\:([0-9]+)(?:,([0-9]+))?)?\}#';
		if(!preg_match_all($regex, $string, $matches, PREG_SET_ORDER)) {
			return;
		}

		$config = LartsConfig::get();

		$logo = $config->logo;

    $logoobj = HTMLHelper::cleanImageURL($config->logo); 
    $logo = $logoobj->url; 

    $logofile = JPATH_SITE . '/' . $logo;
    if(!is_file($logofile)) {
      $logo = '';
    }

		// No logo found, replace all {logo} tags with nothing.
		if(empty($logo)) {
			$string = preg_replace($regex, '', $string);
			return;
		}
    
		$size = getimagesize($logofile);

		$default_width  = ($config->logo_width > 0) ? $config->logo_width : (isset($size['0']) ? $size['0'] : 200);
    // calc height but preserve aspect ratio
		$default_height = isset($size['1']) ? round($default_width / $size['0'] * $size['1']) : $default_width;

		foreach ($matches as $match)
		{
			$width  = isset($match['1']) ? $match['1'] : $default_width;
			$height = isset($match['2']) ? $match['2'] : 0;
			if(!$height)
			{
				$height = isset($match['1'])
					? round($match['1'] / $default_width * $default_height)
					: $default_height;
			}

			$image = '<img src="' . $logo . '" width="' . $width . '" height="' . $height . '" alt="' . $config->company_name . '" />';

			$string = str_replace($match['0'], $image, $string);
		}
	}

	// Makes urls absolute so images/links work in emails/pdfs
	private function makeUrlsAbsolute(&$string)
	{
		$regex = '#((?:href|src)=")([^"]*")#';
		if(!preg_match_all($regex, $string, $matches, PREG_SET_ORDER))
		{
			return;
		}

		foreach ($matches as $match)
		{
			if (preg_match('#^(https?:)?//#', $match['2']))
			{
				continue;
			}

			$string = str_replace(
				$match['0'],
				$match['1'] . Uri::root() . ltrim($match['2'], '/ '),
				$string);
		}
	}

	// Replaces variables in the string that refer to an existing template alias, like {header} and {footer}
	private function replaceTemplates(&$string)
	{
		$templates = $this->getTemplates();

		foreach ($templates as $template)
		{
			$variable = $template->alias;
			if (strpos($variable, '.') !== false)
			{
				list($type, $variable) = explode('.', $variable, 2);
			}

			$string = str_replace(
				[
					'<p>{' . $variable . '}</p>',
					'{' . $variable . '}',
				],
				$template->body,
				$string
			);
		}
	}

	// Returns all templates of the type template
	private function getTemplates()
	{
		if(!is_null($this->templates))
		{
			return $this->templates;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_templates'))
		            ->where($db->quoteName('type') . ' = ' . $db->quote('template'))
		            ->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query);

		$this->templates = $db->loadObjectList();

		return $this->templates;
	}

	// Replaces if structures in the string
	private function replaceIfStructures(&$string)
	{
    // {if : aa = 0 } if text {else} else text {endif} 
		if(!preg_match_all('#\{if\s*:\s*(?<check>[^\}]+)\s*\}(?<if>.*?)(?:\{else\}(?<else>.*?))?\{endif\}#s',
                       $string, $matches, PREG_SET_ORDER))
		{
			return;
		}

		foreach ($matches as $match)
		{
			$replace = $this->getIfPassesCheck($match['check'])
				? $match['if']
				: (isset($match['else']) ? $match['else'] : '');

			$string = str_replace($match['0'], $replace, $string);
		}
	}

	private function getIfPassesCheck($string)
	{
		$string = str_replace(' &amp;&amp; ', ' && ', $string);
		$checks = explode(' && ', $string);

		$pass = true;

		foreach ($checks as $check)
		{
			$pass = $this->getIfPassesSingleCheck($check);

			if(!$pass) {
				return false;
			}
		}

		return $pass;
	}

	private function getIfPassesSingleCheck($string)
	{
		$string = trim($string);

		$key   = $string;
		$value = null;

		if (strpos($string, '=') !== false) {
			list($key, $value) = explode('=', $string);
		}

		$reverse = preg_match('#(^!|!$)#', $key);
		$key     = trim($key, '! ');

		if (is_null($value)) {
			return $this->passNotEmpty($key, $reverse);
		}

		return $this->passValueCheck($key, $value, $reverse);
	}

	private function passNotEmpty($key, $reverse)
	{
		$pass = ! empty($this->variables[$key]);

		if ($pass && $this->tagIsPrice($key, $this->variables[$key]))
		{
			$pass = filter_var($this->variables[$key], FILTER_SANITIZE_NUMBER_INT) > 0;
		}

		return $reverse ? ! $pass : $pass;
	}

	private function passValueCheck($key, $value, $reverse)
	{
		$check_value = isset($this->variables[$key]) ? $this->variables[$key] : $key;

		if ($check_value && $this->tagIsPrice($key, $check_value))
		{
			$check_value = filter_var($check_value, FILTER_SANITIZE_NUMBER_INT) > 0;
		}

		$pass = (trim($check_value) == trim($value));

		return $reverse ? ! $pass : $pass;
	}

	// Replaces the main variables in the string
	private function replaceMainVariables(&$string)
	{
		list($search, $replace) = $this->getVariableSearchReplace();

		$string = str_replace($search, $replace, $string);

		// Remove the empty value placeholder including leading whitespace
		$string = preg_replace('#(&nbsp;|&\#160;|\s)*<\!-- EMPTY VAR -->#', '', $string);
	}

	// Gets the searches (variable tags) and replaces (output values) for the main variables
	private function getVariableSearchReplace()
	{
		if(!empty($this->replacements))
		{
			return [array_keys($this->replacements), array_values($this->replacements)];
		}

		array_walk($this->variables, [$this, 'getSearchReplaceItem']);

		if ($this->show_empty) {
			$this->removeEmptyValuesFromVariables();
		}

		return [array_keys($this->replacements), array_values($this->replacements)];
	}

	// Converts a variable key to a tag syntax and adds it to the replacements array, together with its output value
	private function getSearchReplaceItem($value, $tag)
	{
		if (empty($tag) || $tag['0'] == '@' || strpos($tag, '_short') !== false)
		{
			return;
		}

		// Check if tag is date or foobar_date
		if (strpos($tag, 'date') === 0 || strpos($tag, '_date') !== false)
		{
			// Create replacements for...
      if(!empty($value)) {
  			//$old_tag = str_replace('_date', 'date', $tag);
        $this->replacements['{' . $tag . '}']       = DateEx::_($value);
        $this->replacements['{' . $tag . '_short}'] = DateEx::_($value, Text::_('DATE_FORMAT_LC4'));
        //$this->replacements['{' . $old_tag . '}']   = $this->replacements['{' . $tag . '}'];
      }
      else {
        $this->replacements['{' . $tag . '}']       = '-';
        $this->replacements['{' . $tag . '_short}'] = '-';
      }

			return;
		}

		$this->convertAliases($tag);

		if ($this->tagIsPrice($tag, $value))
		{
			$this->replacements['{' . $tag . '}'] = Price::_($value);

			return;
		}

		// replace empty values with a temporary placeholder to be able to remove the leading spaces later
		if ($value == '')
		{
			$value = '<!-- EMPTY VAR -->';
		}

		$this->replacements['{' . $tag . '}'] = $value;
	}

	private function convertAliases(&$tag)
	{
		$aliases = [
			'gross_price'          => 'brutto_price',
			'net_price'            => '',
			'invoice_gross'        => 'invoice_brutto',
		];

		return isset($aliases[$tag]) ? $aliases[$tag] : $tag;
	}

	private function tagIsPrice($tag, $value)
	{
    if(is_array($value)) {
			return false;
    }
		if (empty($value) || strpos($value, '%')) {
			return false;
		}

		return (
			strpos($tag, 'price') === 0
			|| strpos($tag, '_price')
			|| in_array($tag, [
				'discount',
				'amount',
				'vat_total',
				'invoice_gross',
				'invoice_netto',
				'invoice_netto_pre_vat',
				'invoice_discount',
				'invoice_vat',
				'ticket_brutto',
				'ticket_netto',
				'ticket_net_pre_vat',
				'ticket_discount',
				'ticket_vat',
			])
		);
	}

	// Remove empty values from the replacements
	private function removeEmptyValuesFromVariables()
	{
		foreach ($this->replacements as $tag => $value)
		{
			if(!empty($value) && $value != '<!-- EMPTY VAR -->')
			{
				continue;
			}

			unset($this->replacements[$tag]);
		}
	}
}
