<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
//use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\Database;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Order;
use LartsSubscriptions\Pdf\PdfPrinter;
use LartsSubscriptions\Base\PluginManager;


class Offer
{
  var $config = null;
	var $path = null;
  
  const TYPE_DEBIT = 0;    // db.invoice_type
  const TYPE_CREDIT = 1;   // db.invoice_type
  const UNPAID = 0;        // db.paid
  const PAID = 1;          // db.paid
  
	function __construct()
	{
		$this->path = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/offers/';
		$this->config = LartsConfig::get();
	}

  public function getPath() {
    return $this->path;
  }

  public function generateOfferNumber() {
    return 'Offer_' . strtoupper(uniqid());
  }

  public function store($data, $paid = true)
	{
		$date   = new Date;
		$db     = Factory::getDbo();
		$amount = new OrderAmounts($data['ordercode'], false);

    $invoice_number = $this->generateOfferNumber();

    // Prepare invoice data
		$invoice_data = (object) [
      'invoice_number'  => $invoice_number,
			'userid'          => $data['userid'],
			'ordercode'       => (int) $data['ordercode'],
			'published'       => 1,
			'paid'            => ($paid ? self::PAID : self::UNPAID),
			'provider'        => $data['provider'],
			'net_price'       => round($amount->netprice(), 2),
			'gross_price'     => round($amount->grossprice(), 2),
			'vat_total'       => round($amount->vatprice(), 2),
			'disco_price'     => round($amount->discount(), 2),
			'invoicedate'     => $date->toSql(),
			'vat_percentage'  => $amount->getVatPercentsAppliedStr(),
			'transaction_id'  => $data['transaction_id'],
      'dlkey'           => md5($invoice_number),
      'vat_number'      => $data['vat_number'],
		];

		// Store the invoice information and generate an invoice number;
		Factory::getDbo()->insertObject('#__larts_subs_invoices', $invoice_data, 'id');
		return $invoice_data->id;
	}

  public function createOfferPdf($invoice_id)
	{
  	// load invoice from db
    $invObj = new Invoice;
		$invoice = $invObj->load($invoice_id);
    if(empty($invoice)) {
      return false;
    }
    
		// Variables for the message:
    $variables = Order::getOrderData($invoice->ordercode, false);
		$variables2 = [
			'payment_state'         => 'Offer',
			'date'                  => $invoice->invoicedate,
			'userid'                => $invoice->userid,
			'invoice_number'        => $invoice->invoice_number,
			'transaction_id'        => null,
			'payment_provider'      => 'manual',
		];
    $variables = array_merge($variables, $variables2);
    
    
//		$amount = new OrderAmounts($ordercode);
//    $cart = new Cart($ordercode);
//
//    $ordervars = Order::getOrderData($ordercode);
//
//		// Variables for the message:
//		$variables = [
//			'ordercode'        => $ordercode,
//			'invoice_number'   => 'Q-'. $ordercode,
//			'date'             => 'NOW',
//			'price'            => $amount->netprice(),
//			'orderlist'        => $cart->getOrderedItemsHTML(false),
//			'orderlist_simple' => $cart->getOrderedItemsHTML(true),
//		];
//    $variables = array_merge($variables, $ordervars);

    
		$message = new Message;

		// Getting the invoice template.
		$footer = $message->alias('invoice-footer')
		                ->user($userid)
		                ->variables($variables)
		                ->getBody();

    // Getting the invoice template.
		$body = $message->alias('offer-template')
		                ->user($userid)
		                ->variables($variables)
		                ->getBody();


    
		// Output the HTML content
    $pdf = new PdfPrinter;
    $pdf->setFooter($footer);
    $pdf->doPrint($body);

    $src = $pdf->getfile();
		// The new name for the invoice (destination for saving the file)
		$dest = $this->getPath() . $variables['invoice_number'] . '.pdf';

		// Copy the file now.
		File::copy($src, $dest);
		File::delete($src);

		PluginManager::execPlugins('lartssubscriptions', 'onAfterCreateOffer', $variables);

		return true;
	}

 
  public function sendOffer($invoice_id, $template = 'send-offer')
	{
  	// load invoice from db
    $invObj = new Invoice;
		$invoice = $invObj->load($invoice_id);
    if(empty($invoice)) {
      return false;
    }
    
		// Variables for the message:
    $variables = Order::getOrderData($invoice->ordercode, false);
		$variables2 = [
			'payment_state'         => 'Offer',
			'date'                  => $invoice->invoicedate,
			'userid'                => $invoice->userid,
			'invoice_number'            => $invoice->invoice_number,
			'transaction_id'        => null,
			'payment_provider'      => 'manual',
			'invoice_file'      => $attachment,
		];
    $variables = array_merge($variables, $variables2);

   
		$attachment = $this->getPath() . $variables['invoice_number'] . '.pdf';
		if(!file_exists($attachment)) {
			return false;
		}

//    $cart = new Cart($ordercode);
//    $variables = [
//			'invoice_number'    => $invoice_number,
//			'user_id'           => $userid,
//			'invoice_file'      => $attachment,
//      'orderlist'         => $cart->getOrderedItemsHTML(false),
//      'orderlist_simple'  => $cart->getOrderedItemsHTML(true),
//			'ordercode'         => $ordercode,
//    ];

		Message::getInstance($template)
		             ->variables($variables)
                 ->user($userid)
		             ->attachment($attachment)
		             ->send();

 
		// Trigger plugin to make actions after sending an invoice
    $data = [
			'invoice_number' => $invoice_number,
			'user_id'        => $userid,
			'invoice_file'   => $attachment,
		];
		PluginManager::execPlugins('lartssubscriptions', 'onAfterSendQuote', $data);


		return true;
	}

}