<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;

use LartsSubscriptions\Base\Database;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\GeoIP\GeoIP;
use LartsSubscriptions\Periods;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\ThirdParty\LartsPkgManager;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Invoice;


// Order created at checkput process and updated 
// in payment processing

class Order
{
	public static function create($ordercode, $user_id, $vatnumber, $countrycode, $bAsOffer)
	{
     // create a new order at the db
		$amount = new OrderAmounts($ordercode, true);

    //Order::getOrderByOrdercode($ordercode);
    $cart = new Cart($ordercode);
		$items = $cart->getItems();

 		$order = [
      'id'                 => 0,
			'status'             => OrderStatus::PENDING,
			'userid'             => $user_id,
			//'transaction_number' => md5($ordercode),      // mangled for invoice and payment.response will search for this
			//'transaction_type'   => 0,
			//'transaction_id'     => '',
			'ordercode'          => $ordercode,
			'amounts'            => $amount->getJson(),
			'params'             => '',
      'country_code'       => $countrycode,
      'vat_number'         => $vatnumber,
      'ip_address'         => GeoIP::getIP(),
      'is_offer'           => $bAsOffer ? 1 : 0,
		];

    $obj = (object)$order;
    $db = Factory::getDBO();
    $db->insertObject('#__larts_subs_orders', $obj, 'id');
    
    return $obj->id;
  }

  public static function update($record, $key)
  {
    $dbhlp = new Database;
    return $dbhlp->update('#__larts_subs_orders', $key, $record);
  }

  public static function removeByOrdercodes($ordercodes = [])
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('id')
			->from($db->quoteName('#__larts_subs_orders'))
      ->where($db->quoteName('ordercode') . ' IN (' .implode(',', $ordercodes). ')')
      ;
		$db->setQuery($query);
		$cids = $db->loadObjectList();
    
    self::remove($cids);
  }
  
	public static function remove($cid = [])
	{
    // Remove all orders including
    //   - all period entries with this ordercode
    //   - all transaction entries with this ordercode
    //   - all invoices with this ordercode
    
    // get the ordercodes and create array
    $orders = Order::getOrdercodesByIds($cid);
		if (empty($orders)) {
			return true;
		}
    $ordercodes = array();
    foreach ($orders as $order) {
      $ordercodes[] = $order->ordercode;
    }

    // remove all period entries of the orders
    $periods = new Periods;
    if(!$periods->removeByOrdercode($ordercodes)) {
      return false;
    }

    // remove all transactions of the orders
    $trans = new Transaction;
    if(!$trans->removeByOrdercode($ordercodes)) {
      return false;
    }

    // remove all invoices of the orders
    $invoice = new Invoice;
    if(!$invoice->removeByOrdercode($ordercodes)) {
      return false;
    }
    
		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

    $db = Factory::getDBO();
		$query = $db->getQuery(true);
		$query->delete($db->quoteName('#__larts_subs_orders'));
		$query->where($db->quoteName('id') . ' IN (' . $cids . ')');

		$db->setQuery($query);

		return $db->execute();
	}

  public static function lookup($ordercode, $select = '*')
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName('ordercode') . ' = ' . $db->quote($ordercode));

		$db->setQuery($query);
    //$sql = $query->__toString();
    $result = $db->loadObject();
		return $result;
	}

	public static function lookupEx($value, $key, $select = '*')
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName($key) . ' = ' . $db->quote($value));

		$db->setQuery($query);
    $result = $db->loadObject();
		return $result;
	}

  /**
   * Lookup for all orders of this user
   * @param type $userid
   * @return array
   */
  public static function lookupOrders($userid)
  {
    $db = Factory::getDbo();

    $query = $db->getQuery(true);
    $query->select('*')
          ->from($db->quoteName('#__larts_subs_orders'))
          ->where($db->quoteName('userid') . ' = ' . (int)$userid)
          ;
		$db->setQuery($query);
		return $db->loadObjectList();
  }

  public static function lookupProductInOrders($userid, $productid)
  {    
    $db = Factory::getDbo();

    $query = $db->getQuery(true);
    $query->select('prod.*')
          ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
          ->leftJoin($db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('cart.product_id') . ' = ' . $db->quoteName('prod.id') . ')')
          ->leftJoin($db->quoteName('#__larts_subs_orders', 'ord') . ' ON (' . $db->quoteName('cart.ordercode') . ' = ' . $db->quoteName('ord.ordercode') . ')')
          ->where($db->quoteName('ord.userid') . ' = ' . (int)$userid)
          ->andWhere($db->quoteName('cart.product_id') . ' = ' . (int)$productid)
          ;
		$db->setQuery($query);
		return $db->loadObjectList();
  }
  
	public static function getUserId($ordercode = 0)
	{
		if (empty($ordercode)) {
			return 0;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('userid'))
			->from($db->quoteName('#__larts_subs_orders'))
			->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);
		$db->setQuery($query);

		return (int) $db->loadResult();
	}

	/**
	 * Resetting the order amounts column.
	 *
	 * @return bool
	 */
	public static function resetAmounts($ordercode)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_orders'))
		            ->set($db->quoteName('amounts') . ' = ' . $db->quote(''))
		            ->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);

		return $db->execute();
	}

  public static function storeUserIdInOrder($ordercode = 0, $userid = 0)
	{
		if (empty($ordercode) || empty($userid))
		{
			return 0;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->update($db->quoteName('#__larts_subs_cart_items'))
			->set($db->quoteName('userid') . ' = ' . (int) $userid)
			->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);
		$db->execute();
	}

  public static function getOrders($cids = [], $select = '*')
	{
    //$dbase = new Database;
    return Database::loadObjectList('#__larts_subs_orders', $cids, $select);
	}

  public static function GetOrdersOfUser($userid, $select = '*')
	{
    // SQL Access2PHP (1899.12.30 00-00-00)
    // SELECT x4oa3_larts_subs_orders.created, x4oa3_larts_subs_orders.ordercode, x4oa3_larts_subs_transactions.transaction_id, x4oa3_larts_subs_orders.status, x4oa3_larts_subs_invoices.invoice_number
    // FROM x4oa3_larts_subs_invoices RIGHT JOIN (x4oa3_larts_subs_orders LEFT JOIN x4oa3_larts_subs_transactions ON x4oa3_larts_subs_orders.ordercode = x4oa3_larts_subs_transactions.ordercode) ON x4oa3_larts_subs_invoices.ordercode = x4oa3_larts_subs_orders.ordercode
    // WHERE (((x4oa3_larts_subs_orders.userid)=562))

    $query = ' SELECT #__larts_subs_orders.created, #__larts_subs_orders.ordercode, #__larts_subs_transactions.transaction_id, #__larts_subs_orders.status, #__larts_subs_invoices.id as invoice_id, #__larts_subs_invoices.invoice_number, #__larts_subs_invoices.dlkey'
    .' FROM #__larts_subs_invoices RIGHT JOIN (#__larts_subs_orders LEFT JOIN #__larts_subs_transactions ON #__larts_subs_orders.ordercode = #__larts_subs_transactions.ordercode) ON #__larts_subs_invoices.ordercode = #__larts_subs_orders.ordercode'
    .' WHERE (((#__larts_subs_orders.userid)='.$userid.'))';

    $db = Factory::getDbo();
    //$query = $db->getQuery(true)
    //  ->select($select)
    //  ->from($db->quoteName('#__larts_subs_orders'))
    //  ->where('(' .$db->quoteName('userid') . '=' .$userid. ')');
    $db->setQuery($query);
    //$sql = $query->__toString();
    $result = $db->loadAssocList();
    return $result;
  }

	public static function getOrder($id, $key = 'id')
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName($key) . '=' .$id);

		$db->setQuery($query);
		return $db->loadObject();
	}
  
 	/**
	 * Getting ordercodes by its ID in the database.
	 */
	public static function getOrdercodesByIds($cid = [])
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select(['ordercode', 'userid'])
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName('id') . ' IN (' . implode(',', $cid) . ')');

		$db->setQuery($query);
		return $db->loadObjectList();
	}

  public static function getAmounts($ordercode)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('amounts')
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName('ordercode') . ' = ' . $ordercode)
		            //->where($db->quoteName('status') .'='. OrderStatus::ORDER_PENDING)
                ;

		$db->setQuery($query);
		$amounts = $db->loadResult();

		return ! empty($amounts) ? json_decode($amounts) : false;
	}

//  public static function mergeStatusByIds($cid = [], $status)
//	{
//		$db = Factory::getDbo();
//
//		$query = $db->getQuery(true);
//
//		$conditions = [
//			$db->quoteName('id') . ' IN (' . implode(',', $cid) . ')',
//		];
//
//		$query->update($db->quoteName('#__larts_subs_orders'))
//		      ->set($db->quoteName('status') . '='. $db->quoteName('status') & $status)
//		      ->where($conditions);
//
//		$db->setQuery($query);
//
//		return $db->execute();
//	}
  public static function setBitsStr($db, $var, $value) {
    return $db->quoteName($var) . '='. $db->quoteName($var) .' | '. $value;
  }
  public static function deleteBitsStr($db, $var, $value) {
    return $db->quoteName($var) . '='. $db->quoteName($var) .' & ~'. $value;
  }
  
  public static function setStatus($ordercode, $status, $bMerge = false)
	{
    // Bit on:  UPDATE users SET permission = permission | 16;
    // Bit off: UPDATE users SET permission = permission & ~16;
    
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
    if($bMerge) {
      $query->update($db->quoteName('#__larts_subs_orders'))
            ->set(self::setBitsStr($db, 'status', $status))
            ->where($db->quoteName('ordercode') .'='. $ordercode);
      // if new state is processed, disable pending
      // else if new state is pending, disable processed
      if(($status & OrderStatus::PROCESSED) || 
         ($status & OrderStatus::CANCELED) ||
         ($status & OrderStatus::FAILED))
      {  // clear pending bit
        $query->set(self::deleteBitsStr($db, 'status', OrderStatus::PENDING));
      }
      else if($status & OrderStatus::PENDING) {    // clear processed bit
        $query->set(self::deleteBitsStr($db, 'status', OrderStatus::PROCESSED | OrderStatus::CANCELED | OrderStatus::FAILED));
      }
    }
    else {  // no merge
      if($status & OrderStatus::PROCESSED) {    // clear pending bit
        $status &= ~OrderStatus::PENDING;
      }
      $query->update($db->quoteName('#__larts_subs_orders'))
            ->set($db->quoteName('status') . '='. $status)
            ->where($db->quoteName('ordercode') .'='. $ordercode);
    }
    
    //$sql = $query->__toString();
    
		$db->setQuery($query);

		return $db->execute();
	}

  public static function getStatus($ordercode)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$query->select($db->quoteName('status'))
		      ->from($db->quoteName('#__larts_subs_orders'))
		      ->where($db->quoteName('ordercode') .'='. $ordercode);
		
    $db->setQuery($query);

    return $db->loadResult();
	}

  public static function updateParamsAndStatus($ordercode, $params)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true);

		$fields = [
			$db->quoteName('params') . ' = ' . $db->quote($params),
			$db->quoteName('status') . ' = 3',
		];

		$conditions = [
			$db->quoteName('ordercode') . ' = ' . $ordercode,
		];

		$query->update($db->quoteName('#__larts_subs_orders'))
		      ->set($fields)
		      ->where($conditions);

		$db->setQuery($query);

		return $db->execute();
	}
  
  public static function updatePayed($ordercode)
  {
    $order = Order::getOrder($ordercode, 'ordercode');
    if(empty($order))
      return false;

    // get amounts
    $amounts = new OrderAmounts($ordercode);
    $amounts->setJson($order->amounts);
    
    // get total amount of all transactions
    $amounts->payed_total = Transaction::getTransactionsAmountTotal($ordercode);
   	$record = [
      'id'      => $order->id,
      'amounts' => $amounts->getJson(),
    ];
    if(!Order::update($record, 'id'))
      return false;

    if($amounts->payed_total >= $amounts->netprice) {
      self::setStatus($ordercode, OrderStatus::PAID, true);
      // mark invoice as paid
      Invoice::setPaid($ordercode, Invoice::PAID);
    }
    
    return true;
  }

  public static function getOrderByOrdercode($ordercode, $select = '*')
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select($select)
			->from($db->quoteName('#__larts_subs_orders'))
			->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);
		$db->setQuery($query);
		return $db->loadObject();
  }

	public static function setOfferFlag($id, $flag)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$query->update($db->quoteName('#__larts_subs_orders'))
		      ->set($db->quoteName('is_offer') .'='. ($flag == false ? '0' : '1'))
		      ->where($db->quoteName('id') .'='. $db->quote($id));

		$db->setQuery($query);

		return $db->execute();
	}
  
	/**
   * Collect all data of this order inside an array
   */
  public static function getOrderData($ordercode)
  {
		$db = Factory::getDbo();
    
    $query = $db->getQuery(true)
		            ->select([
			            'cart.*',
			            'prod.name', 'prod.teaser', 'prod.period', 'prod.period_type', 'periods.valid_from', 'periods.valid_to', 'prod.has_licence', 
                  'ord.created', 'ord.status as order_status'
		            ])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'cart'))
		            ->leftJoin($db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('cart.product_id') . ' = ' . $db->quoteName('prod.id') . ')')
		            ->leftJoin($db->quoteName('#__larts_subs_periods', 'periods') . ' ON (' . $db->quoteName('cart.product_id') . ' = ' . $db->quoteName('periods.id') . ')')
		            ->leftJoin($db->quoteName('#__larts_subs_orders', 'ord') . ' ON (' . $db->quoteName('cart.ordercode') . ' = ' . $db->quoteName('ord.ordercode') . ')')
		            ->where($db->quoteName('cart.ordercode') . ' = ' . (int) $ordercode)
		            ->where($db->quoteName('cart.product_id') . ' != 0')   // ignore coupons!
		            ->group($db->quoteName('cart.product_id'));

		$db->setQuery($query);
    //$sql = $query->__toString();
		$order_items = $db->loadObjectList();
    
		// make sure items is empty
		$items = '';

    // Loop through the items:
    $numProducts = sizeof($order_items);
    $numItemsTotal = 0;
		foreach ($order_items as $item)
		{
      $numItemsTotal += $item->quantity;

			// Total item price quantity * price = total_item_price
			$total_item_price = $item->quantity * $item->price;
			//$total_item_price = $refund ? -abs($total_item_price) : $total_item_price;

      // load our expiry
      $period = (new Periods)->Lookup($ordercode);
      if(empty($period)) {
        $period = new \stdClass;
        $period->licence = $item->licence;
        $period->valid_from = null;
        $period->valid_to = null;
      }
      
      if($item->has_licence) {
        if(empty($item->licence)) {
          $item->licence = 'New';
        }
      }
      else {
        $item->licence = '';
      }
       
			$variables = [
        'item_product_id'  => $item->product_id,
				'item_amount'      => $item->quantity,
				'item_name'        => $item->name,
				'item_description' => $item->teaser,
				'item_price'       => $item->price,
				'total_item_price' => $total_item_price,
				'item_start_date'  => $period->valid_from,
				'item_end_date'    => $period->valid_to,
				'item_period'      => $item->period,
				'item_period_type' => $item->period_type,
				//'download_id'      => '',
				'item_has_licence' => $item->has_licence,
				'item_licence'     => $item->licence,
        'extra_info'       => '',
			];

   		$plgReturns = PluginManager::execPlugins('lartssubscriptions', 'onGetInvoiceItemExtraTxt', $variables, false, false);
      if(!empty($plgReturns)) {
        // walk back to front and grab the first licence string found
        foreach($plgReturns as $plgRet) {
          if(!empty($plgRet)) {
            $variables['extra_info'] = $variables['extra_info']. ','. $plgRet;
            break;
          }
        }
      }

			$message = new Message;
			// Append the $items list:
			$items .= $message->alias('invoice-item')
			                  ->variables($variables)
			                  ->getBody();
		}

 		$amount = new OrderAmounts($ordercode);

		// Variables for the message:
		$variables = [
			'ordercode'             => $ordercode,
			'invoice_items'         => $items,
      'invoice_num_products'  => $numProducts,
      'invoice_num_items'     => $numItemsTotal,
			'invoice_gross'         => $amount->grossprice,
			'invoice_net_pre_vat'   => $amount->netprice - $amount->vat_total,
			'invoice_netto'         => $amount->netprice,
			'invoice_netto_pre_vat' => $amount->netprice - $amount->vat_total,
			'invoice_discount'      => $amount->discount,
			'invoice_vat'           => $amount->vat_total,
			'invoice_vat_pct'       => $amount->getVatPercentsAppliedStr(),
			//'payment_state'         => $payment_state,
			//'description'           => 'description from config',
		];

    return $variables;
  }

  public static function getLastPendings($count = 3)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['o.*', 'u.id as user_id', 'u.firstname', 'u.lastname'])
		            ->from($db->quoteName('#__larts_subs_orders', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('o.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->where('(('.$db->quoteName('o.status') .'&'.OrderStatus::PENDING. ') != 0)')
                ->order($db->quoteName('o.created') .' DESC')
		            ->andwhere($db->quoteName('o.is_offer') .'=0')
                  ;
		$db->setQuery($query, 0, $count);
		return $db->loadObjectList();
  }
 
}
