<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Base\Country;
use LartsSubscriptions\Base\Vat;
use LartsSubscriptions\Order;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Price;
use LartsSubscriptions\UserInfo;
use LartsSubscriptions\VAT\VATClasses;


class OrderAmounts
{
	var $ordercode      = 0;
	var $message        = '';
	var $couponcode     = '';
	var $grossprice     = 0;      // no vat
	var $netprice       = 0;      // including vat
	var $discount       = 0;      // discount total
	var $vat_total      = 0;      // vat total
  var $vat_bypercent  = array();
  var $payed_total    = 0;

	var $vat_required = false;
	var $vat_valid    = false;
	var $vat_forced   = false;
  var $country_code = '';
  var $vat_number    = '';
  
	function __construct($ordercode = 0, $bRecalc = false)
	{
		$this->ordercode = $ordercode;
		$this->country_code = '';
		$this->vat_number = '';

    if($this->ordercode != 0)
    {
      $order = Order::getOrder($this->ordercode, 'ordercode');
      if(!empty($order))
      {
        // set country & vatnumber from user
        $userinfo = UserInfo::load($order->userid);
        $this->country_code = empty($userinfo) ? 'DE' : $userinfo->country_code;
        $this->vat_number = empty($userinfo) ? '' : $userinfo->vat_number;

        // init the amounts from order
        $this->setJson($order->amounts);
      }

      if($bRecalc) {
        $this->recalcCart();
      }

      // do we have a item specific coupon?
      //$this->couponcode = $order->coupon_code;
    }
	}

	private function reset()
	{
		$this->grossprice     = 0;
		$this->netprice       = 0;
		$this->discount       = 0;
		$this->vat_total      = 0;
    $this->payed_total    = 0;
  }
  
  public function getJson()
  {
 		$amounts = [
			'gross_price'    => $this->grossprice,
			'net_price'      => $this->netprice,
			'disco_price'    => $this->discount,
			'vat_total'      => $this->vat_total,
      'payed_total'    => $this->payed_total,
      'vat_bypercent'  => $this->vat_bypercent,
		];
    return json_encode($amounts);
  }
  public function setJson($jsonAmounts)
  {
    if(!empty($jsonAmounts)) {
      $amounts = json_decode($jsonAmounts);

      $this->grossprice = round(isset($amounts->gross_price) ? $amounts->gross_price : 0, 2);
      $this->netprice = round(isset($amounts->net_price) ? $amounts->net_price : 0, 2);
      $this->discount = round(isset($amounts->disco_price) ? $amounts->disco_price : 0, 2);
      $this->vat_total = round(isset($amounts->vat_total) ? $amounts->vat_total : 0, 2);
      //$this->vat_percents = isset($amounts->vat_percents) ? $amounts->vat_percents : '';
      $this->payed_total = round(isset($amounts->payed_total) ? $amounts->payed_total : 0, 2);
      $this->vat_bypercent = isset($amounts->vat_bypercent) ? $amounts->vat_bypercent : array();
    }
    else {
      $this->grossprice = 0;
      $this->netprice = 0;
      $this->discount = 0;
      $this->vat_total = 0;
      //$this->vat_percents = '';
      $this->payed_total = 0;
      $this->vat_bypercent = array();
    }
  }

  public function getCartTotals($ordercode)
  {
    // get the cart
    $cart = new Cart($ordercode);
    $cartTotals = $cart->getTotals();
    if (empty($cartTotals)) {
      $this->message = 'No order found with number: ' . $this->ordercode;
      return false;
    }
    $cartTotals->totalprice = round($cartTotals->totalprice, 2);
    $cartTotals->vat_total = round($cartTotals->vat_total, 2);
    $cartTotals->discount = round($cartTotals->discount, 2);
    $this->couponcode = $cart->getFullOrderCoupon();
    return $cartTotals;
  }

  public function recalcCart()
  {
    $this->setAmountsFromCart();
  }

  /**
	 * setting the amounts for an order.
	 */
	private function setAmountsFromCart()
	{
    // get the cart totals
    $cartTotals = $this->getCartTotals($this->ordercode);
    if(empty($cartTotals))
      return;

		// If the ordercountry is still empty..
		// Force the default country to prevent VAT issues.
		if (empty($this->country_code) && LartsConfig::get('use_default_country')) {
			$this->country_code = LartsConfig::get('default_country');
		}
    if (empty($this->country_code)) {
      // last instance -> hard coded
      $this->country_code = 'DE';
    }

		// Prevent to have an higher discount than price.
		// If the discount is higher than the price, the discount will be forced to be the same as the price.
		if ($this->discount > $cartTotals->totalprice) {
			$this->discount = $cartTotals->totalprice;
		}
   
		$this->grossprice = $cartTotals->totalprice;
		$this->discount   = $cartTotals->discount;
		$this->netprice   = round($this->grossprice - $this->discount, 2);
    $this->vat_total  = 0;
    //$this->vat_percents = '';
    $this->vat_bypercent = array();
    
		$this->calcVat();

		$this->vat_required = true;

		// If the user has not been logged in
		if (empty($order->userid) && empty($vat_number)) {
			$this->message = 'VAT has been applied:' . ' No userid set';
			return;
		}

		if (empty($vat_number)) {
			list($vat_number, $country_code) = $this->getVatNumberAndCountryCode($order->userid);
		}

		if (strlen($vat_number) < 4) {
			// VAT number is too short so simply apply VAT
			$this->message = 'VAT has been applied:' . ' VAT number too short (' . $country_code . $vat_number . ')';
			return;
		}

		$vatValidation = new Vat(['debug' => false]);
		$vat_response  = $vatValidation->checkVatNumberAtWebApi($country_code, $vat_number);

    $this->vat_valid = false;
    switch (true) {
    case ($vat_response === true):
      $vat_message     = 'Valid VIES number';
      $this->vat_valid = true;
      break;

    case ($vat_response === false):
      $vat_message = 'Invalid VIES number';
      break;

    default:
      $vat_message = $vat_response;
      break;
		}
		$vat_message .= ' <small>[ '
			. '<a href="http://ec.europa.eu/taxation_customs/vies/vatResponse.html?memberStateCode=' . $country_code . '&number=' . $vat_number . '" target="_blank">'
			. $country_code . $vat_number
			. '</a>'
			. ' ]</small>';

		// Is this country forced to pay VAT?
		if ($country->force_vat_payment) {
			$this->vat_forced = true;
			$this->message = Text::_('COM_LARTSSUBS_VAT_APPLIED') . ':<br>' . $vat_message;
			return;
		}

		// Check if VAT number validates
		if(!$this->vat_valid) {
			$this->message = Text::_('COM_LARTSSUBS_VAT_APPLIED') . ':<br>' . $vat_message;
			return;
		}

		$this->message = Text::_('COM_LARTSSUBS_VAT_NOT_APPLIED') . ':<br>' . $vat_message;
		//$this->calcVat();
	}

  private function getVatNumberAndCountryCode($userid)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select($db->quoteName('vat_number'))
		            ->from($db->quoteName('#__larts_subs_customers'))
		            ->where($db->quoteName('userid') . ' = ' . (int) $userid);

		$db->setQuery($query);
		$user_info = $db->loadObject();

		if (empty($user_info))
		{
			return [0, ''];
		}

		$vat_number = isset($user_info->vat_number) ? $user_info->vat_number : null;

		$string_count = strlen($vat_number);
		$country_code = substr($vat_number, 0, 2);
		$vat_number    = substr($vat_number, 2, $string_count);

		return [$vat_number, $country_code];
	}

	private function calcVat()
	{
    [$country, $vats] = VATClasses::getByCountry($this->country_code);
    
    $cart = new Cart($this->ordercode);
    $items = $cart->getProducts();
    
    
    $this->vat_total = 0;
    //$this->vat_percents = '';
    $this->vat_bypercent = array();
    
		if(!$country->requires_vat) {
			$this->message = 'No VAT applied:' . ' Country has no VAT';
			return;
		}

    foreach($items as $item) {
      $rate = 0;
      $vatclass = $item->vat_class;
      if($vatclass == -1) {   // no VAT?
        if($country->force_vat_payment) {
          $vatclass = 0;    // fallback to std vat
        }
        else {
          continue;
        }
      }
      
      if(isset($vats[$item->vat_class])) {
        $rate = $vats[$item->vat_class];
      }
      if($rate > 0) {
        $vat = round((($item->quantity * $item->price) / 100) * $rate, 2);
        $this->vat_total += $vat;
        
        $vatPecent = isset($this->vat_bypercent[(string)$rate]) ? $this->vat_bypercent[(string)$rate] : 0;
        $vatPecent += $vat;
        $this->vat_bypercent[(string)$rate] = $vatPecent;
      }
    }
    
    $price = $this->grossprice - $this->discount;
		//$this->vat_total      = round(($price / 100) * $this->vatrate_std, 2);
		$this->netprice = $price + $this->vat_total;
    //$this->vat_percents = $this->vatpercentage();
	}

	public function message()
	{
		return isset($this->message) ? $this->message : null;
	}

  public function toPay()
	{
    $toPay = $this->netprice - $this->payed_total;
		return ($toPay > 0) ? round($toPay,2) : 0;
	}
  
	public function price()
	{
		return $this->grossprice();
	}

	public function grossprice()
	{
		return isset($this->grossprice) ? $this->grossprice : 0;
	}

	public function discount()
	{
		return isset($this->discount) ? $this->discount : 0;
	}

	public function netprice()
	{
		return isset($this->netprice) ? $this->netprice : 0;
	}

	public function vatprice()
	{
		return isset($this->vat_total) ? $this->vat_total : 0;
	}

	public function getVatAmountsApplied()
	{
    $strRet = '';
    if(isset($this->vat_bypercent)) {
      foreach($this->vat_bypercent as $percent) {
        if(strlen($strRet) > 0) {
          $strRet .= ' / ';
        }
        $strRet .= $percent .'%';
      }
    }
    return $strRet;
	}

 	public function getVatPercentsAppliedStr()
	{
    $strRet = '';
    if(isset($this->vat_bypercent)) {
      foreach($this->vat_bypercent as $percent => $value) {
        if(strlen($strRet) > 0) {
          $strRet .= ' / ';
        }
        $strRet .= $percent .'%';
      }
    }
    return $strRet;
	}

	public function couponcode()
	{
		return isset($this->couponcode) ? $this->couponcode : '';
	}

  public function bruttoprice()
	{
		return $this->grossprice();
	}

	public function nettoprice()
	{
		return $this->netprice();
	}
 
}


