<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;

defined('_JEXEC') or die;
use Joomla\CMS\Factory;


class Ordercode
{
	protected $db;
	
	public function __construct()
	{
		$this->db = Factory::getDbo();
	}

  	/** Get the ordercode (if it exists) or create a new one */
	public static function getOrCreateOrdercode()
	{
		// Get the ordercode from session if there is one.
		$ordercode = intval(Factory::getSession()->get('ordercode'));
		if(!empty($ordercode)) {
			return $ordercode;
		}

		// No ordercode found in session: create a new ordercode
		$ordercode = self::createNewOrdercode();

		// Creating an ordercode for the client.
		Factory::getSession()->set('ordercode', $ordercode);

		return $ordercode;
	}

	/**
	 * Create a unique ordercode
	 * This will be 10 digits
	 * year, month, day + 4 x random
	 */
	public static function createNewOrdercode()
	{
		$db = Factory::getDbo();

		// Get highest ordercode from database
		$query = $db->getQuery(true)
			->select($db->quoteName('ordercode'))
			->from($db->quoteName('#__larts_subs_cart_items'))
			->order($db->quoteName('ordercode') . ' DESC');

		$db->setQuery($query, 0, 1);

		$ordercode = $db->loadResult();

		if (empty($ordercode) || $ordercode < (date('y') * 1000000 + date('W') * 10000)) {
			$ordercode = date('y') . date('W') . '0000';
		}

		$ordercode++;

    return $ordercode;
	}

}


