<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\PSP;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\Filesystem\File;
use Joomla\Filesystem\Folder;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Logger;
use LartsSubscriptions\PSP\SubsPayment;
use LartsSubscriptions\Message;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Order;
use LartsSubscriptions\Cart;


class PaymentPayPal
{
  public $ipn_logger = 0;

  function __construct()
	{
		$this->ipn_logger = 1;
	}

	/**
	 * Logfile.
	 */
	public function log($text)
	{
		$date = new Date;

		$msg = !empty($text) ? $date->format('Y-m-d H:i:s') . ' - ' . $text : '';
		$msg .= "\n";

		## Open log file:
		$file = JPATH_SITE . '/plugins/lartssubscriptions_payment/paypalexpress/log.txt';

		if(!file_exists($file)) {
			file_put_contents($file, '-- paypalexpress log --'. PHP_EOL);
		}

		$handle = fopen($file, "a");
		fputs($handle, $msg);
		fclose($handle);

		return true;
	}

	public function getTransactionDetails($ordercode)
	{
		// Connecting the database
		$db = Factory::getDbo();

		// Create a new query object.
		$query = $db->getQuery(true)
			->select(['t.*', 'u.name', 'u.email'])
			->from($db->quoteName('#__larts_subs_orders', 't'))
			->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('u.id') . ')')
			->where($db->quoteName('t.ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);

		return $db->loadObject();
	}

	public function getTransactionDetailsByTransactionID($transaction_id)
	{
		// Connecting the database
		$db = Factory::getDbo();

		// Create a new query object.
		$query = $db->getQuery(true)
			->select(['t.*', 'u.name', 'u.email'])
			->from($db->quoteName('#__larts_subs_orders', 't'))
			->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('u.id') . ')')
			->where($db->quoteName('t.transaction_id') . ' = ' . (int) $transaction_id);

		$db->setQuery($query);

		return $db->loadObject();
	}

	public function getTransactionTotal($transaction_id)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select('COUNT(id) AS total')
			->from($db->quoteName('#__larts_subs_transactions'))
			->where($db->quoteName('transaction_id') . ' = ' . $db->quote($transaction_id))
			->group($db->quoteName('transaction_id'));

		$db->setQuery($query);
		return $db->loadResult();
	}

  public function requestExpressCheckout($url, $data = [])
	{
    // Process Paypal...
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));

		$response = urldecode(curl_exec($curl));

    // Process Paypal return...
		$responseNvp = [];

		if (preg_match_all('/(?<name>[^\=]+)\=(?<value>[^&]+)&?/', $response, $matches))
		{
			foreach ($matches['name'] as $offset => $name)
			{
				$responseNvp[$name] = $matches['value'][$offset];
			}
		}

		return $responseNvp;
	}

	public function paypalCheckPaymentReturn($url, $response)
	{
		$request = "cmd=_notify-validate";

		foreach ($response as $varname => $varvalue)
		{
			if (function_exists('get_magic_quotes_gpc') and get_magic_quotes_gpc())
			{
				$varvalue = stripslashes($varvalue);
			}

			$request .= '&' . $varname . '=' . urlencode($varvalue);
		}

		// Make CURL request to PayPal
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);

		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $request);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		$result = curl_exec($ch);
		curl_close($ch);

		return $result;
	}

	// Refunding transactions functionality.

  public function sendNvpRequest($api_endpoint, $requestNvp = [])
	{
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $api_endpoint);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($requestNvp));

		$response = urldecode(curl_exec($curl));

		curl_close($curl);

		$responseNvp = [];

		if (preg_match_all('/(?<name>[^\=]+)\=(?<value>[^&]+)&?/', $response, $matches))
		{
			foreach ($matches['name'] as $offset => $name)
			{
				$responseNvp[$name] = $matches['value'][$offset];
			}
		}

		// Logging
		Logger::_('paypalexpress_refunds_response', [
			'API_ENDPOINT' => $api_endpoint,
			'REQUEST_NVP'  => $requestNvp,
			'RESPONSE'     => $response,
			'RESPONSE_NVP' => $responseNvp,
		]);

		if (isset($responseNvp['ACK'])
			&& $responseNvp['ACK'] != 'Success'
		)
		{
			$this->refundErrorMessage($responseNvp);
		}

		return $responseNvp;
	}

	private function refundErrorMessage($responseNvp)
	{
		for ($i = 0; isset($responseNvp['L_ERRORCODE' . $i]); ++$i)
		{
			$message = Text::sprintf('PLG_LARTSSUBS_PAYPAL_FAILED_REFUND',
				$responseNvp['L_ERRORCODE' . $i],
				$responseNvp['L_LONGMESSAGE' . $i]);
		}

		## Check has not been ticked - No refund will take place:
		Factory::getApplication()->enqueueMessage($message, 'error');

		return;
	}


}
