<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\PSP;
defined('_JEXEC') or die;

use Joomla\CMS\Plugin\CMSPlugin;

class PaymentPlugin extends CMSPlugin
{
	public $helpers = [];
	protected $autoloadLanguage = true;

	public function __construct(&$subject, $config = array())
	{
    parent::__construct($subject, $config);
  }

  public function onSelectPayment($data = array())
  {
  }
  
	/**
	 * Will show the checkout option.
	 */
	public function onRenderMethod($layout)
	{
		return $this->getHelper('checkout')->renderMethod($layout);
	}

	/**
	 * Will process the payment and redirecting the customer to the correct PSP
	 */
	public function onProcess($task = '')
	{
		return $this->getHelper('process')->process($task);
	}

	/**
	 * This method will process the refunds instantiated in the backend when available.
	 */
	public function onRefund($data)
	{
		return $this->getHelper('process')->refund($data);
	}

	/**
	 * This method will show the response when the customers comes back after the payment
	 */
	public function onResponse()
	{
		return $this->getHelper('response')->response();
	}

	/**
	 * This method will trigger and catch IPN payments to process without customer interaction.
	 */
	public function onIPN($layout)
	{
		return $this->getHelper('response')->ipn($layout);
	}

	/**
	 * Triggers the plugin in the backend for checking and activating the update key.
	 */
	public function getExtensionUpdateInfo()
	{
		$plugin_file = JPATH_PLUGINS . '/' . $this->_type . '/' . $this->_name . '/helpers/backend.php';

		if(!file_exists($plugin_file))
		{
			return true;
		}

		return $this->getHelper('backend')->checkUpdateKey();
	}

	/**
	 * Triggers the plugin to update the key in the extensions table.
	 * @return bool
	 */
	public function updateDownloadKeys($data = [])
	{
		$plugin_file = JPATH_PLUGINS . '/' . $this->_type . '/' . $this->_name . '/helpers/backend.php';

		if(!file_exists($plugin_file))
		{
			return true;
		}

		return $this->getHelper('backend')->updateDownloadId($data);
	}

	/**
	 * This function will load the appropiate file to show/do things for the customer.
	 * @return mixed
	 */
	public function getHelper($helper = 'checkout')
	{
		if (isset($this->helpers[$helper])) {
			return $this->helpers[$helper];
		}

		require_once JPATH_PLUGINS. '/' .$this->_type. '/' .$this->_name.'/helpers/' .$helper. '.php';

		$class = $this->_name. 'Helper' . $helper;
		$this->helpers[$helper] = new $class($this);

		return $this->helpers[$helper];
	}
}
