<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\PSP;
defined('_JEXEC') or die;

use LartsSubscriptions\Logger;

class PaypalIPN
{
  public $use_sandbox = true;
  public $ipn_logger = 0;
  public $errno = 0;
  public $errstr = '';

  // Live-URL
  const VERIFY_URI = 'https://ipnpb.paypal.com/cgi-bin/webscr';
  // Sandbox URL
  const SANDBOX_VERIFY_URI = 'https://ipnpb.sandbox.paypal.com/cgi-bin/webscr';

  function __construct()
	{
	}

  public function useSandbox(bool $bSet) {
    $this->use_sandbox = $bSet;
  }

  public function getPaypalUri()
  {
    if ($this->use_sandbox) {
        return self::SANDBOX_VERIFY_URI;
    } else {
        return self::VERIFY_URI;
    }
  }
  
  public function verifyIPN($post_data)
  {
    if(empty($post_data))
      return;
    
		// Make CURL request to PayPal for verification
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->getPaypalUri());
    curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post_data);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
    //curl_setopt($ch, CURLOPT_SSLVERSION, 6);
    //curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
    //curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FORBID_REUSE, 1);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
        'User-Agent: IPN-Verification-Script',
        'Connection: Close',
    ));
    
    // make the request
		$result = curl_exec($ch);
    $info = curl_getinfo($ch);
    curl_close($ch);

    if (!$result) {
        $this->errno = curl_errno($ch);
        $this->errstr = curl_error($ch);
        return false;
    }

    $http_code = $info['http_code'];
    if ($http_code != 200) {
      $this->errstr = 'PayPal responded with ' .$http_code;
      return false;
    }
    
    if ($result != 'VERIFIED') {
     return false;
    }
    return true;
  }
  
  
  
};



