<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\PSP;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\Registration;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\Periods;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Logger;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Message;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\Base\PluginManager;


define("TRANSACTION_UNPROCESSED", 3);
define("TRANSACTION_PROCESSED", 1);

class SubsPayment
{
	var $hasLifetime = false;
	var $hasOnetime  = false;
	var $header = '';
	var $body = '';

	function __construct()
	{
	}

	public function getUserData($transaction_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['tt.*', 'u.name', 'u.email'])
		            ->from($db->quoteName('#__larts_subs_orders', 'tt'))
		            ->where($db->quoteName('transaction_id') . ' = ' . $db->quote($transaction_id))
		            ->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('tt.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->where($db->quoteName('status') . ' != ' . $db->quote(1));

		$db->setQuery($query);

		return $db->loadObject();
	}

	public function store($data, $table)
	{
		if(!$result = Factory::getDbo()->insertObject($table, $data)) {
			return false;
		}

		return true;
	}


	public function sendInvoice($ordercode)
	{
    $config = LartsConfig::get();

		if(!$config->use_invoicing)
		{
			return;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('ordercode') . ' = ' . $ordercode);

		$db->setQuery($query);
		$data = $db->loadObject();

		$invoice = new Invoice((int) $data->id);
		$invoice->createInvoice();

		$args = [];
		// Poulate the items for the invoice uploader.
		$args['id']         = $data->id;
		$args['userid']     = $data->userid;
		$args['ordercode']  = $data->ordercode;
		$args['invoice_number'] = $data->invoice_number;

		// Trigger plugins
		PluginManager::execPlugins('lartssubscriptions', 'onAfterProcessInvoice', $args);

		return $data->id;

	}

	public function clearSession()
	{
		$session = Factory::getSession();
		$session->clear('coupon_code');
		$session->clear('ordercode');

		return true;
	}

	public function setMessage($userid, $variables, $template)
	{
		$message = new Message;

		$this->body = $message->alias($template)
		                      ->user($userid)
		                      ->variables($variables)
		                      ->getBody();

		$this->header = $message->getSubject();

		return;
	}

	public function getMessageBody()
	{
		return $this->body;
	}

	public function getMessageHeader()
	{
		return $this->header;
	}

	/**
	 * Saves the transaction to the database
	 */
	private function saveTransactionData($response)
	{
		$transaction = (object) [
			'transaction_id' => $response['transaction_id'],
			'userid'         => $response['userid'],
			'details'        => $response['payment_details'],
			'amount'         => $response['amount_in_cents'] ? $response['amount'] / 100 : $response['amount'],
			'type'           => $response['payment_type'],
			'plugin'         => $response['plugin'],
			'ordercode'      => $response['ordercode'],
			'amounts'        => $response['amounts'],
			'status'         => $response['status'],
		];

		// Save the transaction and return to main method.
		if (Factory::getDbo()->insertObject('#__larts_subs_transactions', $transaction) != true)
		{
			return false;
		}

		return true;
	}

}
