<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\Utilities\ArrayHelper;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Product;
use LartsSubscriptions\Order;
use LartsSubscriptions\Config as LartsConfig;


class Periods
{
  var $data = null;
  
  // ensure status order at subscription.xml form!
  const STATE_DISABLED = 0;
  const STATE_ACTIVE = 1;
  const STATE_REFUNDED = 2;
  const STATE_CANCELED = 3;

	function __construct()
	{
  }

  public function getItem($id = null)
	{
		if (!is_null($this->data)) {
			return $this->data;
		}

    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('*')
          ->from($db->quoteName('#__larts_subs_periods'))
          ->where($db->quoteName('id') . ' = ' . (int)$id);

    $db->setQuery($query);
    $this->data = $db->loadObject();

    return $this->data;
	}

  public static function insert($data)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->insert($db->quoteName('#__larts_subs_periods'))
          ->columns($db->quoteName(array_keys($data)))
          ->values('\'' .implode('\',\'', $data). '\'');
    $db->setQuery($query);

    $db->execute();

    return $db->insertid();
  }
  
  public static function update($data)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->update($db->quoteName('#__larts_subs_periods'))
          ->set([
            $db->quoteName('product_id') . ' = ' . $db->quote($data->product_id),
            $db->quoteName('order_id') . ' = ' . $db->quote($data->order_id),
            $db->quoteName('userid') . ' = ' . $db->quote($data->userid),
            $db->quoteName('licence') . ' = ' . $db->quote($data->licence),
            $db->quoteName('valid_from') . ' = ' . $db->quote($data->valid_from),
            $db->quoteName('valid_to') . ' = ' . $db->quote($data->valid_to),
          ])
        ->where($db->quoteName('ordercode') . ' = ' . (int) $data->ordercode);

		$db->setQuery($query);
		return $db->execute();
  }

  public static function updateStatus($data)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->update($db->quoteName('#__larts_subs_periods'))
          ->set([
            $db->quoteName('status') . ' = ' . $db->quote($data['status']),
            $db->quoteName('mail_changes') . ' = ' . $db->quote($data['mail_changes']),
            $db->quoteName('user_info') . ' = ' . $db->quote($data['user_info']),
            $db->quoteName('internal_info') . ' = ' . $db->quote($data['internal_info']),
          ])
        ->where($db->quoteName('id') . ' = ' . (int) $data['id']);

		$db->setQuery($query);
		return $db->execute();
  }

  public function remove($cid)
	{
		if (empty($cid)) {
			return true;
		}

    $ids = ArrayHelper::toInteger($cid);
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
                ->delete($db->quoteName('#__larts_subs_periods'))
                ->where($db->quoteName('id') . ' IN ' . '( ' . implode(',', $ids) . ' )');
    $db->setQuery($query);
    $db->execute();

    return $db->execute();
	}

  public function removeByOrdercode($ordercodes = [])
	{
		if (empty($ordercodes)) {
			return true;
		}

    $db = Factory::getDbo();
    $query = $db->getQuery(true)
                ->delete($db->quoteName('#__larts_subs_periods'))
                ->where($db->quoteName('ordercode') . ' IN ' . '( ' . implode(',', $ordercodes) . ' )');
    $db->setQuery($query);
    return $db->execute();
	}

  public static function lookupProduct($userid, $productid)
  {
    $db = Factory::getDbo();

    $query = $db->getQuery(true);
    $query->select('*')
          ->from($db->quoteName('#__larts_subs_periods'))
          ->where($db->quoteName('product_id') . ' = ' . (int)$productid)
          ->andWhere($db->quoteName('userid') . ' = ' . (int)$userid)
          ;
		$db->setQuery($query);
		return $db->loadObjectList();
  }

  public static function LookupLastPeriod($product_id, $userid, $licence = null)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('*')
          ->from($db->quoteName('#__larts_subs_periods'))
          ->where($db->quoteName('product_id') . ' = ' . (int)$product_id) 
          ->andWhere($db->quoteName('userid') . ' = ' . (int)$userid)
          ->order($db->quoteName('id') . ' DESC LIMIT 1')
          ;
    
    if($licence != null) {
      $query->andWhere($db->quoteName('licence') . ' = ' . (int)$licence);
    }
        
    $db->setQuery($query);
    $result = $db->loadObject();

    return $result;
  }
  
  public static function Lookup($ordercode)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('*')
          ->from($db->quoteName('#__larts_subs_periods'))
          ->where($db->quoteName('ordercode') . ' = ' . (int)$ordercode)
          ;

    $db->setQuery($query);
    $result = $db->loadObject();
    return $result;
  }

  public static function getProductsOfUser($userid, $notExpired = false, $sort = 'per.product_id', $sortDir = 'ASC')
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('DISTINCT p.id as product_id, p.content_url, p.extra_url, p.name, p.alias, p.type, p.period_type, p.source_id, p.exclude_renewals, p.pkgmanager_id')
          ->from($db->quoteName('#__larts_subs_products', 'p'))
          ->leftJoin($db->quoteName('#__larts_subs_periods', 'per') . ' ON(' . $db->quoteName('p.id') . ' = ' . $db->quoteName('per.product_id') . ')')
          ->where($db->quoteName('per.userid') . ' = ' . (int)$userid)
          ->order($db->quoteName($sort) . ' ' .$sortDir)
         ;
         if($notExpired == true) {
           $now = new Date;
           $query->andWhere($db->quoteName('per.valid_to') . ' > ' . $db->quote($now->toSql()));
         }
    $db->setQuery($query);
    return $db->loadObjectList();
  }

  public static function getPeriods($userid, $productid, $bMinMax = false)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    
    if($bMinMax) {
      $query->select('min(per.valid_from) as subs_begin, max(per.valid_to) as subs_end, per.status')
            ->from($db->quoteName('#__larts_subs_periods', 'per'))
            ->where($db->quoteName('per.userid') . ' = ' . (int)$userid)
            ->andWhere($db->quoteName('per.product_id') . ' = ' . (int)$productid)
           ;
      $db->setQuery($query);
      return $db->loadObject();
    }

    $query->select('per.*')
          ->from($db->quoteName('#__larts_subs_periods', 'per'))
          ->where($db->quoteName('per.userid') . ' = ' . (int)$userid)
          ->andWhere($db->quoteName('per.product_id') . ' = ' . (int)$productid)
          ->order($db->quoteName('per.valid_from') . ' ASC')
         ;
    
    $db->setQuery($query);
    return $db->loadObjectList();
  }

  public static function getExpired()
  {
    $now = new Date;
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('id')
          ->from($db->quoteName('#__larts_subs_periods'))
          ->where($db->quoteName('valid_to') . ' < ' . $db->quote($now->toSql()))
          ->andWhere($db->quoteName('status') . ' = ' . self::STATE_ACTIVE)
          ;
    $db->setQuery($query);
    return $db->loadObjectList();
  }

  public static function setExpiredInactive()
  {
    $expired = self::getExpired();
    if(empty($expired))
      return;
    $cids  = ArrayHelper::toInteger($expired);
    self::setStatus($cids, $status = self::STATE_DISABLED);
  }

  public static function setStatus($cids, $status = self::STATE_DISABLED)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->update($db->quoteName('#__larts_subs_periods'))
          ->set($db->quoteName('status') . ' = ' . $status)
          ->where($db->quoteName('id') . ' IN ' . '( ' . implode(',', $cids) . ' )');

		$db->setQuery($query);
		$db->execute();
  }
  
  public static function getAllProductsOfUser($userid, $notExpired = false, $sort = 'per.product_id')
  {
    $config = LartsConfig::get();
    $ordering = 'p.id';
    if($config->file_ordering == 'created') {
      $ordering = 'per.valid_from';
    }
    if($config->file_ordering == 'name') {
      $ordering = 'p.name';
    }
    $dir = $config->file_ordering_dir;
    
    //get all products
    $products = self::getProductsOfUser($userid, $notExpired, $ordering, $dir);
    foreach ($products as $item) {
      // get min, max dates...
      $per = self::getPeriods($userid, $item->product_id, true);
      $item->subs_begin = $per->subs_begin; 
      $item->subs_end = $per->subs_end;
      $item->subs_status = $per->status;
    }
    
    return $products;
  }
  
  public static function getAllOfUser($userid, $notExpired = false, $sort = 'per.product_id')
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('per.licence, per.ordercode, per.user_info, per.valid_from as subs_begin, per.valid_to as subs_end, '
            . 'p.id as product_id, p.content_url, p.name, p.alias, p.type, p.source_id, p.pkgmanager_id')
          ->from($db->quoteName('#__larts_subs_periods', 'per'))
          ->leftJoin($db->quoteName('#__larts_subs_products', 'p') . ' ON(' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('p.id') . ')')
          ->where($db->quoteName('per.userid') . ' = ' . (int)$userid)
          ->order($db->quoteName($sort) . ' ASC')
         ;
         if($notExpired == true) {
           $now = new Date;
           $query->andWhere($db->quoteName('per.valid_to') . ' > ' . $db->quote($now->toSql()));
         }

    $db->setQuery($query);
    return $db->loadObjectList();
  }
  
  public static function getLastOfUser($userid, $sourceType, $sourceId)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('per.*, p.type, p.source_id')
          ->from($db->quoteName('#__larts_subs_periods', 'per'))
          ->leftJoin($db->quoteName('#__larts_subs_products', 'p') . ' ON(' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('p.id') . ')')
          ->where($db->quoteName('per.userid') . ' = ' . (int)$userid)
          ->andWhere($db->quoteName('p.type') . ' = ' . (int)$sourceType)
          ->andWhere($db->quoteName('p.source_id') . ' = ' . (int)$sourceId)
          ->order($db->quoteName('per.id') . ' DESC LIMIT 1')
         ;

    $db->setQuery($query);
    return $db->loadObject();
  }

  public static function getAllSubscribedCategoriesOfUser($userid, $notExpired = false)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('per.*, p.type, p.source_id as cat_id')
          ->from($db->quoteName('#__larts_subs_periods', 'per'))
          ->leftJoin($db->quoteName('#__larts_subs_products', 'p') . ' ON(' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('p.id') . ')')
          ->where($db->quoteName('per.userid') . ' = ' . (int)$userid)
          ->andWhere($db->quoteName('p.type') . ' = ' . Product::CATEGORY)
         ;
         if($notExpired == true) {
           $now = new Date;
           $query->andWhere($db->quoteName('per.valid_to') . ' > ' . $db->quote($now->toSql()));
         }

    $db->setQuery($query);
    return $db->loadObjectList();
  }
  
  public function addPeriod($dStart, $ordercode, $productId, $productIdRef, $userid, $licence, $period_type, $nCount,
          $period_status = Periods::STATE_ACTIVE, $userinfo = '', $internalinfo = '')
  {
    if(!empty($dStart)) {
      $tsStart = strtotime($dStart);
    }
    else {
      $date = new Date;
      $today = $date->getTimestamp();
      $tsStart = $today;
    }


    // check if there is an active period
    // Note: We check with the productref here, to grab mixe subscriptions (e.g. year, month)
    $lastPeriod = self::LookupLastPeriod($productIdRef, $userid, $licence);
    if(!empty($lastPeriod)) {
      $tsPeriod_to = strtotime($lastPeriod->valid_to);
      // set start to eof period, if end is beyond start
      if($tsPeriod_to > $tsStart) {
        $tsStart = $tsPeriod_to;
      }
    }
    

    $valid_from = $tsStart;
    $valid_to = self::calcEndDate($valid_from, $period_type, $nCount);

    $valid_fromStr = date('Y-m-d h:i:s', $valid_from);
    $valid_toStr = date('Y-m-d h:i:s', $valid_to);

    // create a new period entry
    $periodValues = [
      'ordercode'       => $ordercode,
      'product_id'      => $productId,
      'product_id_ref'  => $productIdRef,
      'userid'          => $userid,
      'licence'         => $licence,
      'valid_from'      => $valid_fromStr,
      'valid_to'        => $valid_toStr,
      'access'          => (int)0,
      'user_info'       => $userinfo,
      'internal_info'   => $internalinfo,
      'status'          => $period_status,
    ];

    $periods_id = 0;
    $periods_id = self::insert($periodValues);

    return ['id' => $periods_id, 'valid_from' => $valid_fromStr, 'valid_to' => $valid_toStr];
  }
  
  public static function calcEndDate($dStart, $period_type, $nCount)
  {
    $valid_to = new \DateTime;
    $valid_to->setTimestamp($dStart);

    switch ($period_type) {
    case 'days':
      // Expires in xx days
      $valid_to->add(new \DateInterval("P".$nCount."D"));
      break;
    case 'months':
      // Expires in xx months
      //$valid_to->add(new \DateInterval("P".$amount."M"));
      $tsValidTo = DateEx::AddMonthWithoutOvershoot($dStart, $nCount);
      $valid_to->setTimestamp($tsValidTo);
      break;
    case 'years':
      // Expires in xx years
      $valid_to->add(new \DateInterval("P".$nCount."Y"));
      break;
    case 'lifetime':
      // Expires in 250 years :)
      $valid_to->setTimestamp(strtotime("2099-12-31"));
      //Yes, it is a lifetime subscription
      break;
    default:
    case 'onetime':
      // Expires in 1 day
      $valid_to->add(new \DateInterval("P1D"));
      break;
    }

    return $valid_to->getTimestamp();
  }

  public static function str($period, $period_type)
  {
    if($period_type == 'lifetime' || $period_type == 'onetime') {
      return ucfirst($period_type);
    }
      
    if($period == 1) {
      return $period. ' ' . substr($period_type, 0, strlen($period_type)-1);
    }
    else {
      return $period. ' ' .$period_type;
    }
  }
          
  
  public static function isExpired($period)
  {
		$now = new Date('now');
    $expire = new Date($period->valid_to);

    if($now > $expire) {
      return true;
    }
    return false;
  }
  
}


