<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use LartsSubscriptions\Config as LartsConfig;

class Price
{
	public static function _($price = 0, $include_currency = true, $colorize = false)
	{
		$amount = self::formatPrice($price);

		if(!$include_currency) {
			return $amount . ''; // Make sure it is a string, not an int
		}

		$currency = $include_currency ? self::getCurrency() : '';

		if (LartsConfig::get()->currency_position == 'after') {
			$html = $amount . '&nbsp;' . $currency;
		}
    else {
      $html = $currency . '&nbsp;' . $amount;
    }

    if($colorize) {
      $span = '<span style="color:'. (floatval(str_replace(',', '.', $amount)) >= 0.0 ? 'green' : 'red'). '">';
      $html = $span . $html . '</span>';
    }
		return $html;
	}

	public static function getDiscountText($group, $type, $amount, $discount)
	{
		$discount = self::getDiscountAmountText($type, $amount, $discount);

		return Text::sprintf(
			'COM_LARTSSUBS_DISCOUNT_' . ($group ? strtoupper($group) : 'FALLBACK'),
			$discount
		);
	}

	public static function getDiscountAmountText($type, $amount, $discount)
	{
		$discount = Price::_($discount);

		if ($type != 'percent')
		{
			return $discount;
		}

		return Text::sprintf(
			'COM_LARTSSUBS_DISCOUNT_AMOUNT',
			$amount . '%',
			$discount
		);
	}

	private static function formatPrice($price = 0)
	{
		$price = !empty($price) ? $price : 0;

		switch (LartsConfig::get()->price_format)
		{
			case 2:
				return number_format($price, '2', '.', ',');
			case 3:
				return number_format($price, 2, ',', '');
			case 4:
				return number_format($price, 2, '.', '');
			case 5:
				return number_format($price, '0', '', '.');
			case 6:
				return number_format($price, '0', '', ',');
			case 7:
				return number_format($price, '0', '', '');
			case 1:
			default:
				return number_format($price, 2, ',', '.');
		}
	}

	public static function getCurrency()
	{
		return htmlentities(LartsConfig::get()->currency);
	}
}
