<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use DateInterval;
use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Language\Text;

//require_once JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/helpers/groups.php';
//require_once JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/helpers/discount.php';

use LartsSubscriptions\Discount;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\BundledProducts;


class Renewal
{
  const PERIOD_EXPIRED = 0;
  const PERIOD_ACTIVE = 1;
  const PERIOD_REMINDER1 = 2;
  const PERIOD_REMINDER2 = 3;
  const PERIOD_REMINDER3 = 4;
  const PERIOD_GRACE = 5;
  
	var $messages = [];

	var $config        = null;
	var $discount      = null;
	var $user_products = [];

	var $ordercode = null;
	//var $upgrades  = null;
	var $renewals  = null;

	public function __construct($ordercode = null)
	{
		$this->config    = LartsConfig::get();

    if(!empty($ordercode)) {
      $this->ordercode = $ordercode;
      $this->discount  = new Discount($ordercode);
  		$this->user_products = $this->discount->getUserProductIds(true);
    }
	}

  public function getRenewalDaysStr()
	{
    $strRet = '';
    if($this->config->send_first_reminder == 1) {
      $strRet = sprintf('%d', $this->config->first_expiration_days);
    }
    if($this->config->send_second_reminder == 1) {
      if(!empty($strRet)) {
        $strRet .= ', ';
      }
      $strRet .= sprintf('%d', $this->config->second_expiration_days);
    }
    if($this->config->send_third_reminder == 1) {
      if(!empty($strRet)) {
        $strRet .= ', ';
      }
      $strRet .= sprintf('%d', $this->config->third_expiration_days);
    }
    $strRet .= Text::_('COM_LARTSSUBS_RENEW_DAYS');
		return $strRet;
	}

  public function apply(&$orders)
	{
		if(!$this->config->renewal_discount)
		{
			return;
		}

		$this->getRenewalDiscounts($orders);

		foreach ($orders as &$order)
		{
			if(!$discount = $this->getDiscountFromOrder($order)) {
				continue;
			}

			$order->discount_data = $discount;
		}
	}

	private function getDiscountFromOrder($order)
	{
		if(!isset($this->renewals[$order->product_id])) {
			return false;
		}

		$renewal = isset($this->renewals[$order->product_id]) ? $this->renewals[$order->product_id] : 0;

		// get discount
		return (object) [
			'discount'        => $this->getDiscount($order->price, $renewal),
			'discount_amount' => $renewal,
			'discount_type'   => 'percent',
			'type'            => 'renewal',
		];
	}

	private function getRenewalDiscounts(&$orders)
	{
		if(!$this->config->renewal_discount) {
			return;
		}

		foreach ($orders as $i => &$order)
		{
			if (isset($this->renewals[$order->product_id])) {
				continue;
			}

			if(!$this->isRenewal($order)) {
				continue;
			}

			$this->renewals[$order->product_id] = $this->getRenewalPercentage($order);
		}
	}

	private function getRenewalPercentage($order)
	{
		if(!$this->config->renewal_discount
			&& ! $this->config->first_expiration_discount
			&& ! $this->config->second_expiration_discount
			&& ! $this->config->third_expiration_discount
		)
		{
			return 0;
		}

		if(!$this->config->first_expiration_discount
			&& ! $this->config->second_expiration_discount
			&& ! $this->config->third_expiration_discount
		)
		{
			return $this->config->renewal_discount;
		}

		$expiration_date = $this->getExpirationDate($order);

		if (empty($expiration_date))
		{
			return 0;
		}

		$now = new Date;

		// Calculate the amount of days left in the subscription
		$days_left = floor((strtotime($expiration_date) - strtotime($now)) / (3600 * 24));

		if ($days_left < $this->config->third_expiration_days)
		{
			return $this->config->third_expiration_discount;
		}

		if ($days_left < $this->config->second_expiration_days)
		{
			return $this->config->second_expiration_discount;
		}

		if ($days_left < $this->config->first_expiration_days)
		{
			return $this->config->first_expiration_discount;
		}

		return $this->config->renewal_discount;
	}

	private function getExpirationDate($order)
	{
		$db = Factory::getDbo();

		// Create the date to send reminders:
		$valid_to = new Date;
		$status   = '1';

		if ($this->config->expiration_grace_period)
		{
			$valid_to = new Date('- ' . (int) $this->config->expiration_grace_period . 'DAYS');
			$status   = '0, 1';
		}

		$valid_to = $valid_to->toSql();

		$query = $db->getQuery(true)
			->select('valid_to')
			->from($db->quoteName('#__larts_subs_periods', 'p2u'))
			->where($db->quoteName('userid') . ' = ' . (int) $order->userid)
			->where($db->quoteName('status') . ' IN (' . $status . ')')
			->where($db->quoteName('product_id') . ' = ' . $db->quote($order->product_id))
			->where($db->quoteName('valid_to') . ' >= ' . $db->quote($valid_to))
			->order($db->quoteName('valid_to') . ' ASC');

		$db->setQuery($query);

		return $db->loadResult();
	}

  public function getRenewalInfo($dExpire)
  {
		$now = new Date;
		$dExpire = new Date($dExpire);
    
    $info = [
      'state' => self::PERIOD_ACTIVE,
      'expired' => false,
      '$dExpire' => $dExpire,
      'discount' => 0,
      'daysTilExpire' => 0,
    ];

    // expired?
    if($dExpire < $now)
    {
      $info['state'] = self::PERIOD_EXPIRED;
      $info['discount'] = $this->config->renewal_discount;
      $info['expired'] = true;

      $date1 = date_create($now->format('Y-m-d'));
      $date2 = date_create($dExpire);
      $diff  = date_diff($date1, $date2);
      $info['daysTilExpire'] = max(0, $diff->format('%a'));
      
       // check grace period
			$dStart = new Date($dExpire);
      $dEnd = new Date($dExpire);
      $dEnd->add(new DateInterval('P' .(int)$this->config->third_expiration_days. 'D'));
      if($dStart >= $now && $dEnd <= $now) {
        $info['state'] = self::PERIOD_GRACE;
        
        // set discount of last available reminder discount
        if($this->config->send_third_reminder) {
          $info['discount'] = $this->config->third_expiration_discount;
        }
        else if($this->config->send_second_reminder) {
          $info['discount'] = $this->config->second_expiration_discount;
        }
        else if($this->config->send_first_reminder) {
          $info['discount'] = $this->config->first_expiration_discount;
        }
      }
      
      return $info;
    }
    
		// 1st,2nd,3rd reminders
    if ($this->config->send_third_reminder)
		{
			$dStart = new Date($dExpire);
      $dStart->sub(new DateInterval('P' .(int)$this->config->third_expiration_days. 'D'));
      if($dStart <= $now) {
        $info['state'] = self::PERIOD_REMINDER3;
        $info['discount'] = $this->config->third_expiration_discount;
      }
		}
		if ($this->config->send_second_reminder)
		{
			$dStart = new Date($dExpire);
      $dStart->sub(new DateInterval('P' .(int)$this->config->second_expiration_days. 'D'));
      if($dStart <= $now) {
        $info['state'] = self::PERIOD_REMINDER2;
        $info['discount'] = $this->config->second_expiration_discount;
      }
		}
		if ($this->config->send_first_reminder)
		{
			$dStart = new Date($dExpire);
      $dStart->sub(new DateInterval('P' .(int)$this->config->first_expiration_days. 'D'));
      if($dStart <= $now) {
        $info['state'] = self::PERIOD_REMINDER1;
        $info['discount'] = $this->config->first_expiration_discount;
      }
		}

    return $info;
  }

	private function getDiscount($price, $discount, $type = 'percent')
	{
		return $this->discount->getDiscount($price, $discount, $type);
	}

	private function isUpgrade($children)
	{
		$intersect = array_intersect($this->user_products, array_keys($children));

		return ! empty($intersect);
	}

	private function isRenewal($order)
	{
		return in_array($order->product_id, $this->user_products);
	}
}
