<?php
/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Periods;


class Statistics
{
	var $db = null;
  var $stats = null;

  public function __construct()
  {
		$this->db = Factory::getDbo();
    $now = new Date();

    $this->stats = new \stdClass;
    $this->stats->date_time = new Date('-1 day');
    $this->stats->week = $now->format('W');
    $this->stats->month = $now->format('m');
    $this->stats->year = $now->format('Y');
    $this->stats->date_time                    = null;
    $this->stats->week                         = null;
    $this->stats->month                        = null;
    $this->stats->year                         = null;
    $this->stats->subscription_reminders       = 0;
    $this->stats->coupons_disabled             = 0;
    $this->stats->subscriptions_disabled       = 0;
    $this->stats->download_ids_generated       = 0;
    $this->stats->total_clicks_orderbutton     = 0;
    $this->stats->unique_clicks_orderbutton    = 0;
    $this->stats->coupons_used                 = 0;
    $this->stats->active_subscriptions         = 0;
    $this->stats->total_lifetime_subscriptions = 0;
    $this->stats->total_onetime_subscriptions  = 0;
    $this->stats->total_downloads              = 0;
    $this->stats->products_sold                = 0;
    $this->stats->sales_conversion              = 0;
    $this->stats->total_earnings    = 0;
    $this->stats->total_vat         = 0;
    $this->stats->total_refunds     = 0;
    $this->stats->downloads_updater = 0;
    $this->stats->downloads_web     = 0;
  }

  public function load()
  {
		$query = $this->db->getQuery(true)
		                  ->select('*')
		                  ->from($this->db->quoteName('#__larts_subs_statistics'));
		$this->db->setQuery($query);
		$this->stats = $this->db->loadObject();
  }

  public function save()
  {
		Factory::getDbo()->insertObject('#__larts_subs_statistics', $this->stats);
  }
  
	public function calculateStatistics()
	{
		$db = Factory::getDbo();

		$now   = new Date;
		$today = $now->format('Y-m-d');

		$yesterday = new Date('-1 day');
		$yesterday = $yesterday->format('Y-m-d');

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_cart_items'))
		            ->where($db->quoteName('date') . ' >= ' . $db->quote($yesterday))
		            ->where($db->quoteName('date') . ' < ' . $db->quote($today));

		$db->setQuery($query);
		$clicks = $db->loadObject();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName('created') . ' >= ' . $db->quote($yesterday))
		            ->where($db->quoteName('created') . ' < ' . $db->quote($today))
		            ->group($db->quoteName('ip_address'));

		$db->setQuery($query);
		$unique_clicks = $db->loadObjectList();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_cart_items'))
		            ->where($db->quoteName('date') . ' >= ' . $db->quote($yesterday))
		            ->where($db->quoteName('date') . ' < ' . $db->quote($today))
		            ->where($db->quoteName('coupon_code') . ' != ""');

		$db->setQuery($query);
		$coupons = $db->loadObject();

		// Build the query to active user count:
    $user_count = (object)array('total' => 0);

    // Build the query to lifetime subscriptions count:
    $lifetime_count = (object)array('total' => 0);

    $onetime_count = (object)array('total' => 0);

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_logging'))
		            ->where($db->quoteName('date') . ' >= ' . $db->quote($yesterday))
		            ->where($db->quoteName('date') . ' < ' . $db->quote($today));

		$db->setQuery($query);
		$downloads = $db->loadObject();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_logging'))
		            ->where($db->quoteName('date') . ' >= ' . $db->quote($yesterday))
		            ->where($db->quoteName('date') . ' < ' . $db->quote($today))
		            ->where($db->quoteName('request_type') . ' = 0');

		$db->setQuery($query);
		$web_downloads = $db->loadObject();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_logging'))
		            ->where($db->quoteName('date') . ' BETWEEN ' . $db->quote($yesterday) . ' AND ' . $db->quote($today))
		            ->where($db->quoteName('request_type') . ' = 1');

		$db->setQuery($query);
		$updater_downloads = $db->loadObject();

    $sold_products = 0;

		$query = $db->getQuery(true)
		            ->select(['SUM(vat_total) AS vat', 'SUM(gross_price)-SUM(disco_price) AS total'])
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('invoicedate') . ' BETWEEN ' . $db->quote($yesterday) . ' AND ' . $db->quote($today))
		            ->where($db->quoteName('gross_price') . ' >= 0');

		$db->setQuery($query);
		$earnings = $db->loadObject();

		$query = $db->getQuery(true)
		            ->select(['SUM(gross_price) AS total'])
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('invoicedate') . ' BETWEEN ' . $db->quote($yesterday) . ' AND ' . $db->quote($today))
		            ->where($db->quoteName('gross_price') . ' < 0');

		$db->setQuery($query);
		$refunds = $db->loadObject();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS total'])
		            ->from($db->quoteName('#__larts_subs_periods'))
		            ->where($db->quoteName('status')  . ' = ' . Periods::STATE_DISABLED);

		$db->setQuery($query);
		$disabled = $db->loadObject();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id)'])
		            ->from($db->quoteName('#__larts_subs_email_queue'))
		            ->where($db->quoteName('date') . ' = ' . $db->quote($today))
		            ->where($db->quoteName('template') . ' = ' . $db->quote('subscription-expiry-email'));

		$db->setQuery($query);
		$reminder_count = $db->loadResult();

		// Create and populate an object.
		$this->stats->date_time                    = $yesterday;
		$this->stats->week                         = $now->format('W');
		$this->stats->month                        = $now->format('m');
		$this->stats->year                         = $now->format('Y');
		$this->stats->subscription_reminders       = $reminder_count;
		$this->stats->coupons_disabled             = $this->stats->coupons_disabled;
		$this->stats->subscriptions_disabled       = $disabled->total;
		$this->stats->download_ids_generated       = $this->stats->download_ids_generated;
		$this->stats->total_clicks_orderbutton     = $clicks->total;
		$this->stats->unique_clicks_orderbutton    = count($unique_clicks);
		$this->stats->coupons_used                 = $coupons->total;
		$this->stats->active_subscriptions         = $user_count->total;
		$this->stats->total_lifetime_subscriptions = $lifetime_count->total;
		$this->stats->total_onetime_subscriptions  = $onetime_count->total;
		$this->stats->total_downloads              = $downloads->total;
		$this->stats->products_sold                = $sold_products->total;
		if ($sold_products->total > 0)
		{
			$this->stats->sales_conversion = ($sold_products->total / count($unique_clicks)) * 100;
		}
		else
		{
			$this->stats->sales_conversion = 0;
		}
		$this->stats->total_earnings    = $earnings->total;
		$this->stats->total_vat         = $earnings->vat;
		$this->stats->total_refunds     = $refunds->total;
		$this->stats->downloads_updater = $updater_downloads->total;
		$this->stats->downloads_web     = $web_downloads->total;

		$this->save();

		return true;
	}
  
  public static function getOrdersPerMonth()
  {
    $db = Factory::getDbo();

    $query = $db->getQuery(true);
    //            ->select('MONTH(created) as month, count(id)')
    //            ->from($db->quoteName('#__larts_subs_orders'))
    //            ->group($db->quoteName('created'));
    //$db->setQuery($query);
    ////$sql = $query->__toString();
    //$data = $db->loadObjectList();

    //$union = 

    $sql = 'SELECT y, m, Count(id) as num
    FROM (
      SELECT y, m
      FROM
        (SELECT YEAR(CURDATE()) y UNION ALL SELECT YEAR(CURDATE())-1) years,
        (SELECT 1 m UNION ALL SELECT 2 UNION ALL SELECT 3 UNION ALL SELECT 4
          UNION ALL SELECT 5 UNION ALL SELECT 6 UNION ALL SELECT 7 UNION ALL SELECT 8
          UNION ALL SELECT 9 UNION ALL SELECT 10 UNION ALL SELECT 11 UNION ALL SELECT 12) months) ym
      LEFT JOIN #__larts_subs_orders
      ON ym.y = YEAR(created)
         AND ym.m = MONTH(created)
    WHERE
      (y=YEAR(CURDATE()) AND m<=MONTH(CURDATE()))
      OR
      (y<YEAR(CURDATE()) AND m>MONTH(CURDATE()))
    GROUP BY y, m';
    $db->setQuery($sql);
    $data = $db->loadObjectList();
    return $data;
  }
}
