<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Logger;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Message;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\BundledProducts;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\Periods;
use LartsSubscriptions\Template;
use LartsSubscriptions\MassMailTemplates;
use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Coupon;

class SubscriptionManager
{
	public $subscription_reminders;
	public $subscriptions_disabled;
	public $download_ids_generated;

  function __construct()
	{
		// For stats table...
		$this->subscription_reminders = 0;
		$this->subscriptions_disabled = 0;
		$this->download_ids_generated = 0;
	}


	/**
	 * Fee subscriptions aren't reminded unless the user has a registration.
	 * If there is no registration, then remove the reminder email.
	 */
	private function removeQMailWithoutUserId()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_email_queue'))
		            ->where($db->quoteName('userid') . ' = 0');

		$db->setQuery($query);

		return $db->execute();
	}

	public function sendQMail()
	{
    $mails_sent = 0;
    // Removing mails without userid.
		$this->removeQMailWithoutUserId();

    $config = LartsConfig::get();
		$db     = Factory::getDbo();
		$now = new Date;

    if($config->mailer_enabled == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MAILER_DISABLED'), 'info');
      Logger::_('cronjob', Text::_('COM_LARTSSUBS_MAILER_DISABLED'));
      return false;
    }
    
		$query = $db->getQuery(true)
		            ->select(['q.*', 'u.name', 'u.email'])
		            ->from($db->quoteName('#__larts_subs_email_queue', 'q'))
		            ->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('q.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->where($db->quoteName('status') . ' = '. MassMailQueue::MAIL_UNSENT)
		            ->andWhere($db->quoteName('pending_till') . ' > ' . $db->quote($now->format('Y-m-d')))
		            ->order($db->quoteName('id') . ' ASC');

		$db->setQuery($query, 0, (int) $config->send_reminders_batch_limit);

		$data = $db->loadObjectList();
    if(empty($data)) {
      return 0;
    }

    
		foreach ($data as $row)
		{
			if ($row->email == '' || $row->name == '') {
				continue;
			}

			$from_name      = ! empty($row->from_name) ? $row->from_name : ($config->emails_sent_from_name ?: $config->company_name);
			$from_email     = ! empty($row->from_email) ? $row->from_email : ($config->emails_sent_from ?: $config->company_email);
			$reply_to_name  = ! empty($row->reply_to_name) ? $row->reply_to_name : $config->emails_reply_to_name;
			$reply_to_email = ! empty($row->reply_to_email) ? $row->reply_to_email : $config->emails_reply_to;

			$mailer = Factory::getMailer();
      
			$mailer->setSubject($row->subject);
			$mailer->setBody($row->body);
			$mailer->setSender([$from_email, $from_name,]);
			$mailer->addRecipient($row->email, $row->name);
			$mailer->isHtml(true);

			if(empty($reply_to_name)) {
        $site = $config = Factory::getConfig()->get('sitename');
        $reply_to_name = $site;
			}
			if(empty($reply_to_email)) {
        //$site = $config = Factory::getConfig()->get('sitename');
        $host = $_SERVER['HTTP_HOST'];
        $reply_to_email = 'noreply@' .$host;
			}
      $mailer->addReplyTo($reply_to_email, $reply_to_name);


			if(!$mailer->Send()) {
				continue;
			}
      
      $mails_sent++;

			$query = $db->getQuery(true)
			            ->update($db->quoteName('#__larts_subs_email_queue'))
			            ->set([
				            $db->quoteName('status') . ' = '. MassMailQueue::MAIL_SENT,
				            $db->quoteName('date_sent') . ' = ' . $db->quote($now->format('Y-m-d')),
			            ])
			            ->where($db->quoteName('id') . ' = ' . $db->quote($row->id));
			$db->setQuery($query);
			$db->execute();
		}

		return $mails_sent;
	}


	// Removed all temporary transaction after XX days
	// Amount of days needs to be set in the config.

  public function purgePendingTransactions()
	{
		// Getting the configuration
		$config = LartsConfig::get();

		// Check if the cleanup_days and purge settings are set.
		if(!$config->purge_temporary_transactions || ! $config->cleanup_days) {
			return false;
		}

		$db = Factory::getDbo();

		// get expiry date
		$date = new Date('-' . $config->cleanup_days . ' days');
		$removal_date = $date->format('Y-m-d H:i:s');

    Transaction::purgePendingTransactionsBeforeDate($removal_date);
    
		return true;
	}

	public function removeUnfinishedOrders()
	{
		$config = LartsConfig::get();

		if(!$config->auto_clean_orders_table || ! $config->cleanup_days) {
			return false;
		}

		$date = new Date('-' . $config->cleanup_days . ' days');

		$removal_date = $date->format('Y-m-d');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['o.ordercode AS attempted_sale', 't.ordercode AS final_sale'])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_transactions', 't') . ' ON (' . $db->quoteName('o.ordercode') . ' = ' . $db->quoteName('t.ordercode') . ')')
		            ->where($db->quoteName('o.date') . ' < ' . $db->quote($removal_date));

		$db->setQuery($query, 0, $config->auto_clean_limit);
		$items = $db->loadAssocList('attempted_sale', 'final_sale');

		foreach ($items as $attempted_sale => $final_sale)
		{
			if(!empty($final_sale))
			{
				continue;
			}

			$query = $db->getQuery(true)
			            ->delete($db->quoteName('#__larts_subs_cart_items'))
			            ->where([
				            $db->quoteName('ordercode') . ' = ' . $attempted_sale,
			            ]);

			$db->setQuery($query);
			$db->execute();
		}

		return true;
	}

	public function disableSubscriptions()
	{
		$config = LartsConfig::get();

		$date = new Date;
		$date = $date->format('Y-m-d');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['per.*', 'u.email', 'prod.name as productname', 'prod.alias as productalias'])
		            ->from($db->quoteName('#__larts_subs_periods', 'per'))
		            ->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('per.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('prod.id') . ')')
		            ->where($db->quoteName('per.valid_to') . ' < ' . $db->quote($date))
		            ->where($db->quoteName('per.status') . ' = '. Periods::STATE_ACTIVE)
		            ->where($db->quoteName('prod.period_type') . ' NOT IN (' . $db->quote('lifetime') . ',' . $db->quote('onetime') . ')');

		$db->setQuery($query);
		$data = $db->loadObjectList();

		// For the statistics:
		$this->subscriptions_disabled = count($data);

		if (empty($data)) {
			return true;
		}

		$ids = [];

		foreach ($data as $row)
		{
			if (!$this->hasActiveParents($row))
			{
				// Variables for the message:
				$variables = [
					'product'            => $row->productname, // As product is available in all mails, I have added it here too.
					'product_alias'      => $row->productalias,
					'subscription'       => $row->productname, // BC alias for product
					'subscription_alias' => $row->productalias,
					'date'               => 'NOW',
				];

				$message = new Message;
				$message->alias('subscription-expired')
					->user($row->userid)
					->variables($variables);

				$tmpl     = $message->getTemplate();
				$body    = $message->getBody();
				$subject = $message->getSubject();

				$from_name      = ! empty($tmpl->from_name) ? $tmpl->from_name : '';
				$from_email     = ! empty($tmpl->from_email) ? $tmpl->from_email : '';
				$reply_to_name  = ! empty($tmpl->reply_to_name) ? $tmpl->reply_to_name : '';
				$reply_to_email = ! empty($tmpl->reply_to_email) ? $tmpl->reply_to_email : '';

				// Now store all data in the email queue table
				$mail = (object) [
					'template'       => 'subscription-expired',
					'subject'        => $subject,
					'date'           => $date,
					'body'           => $body,
					'userid'         => $row->userid,
					'from_email'     => $from_email,
					'from_name'      => $from_name,
					'reply_to_email' => $reply_to_email,
					'reply_to_name'  => $reply_to_name,
					'receive_bcc'    => $tmpl->receive_bcc,
				];

				$query = $db->getQuery(true);

				$db->insertObject('#__larts_subs_email_queue', $mail);


				$ids[] = $row->id;
			}

			// Fields to update.
			$fields = [$db->quoteName('status') . ' = 0'];
			// Conditions for which records should be updated.
			$conditions = [$db->quoteName('id') . ' = ' . $db->quote($row->id)];
			// The magic query:
			$query->update($db->quoteName('#__larts_subs_product2user'))
			      ->set($fields)
			      ->where($conditions);

			$db->setQuery($query);
			$db->execute();
		}

		PluginManager::execPlugins('lartssubscriptions', 'onAfterRemoveSubscription', $ids);

		return true;
	}

	public function sendReminder($type = 'first', $discount_pct = 15, $max_days = 30, $min_days = 0)
	{
		$db = Factory::getDbo();

		// Create the date range to send reminders:
		$valid_from = new Date('+' . ($min_days ?: $max_days) . ' days');
		$valid_from = $valid_from->format('Y-m-d');

		$valid_to = new Date('+' . $max_days . ' days');
		$valid_to = $valid_to->format('Y-m-d');

    echo ' - Expiry range: '.$valid_from.' - '.$valid_to.'<br>';

		switch ($type) {
    case 'third':
      $type_nr = 3;
      break;
    case 'second':
      $type_nr = 2;
      break;
    default:
      $type_nr = 1;
      break;
		}

		$query = $db->getQuery(true)
		            ->select(['per.*', 'u.email', 'u.name as clientname', 'prod.name as productname', 'prod.alias as productalias', 'prod.exclude_reminders'])
		            ->from($db->quoteName('#__larts_subs_periods', 'per'))
		            ->join('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('per.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('prod.id') . ')')
		            ->where($db->quoteName('per.valid_to') . ' BETWEEN ' . $db->quote($valid_from) . ' AND ' . $db->quote($valid_to))
		            ->where($db->quoteName('per.status') . ' = '. Periods::STATE_ACTIVE)
		            ->where($db->quoteName('per.reminder_sent') . ' < ' . $type_nr)
		            ->where($db->quoteName('prod.period_type') . ' NOT IN (' . $db->quote('lifetime') . ',' . $db->quote('onetime') . ')')
		            ->order($db->quoteName('per.valid_to') . ' ASC');

    
		$db->setQuery($query);
		$data = $db->loadObjectList();
    
    echo ' - Found '.count($data).' subscriptions<br>';
    if(empty($data)) {
      return true;
    }

		foreach ($data as $i => $row)
		{
			// Check for active higher group subscriptions and remove this sub from the list of emails to send
			if ($this->hasActiveParents($row)) {
				unset($data[$i]);
			}
		}

		// For the statistics
		$this->subscription_reminders = $this->subscription_reminders + count($data);

		if (empty($data)) {
			return true;
		}

		$now = new Date;
		$now = $now->format('Y-m-d');

		foreach ($data as $row)
		{
			if($row->exclude_reminders == 1) {
				continue;
			}

      $coup = new Coupon;
      $coupon = '';
      if($discount_pct > 0) {
        $coupon = $coup->GenerateCouponCode();
      }

			$time_left = strtotime($row->valid_to) - strtotime($now);
			$days_left = floor($time_left / (60 * 60 * 24));

			// Variables for the message:
			$variables = [
				'product'            => $row->productname, 
				'product_alias'      => $row->productalias,
				'subscription'       => $row->productname, // BC alias for product
				'subscription_alias' => $row->productalias,
				'days'               => $days_left,
				'date'               => $row->valid_to,
				'coupon'             => $coupon,
				'discount'           => $discount_pct,
				//'download_id'        => $row->download_id,
			];

			$message = new Message;
			$message->alias('subscription-expiry-email')
				->user($row->userid)
				->variables($variables);

			$tmpl     = $message->getTemplate();
			$subject = $message->getSubject();

			$body = $message->getBody();

			$from_name      = ! empty($tmpl->from_name) ? $tmpl->from_name : '';
			$from_email     = ! empty($tmpl->from_email) ? $tmpl->from_email : '';
			$reply_to_name  = ! empty($tmpl->reply_to_name) ? $tmpl->reply_to_name : '';
			$reply_to_email = ! empty($tmpl->reply_to_email) ? $tmpl->reply_to_email : '';

      $template_id = Template::lookupIdByAlias('subscription-expiry-email');
      
			// Now store all data in the table
      $batch_id =  MassMailQueue::getUniqueBatchId($tmpl->id);
      
			$mail = (object) [
				//'template_type'  => Template::TYPE_MAIN,
				//'template_id'    => $template_id,
        'batch_id'         => $batch_id,
				'status'           => MassMailQueue::MAIL_UNSENT,
				'userid'           => $row->userid,
				'subject'          => $subject,
				'body'             => $body,
				'pending_till'     => $valid_to,
        'date_sent'        => null,
				'from_email'     => $from_email,
				'from_name'      => $from_name,
				'reply_to_email' => $reply_to_email,
				'reply_to_name'  => $reply_to_name,
			];

			Factory::getDbo()->insertObject('#__larts_subs_email_queue', $mail);

      if($discount_pct > 0)
      {
        // Now store all data in the discount table
        $coupondata = [
          'limit'         => 1,
          'used'          => 0,
          'coupon_code'   => $coupon,
          'name'          => 'Auto renewal for: ' . $row->clientname,
          'valid_to'      => $row->valid_to,
          'discount_type' => 'percent',
          'discount'      => $discount_pct,
          'product_ids'   => $row->product_id,
          'userid'        => $row->userid,
          'full_order'    => 0,
          'type'          => 1,
          'published'     => 1,
          'created_by'    => 0
        ];
        $coupon = new Coupon;
        $coupon->storeCoupon($coupondata);
        //Factory::getDbo()->insertObject('#__larts_subs_coupons', $discount);
      }
      
			// Update the reminder_sent value
			$query = $db->getQuery(true)
			            ->update($db->quoteName('#__larts_subs_periods'))
			            ->set($db->quoteName('reminder_sent') . ' = ' . $type_nr)
			            ->where($db->quoteName('id') . ' = ' . $db->quote($row->id));

			$db->setQuery($query);
			$db->execute();
		}

		return true;
	}


	private function hasActiveParents($user_product)
	{
		$parent_products = BundledProducts::getParentsById($user_product->product_id);

		if (empty($parent_products))
		{
			return false;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select('id')
			->from($db->quoteName('#__larts_subs_periods'))
			->where($db->quoteName('userid') . ' = ' . $db->quote($user_product->userid))
			->where($db->quoteName('valid_to') . ' > ' . $db->quote($user_product->valid_to))
			->where($db->quoteName('product_id') . ' IN (' . implode(',', array_keys($parent_products)) . ')');

		$db->setQuery($query);
		$parent_subs = $db->loadColumn();

		return ! empty($parent_subs);
	}
}
