<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Order;


class Transaction
{
  const PENDING = 0;
  const PROCESSED = 1;
  const CANCELED = 2;
  const FAILED = 3;
  const REFUNDED = 4;

  public function __construct()
	{
	}

  public static function store($data)
	{
		if(!$result = Factory::getDbo()->insertObject('#__larts_subs_transactions', $data)) {
			return false;
		}

		return true;
	}

  public static function getUserIdFromTransaction($ordercode = 0)
	{
		if (empty($ordercode))
		{
			return 0;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select($db->quoteName('userid'))
			->from($db->quoteName('#__larts_subs_transactions'))
			->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query, 0, 1);

		return (int) $db->loadResult();
	}

  public static function setStatus($transaction_id, $status)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
    $query->update($db->quoteName('#__larts_subs_transactions'))
          ->set($db->quoteName('status') . '='. $status)
          ->where($db->quoteName('transaction_id') .'='. $db->quote($transaction_id));
    $db->setQuery($query);

    return $db->execute();
	}

  public static function setTransactionId($ordercode, $transaction_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
    $query->update($db->quoteName('#__larts_subs_transactions'))
          ->set($db->quoteName('transaction_id') . '='. $db->quote($transaction_id))
          ->where($db->quoteName('ordercode') .'='. $db->quote($ordercode));
    $db->setQuery($query);

    return $db->execute();
	}

  public static function setRefunded($transaction_id)
	{
		$date = new Date;
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
    $query->update($db->quoteName('#__larts_subs_transactions'))
          ->set($db->quoteName('status') . '='. self::REFUNDED)
          ->where($db->quoteName('transaction_id') .'='. $db->quote($transaction_id));
    $db->setQuery($query);
    
    return $db->execute();
	}

 	static public function insert($ordercode, $amount, $comment, $userid, $type = 'PayLater', $plugin = 'Manual')
	{
		if (empty($ordercode)) {
			return false;
		}
    
		$transaction                 = new \stdClass;
		$transaction->transaction_id = md5($ordercode);
		$transaction->userid         = $userid;
		$transaction->details        = 'N/A';
		$transaction->amount         = $amount;
		$transaction->comment        = $comment;
		//$transaction->date           = DateEx::_('now');
		$transaction->type           = $type;
		$transaction->plugin         = $plugin;
		$transaction->ordercode      = $ordercode;
		$transaction->status         = Transaction::PROCESSED;

		return Factory::getDbo()->insertObject('#__larts_subs_transactions', $transaction);
	}

	public static function lookupId($id, $select = '*')
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('id') . ' = ' . $db->quote($id));

		$db->setQuery($query);
    //$sql = $query->__toString();
    $result = $db->loadObject();
		return $result;
	}

  // Getting all transactions by its ordercode
	public static function lookupAll($ordercode, $select = '*')
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('ordercode') . ' = ' . $db->quote($ordercode));

		$db->setQuery($query);
		return $db->loadObjectList();
	}

  // Get last transaction of this ordercode
	public static function lookupLast($ordercode, $select = '*')
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('ordercode') . ' = ' . $db->quote($ordercode))
                ->order($db->quoteName('id'). ' DESC LIMIT 1');

		$db->setQuery($query);
		return $db->loadObject();
	}
  
	public static function lookupTransactionId($transaction_id, $select = '*')
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('transaction_id') . ' = ' . $db->quote($transaction_id));

		$db->setQuery($query);
    //$sql = $query->__toString();
    $result = $db->loadObject();
		return $result;
	}

	public static function lookupTransactionToken($transaction_token, $select = '*')
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select($select)
		            ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('transaction_token') . ' = ' . $db->quote($transaction_token));

		$db->setQuery($query);
    //$sql = $query->__toString();
    $result = $db->loadObject();
		return $result;
	}

  public static function markDeleted($id)
	{
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__larts_subs_transactions'))
			->set($db->quoteName('deleted') . '= 1')
      ->where($db->quoteName('id') .'='. $id);
		$db->setQuery($query);
    //$sql = $query->__toString();
		return $db->execute();
  }

  public static function getTransactionsAmountTotal($ordercode)
	{
		$db = Factory::getDBO();
		$query = $db->getQuery(true)
		            ->select('SUM(amount)')
		            ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('ordercode') . ' = ' . $db->quote($ordercode))
		            ->andWhere($db->quoteName('deleted') . '=0');
		$db->setQuery($query);
    //$sql = $query->__toString();
    $result = $db->loadResult();

    return empty($result) ? 0 : $result;
  }
  
  
	function getTransactionState($transaction_token)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['t.*', 'u.name', 'u.email'])
		            ->from($db->quoteName('#__larts_subs_transactions', 't'))
		            ->leftJoin('LEFT', $db->quoteName('#__users', 'u') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->where($db->quoteName('transaction_token') . ' = ' . $db->quote($transaction_token));

		$db->setQuery($query);

		return $db->loadObject();
	}

  public static function purgePendingTransactionsBeforeDate($removal_date)
  {
    $db = Factory::getDbo();
		// select all transactions to be purged
		$query = $db->getQuery(true)
		            ->select('ordercode')
                ->from($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('date') . ' <= ' . $db->quote($removal_date))
		            ->andWhere($db->quoteName('status') . ' = ' . Transaction::PENDING)
		            ->andWhere($db->quoteName('deleted') . ' = 0')
                ;
		$db->setQuery($query);
		$ordercodes = $db->loadObjectList();
    
    // remove orders
    Order::removeByOrdercodes($ordercodes);
    
    // remove transactions
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('date') . ' <= ' . $db->quote($removal_date))
		            ->andWhere($db->quoteName('status') . ' = ' . Transaction::PENDING)
		            ->andWhere($db->quoteName('deleted') . ' = 0')
                ;
		$db->setQuery($query);
		$db->execute();
  }

  public function removeByOrdercode($ordercodes = [])
	{
		if (empty($ordercodes)) {
			return true;
		}

    $db = Factory::getDbo();
    $query = $db->getQuery(true)
                ->delete($db->quoteName('#__larts_subs_transactions'))
                ->where($db->quoteName('ordercode') . ' IN ' . '( ' . implode(',', $ordercodes) . ' )');
    $db->setQuery($query);
    return $db->execute();
	}
  
}



