<?php

/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Access\Access;
use Joomla\CMS\User\UserHelper;
use Joomla\CMS\Date\Date;
use LartsSubscriptions\Product;


class UserGroups {

  public function __construct() {
    
  }
  
  public static function getUserGroupTitle($groupid)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
        ->select($db->quoteName('title'))
        ->from($db->quoteName('#__usergroups'))
        ->where($db->quoteName('id') . ' = ' . (int) $groupid);
    $db->setQuery($query);
    return $db->loadResult();
  }

  public static function userGroupExists($groupid)
  {
    return !empty(self::getUserGroupTitle($groupid));
  }

  public static function appendToGroup($userid, $groupid)
  {
    try {
      UserHelper::addUserToGroup($userid, $groupid); 
    } catch (\RuntimeException $ex) {
       return false;
    }
    return true;
  }

  public static function removeFromGroup($userid, $groupid)
  {
  	return UserHelper::removeUserFromGroup($userid, $groupid);
  }
  
  public static function isAtGroup($userid, $groupid)
  {
    $groups = Access::getGroupsByUser($userid, false);
    return in_array($groupid, $groups);
  }

  public static function checkExpire($userid)
  {
    // get all active substriptions, expired
    $now = new Date;
    $now = $now->toSql();

    // get active groups of this user
    $groups = UserHelper::getUserGroups($userid);

    $db = Factory::getDbo();

    // get expiry of all products of this user
    $query = ' SELECT product_id, MAX(valid_to) AS date_expire, userid FROM #__larts_subs_periods '
            . ' GROUP BY product_id, userid HAVING (userid='.$userid.')';
    $db->setQuery($query);
    $result = $db->loadObjectList();

    // iter over all products subscribed
    // check prod type if expired
    foreach($result as $item)
    {
      if($item->date_expire < $now) {
        $prod = Product::getProduct($item->product_id);
        if(!empty($prod) && $prod->type == Product::GROUP_ACCESS) {
          $groupid = $prod->source_id;
          UserHelper::removeUserFromGroup($item->userid, $groupid);
        }
      }
    }
  }
  
}
