<?php
/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\VAT;
defined('_JEXEC') or die;

use LartsSubscriptions\Base\Country;

class VATItem
{
	public $country = null;
	public $country_code = null;
	public $moss = null;
	public $vatRate = null;
	public $sales = null;
	public $refunds = null;
	public $vatCharged = null;
	public $vatRefunded = null;
	public $totalCharged = null;
	public $vatTotal = null;
}

class MOSSEntry
{
	public $country = null;
	public $country_code = null;
	public $moss = 0;
	public $vatRate = 0;
	public $sales = 0;
	public $refunds = 0;
	public $vatCharged = 0;
	public $vatRefunded = 0;
	public $totalCharged = 0;
	public $vatTotal = 0;
}

class MOSS
{
  protected $aMOSSbyCountry = null;
  protected $MOSSTotal = null;
  
  public function __construct() {
    $this->MOSSTotal = new MOSSEntry;
    $this->MOSSTotal->country = 'total';
  }

  public function getItems() {
    $ret = $this->aMOSSbyCountry;
    $ret['total'] = $this->MOSSTotal;
    return $ret;
  }
  
  // add VATItem
  public function add($vatitem)
  {
    $refMOSS = $this->lookupCountry($vatitem->country_code, $vatitem->vatRate);

    // add per country
    $refMOSS->sales += $vatitem->sales;
    $refMOSS->refunds += $vatitem->refunds;
    $refMOSS->vatCharged += $vatitem->vatCharged;
    $refMOSS->vatRefunded += $vatitem->vatRefunded;
    $refMOSS->totalCharged += $vatitem->totalCharged;
    $refMOSS->vatTotal += $vatitem->vatCharged;
    $refMOSS->totalCharged = ($refMOSS->sales + $refMOSS->vatCharged) 
                            - ($refMOSS->refunds + $refMOSS->vatRefunded);
    $refMOSS->vatTotal = $refMOSS->vatCharged - $refMOSS->vatRefunded;

    // add total
    $this->MOSSTotal->sales += $vatitem->sales;
    $this->MOSSTotal->refunds += $vatitem->refunds;
    $this->MOSSTotal->vatCharged += $vatitem->vatCharged;
    $this->MOSSTotal->vatRefunded += $vatitem->vatRefunded;
    $this->MOSSTotal->totalCharged += $vatitem->totalCharged;
    $this->MOSSTotal->vatTotal += $vatitem->vatCharged;
    $this->MOSSTotal->totalCharged = ($this->MOSSTotal->sales + $this->MOSSTotal->vatCharged)
                                   - ($this->MOSSTotal->refunds + $this->MOSSTotal->vatRefunded);
    $this->MOSSTotal->vatTotal = $this->MOSSTotal->vatCharged - $this->MOSSTotal->vatRefunded;
  }

  public function lookupCountry($country_code, $vatRate)
  {
    $hash = md5($country_code. $vatRate);
    
    if(!isset($this->aMOSSbyCountry[$hash]))
    {
      $me = new MOSSEntry;
      $me->country_code = $country_code;
      $me->country = $country_code;
      $me->vatRate = (float)$vatRate;
      $me->sales = (float)0;
      $me->refunds = (float)0;
      $me->vatCharged = (float)0;
      $me->vatRefunded = (float)0;
      $me->totalCharged = (float)0;
      $me->vatTotal = (float)0;

      $c = Country::getCountryByCountryCode($country_code);
      if(!empty($c))
        $me->country = $c->country;
      $this->aMOSSbyCountry[$hash] = $me;
    }
    return $this->aMOSSbyCountry[$hash];
  }

}
