<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\VAT;
defined('_JEXEC') or die;

use LartsSubscriptions\Base\Database;
use LartsSubscriptions\Base\Country;

class VATClasses
{
  public static function loadAll()
  {
    return Database::loadObjectList('#__larts_subs_vat_classes');
  }

  public static function remove($id)
	{
    // remove the 'class_xx' column from countries table
    if(!Database::dropColumn('#__larts_subs_countries', 'rateclass_'.$id))
      return false;
    
    // remove class entry
    return Database::deleteObject('#__larts_subs_vat_classes', 'id', $id);
	}

  public static function add($record)
	{
    if($record['id'] == 0) {
      $record['id'] = Database::insert('#__larts_subs_vat_classes', $record);
    }
    else {
      Database::update('#__larts_subs_vat_classes', 'id', $record);
    }

    // insett the 'class_xx' column at the countries table
    if(!Database::addColumn('#__larts_subs_countries', 'rateclass_'.$record['id'], 'FLOAT NOT NULL DEFAULT 0', ''))
      return false;
    
    return $record['id'];
  }

  public static function getByCountry($country_code)
	{
    $country = Country::getCountryByCountryCode($country_code);
    $vat = [0 => 0];
    if(empty($country)) {
      return $vat;
    }
    $vat[0] = $country->vatrate_std;
    
    $classes = self::loadAll();
    foreach($classes as $item) {
      $colname = 'rateclass_'.$item->id;
      $vat[$item->id] = $country->$colname;
    }

    return [$country, $vat];
 }
  
}


