<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\VAT;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Http\HttpFactory;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Base\Database;
use LartsSubscriptions\VAT\EUTaxClient;

class VatRates
{
  const RATE_EU_STANDARD = 0;
  const RATE_EU_REDUCED = 1;
  const RATE_USER = 2;

  public static function getTypeStr($type) {
    switch($type) {
    case self::RATE_EU_STANDARD:
      return 'EU-Standard';
      break;
    case self::RATE_EU_REDUCED:
      return 'EU-Reduced';
      break;
    default:
      return 'User defined';
      break;
    }
  }

  public static function removeEURates()
  {
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete()
		            ->from($db->quoteName('#__larts_subs_vat_rates'))
		            ->where($db->quoteName('type').'!='.self::RATE_USER);
		$db->setQuery($query);
		return $db->execute();
  }

  public static function loadAll()
  {
    return Database::loadObjectList('#__larts_subs_vat_rates');
  }

  public static function loadByCountry($country_2code)
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select('*')
      ->from($db->quoteName('#__larts_subs_vat_rates'))
      ->where($db->quoteName('country_code'). '=' .$db->quote($country_2code))
      ->order($db->quoteName('category'))
            ;

    $db->setQuery($query);
    $result = $db->loadObjectList();
    return $result;
  }

  public static function clearTable()
  {
    return Database::truncate('#__larts_subs_vat_rates');
  }

  public static function addItem($record)
  {
    return Database::insert('#__larts_subs_vat_rates', $record);
  }
  
  public static function updateItem($record)
  {
    return Database::update('#__larts_subs_vat_rates', 'id', $record);
  }

  /**
	 * Resetting the VAT rates in the database to 0
	 */
	public function reset()
	{
		$db = Factory::getDbo();

		$fields = [
			$db->quoteName('requires_vat') . ' = 0',
			$db->quoteName('vatrate_std') . ' = 0',
		];

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_countries'))
		            ->set($fields);

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

  
	/**
	 * Read the current vat rates for all EU countries.
	 * We're not resetting the force_vat_payment in the reset function because we don't want to crash current sites
	 * which has set force vat payments already but not filed out the configuration with this option.
	 *
	 * As soon as the administrator fills out the Forced Vat Updater, this functionality will be used.
	 */
	private function updateForcedVatCountries()
	{
		$forced = (array) LartsConfig::get('forced_vat_payment');

		if (empty($forced))
		{
			return true;
		}

		$db = Factory::getDbo();

		$fields = [
			$db->quoteName('force_vat_payment') . ' = 0',
		];

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_countries'))
		            ->set($fields);

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		foreach ($forced as $force)
		{
			$fields = [
				$db->quoteName('force_vat_payment') . ' = 1',
			];

			$conditions = [
				$db->quoteName('country_2_code') . ' = ' . $db->quote($force),
			];

			$query = $db->getQuery(true)
			            ->update($db->quoteName('#__larts_subs_countries'))
			            ->set($fields)
			            ->where($conditions);

			$db->setQuery($query);

			if(!$db->execute())
			{
				return false;
			}
		}

		return true;
	}


	public static function getVatRate($id)
	{
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_vat_rates'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);
		$db->setQuery($query);
    return $db->loadObject();
  }

	public static function getVatRatesOfCountry($country_code)
	{
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_vat_rates'))
		            ->where($db->quoteName('country_code') . ' = ' . $db->quote($country_code));
		$db->setQuery($query);
    return $db->loadObjectList();
  }
  
	public static function setVatRatesPublished($cids, $publish)
  {
		$ids = implode(',', $cids);

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_vat_rates'))
		            ->set($db->quoteName('published') . ' = ' . (int)$publish)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $ids . ' )');
		$db->setQuery($query);

		if(!$db->execute()) {
			return false;
		}

  }
  
  public static function updateEURates(&$errMsg)
  {
    // remove all EU rates
    self::removeEURates();
    
    $errMsg = '';
    $client = new EUTaxClient;
    $bRet = $client->retrieveVat();
    if(!$bRet) {
      $errMsg = $$client->error;
    }
    return $bRet;
  }
}