
(function() {
    'use strict';

    function triggerChange() {
        var option = document.getElementById('wc_b2b_sales').value;
        var parent = document.getElementById('wc_b2b_sales').closest('tbody');

        if ('none' == option) {
            // Hide all but show only first :)
            Array.from(parent.querySelectorAll('tr')).forEach(function(tr, index) {
                if (index !== 0) {
                    tr.style.display = 'none';
                } else {
                    tr.style.display = 'table-row';
                }
            });
        } else {
            // Show everything now!
            Array.from(parent.querySelectorAll('tr')).forEach(function(tr) {
                tr.style.display = 'table-row';
            });

            // Hide one option for Non-EU stores
            if ('noneu' == option) {
                var euWithVatId = parent.querySelector('#wc_tax_eu_with_vatid').closest('tr');
                euWithVatId.style.display = 'none';
            }
        }
    }

    // AJAX request
    function euVatAjax(req_type, trigger) {
        var req_data = {
            action: 'add_' + req_type,
            nonce: wc_euvat_l10n.nonce
        };

        var xhr = new XMLHttpRequest();
        xhr.open('POST', ajaxurl, true);
        xhr.setRequestHeader('Content-Type', 'application/json');

        xhr.onreadystatechange = function() {
            if (xhr.readyState === XMLHttpRequest.DONE) {
                // Unblock button
                trigger.disabled = false;

                var data = JSON.parse(xhr.responseText);

                // Refresh on a success response
                if (data.status == 'success') {
                    // Refreshes options to latest values
                    window.location.href = window.location.href;
                }
            }
        };

        xhr.send(JSON.stringify(req_data));

        // Disable button
        trigger.disabled = true;
    }

    function addRow(iso_code, tax_rate, tax_label) {
        var taxrates_form = document.querySelector('.wc_tax_rates');
        var tbody = taxrates_form.querySelector('tbody');
        var size = tbody.querySelectorAll('tr').length;

        var possible_existing_lines = tbody.querySelectorAll('tr');
        var was_updated = false;

        possible_existing_lines.forEach(function(line) {
            var p_iso = line.querySelector('td.country input:first-child').value;

            if (!p_iso || p_iso != iso_code) {
                return;
            }

            var p_state = line.querySelector('td.state input:first-child').value;
            var p_postcode = line.querySelector('td.postcode input:first-child').value;
            var p_city = line.querySelector('td.city input:first-child').value;

            if (p_iso == iso_code && (!p_state || p_state == '') && (!p_postcode || p_postcode == '') && (!p_city || p_city == '')) {
                line.querySelector('td.rate input:first-child').value = tax_rate;
                line.querySelector('td.rate input:first-child').dispatchEvent(new Event('change'));

                // Update tax amount in the label
                line.querySelector('td.name input:first-child').value = tax_label;
                line.querySelector('td.name input:first-child').dispatchEvent(new Event('change'));
                was_updated = true;
                return;
            }
        });

        // We are done if the row was updated
        if (was_updated) {
            return;
        }

        taxrates_form.querySelector('.button.insert').click();

        var new_row_parent = tbody.querySelector('tr[data-id^="new"] .country input[value=""]');
        var new_row = new_row_parent.closest('tr');
        new_row.setAttribute('country', iso_code);
        new_row.querySelector('.rate input').value = tax_rate;
        new_row.querySelector('.rate input').dispatchEvent(new Event('change'));
        new_row.querySelector('.name input').value = tax_label;
        new_row.querySelector('.name input').dispatchEvent(new Event('change'));

        return false;
    }

    // Initialize on DOM ready
    document.addEventListener('DOMContentLoaded', function() {
        triggerChange();

        // For tax-rates management section
        var rates = wc_euvat_l10n.tax_rates;
        var known_rates = wc_euvat_l10n.known_rates_key.split(',');
        var known_rate_values = wc_euvat_l10n.known_rates_values.split(',');
        var foot = document.querySelector('table.wc_tax_rates tfoot a.remove_tax_rates');

        foot.insertAdjacentHTML('afterend', '<a href="#" id="wc-euvat-updaterates" class="button wc-euvat-updaterates">' + wc_euvat_l10n.add_update_text + '</a>');
        var rate_selector = '<select id="wc-euvat-rate">';

        for (var i = 0; i < known_rates.length; i++) {
            rate_selector += '<option value="' + known_rates[i].replace('"', '').trim() + '">' + known_rate_values[i].replace('"', '').trim() + '</option>';
        }

        rate_selector += '</select>';
        var tax_description = '&nbsp;&nbsp;' + wc_euvat_l10n.name_text + '<input id="wc-euvat-description" title="' + wc_euvat_l10n.name_desc_text + '" type="text" size="6" value="' + wc_euvat_l10n.name_value_text + '">';

        foot.insertAdjacentHTML('afterend', '&nbsp;&nbsp;' + wc_euvat_l10n.use_rate_text + ' ' + rate_selector + tax_description);

        document.querySelector('table.wc_tax_rates').insertAdjacentHTML('beforebegin', '<p><em>' + wc_euvat_l10n.grab_tax_text + '</em></p>');

        document.querySelector('table.wc_tax_rates').addEventListener('click', function(event) {
            if (event.target.classList.contains('wc-euvat-updaterates')) {
                event.preventDefault();

                var which_rate = document.getElementById('wc-euvat-rate').value;

                if (typeof which_rate == 'undefined' || which_rate == '') {
                    which_rate = wc_euvat_l10n.which_rate;
                }

                // Disable button
                event.target.classList.add('disabled');

                rates.forEach(function(country) {
                    var rate = country.standard_rate;

                    if (which_rate == 'reduced_rate') {
                        var reduced_rate = country.reduced_rate;

                        if (typeof reduced_rate != 'boolean') {
                            rate = reduced_rate;
                        }
                    }

                    // VAT-compliant invoices must show the rate
                    var name = document.getElementById('wc-euvat-description').value + ' (' + rate.toString() + '%)';
                    addRow(country.iso, rate.toString(), name);
                });

                // Unblock button
                event.target.classList.remove('disabled');

                return false;
            }
        });
    });

    // On-change DOM
    document.getElementById('wc_b2b_sales').addEventListener('change', function() {
        triggerChange();
    });

    // Import taxes for Digital Goods
    document.querySelectorAll('.import-digital-tax-rates').forEach(function(button) {
        button.addEventListener('click', function(event) {
            event.preventDefault();
            euVatAjax('digital_taxes', this);
        });
    });
})();


