/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */


document.onreadystatechange = function() {
  if (document.readyState === "complete") {
    onDocReady();
  }
}

function onDocReady()
{
  elems = document.getElementsByName('isbusiness');
  elems.forEach(function(elem) {
    elem.addEventListener('click', () => {
      toggleBusinessFields();
    });
  });  

  toggleBusinessFields();
  setCountryText();
  toggleVatFields();
  checkCountry();

  let elemVatNum  = document.getElementById('vatnumber');
  if(elemVatNum != null) {
    elemVatNum.addEventListener('blur', () => {
      // check on kill focus
      checkVatNumber();
    });
  }

  $combo = document.getElementById("showCountrySelect");
  if($combo != null) {
    $combo.onclick = function(){
      showCountrySelect(true);
    }
  }

  let elemCC = document.getElementById('country_code');
  if(elemCC != null) {
    elemCC.addEventListener('change', () => {
      onChangeCountryCode();
    });
  }
  
  let elemvatCheck = document.getElementById('vatCheck');
  if(elemvatCheck != null) {
    elemvatCheck.addEventListener('click', () => {
      checkVatNumber();
    });
  }
}

function toggleBusinessFields()
{
  checkVatNumber();

  var isbu = getValueChecked('isbusiness');
  if(isbu > 0) {
    showElementById('businessfields', true);
  }
  else {
    showElementById('businessfields', false);
  }
}

function toggleVatFields()
{
  var vies_vat_fields = document.getElementById('vatfields');
  if(vies_vat_fields == null)
    return;

  var country = getValueById('country_code');
  if (country in vat_countries) {
    vies_vat_fields.style.display = '';
  }
  else {
    vies_vat_fields.style.display = 'none';
  }
}

function onChangeCountryCode()
{
  setCountryText();
  checkCountry();
  toggleVatFields();

  // Delay to prevent it from being executed before checkCountry()
  setTimeout(function() {
    checkVatNumber();
  }, 100);
}

function setCountryText()
{
  var combo = document.getElementById('country_code');
  if(combo != null) {
    var text= combo.options[combo.selectedIndex].text;
    document.getElementById("country").value = text;
    showCountrySelect(false);
  }
}

function showCountrySelect(bShow) {
  if(bShow) {
    document.getElementById("country_select").style.display = 'block';
    document.getElementById("country_text").style.display = 'none';
  }
  else {
    document.getElementById("country_select").style.display = 'none';
    document.getElementById("country_text").style.display = 'block';
  }
}

function getVatNumber()
{
  let elemVat = document.getElementById('vatnumber');
  if(elemVat == null)
    return '';
  return elemVat.value;
}
  
function cleanVatNumber()
{
  let elemVat = document.getElementById('vatnumber');
  let elemCountry = document.getElementById('country');
  if(elemVat != null) {

    var vatnumber = elemVat.value.replace(/[^a-zA-Z0-9_]/g, '');
    var country   = elemCountry.value;

    // Strip off the language code
    if (vatnumber.indexOf(country) == 0) {
      vatnumber = vatnumber.substr(country.length);
    }

    elemVat.value = vatnumber;
  }
}

function checkVatNumber()
{
  // Checks if the element is present on the page.
  var elemVatNumber = document.getElementById('vatnumber');

  cleanVatNumber();

  var vatnumber = '';
  var country = '';
  var isbu = getValueChecked('isbusiness');
  if(isbu == 1) {
    let elemCountry = document.getElementById('country');
    vatnumber = elemVatNumber.value;
    country = elemCountry.value;
  }
  if (!vatnumber) {
    return;
  }

  elemVatNumber.classList.remove('invalid');
  let elemVatCheckIcon = document.getElementById('icon_vatcheck');
  elemVatCheckIcon.setAttribute('class', 'icon-search');
  let elemVatCheckMsg = document.getElementById('message_vatcheck');
  elemVatCheckMsg.innerHTML = larts_lang_vat_number_checking;
  elemVatCheckMsg.classList.add('alert-warning');
  elemVatCheckMsg.classList.remove('alert-danger');
  elemVatCheckMsg.classList.remove('alert-success');
  elemVatCheckMsg.style.display = '';

  var data = 'vatnumber=' + vatnumber + '&country=' + country;
  url = larts_root + 'index.php?option=com_lartssubscriptions&task=signup.checkVatNumber';

  // show...
  doAJAXPost(url, data, onAjaxDone_checkVatNumber, onAjaxError_checkVatNumber);


}

function onAjaxDone_checkVatNumber(response)
{
  const data = JSON.parse(response);

  //let elemVatCheck = document.getElementById('larts_cart_has_vat');
  var elemCartVat = document.getElementById('larts_cart_vat');
  var elemCartTotal = document.getElementById('larts_cart_total');
  var elemMsgVatCheck = document.getElementById('message_vatcheck');
  
  elemCartVat.innerHTML = data.carttotal_vat;
  elemCartTotal.innerHTML = data.carttotal_total;
  elemMsgVatCheck.classList.remove('alert-warning');
  
  if (data.msg) {
    elemMsgVatCheck.innerHTML = data.msg;
    show(elemMsgVatCheck);
  }
  
  if (data.status != 666) {
    elemMsgVatCheck.classList.add('alert-danger');

    document.getElementById('vatnumber').classList.add('invalid');
    document.getElementById('icon_vatcheck').setAttribute('class', 'icon-remove');
    return;
  }

  elemMsgVatCheck.classList.add('alert-success');

  document.getElementById('icon_vatcheck').setAttribute('class', 'icon-ok');
}
function onAjaxError_checkVatNumber(response)
{
  let elemVatCheck = document.getElementById('message_vatcheck');
  elemVatCheck.classList.remove('alert-warning');
  document.getElementById('vatnumber').classList.add('invalid');
  elemVatCheck.innerHTML = larts_lang_vat_number_invalid;
  elemVatCheck.classList.add('alert-danger');
}

function onAjaxDone_checkCountry(response)
{
  const data = JSON.parse(response);

  if((elem = document.getElementById('vat_notification')) != null) {
    if (data.requires_vat == 0) {
      elem.style.display = 'none';
    }
    else if (data.requires_vat == 1) {
      elem.style.display = '';
    }
  }

  document.getElementsByName('vatnumber').innerHTML = data.vatnumber;

  elems = document.getElementsByClassName('larts_cart_has_vat');
  for (let i = 0; i < elems.length; i++) {
    elems[i].style.display = '';
  }
  elems = document.getElementsByClassName('larts_cart_vat');
  for (let i = 0; i < elems.length; i++) {
    elems[i].innerHTML = data.carttotal_vat;
  }
  elems = document.getElementsByClassName('larts_cart_total');
  for (let i = 0; i < elems.length; i++) {
    elems[i].innerHTML = data.carttotal_total;
  }
}
  
function onAjaxError_checkCountry()
{
  elems = document.getElementsByClassName('larts_cart_vat');
  for (let i = 0; i < elems.length; i++) {
    elems[i].innerHTML = 'error';
  }
  elems = document.getElementsByClassName('larts_cart_total');
  for (let i = 0; i < elems.length; i++) {
    elems[i].innerHTML = 'error';
  }
}

function checkCountry()
{
  let elemCountry = document.getElementById('country_code');
  if(elemCountry == null) {
    return;   // e.g. empty chart
  }
  var country = elemCountry.value;
  var data = 'country=' + country;

  url = larts_root + 'index.php?option=com_lartssubscriptions&view=checkout&task=checkout.checkCountry';
  doAJAXPost(url, data, onAjaxDone_checkCountry, onAjaxError_checkCountry);
}

