<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Layout\FileLayout;

use LartsSubscriptions\Price;
use LartsSubscriptions\Helper;
use LartsSubscriptions\ThirdParty\LartsLicManager;
use LartsSubscriptions\Product;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Cart;


$item = $displayData['item'];
$product_url = Route::_('index.php?option=com_lartssubscriptions&view=product&id=' . $item->product_id);
$userid = Factory::getApplication()->getIdentity()->id;
?>

<tr>
	<td class="larts-cart-name">
		<span class="d-lg-none">
			<?php echo Text::sprintf('COM_LARTSSUBS_CART_NR_OF_ITEMS', $item->quantity); ?>
		</span>
		<a href="<?php echo $product_url; ?>"><?php echo $item->product_name; ?></a>
		<?php if ($item->quantity > 1): ?>
			<small>(<?php echo Price::_($item->price); ?>)</small>
		<?php endif; ?>
		<br />
    <?php
      $txt = Helper::getProductItemSubtext($item);
      if(!empty($txt)) {
        echo '<small>' .$txt. '</small>';
      }
    ?>
    <?php 
    if($item->has_licence) {
      if(empty($userid)) {
        echo '<br />for new licence';
      }
      else
      {
        // Trigger plugins
        $lics = array();
        $plgReturns = PluginManager::execPlugins('lartssubscriptions', 'onGetUserLicences', ['userid' => $userid], false, false);
        if(!empty($plgReturns)) {
          // walk back to front 
          foreach(array_reverse($plgReturns) as $plgRet) {
            foreach($plgRet as $lic) {
              $lics[] = $lic;
            }
          }
        }

        echo '<div class="larts_cart_licence"><small style="margin-right: 10px;">for licence:</small>';
        echo '<select class="form-select form-select-sm input larts-input-mini larts_cart_licence_select" onchange="updateProductLicInCart('.$item->product_id.', this.value);">';
        echo '<option' . (empty($item->licence) ? ' selected="selected"' : ''). ' value="0">New Licence</option>';
       
        if(!empty($lics)) {
          foreach ($lics as $row) {
            if($row->licence_number == $item->licence) {
              echo '<option selected="selected" value="'.$row->licence_number.'">'.$row->licence_number.'</option>';
            }
            else {
              echo '<option value="'.$row->licence_number.'">'.$row->licence_number.'</option>';
            }
          }
        }
        echo '</select>';
        echo '</div>';

      }
    }

    ?>
 
		<?php
		if ($item->discount > 0) {
			echo '<br/><small>'
				. Price::getDiscountText($item->discount_group, $item->discount_type, $item->discount_amount, $item->discount)
				. '</small>';
		}
		?>
	</td>
	<td class="center nowrap larts-cart-quantity">
		<?php
		$layout = new FileLayout('cart_item_quantity_' . $displayData['quantity_layout']);
		echo $layout->render($item);
		?>
	</td>
	<td class="right nowrap larts-cart-price">
		<?php
		$price = $displayData['quantity_layout'] == 'separate' ? $item->price : $item->quantity * $item->price;
		echo Price::_($price);
		?>
	</td>
</tr>
