<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Router\Route;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Helper;
use LartsSubscriptions\Renewal;
use LartsSubscriptions\Product;
use LartsSubscriptions\Periods;
use LartsSubscriptions\ThirdParty\LartsPkgManager;


// Get user information.
$userid = Factory::getApplication()->getIdentity()->id;

$config    = $displayData['config'];
$item      = $displayData['item'];
$alignment = $displayData['column_alignment'];
$renewInfo = [
  'state' => Renewal::PERIOD_EXPIRED,
  'expired' => true,
  '$dExpire' => '',
  'discount' => 0,
  'daysTilExpire' => 0,
];

if ($config->show_renewal_button && $item->subs_status == Periods::STATE_ACTIVE)
{
  $renewal = new Renewal;
  $renewInfo = $renewal->getRenewalInfo($item->subs_end);
  if($item->exclude_renewals) {
    if($renewInfo['state'] == Renewal::PERIOD_GRACE) {
      $renewInfo['state'] == Renewal::PERIOD_EXPIRED;
    }
    else if($renewInfo['state'] != Renewal::PERIOD_EXPIRED) {
      $renewInfo['state'] == Renewal::PERIOD_ACTIVE;
    }
  }
}

$file = array();
$link = Product::getDownloadLink($item, $file);
$dlkey = '';
if($item->pkgmanager_id > 0 && LartsPkgManager::IsInstalled()) {
  $userid = Factory::getApplication()->getIdentity()->id;
  $api = LartsPkgManager::getAPI();
  $dlkey = $api->getUpdateKey($userid, $item->pkgmanager_id);
}

?>


<div class="panel panel-default" style="width:280px;">
	<div class="panel-heading">
		<h3 class="panel-title"><?php echo $item->name; ?></h3>
	</div>
	<div class="panel-body minheight-mysubscriptions">

		<div style="text-align: <?php echo $alignment; ?>; width:100%; margin-bottom:10px;">
			<?php if(isset($renewInfo['expired'])) {
              echo Helper::getSubscriptionState($renewInfo['expired'], $item->subs_status);
            }
      ?>
		</div>

		<table class="table">
			<tr>
				<td><?php echo Text::_('COM_LARTSSUBS_CREATED'); ?></td>
				<td style="text-align: <?php echo $alignment; ?>;"><?php echo DateEx::_($item->subs_begin); ?></td>
			</tr>
			<tr>
				<td><?php echo Text::_('COM_LARTSSUBS_EXPIRES'); ?></td>
				<td style="text-align: <?php echo $alignment; ?>;">
					<?php if ($item->period_type == 'onetime') : ?>
						<?php echo Text::_('COM_LARTSSUBS_ONE_TIME'); ?>
					<?php elseif ($item->subs_status == Periods::STATE_ACTIVE && $item->period_type == 'lifetime') : ?>
						<span class="badge bg-success"><?php echo Text::_('COM_LARTSSUBS_LIFETIME'); ?></span>
					<?php else: ?>
						<?php echo DateEx::_($item->subs_end); ?>
					<?php endif; ?>
				</td>
			</tr>
			<tr>
				<td><?php echo Text::_('COM_LARTSSUBS_VERSION'); ?></td>
				<td style="text-align: <?php echo $alignment; ?>;"><?php if(!empty($file)) { echo $file->version; } ?></td>
			</tr>
			<tr>
        <td colspan="2"><?php 
          if(!empty($dlkey)) {
            echo Text::_('COM_LARTSSUBS_JOOMLA_UPDATE_KEY');
            echo '<br><div class="small">'. $dlkey. '</div>';
          }
          else {
            echo '&nbsp<br><div class="small">&nbsp</div>';
          }
        ?></td>
			</tr>
			<tr>
        <td colspan="2">
          <div align="center" style="margin-bottom: 0px;">
            <div >
            <?php 
              if($config->show_product_page_button == 1) {
                echo '<div>' .Helper::getShowProductButton($item->product_id, 'larts-btn-mysubs-card'). '</div>';
              }
            ?>
            <?php
            // show renew/by button
            $showRenew = $config->show_renewal_button & ($item->exclude_renewals == 0);
            if($renewInfo['state'] == Renewal::PERIOD_ACTIVE ||
               in_array($item->period_type, ['lifetime', 'onetime',]) || 
               !$showRenew)
            {
              echo Helper::getNoneButton();
            }
            else
            {
              switch($renewInfo['state']) {
              case Renewal::PERIOD_REMINDER1:
              case Renewal::PERIOD_REMINDER2:
              case Renewal::PERIOD_REMINDER3:
              case Renewal::PERIOD_GRACE:
                echo Helper::getRenewSubscriptionButton($item->product_id, $userid, 'larts-btn-mysubs-card');
                break;
              default:
                echo Helper::getBuyButton($item->product_id, 'larts-btn-mysubs-card');
              }
            }
            ?>
            <?php 
              if ($renewInfo['expired'] == false) {
                switch($item->type) {
                case Product::MOVIE_STREAM:
                  $buttxt = 'Watch';
                  break;
                case Product::CATEGORY:
                  $buttxt = 'Show';
                  break;
                default:
                  $buttxt = 'Download';
                }
                if(empty($link)) {
                  echo Helper::getNoneButton();
                }
                else {
                  echo '<div><a href="'.$link.'" target="_self" class="btn btn-sm btn-primary larts-btn-mysubs-card">'
                          . $buttxt. '</a></div>';
                }
              }
              else {
                echo Helper::getNoneButton();
              }
            ?>

            <?php 
              if (!empty($item->extra_url) && $renewInfo['expired'] == false) {
                echo Helper::getExtraUrlButton($item->extra_url, 'larts-btn-mysubs-card');
              }
              else {
                echo Helper::getNoneButton();
              }
            ?>

          </div>
        </td>
			</tr>
		</table>

	</div>
</div>

<div class="clearfix"></div>

