<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Helper;
use LartsSubscriptions\Product;
use LartsSubscriptions\ThirdParty\LartsPkgManager;
use LartsSubscriptions\Renewal;


// Get the config
$config = $displayData['config'];

// Get user information.
$userid = Factory::getApplication()->getIdentity()->id;
$dtNow = (new Date)->toUnix();

$items = $displayData['items'];
?>

<table class="table table-striped">

	<thead>
		<tr>
			<th class="w-40 text-left"><?php echo Text::_('COM_LARTSSUBS_PRODUCT'); ?></th>
			<th class="w-15 text-left">
				<div><?php echo Text::_('COM_LARTSSUBS_CREATED'); ?></div>
			</th>
			<th class="w-15 text-left">
				<div><?php echo Text::_('COM_LARTSSUBS_EXPIRES'); ?></div>
			</th>
			<th class="w-20 text-center">
				<div><?php echo Text::_('COM_LARTSSUBS_STATE'); ?></div>
			</th>
			<th class="w-20 text-center">
				<div><?php echo Text::_('COM_LARTSSUBS_VERSION'); ?></div>
			</th>
			<th class="w-10 text-center">
				<div><?php echo Text::_('COM_LARTSSUBS_DOWNLOAD'); ?></div>
			</th>
		</tr>
	</thead>

	<?php foreach ($items as $i => $subscription) : ?>

		<?php
    $renewal = new Renewal;
    $renewInfo = $renewal->getRenewalInfo($subscription->subs_end);
    if($subscription->exclude_renewals) {
      if($renewInfo['state'] == Renewal::PERIOD_GRACE) {
        $renewInfo['state'] == Renewal::PERIOD_EXPIRED;
      }
      else if($renewInfo['state'] != Renewal::PERIOD_EXPIRED) {
        $renewInfo['state'] == Renewal::PERIOD_ACTIVE;
      }
    }
    
    $file = array();
    $dllink = Product::getDownloadLink($subscription, $file);
    

		if (!empty($subscription->content_url) && $subscription->type == Product::SUBSCRIPTION)
		{
			$url = Route::_($subscription->content_url);
		}
		else
		{
      $url = 'index.php?option=com_lartssubscriptions&view=product&id=' . $subscription->product_id;  // . ':' . $subscription->alias;
			$url = Route::_($url);
		}

		if ($renewInfo['state'] != Renewal::PERIOD_EXPIRED && $config->show_renewal_button)
		{
			$date = new Date;

			$date1 = date_create($date->format('Y-m-d'));
			$date2 = date_create($subscription->subs_end);
			$diff  = date_diff($date1, $date2);
			$days  = $diff->format('%a');
		}
    
    $dlkey = '';
    if($subscription->pkgmanager_id > 0 && LartsPkgManager::IsInstalled()) {
      $userid = Factory::getApplication()->getIdentity()->id;
      $api = LartsPkgManager::getAPI();
      $dlkey = $api->getUpdateKey($userid, $subscription->pkgmanager_id);
    }
    
		?>
		<tr>
			<td class="text-left">
        <a href="<?php echo $url; ?>"><?php echo $subscription->name; ?></a>
        <?php 
          if(!empty($dlkey)) {
            echo '<div class="small">'. Text::_('COM_LARTSSUBS_JOOMLA_UPDATE_KEY'). $dlkey. '</div>';
          }
        ?>
        
<?php
 ?>
 			</td>
			<td class="text-left">
				<div><?php echo DateEx::_($subscription->subs_begin); ?></div>
			</td>
			<td class="text-left">
				<div>
					<?php 
            if ($subscription->subs_status == 1 && $subscription->period_type == 'onetime') {
              echo Text::_('COM_LARTSSUBS_ONE_TIME');
            }
            elseif ($subscription->subs_status == 1 && $subscription->period_type == 'lifetime') {
              echo Text::_('COM_LARTSSUBS_LIFETIME');
            }
            else {
              echo DateEx::_($subscription->subs_end);
            }
          ?>
				</div>
			</td>
			<td class="text-center">
				<div>
					<?php 
             $showRenew = $config->show_renewal_button & ($subscription->exclude_renewals == 0);
             if($renewInfo['state'] == Renewal::PERIOD_ACTIVE ||
               in_array($subscription->period_type, ['lifetime', 'onetime',]) || 
               !$showRenew)
            {
              echo Helper::getSubscriptionState($renewInfo['expired'], $subscription->subs_status);
            }
            else
            {
              switch($renewInfo['state']) {
              case Renewal::PERIOD_REMINDER1:
              case Renewal::PERIOD_REMINDER2:
              case Renewal::PERIOD_REMINDER3:
              case Renewal::PERIOD_GRACE:
                echo Helper::getRenewSubscriptionButton($subscription->product_id, $userid, 'larts-btn-mysubs-tbl');
                break;
              default:
                echo Helper::getBuyButton($subscription->product_id, 'larts-btn-mysubs-tbl');
              }
            }
            ?>
				</div>
			</td>
      <td class="text-center">
        <div><?php if(!empty($file)) { echo $file->version; } ?>
        </div>
      </td>
      <td class="text-center">
				<div>
        <?php 
          if ($renewInfo['state'] != Renewal::PERIOD_EXPIRED && $renewInfo['state'] != Renewal::PERIOD_GRACE)
          {
            switch($subscription->type) {
            case Product::MOVIE_STREAM:
              $buttxt = 'Watch';
              break;
            case Product::CATEGORY:
              $buttxt = 'Show';
              break;
            default:
              $buttxt = 'Download';
            }
            if(!empty($dllink)) {
              echo '<div><a href="'.Route::_($dllink).'" target="_self" class="btn btn-sm btn-primary" style="width:90px">'
                      . $buttxt. '</a></div>';
            }
          }
        ?>
				</div>
      </td>
		</tr>

	<?php endforeach; ?>

</table>
