<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Price;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Base\Redirect;


class CheckoutController extends BaseController
{
	private $ordercode;
	private $userid;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->ordercode = intval(Factory::getSession()->get('ordercode'));
		$this->userid    = Factory::getApplication()->getIdentity()->id;
	}

  public function checkout()
  {
 		$this->checkToken();

    $cm = new CheckoutManager;
    $cm->doCheckout();
    
  }
  
  public function updatelic()
	{
 		$id = $this->input->get('product_id', '0', 'int');
		$lic = $this->input->get('lic', '0', 'int');

		if ($this->getModel('checkout')->updateProductLicenceInCart($this->ordercode, $id, $lic)) {
      if($lic == 0) {
        $lic = 'New licence';
      }
      Factory::getApplication()->enqueueMessage('Licence set: ' . $lic, 'info');
      //$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions'));
		}
		else
		{
      Factory::getApplication()->enqueueMessage('Licence set failed', 'error');
		}
    

 		$this->redirectAfterChange();
		return true;
  }

	// Updating items in cart.
	public function update()
	{
		$id       = $this->input->get('product_id', '0', 'int');
		$quantity = $this->input->get('quantity', '0', 'int');

		if ( ! $quantity) {
			$this->remove();
			return true;
		}

		if ($id == 0 || $id == '') {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_UPDATING_ORDER_FAILED'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions'));
			return false;
		}

		if ($this->getModel('checkout')->updateProductQuantityInCart($this->ordercode, $id, $quantity)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ORDER_HAS_BEEN_UPDATED'), 'success');
		}
		else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_UPDATING_ORDER_FAILED'), 'error');
		}

		$this->redirectAfterChange();

		return true;
	}
  
	// removing a specific product from the cart when somebody has changed the quantity amount to 0 or used the delete button.
	public function remove()
	{
		$id = Factory::getApplication()->input->get('product_id', '0', 'int');

		if ($id == 0 || $id == '')
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_REMOVING_ORDER_FAILED'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions'));

			return false;
		}

		if ($this->getModel('checkout')->removeProduct($this->ordercode, $id))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ORDER_HAS_BEEN_REMOVED'), 'notice');
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_REMOVING_ORDER_FAILED'), 'error');
		}

		$this->redirectAfterChange();

		return true;
	}

	private function redirectAfterChange()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) AS counts'])
		            ->from($db->quoteName('#__larts_subs_cart_items'))
		            ->where([
			            $db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
			            $db->quoteName('product_id') . ' != 0',
		            ]);

		$db->setQuery($query);
		$order = $db->loadObject();

		if ($order->counts)
		{
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=checkout'));

			return;
		}

    // All items needs to be removed from here:
    $cart = new Cart;
    $cart->removeCouponsFromOrder();
    
		if (LartsConfig::get('use_redirect_on_empty_cart') == 0)
		{
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=products'));
			return;
		}

    $page = LartsConfig::get('redirect_on_empty_cart');
    $link = Redirect::getLinkFromMenuId($page);
		$this->setRedirect($link);
	}

	function coupon()
	{
 		$this->checkToken();

    $cm = new CheckoutManager;
    $cm->saveCheckoutFormData();

    // Get the id that needs to be removed:
		$jinput      = Factory::getApplication()->getInput();
		$coupon_code = $jinput->getString('coupon_code', '');

		$link = Route::_('index.php?option=com_lartssubscriptions&view=checkout');

		if ($coupon_code == '') {
			## Redirect the customer back. Nothing to add.
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVALID_COUPON'), 'error');
			$this->setRedirect($link);
		}

		// Get the model to save
		$model = $this->getModel('checkout');

    $coupon_code = filter_input(INPUT_POST, 'coupon_code', FILTER_SANITIZE_STRING);
    $coupon_code = trim($coupon_code);
		if ($model->applyCoupon($coupon_code)) {
			Factory::getApplication()->enqueueMessage($model->showMessage(), 'success');
		}
		else {
			Factory::getApplication()->enqueueMessage($model->showMessage(), 'error');
		}

		$this->setRedirect($link);
	}

  public function cors()
  {
    // Allow from any origin
		//    if (isset($_SERVER['HTTP_ORIGIN'])) {
		//        // Decide if the origin in $_SERVER['HTTP_ORIGIN'] is one
		//        // you want to allow, and if so:
		//        header("Access-Control-Allow-Origin: {$_SERVER['HTTP_ORIGIN']}");
		//        header('Access-Control-Allow-Credentials: true');
		//        header('Access-Control-Max-Age: 86400');    // cache for 1 day
		//    }    

    header('Access-Control-Allow-Origin: *');
    header('Access-Control-Allow-Methods: GET, POST');
    header("Access-Control-Allow-Headers: Content-Type, x-requested-with");    

  }
  
  public function checkCountry()
	{
    $this->cors();
    
		$country_code = $this->input->getString('country');

    $ordercode = intval(Factory::getSession()->get('ordercode'));
    $amount = new OrderAmounts($ordercode);
    $amount->country_code = $country_code;

    $amount->recalcCart();

		$array = [
			'requires_vat'      => $amount->vat_required,
			'carttotal_vat'     => Price::_($amount->vatprice()),
			'carttotal_vat_pct' => $amount->getVatPercentsAppliedStr(),
			'carttotal_total'   => Price::_($amount->netprice()),
		];

		echo json_encode($array);
    exit;
	}


}
