<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\Utilities\ArrayHelper;


use LartsSubscriptions\Base\Redirect;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Download;
use LogicalArts\Joomla\TableContent;
use LartsSubscriptions\GeoIP\GeoIP;
use LartsSubscriptions\Files;
use Joomla\CMS\Filter\OutputFilter;
use LartsSubscriptions\Video\VideoStream;
use LartsSubscriptions\Ticket;
use LartsSubscriptions\Periods;
use LartsSubscriptions\Product;
use LartsSubscriptions\Categories;
use LartsSubscriptions\ThirdParty\LartsPkgManager;


class DownloadController extends BaseController
{
	private $userid;

	private $dlkey;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$user         = Factory::getApplication()->getIdentity();
		$this->userid = $user->id;

		$jinput   = Factory::getApplication()->input;
		$this->dlkey = $jinput->get('dlkey', '');
	}

	public function file()
	{
		$jinput   = Factory::getApplication()->input;
		$hash = $jinput->get('key', '');
		$cat = $jinput->getInt('cat', 0);
    
    // lookup for the file
    $file = Files::lookupByHash($hash);
		if (empty($file))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$link = Route::_('index.php?option=com_lartssubscriptions&view=categories');
			$this->setRedirect($link);

			return false;
		}

    // lookup for the user
		$userid = Factory::getApplication()->getIdentity()->id;

    if ($userid < 1) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PLEASE_LOGIN_FIRST'), 'error');
      $link = Route::_('index.php?option=com_lartssubscriptions&view=categories');
      $this->setRedirect($link);

      return false;
    }
    
    if($cat != 0) {
      // files from category, so we have to check the cat
      
    }
		// check period
    $per = new Periods;
    $period = $per->getLastOfUser($userid, Product::FILE, $file->id);
		if (empty($period) || Periods::isExpired($period))
		{
      // ...failed
      // second try.. file may be member of category
      $fileCats = Categories::getCategoires($file->id);
      $fileCats = ArrayHelper::toInteger($fileCats);
      $products = $per->getAllSubscribedCategoriesOfUser($userid, true);
      $bFound = false;
      foreach($products as $item) {
        if(in_array($item->cat_id, $fileCats)) {
          $bFound = true;
        }
      }
      
      if(!$bFound) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
        $link = Route::_('index.php?option=com_lartssubscriptions&view=categories');
        $this->setRedirect($link);
        return false;
  		}
		}



    $download = new Download;

    // Updating the counter.
		$download->logging($userid, GeoIP::getIP(), $file->filename, Download::DOWNLOAD_FILE, 0);

		// Path to the file depends on the location of the file (local or remote)
		$filepath  = $download->getFilePath();
		//$download  = OutputFilter::stringURLSafe($file->filename);
		//$extension = pathinfo($filepath . $file->filename_physical, PATHINFO_EXTENSION);
		$filename  = $file->filename . '.' . $file->extension;

		// When the own filename is set and not empty
		if ($file->use_own_filename && ! empty($file->use_filename)) {
			$filename = $file->use_filename;
		}
		if($file->keep_original_filename) {
			$filename  = $file->filename_physical;
		}
		if ($file->append_version && ! empty($file->version)) {
			$filename .= '_' . $file->version;
		}

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onBeforeDownload', [$file]);

		// Sending headers and file to the browser.
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/octet-stream");
		header("Content-Disposition: attachment; filename=\"" . $filename . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($filepath . $file->filename_physical));
		ob_end_flush();
		@readfile($filepath . $file->filename_physical);
		die;
	}

  public function stream()
	{
		$jinput   = Factory::getApplication()->input;
		$hash = $jinput->get('key', '');

    $active = Factory::getApplication()->getMenu()->getActive();
    $urlback = $active->link;
    if(empty($urlback)) {
			$urlback = 'index.php?option=com_lartssubscriptions&view=mysubscriptions';
    }
    
    // check subscription here, since user could have copied the url for later use
    
    $file = Files::lookupByHash($hash);
		$userid = Factory::getApplication()->getIdentity()->id;

		if (empty($file))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$link = Route::_($urlback);
			$this->setRedirect($link);
			return false;
    }
    
		// Updating the counter.
    $download = new Download;
		$download->logging($userid, GeoIP::getIP(), $file->filename, Download::DOWNLOAD_STREAM, 0);

		// Path to the file depends on the location of the file (local or remote)
		$path  = $download->getFilePath();
    $filePath = $path . $file->filename_physical;
    
    if(empty($file->filename_physical) || !file_exists($filePath)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FILE_NOT_FOUND'), 'error');
			$link = Route::_($urlback);
			$this->setRedirect($link);
			return false;
    }

    $stream = new VideoStream($filePath);
    $stream->start();    
    
  }
  
  public function invoice()
	{
    $invoice = new Invoice;
    $invloice_id = $invoice->getInvoiceByDownloadKey($this->dlkey);
    
    if($invloice_id == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=myinvoices'));
    }
    
		// Path to the invoices
		$path = $invoice->getPath();

		// Stop if user is not logged in.
		if(!$this->userid || $this->userid == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=myinvoices'));

			return false;
		}

		// Check invoice path
		if(!is_dir($path)) {
			if(!mkdir($path, 0755, true)) {
				die('Failed to create invoice folder...');
			}
		}

		// Initate the download helper.
		$download = new Download;
		// Get required invoice information.
		$data = $download->downloadInvoice((int) $invloice_id);

		// If logged in user != the userid from the invoice data.
		if ($data->userid != $this->userid)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
      //$self = Uri::getInstance()->render();
      // gete jump back article id
      $articleId = Factory::getApplication()->input->get('aid', 0);
      //$jtbl = new TableContent;
      //Redirect::getIdFromAlias('my-invoices');
      $this->setRedirect(Route::_('index.php?option=com_content&view=article&id=' . $articleId));
			//$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=myinvoices'));

			return false;
		}

		$invoice_number  = ! empty($data->invoice_number) ? $data->invoice_number : $data->id;
		$newfilename = 'invoice-' . $invoice_number . '.pdf';

		// Log the download to the logging table:
		$download->logging($this->userid, GeoIP::getIP(), $newfilename, Download::DOWNLOAD_INVOICE, 0);

		// http headers for zip downloads
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/pdf");
		header("Content-Disposition: attachment; filename=\"" . strtolower($newfilename) . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($data->invoice_path . $invoice_number . '.pdf'));
		ob_clean();
		flush();
		@readfile($data->invoice_path . $invoice_number . '.pdf');
		die;
	}
  
  public function ticket()
	{
    $ticket = Ticket::getTicketByDownloadKey($this->dlkey);
    
    if(empty($ticket)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=mytickets'));
    }
    
		// Path to the tickets
		$path = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/tickets/';

		// Stop if user is not logged in.
		if(!$this->userid || $this->userid == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=mytickets'));

			return false;
		}

		// Check tickets path
		if(!is_dir($path)) {
			if(!mkdir($path, 0755, true)) {
				die('Failed to create tickets folder...');
			}
		}

		// Initate the download helper.
		$download = new Download;
		// Get required invoice information.
    $data = $ticket;
		$data->ticket_path = $download->prepareDownloadTicket($ticket);
    if(empty($data->ticket_path) || !file_exists($data->ticket_path)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_MISSING_FILE'), 'error');
      $this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=mytickets'));
      return false;
    }

		// If logged in user != the userid from the ticket data.
		if ($data->userid != $this->userid)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
      //$self = Uri::getInstance()->render();
      // gete jump back article id
      $articleId = Factory::getApplication()->input->get('aid', 0);
      if($articleId != 0) {
        $this->setRedirect(Route::_('index.php?option=com_content&view=article&id=' . $articleId));
      }
      else {
        $this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=mytickets'));
      }
      //$jtbl = new TableContent;
      //Redirect::getIdFromAlias('my-invoices');
			//$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=myinvoices'));

			return false;
		}

		$ticket_number  = ! empty($data->ticket_number) ? $data->ticket_number : $data->id;
		$newfilename = 'ticket-' . $ticket_number . '.pdf';

		// Log the download to the logging table:
		$download->logging($this->userid, GeoIP::getIP(), $newfilename, Download::DOWNLOAD_TICKET, 0);

		// http headers for zip downloads
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/pdf");
		header("Content-Disposition: attachment; filename=\"" . strtolower($newfilename) . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($data->ticket_path));
		ob_clean();
		flush();
		@readfile($data->ticket_path);
		die;
	}

  public function category()
	{
		$jinput   = Factory::getApplication()->input;
		$hash = $jinput->get('key', '');
    
    $cat = Categories::fromHash($hash);
    
    if(empty($cat)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			$this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=mysubscriptions'));
    }

    $this->setRedirect(Route::_('index.php?option=com_lartssubscriptions&view=mysubscriptions&cat='. $cat->id));
  }
  
  public function package()
  {
    $user = Factory::getApplication()->getIdentity();
    $urlback = Route::_('index.php?option=com_lartssubscriptions&view=mysubscriptions');
    
    if($user->id == 0) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NOT_ALLOWED_PLEASE_LOGIN'), 'error');
    	$this->setRedirect($urlback);
      return;
    }

    $key = $this->input->getString('key');
    if(empty($key)) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
    	$this->setRedirect($urlback);
      return;
    }

    if(!LartsPkgManager::IsInstalled()) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
    	$this->setRedirect($urlback);
    }
    
    $api = LartsPkgManager::getAPI();
    if(!$api->downloadPackage($key, true)) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
    }

  	$this->setRedirect($urlback);
  }

}
