<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\UserRegistration;

class MyprofileController extends BaseController
{
	protected $user;
	protected $userid;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->user   = Factory::getApplication()->getIdentity();
		$this->userid = $this->user->id;
	}

	/**
	 * Storing profile information.
	 */
	public function storeProfile()
	{
 		$this->checkToken();

		if (empty($this->user))
		{
			Factory::getApplication()->redirect('index.php');

			return false;
		}

		if ( ! $this->validateInput())
		{
			Factory::getApplication()->redirect('index.php?option=com_lartssubscriptions&view=myprofile');

			return false;
		}

		$post  = Factory::getApplication()->input->post->getArray();
		$model = $this->getModel('myprofile');

		if ($this->user->email != $this->input->getString('emailaddress'))
		{
			$registration = new UserRegistration;

			if ($registration->isExsistingUser('email', $this->input->getString('emailaddress')))
			{
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_ALREADY_IN_DB'), 'error');
				Factory::getApplication()->redirect('index.php?option=com_lartssubscriptions&view=myprofile');

				return false;
			}

			$data = [
				'email' => $this->input->getString('emailaddress'),
			];

			if ( ! $registration->updateJoomlaUserInformation($this->user->id, $data))
			{
				Factory::getApplication()->enqueueMessage($registration->getError(), 'error');
				Factory::getApplication()->redirect('index.php?option=com_lartssubscriptions&view=myprofile');

				return false;
			}
		}

		if (strlen($post['vatnumber']) > 5)
		{
			// Checking the VAT number agains the VIES registartion database.
      $client = new EUTaxClient;
      $check = $client->checkVat($post['country'], $post['vatnumber']);

			$post['vat_validation_date']   = (new Date)->toSql();
			$post['vat_validation_result'] = ($check === false) ? 0 : 1;
		}

		$post['is_business'] = $this->input->getInt('isbusiness');
		$post['ip_address']  = $_SERVER['REMOTE_ADDR'];
		$post['userid']      = $this->user->id;
		$post['vatnumber']   = ! empty($post['vatnumber']) ? $post['country'] . $post['vatnumber'] : (! empty($post['vatnumber2']) ? $post['vatnumber2'] : '');
		$post['published']   = 1;

		if ( ! $model->store($post))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FAILED_SAVING_PROFILE'), 'error');
			Factory::getApplication()->redirect(Route::_('index.php?option=com_lartssubscriptions&view=myprofile'));

			return false;
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SAVING_PROFILE_SUCCESS'));
		Factory::getApplication()->redirect(Route::_('index.php?option=com_lartssubscriptions&view=myprofile'));

		return true;
	}

	/**
	 * Validating the user input.
	 */
	private function validateInput()
	{
		$app    = Factory::getApplication();
		$jinput = Factory::getApplication()->input;

		if ( ! filter_var($jinput->getString('emailaddress'), FILTER_VALIDATE_EMAIL))
		{
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_NOT_VALID'), 'error');

			return false;
		}

		if ($jinput->getString('emailaddress') == '')
		{
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_FILL_EMAIL_ADDRESS'), 'error');

			return false;
		}

		if ($jinput->getString('firstname') == '')
		{
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_ENTER_YOUR_FIRSTNAME'), 'error');

			return false;
		}

		if ($jinput->getString('lastname') == '')
		{
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_ENTER_YOUR_LASTNAME'), 'error');

			return false;
		}

		if ($this->input->getString('address1') == '' && LartsConfig::get('show_1st_addressline') == 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ENTER_YOUR_ADDRESS'), 'error');

			return false;
		}

		if ($this->input->getString('city') == '' && LartsConfig::get('show_city') == 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CHOOSE_A_CITY'), 'error');

			return false;
		}

		if (! preg_match('^[A-Z]{2}$^', $this->input->getString('country_code')) && LartsConfig::get('show_country') == 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CHOOSE_A_COUNTRY'), 'error');

			return false;
		}

		return true;
	}
}