<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Access;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

use LartsSubscriptions\GeoIP\GeoIP;
use LartsSubscriptions\Ordercode;
use LartsSubscriptions\Coupon;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Product;
use LartsSubscriptions\Cart;
use LartsSubscriptions\ThirdParty\LartsLicManager;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Price;
use LartsSubscriptions\Base\PluginManager;


class OrderController extends BaseController
{
	private $product_id;
	private $userid;
  private $ordercode;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$jinput = Factory::getApplication()->input;
		$this->product_id = $jinput->get('id', '0', 'int');

		$this->userid = Factory::getApplication()->getIdentity()->id;
		$this->ordercode = intval(Factory::getSession()->get('ordercode'));
	}

  public function ajaxDeleteItem()
  {
		$product_id = Factory::getApplication()->input->getInt('item', '0');
    $cart = new Cart;
    
    if(!empty($cart->ordercode)) {
      $cart->removeProduct($product_id);
      $amounts = new OrderAmounts($cart->ordercode, true);

      $data['gross']   = Price::_($amounts->grossprice());
      $data['vat']     = Price::_($amounts->vatprice());
      $data['net']   = Price::_($amounts->netprice());

  		die(json_encode($data));
    }
    die('');
  }

	function getLicencePreselect()
	{
    // Trigger plugin to get licences of user
    if(!empty($this->userid)) {
      $plgReturns = PluginManager::execPlugins('lartssubscriptions', 'onGetUserLicences', ['userid' => $this->userid], false, false);
      if(!empty($plgReturns)) {
        // walk back to front and grab the first licence string found
        foreach(array_reverse($plgReturns) as $plgRet) {
          if(!empty($plgRet)) {
            return $plgRet[0]->licence_number;
          }
        }
      }
    }
    return null;
  }
  
	public function addCartItem()
	{
		$app = Factory::getApplication();
		$db  = Factory::getDbo();

		// Getting the view levels for this user to show only products that belongs to the access level.
		$access_levels = Access\Access::getAuthorisedViewLevels(Factory::getApplication()->getIdentity()->get('id'));

		// Link for redirection.
		$redirect_url = $this->getRedirectUrl();

		if ($this->product_id == 0) {
			// Redirect the customer back. No product to add.
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_FAILED_PRODUCT_TO_CART'), 'error');
			$this->setRedirect($redirect_url);

			return false;
		}

    if(!LartsConfig::get('shop_enabled')) {
			$app->enqueueMessage(LartsConfig::get('shop_enabled_msg'), 'error');
			$this->setRedirect($redirect_url);
			return false;
    }
    
		// Get the ordercode if there is one.
    $ordercode = Ordercode::getOrCreateOrdercode();

		// Get the model to save
		$model = $this->getModel('order');

		// Check the product information:
		$product_check = Product::getProduct($this->product_id);

		// Redirect if the product is unpublished
		if ($product_check->published == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_ADD_PRODUCT_TO_CART'), 'error');
			$app->redirect(Route::_('index.php?option=com_lartssubscriptions'));

			return false;
		}

		// Check if this is a one time product and the customer hasn't subscribed before.
		if ($product_check->period_type == 'onetime')
		{
			$previous = $model->hasPreviousSubscription($this->product_id);

			if ($previous) {
				//Redirect the customer back. No product to add as he subscribed before.
				$app->enqueueMessage(Text::sprintf('COM_LARTSSUBS_CANT_SUBSRIBE_MORE_THAN_ONCE', $product_check->name), 'error');
				$this->setRedirect($redirect_url);

				return false;
			}
		}

		// Checking product access level against user view levels.
		if ( ! in_array($product_check->access, $access_levels))
		{
			// Redirect the client as the customer has no access to this product.
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_ACCESS'), 'error');
			Factory::getApplication()->redirect(Route::_('index.php?option=com_lartssubscriptions'));
		}

    $cart = new Cart;
    $cartPproduct = $cart->getProduct($this->product_id);
		$quantity_in_cart = !empty($cartPproduct) ? $cartPproduct->quantity : 0;
		$quantity_max_count = in_array($product_check->period_type, ['lifetime','onetime',]) 
            ? 1 : (int) LartsConfig::get('cart_quantity_max_count');

		if ($quantity_in_cart >= $quantity_max_count)
		{
			Factory::getApplication()->enqueueMessage((in_array($product_check->period_type, ['lifetime', 'onetime',]) 
              ? Text::_('COM_LARTSSUBS_COULD_NOT_ADD_PRODUCT_TWICE_TO_CART') 
              : Text::sprintf('COM_LARTSSUBS_COULD_NOT_ADD_PRODUCT_MORE_TO_CART', $quantity_max_count)), 'notice');
			$app->redirect(Route::_('index.php?option=com_lartssubscriptions&view=checkout'));

			return false;
		}


    $licence = $this->getLicencePreselect();

    // check if product already in here
    $bRet = false;
    if(!empty($cartPproduct)) {
      $bRet = $cart->updateProductQuantity($this->product_id, $cartPproduct->quantity + 1);
    }
    else {
      $data = [
        //'countrycode' => ! empty($country_code) ? $country_code : null,
        'id'          => 0,
        'ordercode'   => $ordercode,
        'product_id'  => $this->product_id,
        'quantity'    => 1,
        'price'       => $product_check->price,
        'discount'       => 0,
        'discount_group' => 'NULL',
        'discount_type'  => 'NULL',
        'licence'        => $licence,
      ];

      $bRet = $cart->addChartItem($data);
    }

    if(!$bRet) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FAILED_PRODUCT_TO_CART'), 'error');
    }
    else {
      Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_ADDED_PRODUCT_TO_CART', $product_check->name));
    }

		// When the session for the coupon is set, the customer added an URL based coupon without products in cart.
		// Now it is time to re-check the coupon and process the coupon.

		if (Factory::getSession()->get('coupon_code')) {
			$coupon = new Coupon;
			// We only want to show a message when the coupon is applied as the check has been done before.
			$coupon->check(Factory::getSession()->get('coupon_code'));
		}

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onAfterAddCartItem', [$data]);

		$this->setRedirect($redirect_url);
	}


  
	private function getRedirectUrl()
	{
		switch (LartsConfig::get('redirect_on_order'))
    {
			case 'products':
        return Route::_('index.php?option=com_lartssubscriptions&view=products');

			case 'product':
				if ( ! $this->product_id) {
					return Route::_('index.php?option=com_lartssubscriptions&view=checkout');
				}

				return Route::_('index.php?option=com_lartssubscriptions&view=product&id=' . $this->product_id);

			case 'current':
				if ( ! $url = $this->getCurrentUrl())
				{
					return Route::_('index.php?option=com_lartssubscriptions&view=checkout');
				}

				return $url;

			default:
			case 'checkout':
				return Route::_('index.php?option=com_lartssubscriptions&view=checkout');
		}
	}

	private function getCurrentUrl()
	{
		if (empty($_SERVER['HTTP_REFERER']) || ! $referer = $_SERVER['HTTP_REFERER'])
		{
			return false;
		}

		$root = Uri::base();

		if (strpos($referer, $root) !== 0)
		{
			return false;
		}

		return $referer;
	}
}
