<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

use LartsSubscriptions\PSP\PaymentMollie;
use LartsSubscriptions\PSP\PaymentPayPal;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\UserRegState;
use LartsSubscriptions\OrderStatus;

class PaymentController extends BaseController
{
	private $ordercode;
	private $userid;
	private $processor;
	private $pay;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->ordercode = intval(Factory::getSession()->get('ordercode'));
		$this->userid    = Factory::getApplication()->getIdentity()->id;
    $this->processor = Factory::getApplication()->input->get('processor');
    $this->pay       = Factory::getApplication()->input->get('pay');
	}

  public function canceled()
  {
  	// When a user clicks the cancel button when arriving at the payment provider pages.
    $app = Factory::getApplication();
		
		UserRegState::postProcessCheckout(OrderStatus::CANCELED);

    $app->enqueueMessage(Text::_('COM_LARTSSUBS_PAYMENT_HAS_BEEN_CANCELLED'), 'warning');

    $redirect = Route::_('index.php?option=com_lartssubscriptions&view=checkout');
    $app->redirect($redirect);
    exit();
  }

  
  public function processed()
  {
    $ain = Factory::getApplication()->getInput()->getArray();


    $processor = Factory::getApplication()->input->getString('processor', 'paylater');
    PluginManager::execPlugin('lartssubscriptions_payment', strtolower($processor), 'onResponse');
    
    PluginManager::execPlugins('lartssubscriptions', 'onAfterPaymentMessage');
  }

  public function payselect()
  {
    $processor = Factory::getApplication()->input->getString('processor', 'amazonpay');
    
    PluginManager::execPlugin('lartssubscriptions_payment', strtolower($processor), 'onSelectPayment');

  }

  public function payprocess()
  {
    $jinput = Factory::getApplication()->input;

    $processor = $jinput->getString('processor', 'amazonpay');
    $access_token = $jinput->post->get('access-token', '', 'raw');
    $orderrefid = $jinput->post->get('orderrefid', '', 'raw');

    $data = [
      'access-token' => $access_token,
      'orderrefid' => $orderrefid,
    ];
    PluginManager::execPlugin('lartssubscriptions_payment', $processor, 'onProcess', $data);
  }
  
};
