<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Router\Route;

use LartsSubscriptions\Base\Country;
use LartsSubscriptions\TemporaryTransaction;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Price;
use LartsSubscriptions\Config as LartsConfig;

use LartsSubscriptions\VAT\EUTaxClient;



class Signupcontroller extends BaseController
{
	public $userid;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		## Check if the user is logged in.
		$this->userid = Factory::getApplication()->getIdentity()->id;
	}

	public function login()
	{
 		$this->checkToken();

    $app = Factory::getApplication();

		// Populate login data
		$data = [
			'username'  => $this->input->post->getString('username'),
			'password'  => $this->input->post->get('password', '', 'raw'),
			'secretkey' => $this->input->post->get('secretkey', '', 'raw'),
			'return'    => 'index.php?option=com_lartssubscriptions&view=checkout',
		];

		// Get the log in options.
		$options = [
			'remember' => $this->input->post->getBool('remember', false),
			'return'   => $data['return'],
		];

		// Get the log in credentials.
		$credentials = [
			'username' => $data['username'],
			'password' => $data['password'],
		];

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onBeforeLogin', ['username' => $data['username']]);

		// Perform the log in.
		$app->login($credentials, $options);

		$redirect = Route::_('index.php?option=com_lartssubscriptions&view=checkout');

		// Check if the log in succeeded.
		if(!$app->login($credentials, $options))
		{
			$app->enqueueMessage(Text::_('COM_LARTSSUBS_LOGIN_UNSUCCESSFUL'), 'error');
			$app->redirect($redirect);

			return false;
		}

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onAfterLogin', ['userid' => Factory::getApplication()->getIdentity()->id]);

		$app->setUserState('users.login.form.data', []);
		$app->enqueueMessage(Text::_('COM_LARTSSUBS_LOGIN_SUCCESSFUL'));

		$app->redirect($redirect);
	}


	function checkVatNumber()
	{
		$vatnumber = $this->input->getString('vatnumber');
		$country   = $this->input->getString('country');
    
		$data = [
      'valid'        => 'NO',
      'status'       => '999',
			'msg'          => '',

			'validator_price'   => '',
			'carttotal_vat'     => Price::_(0),
			'carttotal_total'   => Price::_(0),
		];
    if($vatnumber != 'EMPTY')
    {
      //require_once JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/helpers/vat.php';
      
      $client = new EUTaxClient;
      $ret = $client->checkVat($country, $vatnumber);
      if($ret === false) {
    		$amount = new OrderAmounts();
        $amount->country_code = $country;
        $amount->recalcCart();
        
        $data['msg']               = 'Error';
        $data['validator_price']   = 'VAT applied';
        $data['carttotal_vat']     = Price::_($amount->vatprice());
        $data['carttotal_total']   = Price::_($amount->netprice());
      }
      else {
        $data['valid'] = 'YES';
        $data['status'] = '666';
        $data['msg']               = 'Ok (' .$ret->name. ', ' .$ret->address. ')';
      }
      
    }

		die(json_encode($data));
	}

}
