<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\Filter\OutputFilter;
use LartsSubscriptions\Download;


class UpdaterController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);
		parent::__construct();
	}

	public function download()
	{
		/**
		 * Default userid = 0 (free download by updater)
		 * If a key is provided this value is overwritten
		 */

		$key     = $this->input->getString('key');
		$file_id = $this->input->get('id', '', 'int');

		$userid  = 0;
		$request = strlen($key) == 32 ? 2 : 1;

		$download = new Download;

		$file = $download->fileInformation($file_id);

		if (empty($file))
		{
			exit('No Access to this file.');
		}

		if ($file->ispublic == 0)
		{
			$accesslist = $download->check('', $key);

			if ( ! in_array($file_id, $accesslist))
			{
				PluginHelper::importPlugin('lartssubscriptions');
				Factory::getApplication()->triggerEvent('onForbidDownloadAccess', [$file]);

				header('HTTP/1.0 403 Access forbidden');
				die('Download ID not valid.');
			}

			$userid = $download->getClient($key);
		}

		// Log the information to the logging table.
		$download->logging($userid, $_SERVER['REMOTE_ADDR'], $file->filename, 1, $request);

		// Path to the file. (can be different due to the remote file.
		$filepath  = $download->getFilePath();
		$download  = OutputFilter::stringURLSafe($file->filename);
		$extension = pathinfo($filepath . $file->filename_physical, PATHINFO_EXTENSION);

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onBeforeDownload', [$file]);

		ob_clean();

		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/octet-stream");
		header("Content-Disposition: attachment; filename=\"" . $download . '.' . $extension . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($filepath . $file->filename_physical));
		ob_end_flush();
		@readfile($filepath . $file->filename_physical);
		die();
	}
}