<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Dispatcher;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Dispatcher\ComponentDispatcher;
use Joomla\CMS\Access\Exception\NotAllowed;

use LartsSubscriptions\Cronjob;


// ComponentDispatcher class for com_categories
//
class Dispatcher extends ComponentDispatcher
{
	protected $option = 'com_lartssubscriptions';
	protected $defaultController = 'products';

  public function dispatch()
	{
    // Since handling payments is a dificile task, prevent calling the processing twice!
    // Some browsers will request the page multiple times to collect other data.
    if(isset($_SERVER['HTTP_SEC_FETCH_DEST'])) {
      $sec_fetch_dest = $_SERVER['HTTP_SEC_FETCH_DEST'];
      if($sec_fetch_dest != 'empty' && $sec_fetch_dest != 'document') {
        return;
      }
    }

    // Tell the browser not to cache this page.
    //$app->setHeader('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT', true);

    // load language
    $lang = Factory::getLanguage();
    $lang->load('com_lartssubscriptions', JPATH_ADMINISTRATOR);

    $this->applyViewAndController();

    $jinput = Factory::getApplication()->getInput();
    
    // handle cron jobs here
    $cron = $jinput->get('cron');
    if(!empty($cron))
    {
      $cron = new Cronjob;
      $cron->run();
      exit('END');
    }

    // attached coupon code?
    $coupon_code = $jinput->getString('coupon');

    // coupon attached?
    if ($coupon_code) {
      $coupon = new Coupon;
      $coupon->storeCouponInSession($coupon_code);
    }

    
    parent::dispatch();
  }
  
  // LartsSubscriptions have to check for extension permission
  protected function checkAccess()
  {
      $extension = $this->getApplication()->input->getCmd('extension', '');

      $parts = explode('.', $extension);

      // Check the user has permission to access this component if in the backend
      if ($this->app->isClient('administrator') && !$this->app->getIdentity()->authorise('core.manage', $parts[0])) {
          throw new NotAllowed($this->app->getLanguage()->_('JERROR_ALERTNOAUTHOR'), 403);
      }
  }

	private function applyViewAndController(): void
	{
		// Handle a custom default controller name
		$view       = $this->input->getCmd('view', $this->defaultController);
		$controller = $this->input->getCmd('controller', $view);
		$task       = $this->input->getCmd('task', ''); //, 'main');

		// Check for a controller.task command.
		if (strpos($task, '.') !== false)
		{
			// Explode the controller.task command.
			[$controller, $task] = explode('.', $task);
		}

		$this->input->set('view', $controller);
		$this->input->set('controller', $controller);
    if($task != null) {
      $this->input->set('task', $task);
  	}
	}
    
}
