<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Categories;

class CategoryModel extends ListModel
{
	public $config;
	public $id;
	public $data;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		parent::__construct($config, $factory);

		$this->config = LartsConfig::get();

		// Get the global DB session
		//$session = Factory::getSession();
		// Get the ordercode if there is one.
		//$this->ordercode = $session->get('ordercode');

		$jinput   = Factory::getApplication()->input;
		$this->id = $jinput->getInt('id', '0');
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$app = Factory::getApplication();

		$params = $app->getParams();
		$this->setState('params', $params);

		$limit = $params->get('display_num', $this->config->display_num);
		$this->setState('list.limit', $limit);

		$limitstart = $app->input->get('limitstart', 0, 'uint');
		$this->setState('list.start', $limitstart);
	}

	public function getPagination()
	{
		$limit = (int) $this->getState('list.limit') - (int) $this->getState('list.links');

		return new Pagination($this->getTotal(), $this->getStart(), $limit);
	}

	public function _getListQuery()
	{
		$config = LartsConfig::get();

		$db = Factory::getDbo();

		return $db->getQuery(true)
			->select('*')
			->from($db->quoteName('#__larts_subs_file_categories', 'f2c'))
			->join('LEFT', $db->quoteName('#__larts_subs_files', 'f') . ' ON (' . $db->quoteName('f2c.file_id') . ' = ' . $db->quoteName('f.id') . ')')
			->where($db->quoteName('f2c.category_id') . ' = ' . (int) $this->id)
			->where($db->quoteName('f.published') . ' = 1')
			->group($db->quoteName('f.id'))
			->order($db->quoteName('f.' . $config->file_ordering) . ' ' . $config->file_ordering_dir);
	}

	public function getItems()
	{
		return $this->_getList($this->_getListQuery(), $this->getStart(), $this->getState('list.limit'));
	}

  public function getProducts($category)
	{
    return Categories::getProducts($category);
  }
  
	public function getProduct()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['p2c.*', 'p.*'])
			->from($db->quoteName('#__larts_subs_product_categories', 'p2c'))
			->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p2c.product_id') . ' = ' . $db->quoteName('p.id') . ')')
			->where($db->quoteName('p2c.category_id') . ' = ' . (int) $this->id);

		$db->setQuery($query);

		$this->data = $db->loadObject();

		return $this->data;
	}

	public function getCategory()
	{
		return Categories::load($this->id);
	}

  public function getChildCategories($parentCat = 0)
  {
		return Categories::getChildCategories($parentCat, Categories::TYPE_PRODUCT);
  }

  public function getSubCategories()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['id', 'name'])
			->from($db->quoteName('#__larts_subs_categories'))
			->where($db->quoteName('parentid') . ' = ' . (int) $this->id)
			->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query);

		$this->data = $db->loadObjectList();

		return $this->data;
	}

  public function getCategoryPath($category)
	{
		return Categories::getPathway($category);
	}

  }
