<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\ListModel;


use LartsSubscriptions\Cart;
use LartsSubscriptions\Order;
use LartsSubscriptions\Customer;
use LartsSubscriptions\Coupon;
use LartsSubscriptions\Discount;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\BundledProducts;


class CheckoutModel extends ListModel
{
	private $ordercode;
	private $config;
	private $db;
  public $message;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		parent::__construct($config, $factory);

		$this->db        = Factory::getDbo();
		$this->config    = LartsConfig::get();
		$this->ordercode = intval(Factory::getSession()->get('ordercode'));
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$params = Factory::getApplication()->getParams();
		$this->setState('params', $params);
	}

	public function getProductsList()
	{
		if (empty($this->ordercode)) {
			return [];
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            'o.*',
			            'p.name', 'p.alias', 'p.type', 'p.price', 'p.period', 'p.period_type',
			            'o.quantity AS count_products',
		            ])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('o.product_id') . ' = ' . $db->quoteName('p.id') . ')')
		            ->where($db->quoteName('o.ordercode') . ' = ' . $this->ordercode)
		            ->order('o.discount ASC')
		            ->group('o.product_id');

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	public function getCoupon()
	{
		$coupon = new Coupon($this->ordercode);

		return $coupon->getCoupon();

	}


	function getSingleProcessor()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__extensions'))
		            ->where($db->quoteName('folder') . ' = ' . $db->quote('lartssubscriptions_payment'))
		            ->where($db->quoteName('enabled') . ' = 1');

		$db->setQuery($query);

		$data = $db->loadObject();

		return '<input name="payment_method_value" id="single_payment_method" type="hidden" value="' . $data->element . '">';
	}

	function addProductToCart($orders, $quantity_to_add = 0)
	{
		// Get the last order
		$new_order = array_pop($orders);
		unset($new_order->id);
		unset($new_order->date);

		require_once __DIR__ . '/order.php';
		$model = ListModel::getInstance('order', 'LartssubsModel');

		for ($count = 0; $count < $quantity_to_add; $count++)
		{
			$model->store($new_order);
		}

		return true;
	}

	function applyCoupon($coupon_code)
	{
    $this->message = '';

		//$coupon_code = $data['coupon_code'];
		$replace     = [":", "/", "\\", "@", "#", "@", "!", "$", "?"];
		$coupon_code = str_replace($replace, "", $coupon_code);
		$coupon_code = strtoupper($coupon_code);

		$coupon = new Coupon($this->ordercode);
		$check  = $coupon->check($coupon_code);

		$this->message = $coupon->getMessage();

		return $check;

	}

	function showMessage()
	{
		return $this->message;
	}

  public function updateProductLicenceInCart($ordercode, $product_id, $lic)
	{
		if ($ordercode != $this->ordercode) {
			return false;
		}

    $cart = new Cart();
    if (!$cart->updateCartItemProductLic($product_id, $lic)) {
			return false;
		}
	
 		return true;

  }

	/**
	 * Updating the cart with new cart totals and items in it.
	 */
	public function updateProductQuantityInCart($ordercode, $product_id, $quantity)
	{
		if ($ordercode != $this->ordercode) {
			return false;
		}

		if(!(new Cart())->updateOrderQuantity($product_id, $quantity))
		{
			return false;
		}

		$args = ['ordercode' => $ordercode];

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onAfterChangeOrder', [$args]);

		return true;
	}

	/**
	 * removing a product completely form the cart.
	 */
	public function removeProduct($ordercode, $product_id)
	{
		if ($ordercode != $this->ordercode) {
			return false;
		}
    
    $cart = new Cart;
		if(!$cart->removeProduct($product_id)) {
			return false;
		}

		$args = ['ordercode' => $ordercode];

		PluginHelper::importPlugin('lartssubscriptions');
		Factory::getApplication()->triggerEvent('onAfterRemoveOrder', [$args]);

		return true;
	}

	/**
	 * Return the user object from the table.
	 */
	public function getClient()
	{
		return Customer::getCustomerByUserId(Factory::getApplication()->getIdentity()->id);
	}
}
