<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

use LartsSubscriptions\Periods;

class OrderModel extends BaseDatabaseModel
{
  public $ordercode;
  
	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		parent::__construct($config, $factory);

		## Get the global DB session
		$session = Factory::getSession();
		## Get the ordercode if there is one.
		$this->ordercode = $session->get('ordercode');
	}

	/**
	 * Checks if this customer has been described before to this product.
	 * If multiple subscriptions is set to no then this check is being done.
	 */
	function hasPreviousSubscription($prouctid)
	{
    $userid = Factory::getApplication()->getIdentity()->id;
		// Customer is not logged in, so we need to check it later on.
		if ($userid == 0) {
			return false;
		}

    $result = Periods::lookupProduct($userid, $prouctid);

		return !empty($result);
	}

	function store($data)
	{
		$row = $this->getTable('order');

		// Bind the form fields to the web link table
		if ( ! $row->bind($data)) {
			$this->setError($this->_db->getErrorMsg());

			return false;
		}

		// Make sure the web link table is valid
		if ( ! $row->check()) {
			$this->setError($this->_db->getErrorMsg());

			return false;
		}

		// Store the web link table to the database
		if ( ! $row->store()) {
			$this->setError($this->_db->getErrorMsg());

			return false;
		}
	}
}
