<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Access\Access;

use LartsSubscriptions\Product;


class ProductModel extends BaseDatabaseModel
{
	protected $id;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		parent::__construct($config, $factory);

		$jinput = Factory::getApplication()->input;
    $this->id = $jinput->get('id', '0', 'INT');

    $product_alias = $jinput->get('alias', '');
    if(strlen($product_alias) > 0) {
      // get id by alias
      $this->id = $this->getProductIdByAlias($product_alias);
    }
	}
	
	protected function populateState($ordering = null, $direction = null)
	{
		$params = Factory::getApplication()->getParams();
		$this->setState('params', $params);
	}

	/**
	 * Getting the product information
	 */
	public function getProduct()
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$data = $db->loadObject();

    Product::validateImage($data);

		Factory::getApplication()->triggerEvent('onContentPrepareData', ['com_lartssubscriptions.product', &$data]);

		return $data;
	}

  	public function getProductIdByAlias($alias)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('id')
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('alias') . ' = ' . $db->quote($alias));

		$db->setQuery($query);
		$result = $db->loadResult();
    return empty($result) ? 0 : $result;
	}

	/**
	 * Getting the cross sell products based on the access level for this user.
	 */
	public function getCrossSell()
	{
		$access_levels = Access::getAuthorisedViewLevels(Factory::getApplication()->getIdentity()->get('id'));

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('p.*')
		            ->from($db->quoteName('#__larts_subs_products_related', 'cs'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('cs.related_product_id') . ' = ' . $db->quoteName('p.id') . ')')
		            ->where($db->quoteName('cs.product_id') . ' = ' . (int) $this->id)
		            ->where($db->quoteName('p.access') . ' IN (' . implode(",", $access_levels) . ') ')
		            ->order(['p.ordering ASC', 'p.name ASC']);

		$db->setQuery($query);

		return $db->loadObjectList();
	}
}
