<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Access\Access;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Product;

class ProductsModel extends ListModel
{
  public $config;
  
	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		parent::__construct($config, $factory);

		$this->config = LartsConfig::get();
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$app = Factory::getApplication();

		$params = $app->getParams();
		$this->setState('params', $params);

		$limit = $params->get('display_num', $this->config->display_num);
		$this->setState('list.limit', $limit);

		$limitstart = $app->input->get('limitstart', 0, 'int');
		$this->setState('list.start', $limitstart);

		//$product_ids = $params->get('product_ids', null);
		//$this->setState('product_ids', $product_ids);
	}

	function getPagination()
	{
		$limit = (int) $this->getState('list.limit') - (int) $this->getState('list.links');

		return new Pagination($this->getTotal(), $this->getStart(), $limit);
	}

	function _getListQuery()
	{
		$db = Factory::getDbo();

		// Getting the view levels for this user to show only products that belongs to the access level.
		$access_levels = Access::getAuthorisedViewLevels(Factory::getApplication()->getIdentity()->get('id'));

		$query = $db->getQuery(true)
			->select('*')
			->from($db->quoteName('#__larts_subs_products'))
			->where($db->quoteName('published') . ' = 1')
			->where($db->quoteName('access') . ' IN (' . implode(',', array_unique($access_levels)) . ')')
			->order(['ordering ASC', 'name ASC']);

 		$userid = Factory::getApplication()->getIdentity()->id;
    if($userid == 0) {
      // guest
			$query->where($db->quoteName('ispublic') . ' = 1');
    }
    
		//$product_ids = $this->getState('product_ids');
		//if(!empty($product_ids)) {
		//	$query->where($db->quoteName('id') . ' IN (' . implode(',', $product_ids) . ')');
		//}

		return $query;
	}

	function getItems()
	{
		$products =  $this->_getList($this->_getListQuery(), $this->getStart(), $this->getState('list.limit'));

    Product::prepare($products);

		return $products;
	}
}
