<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\Service;
\defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Application\SiteApplication;
use Joomla\CMS\Categories\CategoryFactoryInterface;
use Joomla\CMS\Categories\CategoryInterface;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Component\Router\RouterView;
use Joomla\CMS\Component\Router\RouterViewConfiguration;
use Joomla\CMS\Component\Router\Rules\MenuRules;
use Joomla\CMS\Component\Router\Rules\NomenuRules;
use Joomla\CMS\Component\Router\Rules\StandardRules;
use Joomla\CMS\Menu\AbstractMenu;
use Joomla\Database\DatabaseInterface;

use Larts\Component\LartsSubscriptions\Site\Service\LartsSubsMenuRules;


// Routing class

class Router extends RouterView
{
	// Flag to remove IDs
	protected $noIDs = false;

	// The category factory
	private $categoryFactory;

	// The category cache
	private $categoryCache = [];

	// The db
	private $db;
  
  protected $orig_query = [];
  protected $lartsSubsMenuRules;

	/**
	 * Content Component router constructor
	 *
	 * @param   SiteApplication           $app              The application object
	 * @param   AbstractMenu              $menu             The menu object to work with
	 * @param   CategoryFactoryInterface  $categoryFactory  The category object
	 * @param   DatabaseInterface         $db               The database object
	 */
	public function __construct(SiteApplication $app, AbstractMenu $menu, CategoryFactoryInterface $categoryFactory, DatabaseInterface $db)
	{
		$this->categoryFactory = $categoryFactory;
		$this->db              = $db;

		$params = ComponentHelper::getParams('com_lartssubscriptions');
		$this->noIDs = (bool) $params->get('sef_ids');

    $cats = new RouterViewConfiguration('categories');
		$this->registerView($cats);
    $cat = new RouterViewConfiguration('category');
    $cat->setKey('id')->setParent($cats, 'id')->setNestable();
		$this->registerView($cat);

    $products = new RouterViewConfiguration('products');
		$this->registerView($products);
    $product = new RouterViewConfiguration('product');
		$product->setKey('id')->setParent($products);
		$this->registerView($product);

    $file = new RouterViewConfiguration('file');
		$file->setKey('id');
		$this->registerView($file);

    $checkout = new RouterViewConfiguration('checkout');
		$this->registerView($checkout);

    $myinvoices = new RouterViewConfiguration('myinvoices');
		$this->registerView($myinvoices);

    $myprofile = new RouterViewConfiguration('myprofile');
		$this->registerView($myprofile);

    $mysubscriptions = new RouterViewConfiguration('mysubscriptions');
		$this->registerView($mysubscriptions);

    $mytickets = new RouterViewConfiguration('mytickets');
		$this->registerView($mytickets);

    $payment = new RouterViewConfiguration('payment');
		$this->registerView($payment);


		parent::__construct($app, $menu);

    $this->lartsSubsMenuRules = new LartsSubsMenuRules($this);
		$this->attachRule($this->lartsSubsMenuRules);
    $this->attachRule(new MenuRules($this));
    $this->attachRule(new StandardRules($this));
    $this->attachRule(new NomenuRules($this));

    //$this->attachRule(new StandardRules($this));
		//$this->attachRule(new NomenuRules($this));
		//$this->attachRule(new LartsSubsMenuRules($this));
	}

  public function queryAlias($table, $id)
  {
    $db = Factory::getDbo();
    $dbquery = $db->getQuery(true);
        
    $dbquery->select($dbquery->qn('alias'))
        ->from($db->qn($table))
        ->where('id = ' . $db->q($id));
            
    $db->setQuery($dbquery);
    return $db->loadResult();
  }
  
  
  public function queryId($table, $segment)
  {
    $db = Factory::getDbo();
    $dbquery = $db->getQuery(true);
        
    $dbquery->select($dbquery->qn('id'))
        ->from($dbquery->qn($table))
        ->where('alias = ' . $dbquery->q($segment));
            
            
    $db->setQuery($dbquery);
    return $db->loadResult();
  }

  public function getCheckoutSegment($id, $query)
  {
		$segments = [];
		if (empty($query['view'])) {
			// We need to have a view in the query or it is an invalid URL
			return $segments;
		}
    return $segments;
  }

  public function getCheckoutId($segment, $query)
  {
		$segments = [];
		if (empty($query['view'])) {
			// We need to have a view in the query or it is an invalid URL
			return $segments;
		} 
    return 0;
  }

  public function getCategoriesId($segment, $query)
  {
    $result = $this->queryId('#__larts_subs_categories', $segment);
    if(empty($result)) {
      return false;
    }
    return (int) $result;
  }
  
  public function getCategoriesSegment($id, $query)
  {
    $result = $this->queryAlias('#__larts_subs_categories', $id);
    if(empty($result)) {
      return array(0 => 'unknown');
    }
    $id .= ':' . $result;
    list($void, $segment) = explode(':', $id, 2);

    return array($void => $segment);
  }

  public function getCategoryId($segment, $query)
  {
    $result = $this->queryId('#__larts_subs_categories', $segment);
    if(empty($result)) {
      return false;
    }
    return (int) $result;
  }
  
  public function getCategorySegment($id, $query)
  {
    $result = $this->queryAlias('#__larts_subs_categories', $id);
    if(empty($result)) {
      return false;
    }
    $id .= ':' . $result;
    list($void, $segment) = explode(':', $id, 2);

    return array($void => $segment);
  }
  
  public function getProductSegment($id, $query)
  {
    $db = Factory::getDbo();
    $dbquery = $db->getQuery(true);
        
    $dbquery->select($dbquery->qn('alias'))
        ->from($db->qn('#__larts_subs_products'))
        ->where('id = ' . $db->q($id));
            
    $db->setQuery($dbquery);

    $id .= ':' . $db->loadResult();

    list($void, $segment) = explode(':', $id, 2);
    //unset($query[$id]); 

    return array($void => $segment);
  }

  public function getProductId($segment, $query)
  {
    $db = Factory::getDbo();
    $dbquery = $db->getQuery(true);
        
    $dbquery->select($dbquery->qn('id'))
        ->from($dbquery->qn('#__larts_subs_products'))
        ->where('alias = ' . $dbquery->q($segment));
            
    $db->setQuery($dbquery);
        
    if (!(int) $db->loadResult())
    {
        return false;
    }
    return (int) $db->loadResult();
  }

  public function getProductsId($segment, $query)
  {
    $db = Factory::getDbo();
    $dbquery = $db->getQuery(true);
        
    $dbquery->select($dbquery->qn('id'))
        ->from($dbquery->qn('#__larts_subs_products'))
        ->where('alias = ' . $dbquery->q($segment));
            
    $db->setQuery($dbquery);
        
    if (!(int) $db->loadResult())
    {
        return false;
    }
    return (int) $db->loadResult();
  }

  public function getProductsSegment($id, $query)
  {
		$segments = [];
		if (empty($query['view'])) {
			// We need to have a view in the query or it is an invalid URL
			return $segments;
		} 

    $db = Factory::getDbo();
    $dbquery = $db->getQuery(true);
        
    $dbquery->select($dbquery->qn('alias'))
        ->from($db->qn('#__larts_subs_products'))
        ->where('id = ' . $db->q($id));
            
    $db->setQuery($dbquery);

    $id .= ':' . $db->loadResult();

    list($void, $segment) = explode(':', $id, 2);

    return array($void => $segment);
  }
  

public function build(&$query)
{
  $this->orig_query = $query;
  
//  if(isset($query['task'])) {
//    $task = $query['task'];
//    if (strpos($task, '.') !== false)
//    {
//      // Explode the controller.task command.
//      [$controller, $task] = explode('.', $task);
//      $query['view'] = $controller;
//    }
//  }

  $ret = parent::build($query);
  return $ret;
}

public function parse(&$segments)
{
  $ret = parent::parse($segments);
  return $ret;
}



  
}

