<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\View\Categories;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Traits\InlineView;

    
class HtmlView extends BaseHtmlView
{
  use InlineView;
  
  var $smartTagParams = null;
  var $config;
  var $params;
  var $cat_id;
  var $category;
  var $cat_path;
  var $categories;
  var $products;

  public function __construct($config = [])
  {
    parent::__construct($config);
  }
  
  function display($tpl = null)
	{
		$this->config = LartsConfig::get();
		$this->params = $this->get('State')->get('params');

    $model = $this->getModel('Categories');

    if($this->smartTagParams != null) {
      $this->cat_id = $this->smartTagParams['cat'] ?: 0;
    }
    else {
      $this->cat_id = Factory::getApplication()->input->getInt('id', 0);
    }
		$this->category = $model->getCategory($this->cat_id);
		$this->cat_path = $model->getCategoryPath($this->cat_id);
    
    $this->categories = $model->getChildCategories($this->cat_id);

    $this->products = array();
    if($this->config->show_cats_all_products || $this->cat_id != 0) {
      $this->products = $model->getProducts($this->cat_id);
    }

    if($this->document == null) {
      return $this->renderInline();
    }

    $this->prepareDocument();
    
		parent::display($tpl);
	}

  public function getBreadcrumb()
  {
    $catUrl = 'index.php?option=com_lartssubscriptions&view=categories&id=';
    $html = '';

    foreach($this->cat_path as $c) {
      $item = '<a hfref="#">&nbsp;'.$c['name'].'</a>';
      if(strlen($html) > 0) {
        $html .= '&nbsp; > &nbsp;';
      }
      $html .= '<a href="'.$catUrl.$c['id'].'">' .$c['name']. '</a>';
    }

    return '<span>' .$html. '</span>';
  }  
  
  protected function prepareDocument()
	{
    // set page title
    $app = Factory::getApplication();
    if ($app->get('sitename_pagetitles', 0) == 1)    {
      // Put the site name before the page title.
      $sitetitle = Text::sprintf('JPAGETITLE', $app->get('sitename'), Text::_('COM_LARTSSUBS_PRODUCT_CATEGORIES'));
    }
    elseif ($app->get('sitename_pagetitles', 0) == 2) {
      // Put the site name after the page title.
      $sitetitle = Text::sprintf('JPAGETITLE', Text::_('COM_LARTSSUBS_PRODUCT_CATEGORIES'), $app->get('sitename'));
    }
    else
    {
      // Nothing in configuration:
      $sitetitle = Text::_('COM_LARTSSUBS_PRODUCT_CATEGORIES');
    }

    $document = Factory::getDocument();
    $document->setTitle($sitetitle);
  }
}
