<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Site\View\Checkout;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;

use LartsSubscriptions\GeoIP\GeoIP;
use LartsSubscriptions\Base\Country;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Traits\InlineView;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\UserRegState;


class HtmlView extends BaseHtmlView
{
  use InlineView;

	protected $processors_amount;
  public $config;
  public $items;
  public $params;
  public $lists;
  public $coupon;
  public $ordercode;
  public $order_amounts;
  public $processors;
  public $amazonbtn;
	public $client;
	public $vat_countries;
	public $cart;

	public function display($tpl = null)
	{
    $app = Factory::getApplication();
		$user = Factory::getApplication()->getIdentity();

		$config = LartsConfig::get();

    // Get some data from request:
		$post = UserRegState::getStateData();
    if($config->use_demouser) {
      if(Factory::getApplication()->getIdentity()->id == 0) {
        if(empty($post['username'])) {
          $name = 'TestUser' . time();
          $post['username'] = $name;
        }
        if(empty($post['firstname'])) {
          $post['firstname'] = 'UserFirstname';
        }
        if(empty($post['lastname'])) {
          $post['lastname'] = 'UserLastname';
        }
        if(empty($post['address1'])) {
          $post['address1'] = 'UserAdress1';
        }
        if(empty($post['address2'])) {
          $post['address2'] = 'UserAdress2';
        }
        if(empty($post['zipcode'])) {
          $post['zipcode'] = '12345';
        }
        if(empty($post['city'])) {
          $post['city'] = 'TheCity';
        }
        if(empty($post['country_code'])) {
          $post['country_code'] = 'DE';
        }
        if(empty($post['emailaddress'])) {
					if(!empty($config->company_email)) {
						$parts = explode('@', $config->company_email);
						if(count($parts) > 1) {
		          $post['emailaddress'] = $name .'@'. $parts[1];
						}
					}
        }
      }
    }

		$client = $this->get('client');
    if($client == null) {
      $client = new \stdClass;
    }
    $client->emailaddress = $user->email;

		// Check user data:
		if(!empty($post))
    {
			foreach ($post as $key => $value)
			{
				if ($key == 'username') {
					$value = str_replace(' ', '', $value);
				}

        if(!empty($value)) {
          $client->$key = $value;
        }
			}
		}

		// Getting the country code from the client object.
		$country_code = isset($client->country_code) ? $client->country_code : '';

		if (empty($country_code))
		{
			// Getting the country code from GeoIP database.
			$country_code = (new GeoIP)->getCountryByIP();

			// When the country code is still empty, we can fill it with the default
			// This can be turned off in the configuration when not needed.
			if (empty($country_code) && LartsConfig::get('use_default_country'))
			{
				$country_code = LartsConfig::get('default_country');
			}
		}

		// Filling a dropdown to show the published countries for the checkout.
		$countries[] = HTMLHelper::_('select.option', '', Text::_('COM_LARTSSUBS_PLEASE_SELECT'), 'id', 'name');
		$countries = array_merge($countries, Country::getCountries(1));
		$lists['country'] = HTMLHelper::_('select.genericlist', $countries, 'country_code', 'class="form-control input input-block country_block" ', 
                                      'id', 'name', !empty($client->country_code) ? $client->country_code : $country_code);

		$db  = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['country_2_code as id, vatrate_std as vat'])
		            ->from($db->quoteName('#__larts_subs_countries'))
		            ->where($db->quoteName('requires_vat') . ' = 1')
		            ->where($db->quoteName('vatrate_std') . ' > 0')
		            ->where($db->quoteName('id') . ' != 1');

		$db->setQuery($query);

		$this->lists         = $lists;
		$this->client        = $client;
		$this->vat_countries = $db->loadAssocList('id', 'vat');

    $this->cart = new Cart;
    $this->cart->prepareCartItems();
    $this->items = $this->cart->getProducts(true);
    //$this->items      = $this->get('Products');
		$this->coupon     = $this->get('Coupon');
		$this->config     = $config;

    $this->ordercode = intval(Factory::getSession()->get('ordercode'));
    $this->order_amounts = new OrderAmounts($this->ordercode, true);

		//$this->params     = $this->get('State')->get('params');
		$this->processors = $this->getProcessors();

 		$this->amazonbtn = $this->getAmazonPayButton();


		if ($this->processors_amount == 1 && ! $config->payment_choice_on_one_processor)
		{
			$model = $this->getModel('checkout');
			$this->processors = $model->getSingleProcessor();
		}

    if($this->document == null) {
      return $this->renderInline();
    }

		$this->prepareDocument();

		parent::display($tpl);
	}

	// Loading payment processors (buttons).
	private function getProcessors()
	{
    $pathLayouts = JPATH_SITE . '/components/com_lartssubscriptions/layouts';
		$layout = new FileLayout('payment_processor', $pathLayouts);
   
    $results = PluginManager::execPlugins('lartssubscriptions_payment', 'onRenderMethod', $layout, false, false);
    if(empty($results)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_WARNING_NO_PAYMENTS'), 'warning');
      return '';
    }

		$this->processors_amount = empty($results) ? 0 : count($results);

		return implode('', $results);
	}

	// Loading Amazon payment button
  private function getAmazonPayButton()
  {
    $result = PluginManager::execPlugin('lartssubscriptions_payment', 'amazonpay', 'onRenderButton', null);
		return (empty($result) ? '' : implode('', $result));
  }
  
	protected function prepareDocument()
	{
    $app = Factory::getApplication();
    $document = Factory::getDocument();

    if ($app->get('sitename_pagetitles', 0) == 1) {
      // Put the site name before the page title.
      $sitetitle = Text::sprintf('JPAGETITLE', $app->get('sitename'), Text::_('COM_LARTSSUBS_YOUR_CART'));
    }
    elseif ($app->get('sitename_pagetitles', 0) == 2) {
      // Put the site name after the page title.
      $sitetitle = Text::sprintf('JPAGETITLE', Text::_('COM_LARTSSUBS_YOUR_CART'), $app->get('sitename'));
    }
    else {
      // Nothing in configuration:
      $sitetitle = Text::_('COM_LARTSSUBS_YOUR_CART');
    }
    $document->setTitle($sitetitle);

		$menus = $app->getMenu();
		// OK, there is a menu item for the checkout - Read the params.
    $jcfg = new \JConfig;
		if(!empty($jcfg->robots)) {
			// Setting the robots meta tag for this view.
			$this->document->setMetadata('robots', $jcfg->robots);
		}
	}
}
