<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Message;


$amount = $this->order_amounts;

$app = Factory::getApplication();

//add a new breadcrumb item with the name "Your Cart"
$pathway = $app->getPathway();
$pathway->addItem(Text::_('COM_LARTSSUBS_YOUR_CART'));

HTMLHelper::script('com_lartssubscriptions/lartsTools.js', ['relative' => true, 'version' => 'auto']);
HTMLHelper::script('com_lartssubscriptions/profile.js', ['relative' => true, 'version' => 'auto']);
HTMLHelper::script('com_lartssubscriptions/checkout.js', ['relative' => true, 'version' => 'auto']);

HTMLHelper::stylesheet('com_lartssubscriptions/stylej3.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::stylesheet('com_lartssubscriptions/cart.css', ['relative' => true, 'version' => 'auto']);


// CORS scripting needs same origin, so we did not use URI:base
// 	var larts_root = '" . Uri::base() . "';

$script = "
	var larts_root = '';
	var vat_countries = " . json_encode($this->vat_countries) . ";
	var larts_lang_vat_number_checking = '" . addslashes(Text::_('COM_LARTSSUBS_VAT_NUMBER_CHECKING')) . "';
	var larts_lang_vat_number_invalid = '" . addslashes(Text::_('COM_LARTSSUBS_VAT_NUMBER_INVALID')) . "';
";
Factory::getDocument()->addScriptDeclaration($script);

$pathLayouts = JPATH_SITE . '/components/com_lartssubscriptions/layouts';

?>
<?php
  if($this->config->shop_enabled == 0) {
    echo '<div><p class="text-error">'. $this->config->shop_enabled_msg. '</p></div>';
    return;
  }
?>

<div class="larts larts-checkout">
  
  <noscript>
    <div><h3 class="text-error">This shop needs Javascript! Please enable Javascript temporarily.</h3></div>
  </noscript>

	<?php
	echo Message::getInstance('checkout-header')
	                  ->user(Factory::getApplication()->getIdentity()->id)
	                  ->getBody();
	?>

	<?php
  $layout = new FileLayout('menu', $pathLayouts);
	echo $layout->render(['position' => 'under']);
	?>

  <?php if($this->cart->isEmpty()): ?>
  
    <div class="row">
      <div class="col-6 span6">

        <div class="bs5card bg-light larts-checkout-cart">
          <?php
          $layout = new FileLayout('cart_empty', $pathLayouts);
          echo $layout->render(null);
          ?>
        </div>
      </div>
    </div>

  <?php else: ?>
  
  <div class="larts-cart-container">
    <div class="larts-cart-left">
          <?php if (!$this->cart->isEmpty()): ?>
            <div class="bs5card bg-light larts-checkout-account">
              <?php echo $this->loadTemplate('account'); ?>
            </div>
          <?php endif; ?>
    </div>
    <div class="larts-cart-right">

        <div class="bs5card bg-light larts-checkout-cart">
          <div class="bs5card-body">
            <?php echo $this->loadTemplate('cart'); ?>
            <?php if (isset($this->config->show_coupon) && $this->config->show_coupon && $amount->nettoprice() > 0): ?>
              <div class="larts-checkout-coupon">
                <?php echo $this->loadTemplate('coupon'); ?>
              </div>
            <?php endif; ?>
          </div>
        </div>

        <div class="bs5card bg-light larts-checkout-checkout">
          <?php echo $this->loadTemplate('checkout'); ?>
        </div>
    </div>
  </div>
	<?php endif; ?>
</div>
