<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\HTML\HTMLHelper;

require_once JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/src/Extension/autoload.php';

use LartsSubscriptions\Price;
use LartsSubscriptions\Cart;
use LartsSubscriptions\OrderAmounts;

HTMLHelper::stylesheet('com_lartssubscriptions/carticon.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::script('com_lartssubscriptions/cart_sidebar.js', ['relative' => true, 'version' => 'auto']);

$url_cart = Route::_('index.php?option=com_lartssubscriptions&view=checkout');
$base_url_product = 'index.php?option=com_lartssubscriptions&view=product&id=';

if(empty($ordercode)) {
  $html = '<i class="fas fa-shopping-cart larts-cart-icon" aria-hidden="true"></i>';
}

$script = "
function itemdelete(id)
{
  ajaxDelete(id);
}

function ajaxDelete(id)
{
  // ajax request
  var xhttp = new XMLHttpRequest();
  var ajax_url = 'index.php?option=com_lartssubscriptions&task=order.ajaxDeleteItem&item='+id;

  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      if(this.responseText.length > 0) {
        var elem = document.getElementById('item-'+id);
        var gross = document.getElementById('sidebar-gross');
        var vat = document.getElementById('sidebar-vat');
        var total = document.getElementById('sidebar-total');
        var obj = JSON.parse(this.responseText);
        
        gross.innerHTML = obj.gross;
        vat.innerHTML = obj.vat;
        total.innerHTML = obj.net;
        
        elem.remove();
      }
    }
  };
  xhttp.open('GET', ajax_url, true);
  xhttp.send();
}";

$doc = Factory::getDocument();
$doc->addScriptDeclaration($script);

$apperance = $params->get('apperance');
$overlay_num_items = $params->get('overlay_num_cartitems');
$overlay_show_zero = $params->get('overlay_show_zero');
$num_items = '';
$amounts = (object)['grossprice' => '',
										'vat_total' => '',
										'netprice' => '',
									 ];
$cart_items = [];

$ordercode = intval(Factory::getSession()->get('ordercode'));
if($ordercode == 0) {
	return;
}

if($ordercode != 0)
{
	$cart = new Cart;
	if($apperance == 'icon') {
		$cart_totals = $cart->getTotals();
		$num_items = $cart_totals->items;
	}
	else {
		$cart_items = $cart->getProductList(false);
		if(empty($cart_items)) 
			return;
		$num_items = sizeof($cart_items);
		$amounts = new OrderAmounts($cart->ordercode, true);
	}
}

?>



<?php if($apperance == 'icon') : ?>
<a class="cart-button" href="<?php echo $url_cart ?>">
  <div class="larts-cart-box">
    <i class="fas fa-shopping-cart larts-cart-icon" aria-hidden="true"></i>
    <?php if($overlay_num_items != 0 && ($overlay_show_zero || ($num_items > 0))) : ?>
      <div class="larts-cart-overlay"><span class="larts-cart-amount"><?php echo $num_items ?></span></div>
    <?php endif; ?>
  </div>
</a>

<?php return;
else : ?>
<nav id="main-nav"></nav>
  <a class="cart-button" href="#">
    <div class="larts-cart-box">
      <i class="fas fa-shopping-cart larts-cart-icon" aria-hidden="true"></i>
      <?php if($overlay_num_items != 0 && ($overlay_show_zero || ($num_items > 0))) : ?>
        <div class="larts-cart-overlay"><span class="larts-cart-amount"><?php echo $num_items ?></span></div>
      <?php endif; ?>
    </div>
  </a>
</nav>
<?php endif; ?>

<aside id="sidebar-cart">
  <main>
    <a href="#" class="close-button"><span class="close-icon">X</span></a>
    <h2 class="bag-title">Shopping Bag</h2>
    <div class="products">
      <ul class="cart-items">
        <?php foreach($cart_items as $item) : ?>
        <li class="product" id="item-<?php echo $item->product_id; ?>">
          <a href="<?php echo Route::_($base_url_product. $item->product_id); ?>" class="product-link">
            <span class="product-details">
              <div><?php echo $item->product_name; ?></div>
              <span class="qty-price">
                <span class="qty"><?php echo $item->quantity; ?>&nbsp;x</span>
                <span class="price"><?php echo Price::_($item->price) ?></span>
              </span>
            </span>
          </a>
          <a href="#remove" class="remove-button" onclick="itemdelete(<?php echo $item->product_id; ?>);"><span class="remove-icon">X</span></a>
        </li>
        <?php endforeach; ?>
      </ul>
    </div>
    <div class="totals">
      <div class="subtotal">
        <span>Subtotal:</span> <span class="amount" id="sidebar-gross"><?php echo Price::_($amounts->grossprice) ?></span>
      </div>
      <div class="tax">
        <span>Total VAT:</span> <span class="amount" id="sidebar-vat"><?php echo Price::_($amounts->vat_total) ?></span>
      </div>
      <div class="ordertotal">
          <span>Order Total:</span> <span class="amount" id="sidebar-total"><?php echo Price::_($amounts->netprice) ?></span>
      </div>
    </div>
    <div class="action-buttons">
      <a class="view-cart-button" href="<?php echo $url_cart?>">Cart</a>
    </div>

  </main>
</aside>



