<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Base\Image;
use LartsSubscriptions\Helper;
use LartsSubscriptions\Message;


$app = Factory::getApplication();

$category_id   = $app->getUserState('category_id', '');
$category_name = $app->getUserState('category_name', '');
$pathway = $app->getPathway();
if(!empty($category_id)) {
  $pathway->addItem($category_name, Route::_('index.php?option=com_lartssubscriptions&view=category&id=' . $category_id));
}
$pathway->addItem($this->item->filename);


## Obtain user information.
$user   = Factory::getApplication()->getIdentity();
$userid = $user->id;

HTMLHelper::stylesheet('com_lartssubscriptions/stylej4.css', ['version' => 'auto', 'relative' => true]);

$has_access = false;
if ($this->item->ispublic || in_array($this->item->id, $this->access, true))
{
	$download   = 'index.php?option=com_lartssubscriptions&view=download&task=file&id=' . (int) $this->item->id;
	$has_access = true;
}
?>
<div class="larts larts-file">

	<?php
	$layout = new FileLayout('menu');
	echo $layout->render(['position' => 'above', 'active' => 'categories']);
	?>

	<?php
	echo Message::getInstance('file-header')
		->user(Factory::getApplication()->getIdentity()->id)
		->getBody();
	?>

	<?php
	$layout = new FileLayout('menu');
	echo $layout->render(['position' => 'under', 'active' => 'categories']);
	?>

	<div class="rows">
		<div class="col-sm-4 span4">
			<div class="larts-image-container">
				<img src="<?php echo $this->product->image; ?>" class="larts-image">
			</div>
		</div>

		<div class="col-sm-8 span8">

			<?php if ($has_access) : ?>
				<div class="pull-right">
					<a href="<?php echo $download; ?>" class="btn btn-success btn-large">
						<span class="icon-download icon-white"></span>
						<?php echo Text::_('COM_LARTSSUBS_DOWNLOAD'); ?>
					</a>
				</div>
			<?php endif; ?>

			<h1><?php echo $this->item->filename; ?></h1>

			<table class="table">
				<tr>
					<td style="width:20%" class="nowrap">
						<?php echo Text::_('COM_LARTSSUBS_VERSION'); ?>
					</td>
					<td>
						<?php echo $this->item->version; ?>
					</td>
				</tr>
				<tr>
					<td class="nowrap">
						<?php echo Text::_('COM_LARTSSUBS_RELEASE_DATE'); ?>
					</td>
					<td>
						<?php if ($this->item->created == $this->item->release_date || $this->item->release_date == '0000-00-00'):
							echo DateEx::_($this->item->created);
						else:
							echo DateEx::_($this->item->release_date);
						endif; ?>
					</td>
				</tr>
			</table>
		</div>
	</div>

	<div class="clearfix"></div>

	<?php if ( ! $has_access) : ?>
		<div class="alert alert-warning">
			<?php echo Text::_('COM_LARTSSUBS_FILE_NO_ACCESS'); ?>
		</div>
	<?php endif; ?>

	<hr class="hr_styling">

	<?php if ($this->item->releasenotes || $this->item->installnotes): ?>
		<ul class="nav nav-tabs" id="myTab">
			<?php if ($this->item->releasenotes): ?>
				<li class="active">
					<a href="#release_notes"><?php echo Text::_('COM_LARTSSUBS_RELEASE_NOTES'); ?></a>
				</li>
			<?php endif; ?>

			<?php if ($this->item->installnotes): ?>
				<li>
					<a href="#installation_notes"><?php echo Text::_('COM_LARTSSUBS_INSTALL_NOTES'); ?></a>
				</li>
			<?php endif; ?>
		</ul>

		<div class="tab-content">

			<?php if ($this->item->releasenotes): ?>
				<div class="tab-pane active" id="release_notes">
					<?php echo $this->item->releasenotes; ?>
				</div>
			<?php endif; ?>

			<?php if ($this->item->installnotes): ?>
				<div class="tab-pane" id="installation_notes">
					<?php echo $this->item->installnotes; ?>
				</div>
			<?php endif; ?>

		</div>

		<script>
      document.addEventListener('DOMContentLoaded', function() {
        const myTabLinks = document.querySelectorAll('#myTab a');
        myTabLinks.forEach(function(link) {
          link.addEventListener('click', function(e) {
            e.preventDefault();
            new bootstrap.Tab(this).show();
          });
        });
      });
		</script>
	<?php endif; ?>
</div>
