<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Message;


$app     = Factory::getApplication();
$pathway = $app->getPathway();
$pathway->addItem(Text::_('COM_LARTSSUBS_MYPROFILE'));

HTMLHelper::stylesheet('com_lartssubscriptions/stylej3.css', ['version' => 'auto', 'relative' => true]);

//get user information.
$userid = Factory::getApplication()->getIdentity()->id;

$script = "
	var larts_root = '" . Uri::base() . "';
	var vat_countries = " . json_encode($this->vat_countries) . ";
	var larts_lang_vat_number_checking = '" . addslashes(Text::_('COM_LARTSSUBS_VAT_NUMBER_CHECKING')) . "';
	var larts_lang_vat_number_invalid = '" . addslashes(Text::_('COM_LARTSSUBS_VAT_NUMBER_INVALID')) . "';
";
Factory::getDocument()->addScriptDeclaration($script);
HTMLHelper::script('com_lartssubscriptions/profile.js', ['relative' => true, 'version' => 'auto']);

?>

<div class="larts larts-myprofile">

	<?php
	$layout = new FileLayout('menu');
	echo $layout->render(['position' => 'above']);
	?>

	<?php
	echo Message::getInstance('myprofile-header')
	                  ->user(Factory::getApplication()->getIdentity()->id)
	                  ->getBody();
	?>

	<?php
	$layout = new FileLayout('menu');
	echo $layout->render(['position' => 'under']);
	?>

	<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&task=myprofile.storeProfile'); ?>" method="post" name="profileForm" id="profileForm" class="form-vertical">

		<div class="row">
			<div class="col-md-6">
				<div class="bs5card bg-light">
          <div class="bs5card-body">
            <?php
            $layout = new FileLayout('account_fields');
            echo $layout->render([
              'client'         => $this->client,
              'config'         => $this->config,
              'countries'      => $this->lists['country'],
            ]);
            ?>
          </div>
					<button type="submit" class="btn btn-success btn-block btn-large" type="button">
            <i class="fas fa-check-square"></i>
						<?php echo Text::_('COM_LARTSSUBS_SAVE_PROFILE'); ?>
					</button>
				</div>
			</div>
		</div>

		<div class="clearfix"></div>

		<input type="hidden" name="userid" value="<?php echo $userid; ?>" />
		<input type="hidden" name="id" value="<?php echo isset($this->client->id) ? $this->client->id : 0; ?>" />

		<?php echo HTMLHelper::_('form.token'); ?>
	</form>

</div>
