<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Base\PluginManager;

// Requires the amount to check if the price is still > 0
// Otherwise the session has probably been expired and the customer needs to be redirected.

$app = Factory::getApplication();
$ordercode = Factory::getSession()->get('ordercode');
$amount = new OrderAmounts($ordercode, false);
$processor = $app->input->getString('processor', 'paypal');

if ($processor != 'freecheckout')
{
  // session expired (amount == 0)?
  if ($amount->netprice() == 0) {
    $app->enqueueMessage(Text::_('COM_LARTSSUBS_SESSION_HAS_BEEN_EXPIRED'), 'error');
    $app->redirect(Route::_('index.php?option=com_lartssubscriptions&view=products'));
  }

  $parts     = explode('.', $processor);
  $processor = $parts['0'];
  $task      = isset($parts['1']) ? $parts['1'] : 'cancel';

  PluginManager::execPlugin('lartssubscriptions_payment', $processor, 'onProcess', $task);
}
else
{
	echo '<h1>' . $this->message['subject'] . '</h1>';
	echo $this->message['message'];
}

?>

<script type="text/javascript">
  document.addEventListener('DOMContentLoaded', function() {
    const paymentButton = document.querySelector('#paymentButton');
    paymentButton.click();
  });
</script>
