
--
-- Table structure for table `#__larts_subs_cart_items`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_cart_items` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `date` timestamp NOT NULL DEFAULT current_timestamp() ON UPDATE current_timestamp(),
  `ordercode` int(11) NOT NULL,
  `product_id` int(11) NOT NULL,
  `coupon_code` varchar(15) NULL DEFAULT NULL,
  `licence` TINYTEXT NULL,
  `price` decimal(10,2) NOT NULL DEFAULT 0.00 COMMENT 'Item price',
  `quantity` int(11) NOT NULL DEFAULT 1 COMMENT 'Number of items ',
  `discount` decimal(10,2) NOT NULL DEFAULT 0.00,
  `discount_group` varchar(15) NULL DEFAULT NULL,
  `discount_type` varchar(15) NULL DEFAULT NULL,
  `discount_amount` int(11) NOT NULL DEFAULT 0 COMMENT ' Heigt of amount in percent or price',
  `period_id` int(11) DEFAULT 0,
  `vat` decimal(10,2) NOT NULL DEFAULT 0.00,
  `vat_percent` decimal(10,2) NOT NULL DEFAULT 0.00,
  PRIMARY KEY (`id`),
  KEY `ordercode` (`ordercode`,`product_id`),
  KEY `product_id` (`product_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_categories`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_categories` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `cattype` tinyint(4) NOT NULL COMMENT '0: Product\r\n1: Subscription',
  `name` varchar(100) NOT NULL,
  `alias` TINYTEXT NOT NULL,
  `image` varchar(250) NULL DEFAULT NULL,
  `hash` char(32) NULL DEFAULT NULL,
  `parentid` int(11) NULL DEFAULT NULL,
  `teaser` tinytext NULL, 
  `description` text NULL DEFAULT NULL,
  `ordering` int(11) NOT NULL DEFAULT 0,
  `published` tinyint(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_countries`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_countries` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `country` varchar(64) CHARACTER SET latin1 NULL DEFAULT NULL,
  `country_3_code` char(3) CHARACTER SET latin1 NULL DEFAULT NULL,
  `country_2_code` char(2) CHARACTER SET latin1 NULL DEFAULT NULL,
  `requires_vat` tinyint(1) NOT NULL DEFAULT 0,
  `vatrate_std` float NOT NULL DEFAULT 0,
  `force_vat_payment` int(11) NOT NULL DEFAULT 0,
  `published` tinyint(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`id`),
  KEY `country_id` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT='Country records';

--
-- Table structure for table `#__larts_subs_coupons`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_coupons` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `limit` int(11) NULL DEFAULT 1,
  `coupon_code` varchar(15) NULL DEFAULT NULL,
  `name` varchar(100) NOT NULL,
  `valid_from` date NULL DEFAULT NULL,
  `valid_to` date NULL DEFAULT NULL,
  `created_by` int(11) DEFAULT 0,
  `created` timestamp NOT NULL DEFAULT current_timestamp() ON UPDATE current_timestamp(),
  `discount` int(11) unsigned DEFAULT 0,
  `discount_type` varchar(15) DEFAULT 0,
  `used` int(11) NULL DEFAULT 0,
  `type` tinyint(1) NOT NULL DEFAULT 0,
  `full_order` tinyint(1) NULL DEFAULT 0 COMMENT '1=Yes 0=No',
  `product_ids` text NULL DEFAULT NULL COMMENT 'Assignment to specific products',
  `userid` int(11) NULL DEFAULT NULL COMMENT 'Assignment to a specific user',
  `published` tinyint(1) NULL DEFAULT 1,
  PRIMARY KEY (`id`),
  KEY `coupon_code` (`coupon_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_croninfo`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_croninfo` (
  `id` int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `name` varchar(50) NOT NULL,
  `period` int(11) NOT NULL DEFAULT 1440 COMMENT '[Minutes]',
  `period_min` int(11) NOT NULL DEFAULT 5 COMMENT 'Minimum period size',
  `lastrun` datetime NULL DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB CHARSET=utf8 DEFAULT COLLATE=utf8_unicode_ci;

--
-- Table structure for table `#__larts_subs_customers`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_customers` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `userid` int(11) NOT NULL,
  `firstname` varchar(100) NOT NULL,
  `lastname` varchar(100) NOT NULL,
  `address1` varchar(100) NOT NULL,
  `address2` varchar(100) NOT NULL,
  `zipcode` varchar(100) NOT NULL,
  `city` varchar(100) NOT NULL,
  `country_code` char(2) NOT NULL,
  `phone` varchar(100) NOT NULL,
  `is_business` tinyint(1) NOT NULL,
  `companyname` varchar(100) NOT NULL,
  `vat_number` varchar(20) NULL DEFAULT NULL,
  `vat_validation_result` tinyint(1) DEFAULT 0 COMMENT '0 = Invalid 1 = Valid',
  `vat_validation_date` datetime NULL DEFAULT NULL,
  `newsletter` tinyint(1) DEFAULT 1,
  `published` tinyint(1) DEFAULT 1,
  PRIMARY KEY (`id`),
  KEY `firstname` (`firstname`),
  KEY `lastname` (`lastname`),
  KEY `id` (`id`),
  KEY `userid` (`userid`),
  KEY `user_country` (`userid`,`country_code`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_email_queue`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_email_queue` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `batch_id` varchar(50) NOT NULL,
  `status` tinyint(1) DEFAULT 0,
  `pending_till` date NULL DEFAULT NULL,
  `date_sent` date NULL DEFAULT NULL,
  `userid` int(11) DEFAULT 0,
  `subject` varchar(255) NOT NULL,
  `body` text NOT NULL,
  `from_email` varchar(255) NOT NULL,
  `from_name` varchar(255) NOT NULL,
  `reply_to_email` varchar(255) NOT NULL,
  `reply_to_name` varchar(255) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_email_templates`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_email_templates` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `template` varchar(50) NOT NULL,
  `subject` varchar(255) NULL DEFAULT NULL,
  `body` text NULL DEFAULT NULL,
  `from_email` varchar(255) NULL DEFAULT NULL,
  `from_name` varchar(255) NULL DEFAULT NULL,
  `reply_to_email` varchar(255) NULL DEFAULT NULL,
  `reply_to_name` varchar(255) NULL DEFAULT NULL,
  `receive_bcc` tinyint(1) NULL DEFAULT NULL,
  `published` tinyint(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_files`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_files` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `filename` varchar(100) NULL DEFAULT NULL,
  `filename_physical` varchar(100) NULL DEFAULT NULL,
  `filename_orig` VARCHAR(40) NULL DEFAULT NULL , 
  `hash` CHAR(32) NULL DEFAULT NULL , 
  `keep_original_filename` TINYINT(1) DEFAULT 0 , 
  `download_url` varchar(255) NULL DEFAULT NULL,
  `download_link` varchar(255) NULL DEFAULT NULL,
  `source_file_type` int(1) NOT NULL,
  `use_filename` varchar(255) NULL DEFAULT NULL,
  `use_own_filename` tinyint(1) NULL DEFAULT NULL,
  `alias` varchar(100) NOT NULL,
  `extension` varchar(100) NULL DEFAULT NULL,
  `created` datetime NULL DEFAULT NULL,
  `ispublic` TINYINT NOT NULL DEFAULT 0,
  `ordering` int(11) DEFAULT 0,
  `version` varchar(10) NULL DEFAULT NULL,
  `append_version` tinyint NULL DEFAULT 0,
  `installnotes` text NULL DEFAULT NULL,
  `releasenotes` text NULL DEFAULT NULL,
  `licence` TINYTEXT NULL,
  `demo_link` varchar(255) NULL DEFAULT NULL,
  `review_link` varchar(255) NULL DEFAULT NULL,
  `show_review_link` tinyint(1) NULL DEFAULT NULL,
  `views` int(11) NULL DEFAULT NULL,
  `downloads` int(11) DEFAULT 0,
  `sha256_checksum` varchar(255) NULL DEFAULT NULL,
  `download_type` varchar(15) NULL DEFAULT NULL,
  `release_date` date NULL DEFAULT NULL,
  `published` tinyint(1) DEFAULT 1,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_file_categories`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_file_categories` (
  `file_id` int(11) NOT NULL,
  `category_id` int(11) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_invoices`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_invoices` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `invoice_number` varchar(50) NOT NULL DEFAULT '0',
  `userid` int(11) NOT NULL,
  `ordercode` int(11) NOT NULL,
  `invoicedate` timestamp NOT NULL DEFAULT current_timestamp(),
  `vat_number` varchar(20) NOT NULL,
  `gross_price` double NOT NULL COMMENT 'price before discount and vat',
  `net_price` double NOT NULL COMMENT 'price after discount and vat',
  `disco_price` double NOT NULL,
  `vat_total` double NOT NULL,
  `vat_percentage` tinytext NULL,
  `paid` tinyint(1) NULL DEFAULT '0' COMMENT '0: TYPE_DEBIT, 1: TYPE_CREDIT',
  `provider` varchar(50) NOT NULL,
  `transaction_id` varchar(255) NULL DEFAULT NULL COMMENT 'PayPal-Token or ordercode',
  `invoice_type` tinyint(1) NULL DEFAULT '0' COMMENT '0: UNPAID, 1: PAID',
  `refund_invoice` int(11) NULL DEFAULT NULL,
  `refund_reason` text CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NULL DEFAULT NULL,
  `refund_date` datetime NULL DEFAULT NULL,
  `published` tinyint(1) NULL DEFAULT '0' COMMENT '0: not sent, 1: sent',
  `dlkey` char(32) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NULL DEFAULT NULL COMMENT 'Download key = md5(invoice-number)',
  PRIMARY KEY (`id`),
  KEY `userid` (`userid`),
  KEY `user_order` (`userid`,`ordercode`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

CREATE TABLE IF NOT EXISTS `#__larts_subs_logging` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `type` int(11) DEFAULT 1 COMMENT '1 = file and 2 is invoice',
  `request_type` tinyint(1) NOT NULL DEFAULT 0 COMMENT '0 = website and 1 = Joomla Updater',
  `userid` int(11) NULL DEFAULT NULL,
  `country_code` varchar(3) NULL DEFAULT NULL,
  `ip_address` varchar(25) NULL DEFAULT NULL,
  `action` varchar(255) NULL DEFAULT NULL,
  `date` timestamp NOT NULL DEFAULT current_timestamp() ON UPDATE current_timestamp(),
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_orders`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_orders` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `status` SMALLINT(1) NOT NULL DEFAULT 0 COMMENT 'Bitfield',
  `userid` int(11) NOT NULL,
  `ordercode` int(11) NOT NULL,
  `amounts` varchar(255) NOT NULL,
  `params` varchar(255) NULL DEFAULT NULL COMMENT 'Params can be used by custom integrations and the paylater plugin for status updates',
  `vat_number` varchar(20) NULL DEFAULT NULL,
  `country_code` char(2) NULL DEFAULT NULL COMMENT 'Country the VAT is applied.',
  `ip_address` varchar(40) NULL DEFAULT NULL,
  `is_offer` tinyint(1) NOT NULL DEFAULT 0 COMMENT '0: order 1: offer',
  `created` timestamp NOT NULL DEFAULT current_timestamp(),
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_periods`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_periods` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `ordercode` int(11) NOT NULL,
  `product_id` int(11) NOT NULL,
  `product_id_ref` INT NOT NULL,
  `userid` int(11) NOT NULL,
  `licence` tinytext NULL,
  `valid_from` datetime NULL DEFAULT NULL,
  `valid_to` datetime NULL DEFAULT NULL,
  `access` int(10) NULL DEFAULT NULL COMMENT 'Joomla ACL access',
  `user_info` tinytext NULL,
  `internal_info` tinytext NULL,
  `status` tinyint(1) NOT NULL DEFAULT 1 COMMENT '0: disabled\r\n1: active\r\n2: refunded\r\n3: canceled',
  `mail_changes` tinyint(1) NOT NULL DEFAULT 1,
  `created` timestamp NOT NULL DEFAULT current_timestamp(),
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_products`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_products` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `type` int(11) NOT NULL,
  `source_id` int(11) NULL DEFAULT NULL,
  `product_ref` int(11) NOT NULL DEFAULT 0 COMMENT 'The product to calculate the period',
  `name` varchar(100) NOT NULL,
  `alias` varchar(255) NOT NULL,
  `teaser` varchar(255) NULL DEFAULT NULL,
  `content_url` varchar(255) NULL DEFAULT NULL,
  `extra_url` varchar(255) NULL DEFAULT NULL,
  `downloads_menu_item` int(11) NULL DEFAULT NULL,
  `short_description` text NULL DEFAULT NULL,
  `description` text NULL DEFAULT NULL,
  `image` varchar(250) NULL DEFAULT NULL,
  `created` datetime NOT NULL DEFAULT current_timestamp(),
  `updated` timestamp NOT NULL DEFAULT current_timestamp(),
  `ispublic` tinyint(1) DEFAULT 1,
  `access` tinyint(1) DEFAULT 0,
  `exclude_reminders` tinyint(1) DEFAULT 0,
  `exclude_renewals` tinyint(1) DEFAULT 0,
  `period` int(11) DEFAULT 0,
  `period_type` varchar(20) NULL DEFAULT NULL,
  `price` double DEFAULT 0,
  `vat_class` tinyint(4) NOT NULL DEFAULT 0 COMMENT '0: No VAT\r\n1: Standard VAT\r\n2-x: VAT class',
  `special_offer` tinyint(1) DEFAULT 0,
  `recurring` tinyint(1) DEFAULT 0,
  `link_demo` varchar(255) NULL DEFAULT NULL,
  `link_to_review` varchar(255) NULL DEFAULT NULL,
  `link_to_support` varchar(255) NULL DEFAULT NULL,
  `pkgmanager_id` int(11) DEFAULT 0,
  `has_licence` tinyint(1) DEFAULT 0,
  `custom_buttons` text NULL DEFAULT NULL,
  `metakey` text NULL DEFAULT NULL,
  `metatitle` varchar(255) NULL DEFAULT NULL,
  `metadesc` text NULL DEFAULT NULL,
  `ordering` int(11) DEFAULT 0,
  `published` tinyint(1) DEFAULT 1,
  PRIMARY KEY (`id`),
  KEY `name` (`name`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_products_bundled`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_products_bundled` (
  `product_id` int(11) NOT NULL,
  `bundled_product_id` int(11) NOT NULL,
  PRIMARY KEY (`product_id`,`bundled_product_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_products_related`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_products_related` (
  `product_id` int(11) NOT NULL,
  `related_product_id` int(11) NOT NULL,
  PRIMARY KEY (`product_id`,`related_product_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_product_categories`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_product_categories` (
  `product_id` int(11) NOT NULL,
  `category_id` int(11) NOT NULL,
  PRIMARY KEY (`product_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_recurring`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_recurring` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `userid` int(11) NULL DEFAULT NULL COMMENT 'The userid who made the transaction',
  `name` int(255) NULL DEFAULT NULL COMMENT 'The name value for this record. EG: MollieSubscription',
  `value` int(255) NULL DEFAULT NULL COMMENT 'The value which belongs to the name so we have a key pair combination',
  `product_id` int(11) NULL DEFAULT NULL COMMENT 'The product_id related to this name-value pair if required',
  `added` timestamp NOT NULL DEFAULT current_timestamp() COMMENT 'The time in seconds when it was dadded to the database.',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_statistics`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_statistics` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `date_time` date NOT NULL,
  `week` int(5) NOT NULL,
  `month` int(3) NOT NULL,
  `year` int(5) NOT NULL,
  `subscription_reminders` int(11) NOT NULL,
  `coupons_disabled` int(11) NOT NULL,
  `subscriptions_disabled` int(11) NOT NULL,
  `total_earnings` float NOT NULL,
  `total_vat` float NOT NULL,
  `total_refunds` float NOT NULL,
  `products_sold` int(11) NOT NULL,
  `downloads_web` int(11) NOT NULL,
  `downloads_updater` int(11) NOT NULL,
  `total_downloads` int(11) NOT NULL,
  `active_subscriptions` int(11) NOT NULL,
  `total_lifetime_subscriptions` int(11) NOT NULL,
  `total_onetime_subscriptions` int(11) NOT NULL,
  `total_clicks_orderbutton` int(11) NOT NULL,
  `sales_conversion` float NOT NULL,
  `coupons_used` int(11) NOT NULL,
  `unique_clicks_orderbutton` int(11) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_templates`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_templates` (
  `id` int(10) NOT NULL AUTO_INCREMENT,
  `alias` varchar(255) NOT NULL,
  `type` varchar(15) NULL DEFAULT NULL,
  `description` varchar(255) NULL DEFAULT NULL,
  `subject` varchar(255) NULL DEFAULT NULL,
  `body` text NULL DEFAULT NULL,
  `secured` tinyint(1) NULL DEFAULT NULL,
  `userid` int(10) NULL DEFAULT NULL,
  `lastchange` timestamp NOT NULL DEFAULT current_timestamp(),
  `from_email` varchar(255) NULL DEFAULT NULL,
  `from_name` varchar(255) NULL DEFAULT NULL,
  `receive_bcc` tinyint(1) NULL DEFAULT 0,
  `bcc1_name` varchar(150) NULL DEFAULT NULL,
  `bcc1_email` varchar(150) NULL DEFAULT NULL,
  `bcc2_name` varchar(150) NULL DEFAULT NULL,
  `bcc2_email` varchar(150) NULL DEFAULT NULL,
  `reply_to_email` varchar(255) NULL DEFAULT NULL,
  `reply_to_name` varchar(255) NULL DEFAULT NULL,
  `language` varchar(7) NULL DEFAULT NULL,
  `send_to` varchar(25) NULL DEFAULT NULL,
  `published` tinyint(1) DEFAULT 1,
  PRIMARY KEY (`id`),
  KEY `alias` (`alias`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_tickets`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_tickets` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `userid` int(11) NULL DEFAULT NULL,
  `ordercode` int(11) NOT NULL,
  `product_id` int(11) NOT NULL,
  `ticket_number` varchar(20) NULL DEFAULT NULL,
  `date` datetime NULL DEFAULT NULL,
  `published` tinyint(1) DEFAULT 1,
  `dlkey` char(32) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_transactions`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_transactions` (
  `id` int(10) NOT NULL AUTO_INCREMENT,
  `status` tinyint(1) DEFAULT 0 COMMENT '0: pending\r\n1: processed',
  `transaction_token` varchar(64) NULL DEFAULT NULL,
  `transaction_id` varchar(64) NULL DEFAULT NULL,
  `userid` int(10) NOT NULL,
  `details` text NULL DEFAULT NULL,
  `amount` decimal(10,2) NOT NULL DEFAULT 0.00,
  `date` timestamp NOT NULL DEFAULT current_timestamp(),
  `type` varchar(50) NULL DEFAULT NULL,
  `email_paypal` varchar(150) NULL DEFAULT NULL,
  `ordercode` int(11) NOT NULL,
  `plugin` varchar(255) NOT NULL,
  `comment` text NULL DEFAULT NULL,
  `deleted` tinyint(1) DEFAULT 0,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_vat_classes`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_vat_classes` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(50) NOT NULL,
  `ordering` int(11) NOT NULL,
  `published` int(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

--
-- Table structure for table `#__larts_subs_vat_rates`
--

CREATE TABLE IF NOT EXISTS `#__larts_subs_vat_rates` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `country_code` char(2) NOT NULL,
  `type` int(11) NOT NULL COMMENT '0: Standard, 1:Reduced',
  `category` varchar(50) NULL DEFAULT NULL,
  `category_desc` text NULL DEFAULT NULL,
  `rate` float NOT NULL,
  `rate_desc` text NULL DEFAULT NULL,
  `published` tinyint(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;



