<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\VAT\VatRates;

class CountriesController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		## Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'countries');

		return parent::display();
	}

	function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'countries');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		## Get the task (publish/upnpublish)
		if ($this->getTask() == 'publish')
		{
			$publish = 1;
		}
		else
		{
			$publish = 0;
		}

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SELECT_ITEM'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
		}

		$model = $this->getModel('countries');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_COUNTRY'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
		}

		if ($this->getTask() == 'publish')
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUNTRY_PUBLISHED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUNTRY_UNPUBLISHED'));
		}
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
	}

	function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('countries');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_COUNTRY'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUNTRIES_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
	}

	function save()
	{
		$post = Factory::getApplication()->GetInput()->post->getArray();

    // ensure vat rates are floats
    $post['vatrate_std'] = empty($post['vatrate_std']) ? 0 : $post['vatrate_std'];
    $keys = array_keys($post);
    foreach ($keys as $key => $val) {
      if(str_starts_with($val, 'rateclass_')) {
        $post[$val] = empty($post[$val]) ? 0 : $post[$val];
      }
    }
    
    $model = $this->getModel('countries');

		if ($id = $model->store($post))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUNTRY_SAVED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUNTRY_SAVE_FAILURE'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=countries';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=countries.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}

  public function getAjaxVATSel()
  {
    $input = Factory::getApplication()->GetInput();
    $country = $input->get('cid');
    $col = $input->get('col');
    $result = VatRates::loadByCountry($country);
    
    
    if(empty($result)) {
      $body = '<h3>No result</h3>';
    }
    else {
      $body = '';
      $body .= '<div>';
      $body .= '<table class="table table-hover">';
      $body .= '  <thead>';
      $body .= '    <th class="center" width="15px">Sel</th>';
      $body .= '    <th class="left" width="80px">Type</th>';
      $body .= '    <th class="left" width="80px">rate</th>';
      $body .= '    <th class="left" width="20%">category</th>';
      $body .= '    <th class="left" width="20%">category_desc</th>';
      $body .= '    <th class="left">rate_desc</th>';
      $body .= '  </thead>';
      $body .= '  <tbody>';
      
      $body .= '  <tr>';
      $body .= '    <td class="center">'
                    . '<a class="btn btn-micro tbody-icon active hasTooltip" href="javascript:void(0);" onclick="onVATSel(\'0\')">'
                    . '<i class="fas fa-check green" aria-hidden="true"></i>'
                    . '</a>'
                  . '</td>';
      $body .= '    <td class="left">'.Text::_('JCLEAR').'</td>';
      $body .= '    <td class="left" colspan="4"></td>';
      $body .= '  </tr>';

        
      foreach($result as $item) {
        $type = ($item->type == 0) ? 'Standard' : 'Reduced';
        $body .= '  <tr>';
        $body .= '    <td class="center">'
                      . '<a class="btn btn-micro tbody-icon active hasTooltip" href="javascript:void(0);" onclick="onVATSel(\''.$item->rate.'\')">'
                      . '<i class="fas fa-check green" aria-hidden="true"></i>'
                      . '</a>'
                    . '</td>';
        $body .= '    <td class="left">'.$type.'</td>';
        $body .= '    <td class="left">'.$item->rate.'</td>';
        $body .= '    <td class="left">'.$item->category.'</td>';
        $body .= '    <td class="left">'.$item->category_desc.'</td>';
        $body .= '    <td class="left">'.$item->rate_desc.'</td>';
        $body .= '  </tr>';
      }
      $body .= '  </tbody>';
      $body .= '</table';
      $body .= '</div>';
    }
    
    $action = 'index.php?option=com_lartssubscriptions&task=countries.onSelVAT&cid='.$country;
    $html = '<div class="modal-header">'
          .'  <h3 class="modal-title">'.Text::_('COM_LARTSSUBS_SELECT_VAT').'</h3>'
          .'  <button type="button" class="btn-close" data-bs-dismiss="modal" aria-hidden="true"></button>'
          .' </div>'
          .' <div class="modal-body" style="max-height: calc(100vh - 210px); overflow-y: auto;">'
          .'   <div class="container">'
          . '    <form id="form-selVAT" action="'.$action.'" method="POST">' .$body
          . '    <input name="col" type="hidden" value="'.$col.'" />'
          . '    </form>'
          . '  </div>'
          .' </div>'
          .' <div class="modal-footer">'
          .'  <button type="button" class="btn btn-primary" data-bs-dismiss="modal">Close</button>'
          .' </div>';
    echo $html;
    exit;
  }

  public function onSelVAT()
  {
    $input = Factory::getApplication()->GetInput();
    $country = $input->get('cid');
    $VATrate = $input->get('rate', 0);
    $rateClassCol = $input->get('col');
    
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
                ->where($db->quoteName('country_2_code') .'='. $db->quote($country));

    if($rateClassCol == 0) {
      // update countries.vatrate_std
      $query->update($db->quoteName('#__larts_subs_countries'))
            ->set($db->quoteName('vatrate_std'). '=' .$db->quote($VATrate));
    }
    else {
      // update countries.rateclass_$rateClassCol
      $query->update($db->quoteName('#__larts_subs_countries'))
            ->set($db->quoteName('rateclass_'.$rateClassCol). '=' .$db->quote($VATrate));
    }
		$db->setQuery($query);
    $sql = $query->__toString();
		$db->execute();
    
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
  }
}


						