<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;

class CouponsController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		## Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function cancel()
	{
    // reset user state
    Factory::getApplication()->setUserState('com_lartssubscriptions.coupon', null);
  	$this->display();
  }

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'coupons');

		return parent::display();
	}

	public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'coupons');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	public function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		// Get the task (publish/upnpublish)
		$publish = ($this->getTask() == 'publish') ? 1 : 0;

		if (count($cid) < 1) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_NO_CATEGORY_SELECTED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=coupons');
		}

		$model = $this->getModel('coupons');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_COUPON'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=coupons');
		}

		if ($this->getTask() == 'publish') {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_PUBLISHED'));
		}
		else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_UNPUBLISHED'));
		}

		$this->setRedirect('index.php?option=com_lartssubscriptions&view=coupons');
	}

	public function duplicate()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('coupons');

		foreach ($cid as $item)
		{
			$model->copyCoupon($item);
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPONS_COPIED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=coupons');
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('coupons');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_COUPON'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=coupons');
      return false;
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=coupons');
	}

	public function save()
	{
    $post = Factory::getApplication()->GetInput()->post->getArray();
    Factory::getApplication()->setUserState('com_lartssubscriptions.coupon', $post);

		// Check on product_ids - otherwise you can never empty the value.
		$post['product_ids'] = isset($post['product_ids']) ? $post['product_ids']: '';
    $post['created_by'] = Factory::getApplication()->getIdentity()->id;
    $post['valid_from'] = empty($post['valid_from']) ? null : $post['valid_from'];
    $post['valid_to'] = empty($post['valid_to']) ? null : $post['valid_to'];
    $post['userid'] = empty($post['userid']) ? 0 : $post['userid'];
    $post['limit'] = empty($post['limit']) ? 0 : $post['limit'];
    
    $id = $post['id'];
    
    $bError = false;
    if(empty($post['coupon_code'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_MISSING_CODE'), 'error');
      $bError = true;
    }
    // check discount
    if((empty($post['discount'])) ||
       ((int)$post['discount'] <= 0) || 
       ($post['discount_type'] == 'percent' && (int)$post['discount'] > 100))
    {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_INVALID_DISCOUNT'), 'error');
      $bError = true;
    }

    if(!$bError) {
      $model = $this->getModel('coupons');

      if ($id = $model->store($post)) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_SAVED'));
        
        // reset user state
        Factory::getApplication()->setUserState('com_lartssubscriptions.coupon', null);
      }
      else {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COUPON_SAVE_FAILURE'), 'error');
        $bError = true;
      }
    }

		$redirect = 'index.php?option=com_lartssubscriptions&view=coupons';
		if ($bError || $this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=coupons.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}
}
