<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

//use LartsSubscriptions\Base\Portabillity;
// use LartsSubscriptions\Base\Gdpr;
use LartsSubscriptions\Message;

class CustomersController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		// Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	// This function will display if there is no choice.
	function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'customers');

		return parent::display();
	}

	function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'customers');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}


	/**
	 * Selecting a country code by the country id.
	 */
	private function getCountryById($id)
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true);

		$query->select(['country_2_code'])
			->from($db->quoteName('#__larts_subs_countries'))
			->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);

		$result = $db->loadObject();

		return ! empty($result->country_2_code) ? $result->country_2_code : null;
	}

	function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('customers');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_USER'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=customers');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUCCESS_DELETE_USER'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=customers');
	}

	function save()
	{
		// Obtain post data
		$post = Factory::getApplication()->GetInput()->post->getArray();
    
    // precheck input
    $bErr = false;
    if(empty($post['userid'])) {
      $bErr = true;
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_REQUIRE_JUSER'), 'error');
    }
    if(empty($post['firstname'])) {
      $bErr = true;
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_REQUIRE_FIRSTNAME'), 'error');
    }
    if(empty($post['lastname'])) {
      $bErr = true;
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_REQUIRE_LASTNAME'), 'error');
    }
    
    if($bErr) {
			$this->setRedirect('index.php?option=com_lartssubscriptions&task=customers.edit');
			return false;
    }

		$model = $this->getModel('customers');

		// New user Checks
		if (empty($post['id']))
		{
			if ($model->checkOnDoubleAccounts($post['userid']))
			{
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_EXSISTS_IN_DB'), 'error');
				$this->setRedirect('index.php?option=com_lartssubscriptions&task=customers.edit');

				return false;
			}

			// Unique key for this user.
			//$post['watchful_key'] = md5($post['userid']);
		}

    $redirect = 'index.php?option=com_lartssubscriptions&view=customers';
		if ($id = $model->store($post)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_SAVED'));
		}
		else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_SAVE_FAILED'), 'error');
      $redirect = 'index.php?option=com_lartssubscriptions&task=customers.edit';
		}

		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . $id;
		}

		$this->setRedirect($redirect);
	}


}