<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Larts\Component\LartsSubscriptions\Administrator\Traits\ControllerRegisterTasksTrait;
use Larts\Component\LartsSubscriptions\Administrator\Traits\ControllerReusableModelsTrait;

use Larts\Component\LartsSubscriptions\Administrator\Model\ControlpanelModel;
use LartsSubscriptions\Base\UpdateKey;
use LartsSubscriptions\Config as LartsConfig;


class DashboardController extends BaseController
{
	use ControllerRegisterTasksTrait;
  use ControllerReusableModelsTrait;

    
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);
    
 		$this->registerControllerTasks('main');

	}

  	public function main($cachable = false, $urlparams = [])
	{
		/** @var DashboardModel $model */
		$model = $this->getModel('Dashboard', 'Administrator');
  
		return parent::display($cachable, $urlparams);
  }
  
	/**
	 * Display functionality for the control panel.
	 */
	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'dashboard');

		return parent::display();
	}

	/**
	 * Updating the keys when the updater key has been entered.
	 */
	public function setUpdateKey()
	{
		$jinput = Factory::getApplication()->GetInput();

		$post = Factory::getApplication()->GetInput()->post->getArray();

		if ($jinput->get('key', '', 'string') == '')
		{
			Factory::getApplication()->enqueueMessage(Text::_('JNO_KEY_PROVIDED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=dashboard');

			return true;
		}

		// set key to j-db
    $url = null;
		$update = new UpdateKey;
    $bError = !$update->updateOrAddDownloadId('com_lartssubscriptions', $post['key'], $url, 'extension');

		// Updating the configuration
		$bError |= !LartsConfig::save('update_key', $post['key']);

    if($bError) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
		}
    else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_UPDATE_KEY_SET'), 'success');
    }

		// Done. Return to the control panel
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=dashboard');
	}

}
