<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Filter\OutputFilter;

use LartsSubscriptions\Download;
use LartsSubscriptions\Base\Image;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Files;


class FilesController extends BaseController
{
  public $hash = null;
  
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();

		$jinput->set('layout', 'default');
		$jinput->set('view', 'files');
    $vName   = $this->input->get('view', 'files');


		return parent::display();
	}

	public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();

		$jinput->set('layout', 'form');
		$jinput->set('view', 'files');
		$jinput->set('hidemainmenu', '1');

		parent::display();
	}

	/**
	 * Download a file from the backend.
	 *
	 * @since 2.5.0
	 */
	public function download()
	{
		$download = new Download;
		$file_id  = $this->input->get('file', '', 'int');
    $this->hash = md5(time());

		$fileinfo = $download->fileInformation($file_id);

//    if($fileinfo->source_file_type != Files::SOURCE_TYPE_LOCAL) {
//  		$filepath  = $download->getFilePath();
//  		$download  = OutputFilter::stringURLSafe($fileinfo->filename_physical);
//    }
//    else if($fileinfo->source_file_type == Files::SOURCE_TYPE_EXTERN) {
//      
//    }
    
		// Path to the file depends on the location of the file (local or remote)
    $filepath  = $download->getFilePath();
    $filename  = $fileinfo->filename_physical;
    $file = $filepath . $fileinfo->filename_physical;
		$extension = $fileinfo->extension;

    //$filepath  = $download->getFilePath();
		//$download  = OutputFilter::stringURLSafe($fileinfo->filename);
		//$extension = pathinfo($filepath . $fileinfo->filename_physical, PATHINFO_EXTENSION);
		//$filename  = $download . '.' . $extension;

		$filename  = $fileinfo->filename_orig;

    if ($fileinfo->use_own_filename && !empty($fileinfo->use_filename)) {
			$filename = $fileinfo->use_filename;
		}

		// Push the download to the browser.
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/octet-stream");
		header("Content-Disposition: attachment; filename=\"" . $filename . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($file));
		ob_end_flush();
		@readfile($file);
		die;
	}

	public function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$publish = ($this->getTask() == 'publish') ? 1 : 0;

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SELECT_ITEM'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=files');
		}

		$model = $this->getModel('files');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_FILE'), 'error');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FILES_PUBLISHED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=files');
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('files');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_FILE'), 'error');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FILES_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=files');
	}

	public function prepopulatefile()
	{
		// Get the ID of the send checkbox:
		$cid = $this->input->get('cid', [], 'array');

		// Make the cid safe from here:
		$file_to_copy = intval($cid['0']);

		// Load file data:
		$model = $this->getModel('files');
		$file = $model->getFileInformation($file_to_copy);

		// Get date:
		$now = new Date;

		$file['published'] = 0;
		$file['created'] = $now->toSql();

		// unset some data, as we want to start a new file:
		unset($file['id'], $file['filename_physical'], $file['downloads']);

		// Get the file categories:
		$categories = Files::getCategories($file_to_copy);
		$categories = ArrayHelper::toInteger($categories);

		// Store the file information:
		if ($model->store($file, $categories))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COPIED_FILE_SAVED'), 'message');
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COPIED_FILE_SAVED_FAILED'), 'error');
		}

		// Redirect to the new created file toedit the file:filename_physical
		$link = 'index.php?option=com_lartssubscriptions&task=files.edit&cid[]=' . $model->getFileID();
		$this->setRedirect($link);
	}

	public function save()
	{
		$id = $this->input->getInt('id');
		$post    = Factory::getApplication()->GetInput()->post->getArray();

		$post['installnotes'] = $this->input->get('installnotes', '', 'raw');
		$post['releasenotes'] = $this->input->get('releasenotes', '', 'raw');

    if(empty($post['filename'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FILE_NAME_REQUIRED'), 'error');
  		$redirect = 'index.php?option=com_lartssubscriptions&task=files.edit&cid[]=' . $id;
  		Factory::getApplication()->redirect($redirect);
    }
    
		if ($this->input->getCmd('alias') == '')
    {
			$post['alias'] = OutputFilter::stringURLSafe($this->input->getString('filename'));
		}

		if(!$id) {
			// Only add create date when new added.
			$date = new Date;
			$post['created'] = $date->toSql();
		}

		// Removing the image as it is requested.
		if(isset($post['remove_image']))
		{
			if(!Image::remove($post['id'], 'file'))
			{
				// Only throws a message because this public function is not critical for the rest of this page
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_REMOVE_PRODUCT_IMAGE'), 'error');
			}
		}

		$categories = $this->input->get('categories', [], 'array');
		$categories = ArrayHelper::toInteger($categories);

		$products = $this->input->get('product', [], 'array');
		$products = ArrayHelper::toInteger($products);

		$model = $this->getModel('files');

		if(!$model->aliasCheck($post['alias'], $id)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_DUPLICATED_ALIAS_DETECTED'), 'warning');
		}

		if ($model->store($post, $categories, $products)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FILE_SAVED'), 'message');
		}
		else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FILE_SAVED_FAILED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=files';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=files.edit&cid[]=' . $model->getFileID();
		}

		$this->setRedirect($redirect);
	}

	public function saveOrderAjax()
	{
		// Get the input
		$pks   = $this->input->post->get('cid', [], 'array');
		$order = $this->input->post->get('order', [], 'array');

		// Sanitize the input
		$pks   = ArrayHelper::toInteger($pks);
		$order = ArrayHelper::toInteger($order);

		// Get the model
		$model = $this->getModel('files');

		// Save the ordering
		$return = $model->saveorder($pks, $order);
		if ($return) {
			echo "1";
		}

		// Close the application
		Factory::getApplication()->close();
	}

}
