<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\AdminController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\GeoIP\GeoIP;

class GeoipController extends AdminController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

  }

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();

		$jinput->set('layout', 'default');
		$jinput->set('view', 'geoip');

		return parent::display();
	}

  public function config()
	{
		$jinput = Factory::getApplication()->GetInput();

		$jinput->set('layout', 'default');
		$jinput->set('view', 'geoip');

		parent::display();
	}

  public function update()
	{
    $this->saveForm();
    
    $config = LartsConfig::get();

    $key = '';
    if($config->use_geoip == 'maxmind') {
      $key = $config->maxmind_key;
    }
    else if($config->use_geoip == 'ipstack') {
      $key = $config->ipstack_key;
    }
    else {
      $this->setRedirect('index.php?option=com_lartssubscriptions&task=geoip.config');
      $this->redirect();
    }
    
    if(empty($key)) {
   		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_GEO2IP_KEY_REQUIRED'), 'error');
      $this->setRedirect('index.php?option=com_lartssubscriptions&task=geoip.config');
      $this->redirect();
      return;
    }
    
    if($config->use_geoip == 'ipstack') {
   		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_GEOIP_IPSTACK_NO_DB'), 'info');
    }
    else if($config->use_geoip == 'maxmind') {
      $g = new GeoIP;
      if(!$g->UpdateMaxmindDB()) {
        $msg = Text::_('COM_LARTSSUBS_ERROR'). ' '. $g->message;
        Factory::getApplication()->enqueueMessage($msg, 'error');
      }
      else {
        $msg = Text::_('COM_LARTSSUBS_GEOIP_MAXMIND_UPDATED'). ' '. $g->message;
        Factory::getApplication()->enqueueMessage($msg, 'success');
      }

    }

		$this->setRedirect('index.php?option=com_lartssubscriptions&task=geoip.config');
    $this->redirect();
	}

  public function saveForm()
	{
    $formdata = Factory::getApplication()->GetInput()->post->get('jform', array(), 'array');
    $postData = Factory::getApplication()->GetInput();

    LartsConfig::save('maxmind_key', $formdata['maxmind_key']);
    LartsConfig::save('ipstack_key', $formdata['ipstack_key']);
    LartsConfig::save('use_default_country', $formdata['use_default_country']);
    LartsConfig::save('default_country', $formdata['default_country']);
    LartsConfig::save('use_geoip', $formdata['use_geoip']);

 		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CONFIG_SAVED'));
  }
  
  public function apply()
	{
    $key = $this->saveForm();

		$this->setRedirect('index.php?option=com_lartssubscriptions&task=geoip.config');
    $this->redirect();
	}

  public function cancel()
	{
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=countries');
    $this->redirect();
  }
}
