<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;

use LartsSubscriptions\Download;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Export\InvoiceExport;
use LartsSubscriptions\Base\PluginManager;


class InvoicesController extends BaseController
{
	var $id = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		//$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');

		$cid      = $this->input->get('cid', ['0'], 'array');
		$this->id = (int) $cid['0'];
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'invoices');

		return parent::display();
	}

	public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'invoices');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	/**
	 * Exporting the invoice finance report.
	 */
	public function report()
	{
		$invoice = new InvoiceExport;
		if(!$invoice->report()) {
			Factory::getApplication()->enqueueMessage($invoice->message, 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
			return false;
    }

		if(!file_exists($invoice->getFilename()))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CREATED_XLS_FILE_DOES_NOT_EXIST'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
			return false;
		}

		$this->downloadExcelFile($invoice->getFilename());

		return true;
	}

	/**
	 * Exporting the invoice list to a readable Excel File.
	 */
	public function export()
	{
		$invoice = new InvoiceExport;
		$invoice->export();

		if(!file_exists($invoice->getFilename()))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CREATED_XLS_FILE_DOES_NOT_EXIST'), 'error');

			$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');

			return false;
		}

		$this->downloadExcelFile($invoice->getFilename());

		return true;
	}

	/**
	 * Exporting the invoice list to a readable Excel File.
	 */
	private function downloadExcelFile($filename)
	{
		PluginManager::execPlugins('lartssubscriptions', 'onBeforeDownloadInvoicesCSV', $filename);

		header('Pragma: public');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($filename)) . ' GMT');
		header('Cache-Control: private', false);
		header('Content-Type: application/vnd.ms-excel');
		header('Content-Disposition: attachment; filename="' . basename($filename) . '"');
		header('Content-Transfer-Encoding: binary');
		header('Content-Length: ' . filesize($filename));
		header('Connection: close');

		ob_clean();
		flush();

		readfile($filename);
		exit();
	}

	/**
	 * Downloading an invoice in the backend and opening it.
	 */
	public function getinvoice()
	{
		$invoice = new Download;

		// Get invoice information;
		$data = $invoice->downloadInvoice((int) $this->id);

		// Prpearing the new file name which will be popped.
		$newfilename = ! empty($data->invoice_number) ? $data->invoice_number . '.pdf' : 'invoice-' . $data->id . '.pdf';

		// Push to PDF file to the browser.
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/pdf");
		header("Content-Disposition: attachment; filename=\"" . strtolower($newfilename) . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($data->invoice_path . $data->invoice_number . '.pdf'));

		ob_clean();
		flush();

		@readfile($data->invoice_path . $data->invoice_number . '.pdf');
		die;
	}

	// Loading the refund form and show it in screen.
	public function getRefundForm()
	{
		// Getting the ID from the post.
		$id = Factory::getApplication()->GetInput()->get('id', '0', 'int');

		// Getting invoice information for the refund
    $inv = new Invoice;
		$results = $inv->getInvoiceById($id);

		$layout = new FileLayout('refund');
		echo $layout->render(['invoice' => $results]);

		exit();
	}

	// Loading the export form on screen
	public function getExportForm()
	{
		$model = $this->getModel('invoices');
		$form  = $model->getFormExport();

		$layout = new FileLayout('report');
		echo $layout->render(['form' => $form]);

		exit();
	}

	// Refunding and sending an invoice to the customer.
	public function refund()
	{
		$jinput     = Factory::getApplication()->GetInput();
		$send_email = $jinput->getWord('send_email') == 'yes';
		$invoice    = $jinput->getInt('id');
		$reason     = $jinput->get('refund_reason', '0', 'raw');


		$model = $this->getModel('invoices');

		if(!$model->creditInvoice($invoice, $reason, $send_email))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_CREDIT_INVOICES'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
      return;
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CREATED_CREDIT_INVOICE'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
	}

  public function send()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('invoices');

		if(!$model->send($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_SENDING_INVOICES'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVOICES_SENT'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
  }

  public function save()
	{
		$post = Factory::getApplication()->GetInput()->post->getArray();

		$model = $this->getModel('invoices');

		if ($id = $model->store($post))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVOICE_SAVED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVOICE_SAVED_FAILED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=invoices';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=invoices.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('invoices');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_INVOICE'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVOICE_REMOVED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=invoices');
	}
}
