<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\Utilities\ArrayHelper;

use LartsSubscriptions\Coupon;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Ordercode;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\Message;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\GeoIP\GeoIP;
use LartsSubscriptions\VAT\EUTaxClient;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\ThirdParty\LartsLicManager;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Customer;
use LartsSubscriptions\Product;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Offer;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Base\UserRegistration;
use LartsSubscriptions\UserRegState;


class OrderController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		// Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'order');

		// Starting a session.
		$session = Factory::getSession();

		// Get the ordercode if there is one.
		$ordercode = Ordercode::getOrCreateOrdercode();

		return parent::display();
	}

  
  public function show()
	{
 		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'show');
		$jinput->set('view', 'order');

 		parent::display();
  }
  
	public function cancelOrder()
	{
		// Get the global DB session
		$session = Factory::getSession();
    $ordercode = $session->get('ordercode');
    
    $cart = new Cart;
    $cart->remoeAll($ordercode);
    
    $session->clear('ordercode');
    // Reset the userstate
		UserRegState::clearStateData();

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ORDER_CANCELLED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions');
	}

  public function redirect2Order()
  {
    $jinput = Factory::getApplication()->GetInput();
    $userid = $jinput->get('userid', 0);
    $link = 'index.php?option=com_lartssubscriptions&view=order';
    if($userid > 0) {
      $link .= '&userid=' .$userid;
    }
		$this->setRedirect($link);
  }

  public function loadCustomer()
	{
		$jinput = Factory::getApplication()->getInput();
    $userid = $jinput->get('userid', 0);

    // save form data into a registry entry
    $post = $jinput->post->getArray();

    if($userid > 0) {
      $cust = Customer::getCustomerByUserId($userid);

      $post['newuser'] = 0;
      $post['userid'] = $userid;
      $post['username'] = '';
      $post['password'] = '';

      $post['firstname'] = empty($cust) ? '' : $cust->firstname;
      $post['lastname'] = empty($cust) ? '' : $cust->lastname;
      $post['emailaddress'] = empty($cust) ? '' : $cust->email;
      $post['address1'] = empty($cust) ? '' : $cust->address1;
      $post['address2'] = empty($cust) ? '' : $cust->address2;
      $post['zipcode'] = empty($cust) ? '' : $cust->zipcode;
      $post['city'] = empty($cust) ? '' : $cust->city;
      $post['country_code'] = empty($cust) ? '' : $cust->country_code;
    }
		UserRegState::setStateData($post);
    
		$redirect = 'index.php?option=com_lartssubscriptions&view=order&userid=' .$userid;
		$this->setRedirect($redirect);
	}

  public function onSubmit()
  {
    // save form data into a registry entry
    $input = Factory::getApplication()->getInput();
    $post = $input->post->getArray();

		UserRegState::setStateData($post);
 
    if(isset($post['addprod'])) {
      $this->addToCart();
    }
    else if(isset($post['coupon'])) {
      $this->applyCoupon();
    }
    else if(isset($post['setuser'])) {
      $this->loadCustomer();
    }
    else if(isset($post['filter'])) {
      $filter = $this->input->get('product_filter', '');
      $link = 'index.php?option=com_lartssubscriptions&view=order&filter='. $filter;
      $this->setRedirect($link);
      return;
    }

		$redirect = 'index.php?option=com_lartssubscriptions&view=order';
		$this->setRedirect($redirect);
  }
  
	public function addToCart()
	{
    $db = Factory::getDbo();

		// Get the global DB session
		$session = Factory::getSession();
		// Get the ordercode if there is one.
		$ordercode = $session->get('ordercode');

		$cid = $this->input->get('product', [], 'array');
		$prod_ids = ArrayHelper::toInteger($cid);


		$model  = $this->getModel('order');
		$userid = $session->get('order_userid', 0);

		foreach ($prod_ids as $product_id)
		{
      $product = Product::getProduct($product_id);

      $cart = new Cart;
      $cartPproduct = $cart->getProduct($product_id);
      $quantity_in_cart = !empty($cartPproduct) ? $cartPproduct->quantity : 0;
      $quantity_max_count = in_array($product->period_type, ['lifetime','onetime',]) 
              ? 1 : (int) LartsConfig::get('cart_quantity_max_count');

      if ($quantity_in_cart >= $quantity_max_count)
      {
        Factory::getApplication()->enqueueMessage((in_array($product->period_type, ['lifetime', 'onetime',]) 
                ? Text::_('COM_LARTSSUBS_COULD_NOT_ADD_PRODUCT_TWICE_TO_CART') 
                : Text::sprintf('COM_LARTSSUBS_COULD_NOT_ADD_PRODUCT_MORE_TO_CART', $quantity_max_count)), 'notice');
        $app->redirect('index.php?option=com_lartssubscriptions&view=checkout');

  			$this->redirect2Order();
        return false;
      }

      // check if product already in here
      $bRet = false;
      if(!empty($cartPproduct)) {
        $bRet = $cart->updateProductQuantity($product_id, $cartPproduct->quantity + 1);
      }
      else {
        $data = [
          //'countrycode' => ! empty($country_code) ? $country_code : null,
          'id'          => 0,
          'ordercode'   => $ordercode,
          'product_id'  => $product_id,
          'quantity'    => 1,
          'price'       => $product->price,
          'discount'    => 0,
          //'finished'    => 0,
          'licence'     => $licence,
        ];

        $bRet = $cart->addChartItem($data);
      }

      if(!$bRet) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_FAILED_PRODUCT_TO_CART'), 'error');
      }
      else {
        Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_ADDED_PRODUCT_TO_CART', $product_check->name));
      }
      
		}

		$this->redirect2Order();
	}

	public function remove()
	{
		$db  = Factory::getDbo();
		$app = Factory::getApplication();

		// Get the global DB session
		$session = Factory::getSession();
		// Get the ordercode if there is one.
		$ordercode = $session->get('ordercode');

		// Get the id that needs to be removed:
		$jinput = Factory::getApplication()->GetInput();
		$product_id = $jinput->get('pid', '0', 'INT');

		$link = 'index.php?option=com_lartssubscriptions&view=order';

		$model = $this->getModel('order');

		// Perform action:
		if ($model->removeFromCart($ordercode, $product_id))
		{
			$msg = Text::_('COM_LARTSSUBS_ORDER_HAS_BEEN_REMOVED');
			Factory::getApplication()->enqueueMessage($msg, 'notice');
		}
		else
		{
			$msg = Text::_('COM_LARTSSUBS_ERROR_DELETE_ORDER');
			Factory::getApplication()->enqueueMessage($msg, 'error');
		}


		$this->setRedirect($link);
	}

	public function applyCoupon()
	{
    $jinput = Factory::getApplication()->GetInput();

    // Get the id that needs to be removed:
		$coupon_code = $jinput->getString('coupon_code');

		// Get the model to save
		$model = $this->getModel('order');
		if ($model->applyCoupon($coupon_code)) {
			Factory::getApplication()->enqueueMessage($model->getMessage(), 'notice');
		}
		else {
			Factory::getApplication()->enqueueMessage($model->getMessage(), 'error');
		}

  	$this->redirect2Order();
	}

  public function getAjaxLicences()
  {
    LartsLicManager::getAjaxLicences();
  }

  public function completeOffer()
  {
    $this->completeCart(true);
  }
  
  public function completeOrder()
  {
    $this->completeCart(false);
  }

  public function completeCart($bAsOffer)
  {
		$app = Factory::getApplication();
    $jinput = Factory::getApplication()->getInput();

    $userid = $jinput->get('userid');
    $newuser = $jinput->get('newuser');
    if($newuser) {
      $userid = 0;
    }

    
    $session = Factory::getSession();
		$ordercode = $session->get('ordercode');

		$model = $this->getModel('order');
		$order = $model->getOrder();
    

    $formdata = new \stdClass;
    $formdata->username = $jinput->getString('username', '');
    $formdata->password = $jinput->get('password', '', 'raw');
    $formdata->password2 = $jinput->get('password2', '', 'raw');
    $formdata->accepted_tos = true;
    $formdata->firstname = $jinput->getString('firstname', '');
    $formdata->lastname = $jinput->getString('lastname', '');
    $formdata->address1 = $jinput->getString('address1', '');
    $formdata->address2 = $jinput->getString('address2', '');
    $formdata->zipcode = $jinput->getString('zipcode', '');
    $formdata->city = $jinput->getString('city', '');
    $formdata->phone = $jinput->getString('phone', '');
    $formdata->emailaddress = $jinput->getString('emailaddress', '');
    $formdata->country_code = $jinput->get('country_code', 'DE');
    $formdata->isbusiness = $jinput->get('isbusiness', 0);
    $formdata->companyname = $jinput->getString('companyname', '');
    $formdata->vatnumber = $jinput->getString('vatnumber', '');
    $formdata->vatnumber2 = $jinput->getString('vatnumber2', '');
    $formdata->processor = $jinput->getString('payment_method', 'paylater');

    if (count($order) < 1)
		{
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ORDER_HAS_NO_PRODUCTS'), 'error');
			$this->redirect2Order();
			return false;
		}

    $bNeedRegister = empty($userid);

		$cm = new CheckoutManager;
    if(!$cm->validateUserInput($formdata, $bNeedRegister)) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVALID_CUSTOMER'), 'error');
			$this->redirect2Order();
			return false;
		}


    // new user?
    if($bNeedRegister)
    {
      $juser_data = [
        'username' => $jinput->getUsername('username'),
        'password' => $jinput->get('password', '', 'raw'),
        'name'     => $jinput->getString('firstname') . ' ' . $jinput->getString('lastname'),
        'email'    => $jinput->getString('emailaddress'),
      ];

      if(!$cm->RegisterNewUser($juser_data)) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_REGISTER_FAILED'), 'error');
  			$this->redirect2Order();
   			return false;
      }
			
      $userid = $cm->userid;
      
      // Sending the user details to the customer.
      UserRegistration::sendUserRegistration($ordercode, $userid);
    }

    // update user informations
    $cm->saveUserDetails($userid, $formdata);
    
    // create order as offer
    $cm->userid = $userid;
    $orderid = $cm->createOrder($formdata, $bAsOffer);

		// Now create and send the offer

    $data = [
      'userid'           => $userid,
      'ordercode'        => $ordercode,
      'provider'         => 'manual',
      'brutto_price'     => $amounts['gross_price'],
      'transaction_id'   => '0',
      'vat_number'       => $order->vat_number,
      'vat_number'       => $order->vat_number,
      'paid'             => Invoice::UNPAID,
      'invoice_type'     => Invoice::TYPE_OFFER,
    ];

    $orderstateAppend = 0;
    if($bAsOffer) {
      // save offer
      $invoice_id = $cm->createOffer($data, false);
      $orderstateAppend |= OrderStatus::INVOICE_CREATED;

      if($cm->sendOffer($invoice_id)) {
        $orderstateAppend |= OrderStatus::INVOICE_SENT;
      }
    }
    else {
      $invoice_id = $cm->createInvoice($data, false);
      $orderstateAppend |= OrderStatus::INVOICE_CREATED;

      if($cm->sendInvoice($invoice_id)) {
        $orderstateAppend |= OrderStatus::INVOICE_SENT;
      }
    }


    Order::setStatus($ordercode, $orderstateAppend, true);
    
    

		// Removing the session, it's not needed anymore.
		$session = Factory::getSession();
		$session->clear('ordercode');
		$session->clear('order_userid');
    // Reset the userstate
		UserRegState::clearStateData();
		
    $msg = $bAsOffer ? 'COM_LARTSSUBS_OFFER_HAS_BEEN_FINISHED' : 'COM_LARTSSUBS_ORDER_HAS_BEEN_FINISHED';
		Factory::getApplication()->enqueueMessage(Text::_($msg), 'success');

		$link = 'index.php?option=com_lartssubscriptions';
		$this->setRedirect($link);
  }
  
}


