<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Invoice;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\Export\OrdersExport;


class OrdersController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);
	}

	/**
	 * @param bool  $cachable
	 * @param array $urlparams
	 *
	 * @return JControllerLegacy|void
	 *
	 * @since 2.9.2
	 * @throws Exception
	 */
	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'orders');

		parent::display();
	}

	/**
	 * Removing the selected items from the database.
	 */
	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_SELECTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=orders');
		}

		if(!Order::remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_REMOVE_ORDERS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=orders');
			return false;
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_REMOVAL_COMPLETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=orders');

		return true;
	}

  public function export()
	{
		$export = new OrdersExport;

    $err = false;
		if(!$export->queryData()) {
      $err = true;
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUBSCRIPTIONS_EXPORT_FAILED'), 'error');
		}

    if(!$err && !$export->download(null)) {
      $err = true;
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CREATED_XLS_FILE_DOES_NOT_EXIST'), 'error');
		}
    
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=orders');
    return !$err;
  }
  
}

