<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Invoice;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\Transaction;



class PendingsController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'pendings');

		return parent::display();
	}

	/**
	 * Marking the pending transactions as paid.
	 */
	public function markPaid()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		if (count($cid) < 1) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_SELECTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
			return false;
		}

    // set all orders as pais...
		$orders = Order::getOrdercodesByIds($cid);
		foreach ($orders as $order)
		{
      // get all trasnactions of this order
      $amounts = new OrderAmounts($order->ordercode);
      $mssing = $amounts->netprice - $amounts->payed_total;
     
      // create a manual transaction...
      if(!Transaction::insert($order->ordercode, $mssing, 'PayLater', 'Manual')) {
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_SET_INVOICE_AS_PAID'), 'error');
				$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
				return false;
      }

      // set order status
      if(!Order::setStatus($order->ordercode, OrderStatus::PAID, true))
      {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_MARK_TEMPORARY_TRANSACTIONS_AS_PAID'), 'error');
        $this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
        return false;
      }
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PENDING_TRANSACTIONS_MARKED_PAID'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');

		return true;
	}

  public function markProcessed()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);
  
 		if (count($cid) < 1) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_SELECTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
			return false;
		}

		$orders = Order::getOrdercodesByIds($cid);
		foreach ($orders as $order)
		{
     // set order status
     if(!Order::setStatus($order->ordercode, OrderStatus::PROCESSED, true))
      {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_MARK_TEMPORARY_TRANSACTIONS_AS_PAID'), 'error');
        $this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
        return false;
      }

		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PENDING_TRANSACTIONS_PROCESSED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');

		return true;

  }
  
	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		if (count($cid) < 1) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_SELECTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
		}

    if(!Order::remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_REMOVE_ORDERS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
			return false;
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_REMOVAL_COMPLETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');

		return true;
	}

  public function processOrder() 
  {
		$id = $this->input->getInt('id', 0);

		if ($id == 0) {
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
      return;
		}
  
    $this->doProcess(array($id));
  }

  /**
	 * Processing pending orders and activate the subscription and trigger an invoice if required.
	 */
	public function process()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		if (count($cid) < 1) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_SELECTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
		}
  
    $this->doProcess($cid);
  }
  
  public function doProcess($cid)
	{
		$orders = Order::getOrders($cid, 'ordercode, status');
    foreach($orders as $order)
    {
      $cm = new CheckoutManager($order->ordercode);

       // has to be paid
      if(($order->status & OrderStatus::PAID) == false) {
        $msg = Text::_('COM_LARTSSUBS_COULD_NOT_PROCESS_ORDERED_PRODUCTS'). $order->ordercode 
                .'<br>The order is not paid!'; 
        Factory::getApplication()->enqueueMessage($msg, 'error');
        $this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
        //continue;
     		return false;
      }
      
      // transaction_id is always md5(ordercode);
      $transaction_id = md5($order->ordercode); 
      
       // process...
      if(!$cm->processOrderedItems($transaction_id)) {
        $msg = Text::_('COM_LARTSSUBS_COULD_NOT_PROCESS_ORDERED_PRODUCTS'). $order->ordercode
                .'<br>Error while processing!';  
        Factory::getApplication()->enqueueMessage($msg, 'error');
        $this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
        //continue;
     		return false;
      }
      else {
        if(!Order::setStatus($order->ordercode, OrderStatus::PROCESSED, true)) {
          Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_PROCESS_ORDERED_PRODUCTS'), 'error');
          $this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
          return false;
        }
      }
    }
   

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PENDING_TRANSACTIONS_PROCESSED'), 'success');
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');

		return false;
	}

	public function payment()
	{
		$input = $this->input;
		$id = $this->input->getInt('id', 0);
		$amount = $this->input->getFloat('amount', 0);
		$comment = $this->input->getString('comment', '');
    $order = Order::getOrder($id);
    if(empty($order))
      return;

    // create transaction
    $user = Factory::getApplication()->getIdentity();
		$transaction = new Transaction;
    $transaction->insert($order->ordercode, $amount, $comment, $user->id, 'manual', 'manual');

    // update amounts
    if(!Order::updatePayed($order->ordercode)) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PAYMENT_FAILED'), 'error');
      $this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
      return;
    }

 		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PAYMENT_SAVED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
  }
  
  /**
   * Create order from offer
   */
  public function creatOrder()
  {
		$input = $this->input;
		$orderid = $this->input->getInt('id', 0);

    // at this point, all we have to do ist to switch the 'is_offer' flag to false
    $ret = Order::setOfferFlag($orderid, false);
    
 		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ORDER_CREATED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=pendings');
  }
}


