<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\MVC\Controller\AdminController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;

use Larts\Component\LartsSubscriptions\Administrator\Traits\ControllerRegisterTasksTrait;
use Larts\Component\LartsSubscriptions\Administrator\Traits\ControllerReusableModelsTrait;
use LartsSubscriptions\Base\Image;
use LartsSubscriptions\Product;
use LartsSubscriptions\Base\PluginManager;


class ProductsController extends AdminController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		## Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}
	
	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'products');

		return parent::display();
	}

	function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'products');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	// Copies the product information including all relations.

  public function copyProduct()
	{
		$jNow = new Date;

		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_NO_PRODUCT_SELECTED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
		}

		// Connect the model to get the info.
		$model = $this->getModel('products');

		foreach ($cid as $productid)
		{
			// Retreive some product information
			$product    = $model->getProductInformation($productid);
			$categories = $model->getProductCategories($productid);
			$bundled_products = $model->getBundledProducts($productid);
			$related_products = $model->getProductChildren($productid);

			// Set some required data and make sure everything is unique.
			$product['name']      = $product['name'] . ' [COPIED]';
			$product['alias']     = OutputFilter::stringURLSafe($product['name']);
			$product['created']   = $jNow->toSql();
			$product['updated']   = $jNow->toSql();
			$product['published'] = 0;

			// Unset product ID:
			unset($product['id']);

			// Save the new product.
			if ($id = $model->store($product, $categories, $bundled_products, $related_products))
			{
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_SAVED'));

				$data = [
					'productid' => $id,
					'postdata'  => $product,
				];

    		PluginManager::execPlugins('lartssubscriptions', 'onAfterSaveProduct', $data);
			}
			else
			{
				Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_SAVED_FAILED'), 'error');
			}
		}

		$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
	}

	/**
	 * (Un)publish products in the list.
	 * Requires an array with productid's
	 *
	 * @throws Exception
	 */
	public function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		## Get the task (publish/upnpublish)
		if ($this->getTask() == 'publish') {
			$publish = 1;
		}
		else {
			$publish = 0;
		}

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_NO_PRODUCT_SELECTED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
		}

		$model = $this->getModel('products');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_PRODUCTS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
		}

		if ($this->getTask() == 'publish')
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_PUBLISHED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_UNPUBLISHED'));
		}

		$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('products');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_PRODUCTS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCTS_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=products');
	}

  public function save2new()
	{
    $this->save();

    $redirect = 'index.php?option=com_lartssubscriptions&task=products.add';
    $this->setRedirect($redirect);
  }
  
	public function save()
	{
		$post = Factory::getApplication()->GetInput()->post->getArray();

		$post['description']       = $this->input->get('description', '', 'raw');
		$post['short_description'] = $this->input->get('short_description', '', 'raw');
		$post['custom_buttons']    = $this->input->get('custom_buttons', '', 'raw');
    
    if(!isset($post['vat_class'])) {
      $post['vat_class'] = 1;
    }

    $image = explode('#', $post['image']);
    $post['image'] = $image[0];
    
		if ($this->input->getCmd('alias') == '') {
			$post['alias'] = OutputFilter::stringURLSafe($this->input->getString('name'));
		}

		$date = new Date;

		if(!$post['created']) {
			$post['created'] = $date->toSql();
		}

		if(!$post['updated']) {
			$post['updated'] = $date->toSql();
		}

		$categories = $this->input->get('categories', [], 'array');
		$categories = ArrayHelper::toInteger($categories);

		$bundled_products = $this->input->get('bundled_products', [], 'array');
		$bundled_products = ArrayHelper::toInteger($bundled_products);

 		$related_products = $this->input->get('related_products', [], 'array');
		$related_products = ArrayHelper::toInteger($related_products);

		$type = $this->input->getInt('type', 0);
    $file_id = $this->input->getInt('file_id', 0);
		$template_id = $this->input->getInt('template_id', 0);
		$category_id = $this->input->getInt('category_id', 0);
		$group_id = $this->input->getInt('group_id', 0);

    if($type == Product::UNKNOWN) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_TYPE_REQUIRED'), 'error');
			$redirect = 'index.php?option=com_lartssubscriptions&task=products.edit';
  		Factory::getApplication()->redirect($redirect);
    }

    switch($type) {
    case Product::FILE:
    case Product::MOVIE_STREAM:
			$post['source_id'] = $file_id;
      if($file_id == 0) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_SOURCE_FILE'), 'error');
      }
      break;
    case Product::TICKET:
			$post['source_id'] = $template_id;
      if($template_id == 0) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_SOURCE_TICKET'), 'error');
      }
      break;
    case Product::CATEGORY:
			$post['source_id'] = $category_id;
      if($category_id == 0) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_SOURCE_CAT'), 'error');
      }
      break;
    case Product::GROUP_ACCESS:
			$post['source_id'] = $group_id;
      if($group_id == 0) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_SOURCE_CAT'), 'error');
      }
      break;
    }

    
		if(empty($post['name'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_NAME_REQUIRED'), 'error');
			$redirect = 'index.php?option=com_lartssubscriptions&task=products.edit';
  		Factory::getApplication()->redirect($redirect);
    }
    
		$model = $this->getModel('products');

		if ($model->aliasCheck($post['id'], $post['alias']))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_DOUBLE_ALIAS'), 'error');
		}

		if ($id = $model->store($post, $categories, $related_products, $bundled_products))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_SAVED'));

			$data = [
				'productid' => $id,
				'postdata'  => $post,
			];

   		PluginManager::execPlugins('lartssubscriptions', 'onAfterSaveProduct', $data);
		}
		else
		{
			$this->task = 'save';	// change task to prevent using invalid $id
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PRODUCT_SAVED_FAILED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=products';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=products.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}

	public function saveOrderAjax()
	{
		$pks   = $this->input->post->get('cid', [], 'array');
		$order = $this->input->post->get('order', [], 'array');

		// Sanitize the input
		$pks   = ArrayHelper::toInteger($pks);
		$order = ArrayHelper::toInteger($order);

    // Get the model
		$model = $this->getModel('products');

		// Save the ordering
		$return = $model->saveOrdering($pks, $order);
		if ($return) {
			echo "1";
		}

		// Close the application
		//Factory::getApplication()->close();
    exit();
	}

  public function removeimage()
  {
		$id = $this->input->get('id', );
    
    Product::removeImage($id);
    
		$redirect = 'index.php?option=com_lartssubscriptions&task=products.edit&cid[]=' . $id;
		$this->setRedirect($redirect);
  }

}
