<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Layout\FileLayout;

use LartsSubscriptions\MassMailTemplates;
use LartsSubscriptions\MassMailQueue;
use LartsSubscriptions\Periods;
use LartsSubscriptions\SubscriptionManager;


class QueueController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		// Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();

		$jinput->set('layout', 'default');
		$jinput->set('view', 'queue');

		return parent::display();
	}

	function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		// Get the task (publish/upnpublish)
		$publish = ($this->getTask() == 'publish') ? 1 : 0;

		if (count($cid) < 1) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NOTHING_SELECTED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
		}

		$model = $this->getModel('queue');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
		}

		$msg = ($publish == 1) ? 'COM_LARTSSUBS_TEMPLATE_PUBLISHED' : 'COM_LARTSSUBS_TEMPLATE_UNPUBLISHED';
		Factory::getApplication()->enqueueMessage(Text::_($msg));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
	}

  public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();

		$jinput->set('layout', 'form');
		$jinput->set('view', 'queue');

		parent::display();
	}

	public function save()
	{
		$db = Factory::getDbo();

		$post = Factory::getApplication()->GetInput()->post->getArray();
		$post['body'] = $this->input->get('body', '', 'raw');
		$post['date'] = $this->input->getCmd('date');
		$id = $this->input->get('id', 0);

    // Save the data in the session.
    Factory::getApplication()->setUserState('com_lartssubscriptions.queue', $post);

    if(strlen($post['template']) == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_TEMPLATE_NAME'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&task=queue.add');
    }
    if(strlen($post['subject']) == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_MISSING_SUBJECT'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&task=queue.add');
    }

    // Reset the userstate
    Factory::getApplication()->setUserState('com_lartssubscriptions.queue', null);

    // save template
		$model = $this->getModel('queue');

    $record = [
        'id'             => $id,
				'template'       => $post['template'],
				'subject'        => $post['subject'],
				'body'           => $post['body'],
				'from_email'     => $post['from_email'],
				'from_name'      => $post['from_name'],
				'reply_to_email' => $post['reply_to_email'],
				'reply_to_name'  => $post['reply_to_name'],
				'published'      => $post['published'],
			];

    // save template
    $id = MassMailTemplates::saveTemplate($record);
    if($id == 0) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
  		$redirect = 'index.php?option=com_lartssubscriptions&view=queue';
    }
    else {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TEMPLATE_SAVED'), 'message');

      $redirect = 'index.php?option=com_lartssubscriptions&view=queue';
      if ($this->task == 'apply') {
        $redirect = 'index.php?option=com_lartssubscriptions&task=queue.edit&cid[]=' . $id;
      }
    }

		$this->setRedirect($redirect);
  }

  public function queue()
	{
    $template_id = $this->input->get('id', 0);
    // Save the data in the session.
		$post = Factory::getApplication()->GetInput()->post->getArray();
    Factory::getApplication()->setUserState('com_lartssubscriptions.queue', $post);
    
 		if (empty($post['product']) || count($post['product']) == 0) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_NO_PRODUCTS_HAS_BEEN_SELECTED'), 'error');
			$redirect = 'index.php?option=com_lartssubscriptions&task=queue.add';
      if ($template_id != 0) {
        $redirect = 'index.php?option=com_lartssubscriptions&task=queue.edit&cid[]=' . $template_id;
      }
  		$this->setRedirect($redirect);
			return false;
		}
  
    
    // send mass mail...

    $active_only = $post['active_only'];
    $post['product'] = ArrayHelper::toInteger($post['product']);
		$cids = implode(',', $post['product']);

    $date = new Date('-1 year');
		$valid_to = $date->format('Y-m-d');

    $db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('per.product_id, per.userid, per.licence, per.valid_from, per.valid_to, '
              . 'cust.firstname, cust.lastname, cust.newsletter, u.name, u.username, u.email')
			->from($db->quoteName('#__larts_subs_periods', 'per'))
			->join('LEFT', $db->quoteName('#__users', 'u'). ' ON (' . $db->quoteName('per.userid') . ' = ' . $db->quoteName('u.id') . ')')
			->join('LEFT', $db->quoteName('#__larts_subs_customers', 'cust') . ' ON (' . $db->quoteName('per.userid') . ' = ' . $db->quoteName('cust.userid') . ')')
			->where($db->quoteName('per.valid_to'). ' >= ' . $db->quote($valid_to))
			->andwhere($db->quoteName('per.product_id'). ' IN ' . '( ' . $cids . ' )')
			->andwhere($db->quoteName('u.block'). ' = false')
			->andwhere($db->quoteName('cust.newsletter'). ' = true')
            ;

    if($active_only == 1) {
			$query->andwhere($db->quoteName('per.status'). ' = '. Periods::STATE_ACTIVE);
    }

    $db->setQuery($query);
		$data = $db->loadObjectList();

		// Generate a unique mail id:

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_QUEUE_PROCESSED', $mails_sent));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
	}

  public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
    
    if(!empty($cid))
    {
      $cid = ArrayHelper::toInteger($cid);
      $model = $this->getModel('queue');

      if(!$model->remove($cid)) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_QUEUE_DELETED_ERROR'), 'error');
      }
      else {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_QUEUE_DELETED'));
      }
    }
    
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
	}

	public function removePending()
	{
		$model = $this->getModel('queue');

    if(!$model->removePending())
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_QUEUE_REMOVED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
	}

	public function removeSent()
	{
		$model = $this->getModel('queue');

    if(!$model->removeSent())
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_QUEUE_REMOVED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
	}
  
  public function getTemplatesModal()
  {
    $results = MassMailTemplates::getTemplates();
    
		$layout = new FileLayout('modal-templates');
		echo $layout->render(['templates' => $results]);
    exit;
  }
  
 	function removeTemplates($cid)
	{
		$cid = $this->input->get('cid', [], 'array');
    
    if(!empty($cid))
    {
      $cid = ArrayHelper::toInteger($cid);
      $model = $this->getModel('queue');

      if(!$model->removeTemplates($cid)) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
      }
      else {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_QUEUE_TEMPLATE_DELETED'));
      }
    }
    
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=queue');
	}

}
