<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Periods;
use LartsSubscriptions\Logger;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Export\SubscriberExport;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\ThirdParty\LartsLicManager;
use LartsSubscriptions\Product;


class SubscriptionsController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'subscriptions');

		return parent::display();
	}

	function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'subscriptions');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		## Get the task (publish/upnpublish)
		if ($this->getTask() == 'publish')
		{
			$publish = 1;
		}
		else
		{
			$publish = 0;
		}

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SELECT_ITEM'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
		}

		$model = $this->getModel('subscriptions');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_SUBSCRIPTIONS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUCCESS_PUBLISH_SUBSCRIPTIONS'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
	}

	function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('subscriptions');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_SUBSCRIPTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUBSCRIPTION_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
	}

	function save()
	{
    $config = LartsConfig::get();
		$post = Factory::getApplication()->GetInput()->post->getArray();

    $id = $this->input->getInt('id');
    $internalinfo = $this->input->getString('private_information');
    $userinfo = $this->input->getString('user_info');
    $product_id = $this->input->getInt('product_id');
    $userid = $this->input->getInt('userid');
    $period_status = $this->input->getInt('status');
    $mail_changes = $this->input->getInt('mail_changes');
    $action = 'save';
    
    if(!empty($id)) {
      // modify subscription

      $model = $this->getModel('subscriptions');

      // save new state
      $data = ['id' => $id,
        'status' => $period_status,
        'mail_changes' => $mail_changes,
        'user_info' => $userinfo,
        'internal_info' => $internalinfo,
        ];

      if (Periods::updateStatus($data)) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUB_SAVED'));
      }
      else {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUB_SAVED_FAILED'), 'error');
      }
      PluginManager::execPlugins('lartssubscriptions', 'onAfterPeriodChanged', ['action' => $action, 'product' => $product_id, 'userid' => $userid]);

      // mail changes?
      if($mail_changes) {
        // Obtain some information:
        switch($period_status) {
        case Periods::STATE_ACTIVE:
          $action = Text::_('COM_LARTSSUBS_SUB_ACTIVATION');
          break;
        case Periods::STATE_CANCELED:
          $action = Text::_('COM_LARTSSUBS_SUB_CANCEL_ACTION');
          break;
        case Periods::STATE_REFUNDED:
          $action = Text::_('COM_LARTSSUBS_SUBSCRIPTION_REFUNDED');
          break;
        case Periods::STATE_DISABLED:
        default:
          $action = Text::_('COM_LARTSSUBS_SUB_DEACTIVATION');
          break;
        }

        // Perform the action:
        $model->sendMessage($product_id, $userid, $action);
      }

    }
    else
    {
      // add new subscription
      
      $redirect = 'index.php?option=com_lartssubscriptions&task=subscriptions.edit';
      
      if($userid != 0) {
        $redirect .= '&uid=' .$userid;
      }

      if($userid == 0) {
        Factory::getApplication()->enqueueMessage(Text::_('User?'), 'error');
        $this->setRedirect($redirect);
        return;
      }

      $prod = Product::getProduct($product_id);
      if (empty($prod)) {
        Factory::getApplication()->enqueueMessage(Text::_('Product?'), 'error');
        $this->setRedirect($redirect);
        return;
      }


      if($prod->has_licence && $config->LicMngrInstalled) {
        if(empty($post['licence'])) {
          Factory::getApplication()->enqueueMessage(Text::_('licence?'), 'error');
          $this->setRedirect($redirect);
          return;
        }
      }

      if($prod->has_licence && $config->LicMngrInstalled)
      {
        $api = LartsLicManager::getAPI();
        $lic = $api->lookupLicence($post['licence']);
        if(empty($lic)) {
          Factory::getApplication()->enqueueMessage(Text::_('Licence not found!'), 'error');
          $this->setRedirect($redirect);
          return;
        }
        //$product_id = $lic->product_id;
      }


      $periods = new Periods;
      $nCount = intval($post['period']);
      $ordercode = 0;
      $newPeriod = $periods->addPeriod($post['dstart'], $ordercode, $prod->id, $prod->product_ref, $post['userid'], $lic->licence_number, 
                                       $post['period_type'], $nCount, $period_status,
                                       $userinfo, $internalinfo);
      Logger::_('add_new_period', ['period' => $newPeriod]);

      PluginManager::execPlugins('lartssubscriptions', 'onNewPeriodAdded', ['period' => $newPeriod]);
      
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUB_ADDED'));

    }

    
    $redirect = 'index.php?option=com_lartssubscriptions&view=subscriptions';
		if ($this->task == 'apply') {
      $redirect = 'index.php?option=com_lartssubscriptions&task=subscriptions.edit&cid[]=' . $id;
		}

		$this->setRedirect($redirect);
    return;
    
	}

	/**
	 * Exportig the list of subscribers to Excel.
	 */
	public function export()
	{
		$export = new SubscriberExport;

		if(!$export->queryData())
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SUBSCRIPTIONS_EXPORT_FAILED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
			return false;
		}

    if(!$export->download())
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CREATED_XLS_FILE_DOES_NOT_EXIST'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=subscriptions');
			return false;
		}
	}
  
  public function getAjaxLicences()
  {
    LartsLicManager::getAjaxLicences();
  }
  
}
