<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

class TemplatesController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		## Register Extra tasks
		$this->registerTask('add', 'edit');
		$this->registerTask('apply', 'save');
	}

	function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'templates');

		return parent::display();
	}

	function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'templates');

		parent::display();
	}

	function save()
	{
		$post         = Factory::getApplication()->GetInput()->post->getArray();
		$post['body'] = $this->input->get('body', '', 'raw');

		// Check if the BCC emails are set and correctly.
		if(!filter_var($post['bcc1_email'], FILTER_VALIDATE_EMAIL) || empty($post['bcc1_name']))
		{
			// Reset values for BCC
			$post['bcc1_email'] = '';
			$post['bcc1_name']  = '';
		}

		// Check if the BCC emails are set and correctly.
		if(!filter_var($post['bcc2_email'], FILTER_VALIDATE_EMAIL) || empty($post['bcc2_name']))
		{
			// Reset values for BCC
			$post['bcc2_email'] = '';
			$post['bcc2_name']  = '';
		}

		$model = $this->getModel('templates');

		if ($id = $model->store($post))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TEMPLATE_SAVED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TEMPLATE_NOTSAVED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=templates';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=templates.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}

	/*
	 * Currently not used
	 */
	function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		## Get the task (publish/upnpublish)
		$publish = ($this->getTask() == 'publish');

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_SELECT_ITEM'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=templates');
		}

		$model = $this->getModel('templates');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISHING_ACTIONS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=templates');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_PUBLISHED_MESSAGE'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=templates');
	}

	function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('templates');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_COULD_NOT_REMOVE_ITEMS'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=templates');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ITEMS_REMOVED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=templates');
	}
}
