<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Filter\OutputFilter;

use LartsSubscriptions\Download;
use LartsSubscriptions\Ticket;
use LartsSubscriptions\Categories;


class TicketsController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'tickets');

		return parent::display();
	}

	public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'tickets');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	public function publish()
	{
		$cid     = $this->input->get('cid', [], 'array');
		$cid     = ArrayHelper::toInteger($cid);
		$publish = $this->getTask() == 'publish' ? 1 : 0;

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_NO_TICKET_SELECTED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=tickets');
		}

		$model = $this->getModel('tickets');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_TICKET'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=tickets');
		}

		if ($this->getTask() == 'publish')
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TICKET_PUBLISHED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TICKET_UNPUBLISHED'));
		}
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=tickets');
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('tickets');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_TICKET'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=tickets');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TICKETS_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=tickets');
	}

  public function removeimage()
  {
		$id = $this->input->get('id', );
    
    Categories::removeImage($id);
    
		$redirect = 'index.php?option=com_lartssubscriptions&task=tickets.edit&cid[]=' . $id;
		$this->setRedirect($redirect);
  }

  public function save()
	{
		$post = Factory::getApplication()->GetInput()->post->getArray();
		$post['description'] = $this->input->get('description', '', 'raw');

		if ($this->input->getCmd('alias') == '')
		{
			$post['alias'] = OutputFilter::stringURLSafe($this->input->getString('name'));
		}

		if ($this->input->get('id', 0, 'int') == 0)
		{
			$date = new Date;

			$post['created'] = $date->toSql();
		}

		$model = $this->getModel('tickets');

		if ($id = $model->store($post))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TICKET_SAVED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TICKET_SAVED_FAILED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=tickets';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=tickets.edit&cid[]=' . $id;
		}

		$this->setRedirect($redirect);
	}

	/**
	 * Downloading an invoice in the backend and opening it.
	 */
	public function getticket()
	{
    $ticketid = $this->input->getInt('id', 0);
		$ticket = new Download;

		// Get invoice information;
		$data = $ticket->downloadTicket($ticketid);
    if($data === false) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TICKET_NOT_FOUND'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=tickets');
      return;
    }

		// Prpearing the new file name which will be popped.
		$newfilename = 'ticket-' . !empty($data->ticket_number) ? $data->ticket_number . '.pdf' : $data->id . '.pdf';

		// Push to PDF file to the browser.
		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-type: application/pdf");
		header("Content-Disposition: attachment; filename=\"" . strtolower($newfilename) . "\"");
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: " . filesize($data->ticket_file));

		ob_clean();
		flush();

		@readfile($data->ticket_file);
		die;
	}
}
