<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;


class TransactionsController extends BaseController
{
	/**
	 * LartssubsControllerTransactions constructor.
	 */
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	/**
	 * Displaying the page with a table
	 *
	 * @param bool  $cachable
	 * @param array $urlparams
	 *
	 * @return JControllerLegacy|void
	 *
	 * @since 2.9.2
	 * @throws Exception
	 */
	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'transactions');

		parent::display();
	}

	/**
	 * Display an editoral page.
	 *
	 * @param bool  $cachable
	 * @param array $urlparams
	 *
	 *
	 * @since 2.9.2
	 * @throws Exception
	 */
	public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'transactions');

		parent::display();
	}

	/**
	 * Removing transactions or pending temporay transactions.
	 * cid is being sent by the transactions table
	 *
	 * @since 2.9.2
	 * @throws Exception
	 */
	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('transactions');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_TRANSACTION'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=transactions');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_TRANSACTION_TRUNCATED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=transactions');

		return true;
	}
}
