<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

class VatclassesController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'default');
		$jinput->set('view', 'vatclasses');

		return parent::display();
	}

  public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'vatclasses');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	public function publish()
	{
		$cid     = $this->input->get('cid', [], 'array');
		$cid     = ArrayHelper::toInteger($cid);
		$publish = $this->getTask() == 'publish' ? 1 : 0;

		if (count($cid) < 1)
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_NO_CATEGORY_SELECTED'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatclasses');
		}

		$model = $this->getModel('vatclasses');

		if(!$model->publish($cid, $publish))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_PUBLISH_CATEGORY'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatclasses');
		}

		if ($this->getTask() == 'publish')
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CATEGORY_PUBLISHED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CATEGORY_UNPUBLISHED'));
		}
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatclasses');
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('vatclasses');
		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_CATEGORY'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatclasses');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CATEGORIES_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatclasses');
	}

	public function save()
	{
		$post = Factory::getApplication()->GetInput()->post->getArray();

    $record['id'] = $post['id'];
    $record['name'] = $post['name'];
    $record['ordering'] = intval($post['ordering']);
    $record['published'] = intval($post['published']);

    $model = $this->getModel('vatclasses');

		if ($id = $model->store($record)) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CAT_SAVED'));
		}
		else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CAT_SAVED_FAILED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=vatclasses';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=vatclasses.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}

}
