<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Controller;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Application\CMSApplication;
use Joomla\Input\Input;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\VAT\VatRates;

class VatratesController extends BaseController
{
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('add', 'edit');
		$this->registerTask('unpublish', 'publish');
		$this->registerTask('apply', 'save');
	}

	public function display($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		//$jinput->set('layout', 'default');
		$jinput->set('view', 'vatrates');

		return parent::display();
	}

  public function edit($cachable = false, $urlparams = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		$jinput->set('layout', 'form');
		$jinput->set('view', 'vatrates');
		$jinput->set('hidemainmenu', 1);

		parent::display();
	}

	public function remove()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

		$model = $this->getModel('vatrates');

		if(!$model->remove($cid))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR_DELETE_CATEGORY'), 'error');
			$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatrates');
		}

		Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_CATEGORIES_DELETED'));
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatrates');
	}

	public function save()
	{
		$post = Factory::getApplication()->GetInput()->post->getArray();

		$model = $this->getModel('vatrates');

		if ($id = $model->store($post))
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_VAT_SAVED'));
		}
		else
		{
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_VAT_SAVED_FAILED'), 'error');
		}

		$redirect = 'index.php?option=com_lartssubscriptions&view=vatrates';
		if ($this->task == 'apply') {
			$redirect = 'index.php?option=com_lartssubscriptions&task=vatrates.edit&cid[]=' . $id;
		}
		$this->setRedirect($redirect);
	}

  function publish()
	{
		$cid = $this->input->get('cid', [], 'array');
		$cid = ArrayHelper::toInteger($cid);

  	$publish = ($this->getTask() == 'publish') ? 1 : 0;
    
    VatRates::setVatRatesPublished($cid, $publish);

    $this->setRedirect('index.php?option=com_lartssubscriptions&view=vatrates');
	}

public function updateRates()
	{
    $errMsg = '';
    $bRet = VatRates::updateEURates($errMsg);
		if ($bRet) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_VATRATES_HAS_BEEN_RENEWED'));
		}
		else {
      $msg = Text::_('COM_LARTSSUBS_VATRATES_RENEW_FAILED'). ': '.$errMsg; 
			Factory::getApplication()->enqueueMessage($msg, 'error');
		}

		$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatrates');
	}

  public function removeEURates()
	{
    $bRet = VatRates::removeEURates();
		if ($bRet) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_VATRATES_HAS_BEEN_REMOVED'));
		}
		else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_VATRATES_REMOVE_FAILED'), 'error');
		}
		$this->setRedirect('index.php?option=com_lartssubscriptions&view=vatrates');
  }
}


