<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Dispatcher;
defined('_JEXEC') or die;

use Joomla\CMS\Dispatcher\ComponentDispatcher;
use Joomla\CMS\Access\Exception\NotAllowed;

// ComponentDispatcher class for LartsSubscriptions
//
class Dispatcher extends ComponentDispatcher
{
	protected $option = 'com_lartssubscriptions';
	protected $defaultController = 'dashboard';

  public function dispatch()
	{
    
    $this->applyViewAndController();
    
    parent::dispatch();
  }
  
  // LartsSubscriptions have to check for extension permission
  protected function checkAccess()
  {
    $extension = $this->getApplication()->input->getCmd('extension', '');

    $parts = explode('.', $extension);

    // Check the user has permission to access this component if in the backend
    if ($this->app->isClient('administrator') && !$this->app->getIdentity()->authorise('core.manage', $parts[0])) {
        throw new NotAllowed($this->app->getLanguage()->_('JERROR_ALERTNOAUTHOR'), 403);
    }
  }

	private function applyViewAndController(): void
	{
		// Handle a custom default controller name
		$view       = $this->input->getCmd('view', $this->defaultController);
		$controller = $this->input->getCmd('controller', $view);
		$task       = $this->input->getCmd('task', 'main');

		// Check for a controller.task command.
		if (strpos($task, '.') !== false)
		{
			// Explode the controller.task command.
			[$controller, $task] = explode('.', $task);
		}

		$this->input->set('view', $controller);
		$this->input->set('controller', $controller);
		$this->input->set('task', $task);
	}
    
}
