<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\EditorsXtd\LartsSubscriptions\Extension;

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Event\DispatcherInterface;
use Joomla\Event\SubscriberInterface;
use Joomla\CMS\Event\Editor\EditorButtonsSetupEvent;
use Joomla\CMS\Editor\Button\Button;


class LartsSubscriptions extends CMSPlugin implements SubscriberInterface
{
	protected $autoloadLanguage = true;
  protected $app;

	public function __construct(DispatcherInterface $dispatcher, array $config)
	{
    parent::__construct($dispatcher, $config);
  }

	public static function getSubscribedEvents(): array
	{
			return ['onEditorButtonsSetup' => 'onEditorButtonsSetup'];
	}

	public function onEditorButtonsSetup(EditorButtonsSetupEvent $event)
	{
			$subject  = $event->getButtonsRegistry();
			$disabled = $event->getDisabledButtons();

			if (\in_array($this->_name, $disabled)) {
					return;
			}

			$this->loadLanguage();

			$button = $this->onDisplay($event->getEditorId());

			if ($button) {
					$subject->add($button);
			}
	}

	public function onDisplay($name)
  {
		// loading css here has no effect

 		$component = $this->app->input->getCmd('option');
		$basePath  = $this->app->isClient('administrator') ? '' : 'administrator/';
		$link = $basePath . 'index.php?option=com_lartssubscriptions&amp;view=editorbutton&amp;layout=modal&amp;tmpl=component&amp;e_name=' . $name;

		$button = new Button(
				$this->_name,
				[
						'action'  => 'modal',
						'link'    => $link,
						'text'    => Text::_('PLG_EDITORS_XTD_LARTSSUBSCRIPTIONS_BUTTON_TEXT'),
						'icon'    => '',
						'iconSVG' => '',
						//'iconSVG' => '<svg viewBox="0 0 32 32" width="24" height="24"><path d="M28 24v-4h-4v4h-4v4h4v4h4v-4h4v-4zM2 2h18v6h6v10h2v-10l-8-'
						//		. '8h-20v32h18v-2h-16z"></path></svg>',
						// This is whole Plugin name, it is needed for keeping backward compatibility
						'name' => $this->_type . '_' . $this->_name,
				]
		);

//		$button = new JObject;
//		$button->modal = true;
//		$button->class = 'btn';
//		$button->link = $link;
//		$button->text = Text::_('PLG_EDITORS_XTD_LARTSSUBSCRIPTIONS_BUTTON_TEXT');
//		$button->name = 'checkbox-unchecked';
//		$button->options = "{handler: 'iframe', size: {x: 300, y: 300}}";

		return $button;
	}

}

?>