<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\System\LartsSubscriptions\Extension;

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();
require_once(JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/src/Extension/autoload.php');

use Joomla\CMS\Factory;
use Joomla\Event\DispatcherInterface;
use Joomla\Event\SubscriberInterface;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Event\Content\ContentPrepareEvent;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\UserGroups;
use LartsSubscriptions\Cronjob;
use LartsSubscriptions\Config as LartsConfig;
use Larts\Component\LartsSubscriptions\Site\View\Categories\HtmlView as CategoriesView;
use Larts\Component\LartsSubscriptions\Site\Model\CategoriesModel;
use Larts\Component\LartsSubscriptions\Site\View\Checkout\HtmlView as CheckoutView;
use Larts\Component\LartsSubscriptions\Site\Model\CheckoutModel;
use Larts\Component\LartsSubscriptions\Site\View\Products\HtmlView as ProductsView;
use Larts\Component\LartsSubscriptions\Site\Model\ProductsModel;


class LartsSubscriptions extends CMSPlugin implements SubscriberInterface
{
  protected $app;
  protected $param;
	var $cfg;			// component config
	var $plgCfg;	// plugin config
  var $articleid;

	public function __construct(DispatcherInterface $dispatcher, array $config)
	{
    $component = ComponentHelper::getComponent('com_lartssubscriptions', true);
    if (!$component->enabled) {   
      return;
    }

    $this->param = $component->params;
    $this->app = Factory::getApplication();
    
		// load language
		$lang = Factory::getLanguage();
    //$lang->load('com_lartssubscriptions', JPATH_ADMINISTRATOR);
    $lang->load('com_lartssubscriptions', JPATH_SITE);

    parent::__construct($dispatcher, $config);
  }

  public static function getSubscribedEvents(): array
	{
		if(defined('verJ4')) {
			return [
				'onContentPrepare' => 'ContentPrepareJ4',
				'onAfterInitialise' => 'AfterInitialise',
				'onUserAfterLogin' => 'UserAfterLogin',
			];
		}
	
		return [
			'onContentPrepare' => 'ContentPrepare',
			'onAfterInitialise' => 'AfterInitialise',
			'onUserAfterLogin' => 'UserAfterLogin',
		];
	}

	public function ContentPrepareJ4($context, &$article, &$params, $page = 0)
  {
    // Don't run this plugin when the content is being indexed
		if ($context == 'com_finder.indexer') {
			return true;
		}
    // Return if we are in backend
    if (!$this->app->isClient('site')) {
      return true;
    }
    
		$view	= $this->app->input->get('view');
		if ($view == 'tag')
      { return; }
    
		$this->articleid = $this->app->input->get('id');
    

    // check it button is inserted
    if(strpos($article->text, 'larts-button') !== false) {
      HTMLHelper::stylesheet('com_lartssubscriptions/product.css', array('version' => 'auto', 'relative' => true));
    }
    
    // Start Plugin
		$regex = '/{(lartssubscriptions)\s*(.*?)}/i';
		$matches = array();
		//$count_matches	= preg_match_all($regex_all,$article->text,$matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER);
		if (isset($article->text)) {
			$count_matches = @preg_match_all($regex, $article->text, $matches, PREG_SET_ORDER);
		} else {
			$count_matches = @preg_match_all($regex, $article->introtext, $matches, PREG_SET_ORDER);
		}

 		foreach ($matches as $elm) {
			//$elmnum += 1;
			$line = strip_tags($elm[2]); // get rid of most of the embedded html tags, if any. Only tags directly after the { will not be cleaned as regex will not pick those up
			$line = preg_replace('/\s\s+/', ' ', $line);	 // remove multiple spaces
			$line = str_replace("&nbsp;", " ", $line);	   // replace HTML non breaking space
			$line = str_replace("\xc2\xa0", ' ', $line);	 // replace UTF-8 non breaking space
			//$line = str_replace(" ", "&", $line);			     // replace spaces with '&'

			$line = strtolower($line);
      $cmd = $line;
      $extra_params = array();
      $pos = strpos($line, ' ');
      if($pos > 0) {
        $cmd = substr($line, 0, $pos);
        // splitup name/value pairs
        $extra = substr($line, -(strlen($line) - $pos-1));
        preg_match_all("/([^,= ]+)=([^,= ]+)/", $extra, $r); 
        $extra_params = array_combine($r[1], $r[2]);
      }

			$html = "";
      
			switch ($cmd) {
      case 'products':
        $html .= $this->renderProducts();
        break;
      case 'categories':
        $html .= $this->renderCategories($extra_params);
        break;
      case 'checkout':
        $html .= $this->renderCheckout();
        break;
      case 'myinvoices':
        $html .= $this->renderInvoices();
        break;
      case 'myprofile':
        $html .= $this->renderProfile();
        break;
      case 'mysubscriptions':
        $html .= $this->renderSubscriptions();
        break;
      }

      if (isset($article->text)) {
        $article->text = preg_replace($regex, $html, $article->text, 1);
      } else {
        $article->introtext = preg_replace($regex, $html, $article->introtext, 1);
      }
    }


    return false;
	}
	
	public function ContentPrepare(ContentPrepareEvent $event)
  {
		$context = $event->getContext();
    // Don't run this plugin when the content is being indexed
		if ($context == 'com_finder.indexer') {
			return true;
		}

		// Return if we are in backend
    if (!$this->app->isClient('site')) {
      return true;
    }
    
		// Get content item
		$item = $event->getItem();

		// If the item does not have a text property there is nothing to do
		if (!property_exists($item, 'text')) {
				return;
		}

		$view	= $this->app->input->get('view');
		if ($view == 'tag')
      { return; }
    
		$this->articleid = $this->app->input->get('id');
    

    // check it button is inserted
		if(!empty($item->text) && strpos($item->text, 'larts-button') !== false) {
			HTMLHelper::stylesheet('com_lartssubscriptions/product.css', array('version' => 'auto', 'relative' => true));
		}
    
    // Start Plugin
		$regex = '/{(lartssubscriptions)\s*(.*?)}/i';
		$matches = array();
		//$count_matches	= preg_match_all($regex_all,$article->text,$matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER);
		if (isset($item->text)) {
			$count_matches = @preg_match_all($regex, $item->text, $matches, PREG_SET_ORDER);
		} else {
			$count_matches = @preg_match_all($regex, $item->introtext, $matches, PREG_SET_ORDER);
		}

 		foreach ($matches as $elm) {
			//$elmnum += 1;
			$line = strip_tags($elm[2]); // get rid of most of the embedded html tags, if any. Only tags directly after the { will not be cleaned as regex will not pick those up
			$line = preg_replace('/\s\s+/', ' ', $line);	 // remove multiple spaces
			$line = str_replace("&nbsp;", " ", $line);	   // replace HTML non breaking space
			$line = str_replace("\xc2\xa0", ' ', $line);	 // replace UTF-8 non breaking space
			//$line = str_replace(" ", "&", $line);			     // replace spaces with '&'

			$line = strtolower($line);
      $cmd = $line;
      $extra_params = array();
      $pos = strpos($line, ' ');
      if($pos > 0) {
        $cmd = substr($line, 0, $pos);
        // splitup name/value pairs
        $extra = substr($line, -(strlen($line) - $pos-1));
        preg_match_all("/([^,= ]+)=([^,= ]+)/", $extra, $r); 
        $extra_params = array_combine($r[1], $r[2]);
      }

			$html = "";
      
			switch ($cmd) {
      case 'products':
        $html .= $this->renderProducts();
        break;
      case 'categories':
        $html .= $this->renderCategories($extra_params);
        break;
      case 'checkout':
        $html .= $this->renderCheckout();
        break;
      case 'myinvoices':
        $html .= $this->renderInvoices();
        break;
      case 'myprofile':
        $html .= $this->renderProfile();
        break;
      case 'mysubscriptions':
        $html .= $this->renderSubscriptions();
        break;
      }

      if (isset($item->text)) {
        $item->text = preg_replace($regex, $html, $item->text, 1);
      } else {
        $item->introtext = preg_replace($regex, $html, $item->introtext, 1);
      }
    }


    return false;
	}

  public function renderProducts()
  {
    // load language
    $lang = Factory::getLanguage();
    $lang->load('com_lartssubscriptions', JPATH_ADMINISTRATOR);


    $view = new ProductsView;
    $model = new ProductsModel;

    $view->setArticleId($this->articleid);
    $view->setModel($model, true);
    $view->setLayout('default');
    
    $path = JPATH_SITE . '/components/com_lartssubscriptions/tmpl/products/default.php';
    $view->setRenderLayoutPath($path);
    return $view->display();
  }

  public function renderCategories($extra_params)
  {
    // load language
    $lang = Factory::getLanguage();
    $lang->load('com_lartssubscriptions', JPATH_ADMINISTRATOR);

    //$sdf = new \Larts\Component\LartsSubscriptions\View\Categories\HtmlView();
    $view = new CategoriesView;
    $model = new CategoriesModel;

    $view->setArticleId($this->articleid);
    $view->setModel($model, true);
    $view->setLayout('default');
    $view->smartTagParams = $extra_params;

    $path = JPATH_SITE . '/components/com_lartssubscriptions/tmpl/categories/default.php';
    $view->setRenderLayoutPath($path);
    return $view->display();
  }

  public function renderCheckout()
  {
    // load language
    $lang = Factory::getLanguage();
    $lang->load('com_lartssubscriptions', JPATH_ADMINISTRATOR);


    $view = new CheckoutView;
    $model = new CheckoutModel;

    $view->setArticleId($this->articleid);
    $view->setModel($model, true);
    $view->setLayout('default');

    $path = JPATH_SITE . '/components/com_lartssubscriptions/tmpl/checkout/default.php';
    $view->setRenderLayoutPath($path);
    $view->addTemplatePath(JPATH_SITE . '/components/com_lartssubscriptions/views/checkout/tmpl');

    return $view->display();
  }
          
  public function UserAfterLogin($options = array())
  {
    $user = Factory::getUser();
    UserGroups::checkExpire($user->id);
  }
  
  public function AfterInitialise()
  {
    if(LartsConfig::get('use_pseudo_cron')) {
      $cron = new Cronjob;
      $cron->runPseudo();
    }
  }

}








