<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\LartsSubscriptions\Subscriptions\Extension;

defined('_JEXEC') or die();

if (!@include_once(JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/src/Extension/autoload.php'))
  return;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\Event\SubscriberInterface;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Product;
use LartsSubscriptions\UserGroups;


class SubsSubscriptions extends CMSPlugin implements SubscriberInterface
{
	var $param;
	var $comcfg;		// component config
	var $plgCfg;    // plugin config

	public function __construct(&$subject, $config)
  {
		parent::__construct($subject, $config);

    $component = ComponentHelper::getComponent('com_lartssubscriptions', true);
    if (!$component->enabled) {   
      return;
    }
    $this->param = $component->params;
    
    $this->comcfg = LartsConfig::get();

    // load language
    $lang = Factory::getLanguage();
    $lang->load('com_lartssubscriptions', JPATH_ADMINISTRATOR.'/components/com_lartssubscriptions');
  }

  public static function getSubscribedEvents(): array
	{
		return [
			'onAfterAddCartItem' => 'onAfterAddCartItem',
			'onStartRegister' => 'onStartRegister',
			'onPostRegister' => 'onPostRegister',
			'onBeforeSaveUserDetails' => 'onBeforeSaveUserDetails',
			'onAfterSaveUserDetails' => 'onAfterSaveUserDetails',
			'onGetInvoiceItemExtraTxt' => 'onGetInvoiceItemExtraTxt',
			'onAfterCreateInvoice' => 'onAfterCreateInvoice',
			'onAfterCreateOffer' => 'onAfterCreateOffer',
			'onAfterSendInvoice' => 'onAfterSendInvoice',
			'onBeforePaymentRedirect' => 'onBeforePaymentRedirect',
			'onCreateNewLicence' => 'onCreateNewLicence',
			'onAfterProcessOrderedItem' => 'onAfterProcessOrderedItem',
			'onAfterProcessedAllItems' => 'onAfterProcessedAllItems',
			'onShowPaymentMessage' => 'onShowPaymentMessage',
			'onAfterPaymentMessage' => 'onAfterPaymentMessage',
			'onBeforeRemoveSubscription' => 'onBeforeRemoveSubscription',
			'onAfterRemoveSubscription' => 'onAfterRemoveSubscription',
			'onGenerateTicketNumber' => 'onGenerateTicketNumber',
			'onPrintTicket' => 'onPrintTicket',
			'onAfterCreateTicket' => 'onAfterCreateTicket',
			'onAfterSendTicket' => 'onAfterSendTicket',
			'onForbidDownloadAccess' => 'onForbidDownloadAccess',
			'onBeforeDownloadsExport' => 'onBeforeDownloadsExport',
			'onBeforeMossExport' => 'onBeforeMossExport',
			'onBeforeUpdateOrderDiscounts' => 'onBeforeUpdateOrderDiscounts',
		];
	}

	public function onBeforeUpdateOrderDiscounts($event)
  {
		$data = $event->getArgument(0);
    //var_dump($data);
  }

	public function onAfterAddCartItem($event)
  {
		$data = $event->getArgument(0);
  }
  
	public function onStartRegister($event)
	{
		$data = $event->getArgument(0);
    return true;
  }

  public function onPostRegister($event)
  {
		$userid = $event->getArgument(0);
  }

  public function onBeforeSaveUserDetails($event)
  {
		$data = $event->getArgument(0);
  
  }
  
 	public function onAfterSaveUserDetails($event)
	{
		$data = $event->getArgument(0);
  }

  public function onGetInvoiceItemExtraTxt($event)
  {
		$data = $event->getArgument(0);
    return '';
  }

  public function onAfterCreateInvoice($event)
  {
		$data = $event->getArgument(0);
    // not check
  }
  
  public function onAfterCreateOffer($event)
  {
		$data = $event->getArgument(0);
    // not check
  }
  
  public function onAfterSendInvoice($event)
  {
		$data = $event->getArgument(0);
    //    $data = [
    //			'invoice_id'
    //			'invoice_number'
    //			'user_id'   
    //			'invoice_file'
    //		];

    // not check
  }

  public function onBeforePaymentRedirect($event)
  {
		$data = $event->getArgument(0);
    //		$data = [
    //      'ordercode' => $ordercode,
    //			'userid'    => $this->userid,
    //			'method'    => $method,
    //			'amount'    => $amount,
    //		];
  
  }
  
	public function onCreateNewLicence($event)
	{
		$data = $event->getArgument(0);
    return false;
  }
  
	public function onAfterProcessOrderedItem($event)
	{
		$data = $event->getArgument(0);
	/*
			$plg_data = [
				'product_id'    => (int) $item->product_id,
				'product_type'  => (int) $item->product_type,
				'product_ref'   => (int) $item->product_ref,
        'is_jextension' => $item->is_jextension,
        'has_licence'   => $item->has_licence,
				'userid'        => (int) $userid,
				'ordercode'     => (int) $ordercode,
				'licence'       => $item->licence,
        'valid_from'    => $newPeriod['valid_from'],
				'valid_to'      => $newPeriod['valid_to'],
				'payment_id'    => $payment_id,
        'period_id'     => (int)$newPeriod['id'],
				'order_id'      => (int) $item->id,
        'access_group'  => $item->access_group,
			];
   */

    if($data['product_type'] == Product::GROUP_ACCESS) {
      UserGroups::appendToGroup($data['userid'], $data['access_group']);
    }
    
  }
  
  public function onAfterProcessedAllItems($event)
  {
  	$data = $event->getArgument(0);
	}
  
	public function onShowPaymentMessage($event)
	{
		$data = $event->getArgument(0);
    // not check
    //		$variables = [
    //			'brutto_price'      => $amounts['gross_price'],
    //			'transaction_id'   => $transaction_data->transaction_id,
    //			'ordercode'        => $transaction_data->ordercode,
    //			'orderlist'        => $payment->showOrderedItems((int) $transaction_data->ordercode),
    //			'orderlist_simple' => $payment->showOrderedItems((int) $transaction_data->ordercode, true),
    //			'date'             => 'NOW',
    //			'userid'           => $transaction_data->userid,
    //			'download_id'      => md5($transaction_data->userid),
    //		];
	}

  public function onAfterPaymentMessage($event)
  {
		$data = $event->getArgument(0);
    
  }
  
  public function onBeforeRemoveSubscription($event)
	{
    // $data: cids
		$data = $event->getArgument(0);
	}

	public function onAfterRemoveSubscription($event)
	{
		$data = $event->getArgument(0);
		if (empty($data)) {
			return;
		}
	}

  public function onGenerateTicketNumber($event)
	{
  		$data = $event->getArgument(0);
  return null;
	}

  public function onPrintTicket($event)
	{
		$data = $event->getArgument(0);
	}

  public function onAfterCreateTicket($event)
	{
		$data = $event->getArgument(0);
	}

  public function onAfterSendTicket($event)
	{
		$data = $event->getArgument(0);
	}

  public function onForbidDownloadAccess($event)
	{
		$data = $event->getArgument(0);
	}
  
  public function onBeforeDownloadsExport($event)
	{
		$data = $event->getArgument(0);
	}

  public function onBeforeMossExport($event)
	{
		$data = $event->getArgument(0);
	}
  
}

