<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\Field\ListField;


class JFormFieldLarts_Period extends ListField
{
	public $type = 'Period';

	protected function getInput()
	{
		list($period, $type) = (strpos($this->value, ':') !== false)
			? explode(':', $this->value, 2)
			: ['', ''];

		$period_name = isset($this->element['period_name']) ? $this->element['period_name'] : 'period';
		$type_name   = isset($this->element['type_name']) ? $this->element['type_name'] : 'period_type';

		if (isset($this->form->bReadOnly) && $this->form->bReadOnly)
		{
			$text = Text::plural('COM_LARTSSUBS_' . strtoupper($type), (int) $period);

			if (!in_array($type, array('lifetime', 'onetime'))) {
				$text = (int) $period . ' ' . $text;
			}

			return '<span class="hasPopover" data-content="' . Text::_('COM_LARTSSUBS_PERIOD_FIXED') . '">'
				. '<input type="text" disabled="disabled" value="' . $text . '" />'
				. '<input type="hidden" name="' .$period_name. '" value="' . $period . '" />'
				. '<input type="hidden" name="' .$type_name. '" value="' . $type . '" />'
				. '</span>';
		}

		$script = "
			(function($) {
				$(document).ready(function() {
					updatePeriodField();
					$('#" . $type_name . "').change(function() {
						updatePeriodField();
					});
					function updatePeriodField() {
						if ($('#" . $type_name . "').val() == 'lifetime' || $('#" . $type_name . "').val() == 'onetime') {
							$('#" . $period_name . "').hide();
							return;
						}

						$('#" . $period_name . "').show();
					}
				});
			})(jQuery);";
		Factory::getDocument()->addScriptDeclaration($script);

		$options = (array) $this->getOptions();

		$html = [];

		$html[] = '<input type="text" name="' . $period_name . '" id="' . $period_name . '"'
			. ' class="form-control input-mini" size="5" maxlength="3"'
			. ' value="' . (int) $period . '" />';

		$html[] = ' ' . HTMLHelper::_('select.genericlist', $options, $type_name, ' class="form-select input-medium"', 'value', 'text', $type, $type_name);

		return implode('', $html);
	}
}
