<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\Field\ListField;

use LartsSubscriptions\Files;


class JFormFieldLarts_ServerFile extends ListField
{
	public $type      = 'ServerFile';
	public $downloads = [];

	protected function getOptions()
	{
		$options = [];

		// Set the choose option
		$options[] = HTMLHelper::_('select.option', 'none', '- ' . Text::_('COM_LARTSSUBS_SELECT_FILE') . ' -');

		// Loop through file
		foreach ($this->getZipFilesFromDirectories() as $file)
		{
			// Add the files to the dropdown box.
			$options[] = HTMLHelper::_('select.option', $file['path'], $file['name']);
		}

		// return files
		return $options;
	}

	/**
	 * Looping throug the files i nthe downloads folder.
	 *
	 * @return array
	 *
	 * @since 2.9.2
	 */
	private function getZipFilesFromDirectories()
	{
		$files = [];
    $f = new Files;
    $path = $f->getBasePath();

		// Especially for windows computers/servers.
		// Replacing the backslash with a forward slash
		$spath = str_replace('\\', '/', $path);

		// Running trough the files i nthe given folder.
		$directory = new RecursiveDirectoryIterator($path, RecursiveDirectoryIterator::SKIP_DOTS);
		$iterator  = new RecursiveIteratorIterator($directory, RecursiveIteratorIterator::LEAVES_ONLY);

		// Add the files to an array
		foreach ($iterator as $fileinfo)
		{
			if (in_array($fileinfo->getExtension(), ["zip"]))
			{
				// Make sure we're using forward slashes
				$site = str_replace('\\', '/', $fileinfo->getPathname());

				// Remove the $path from the string.
				$file = str_replace($spath, '', $site);

				// Obtain the filename as this one is shown
				$name = explode('/', $file);

				$files[] = ['path' => $file, 'name' => end($name)];
			}
		}

		return $files;
	}
}
