<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\Field\ListField;


class JFormFieldLarts_Updater extends ListField
{
	public $type = 'Updater';

	protected function getOptions()
	{
		$db = Factory::getDbo();

		//Loading the parent menu items.
		$query = $db->getQuery(true)
			->select(['id as value', 'name as text'])
			->from($db->quoteName('#__larts_subs_categories'))
			->where($db->quoteName('parentid') . ' = ' . 0)
			->order($db->quoteName('name') . ' ASC');

		$db->setQuery($query);

		$parents = $db->loadObjectList();

		// Loading the child menu items:
		$query = $db->getQuery(true)
			->select(['id as value', 'name as text', 'parentid'])
			->from($db->quoteName('#__larts_subs_categories'))
			->where($db->quoteName('parentid') . ' > ' . 0)
			->order($db->quoteName('name') . ' ASC');

		$db->setQuery($query);

		$childs = $db->loadObjectList();

		$options = parent::getOptions();

		if(!$this->multiple && empty($options))
		{
			$options[] = HTMLHelper::_('select.option', '', '- ' . Text::_('COM_LARTSSUBS_SELECT_CATEGORY') . ' -');
		}

		// Loop through the parent categories.
		foreach ($parents as $category)
		{
			// Adding the option to the menu.
			$options[] = HTMLHelper::_('select.option', $category->value, $category->text);

			// Loop through childs and show only for this parent.
			foreach ($childs as $child)
			{
				if ($category->value != $child->parentid)
					continue;

				// Adding the option to the menu.
				$options[] = HTMLHelper::_('select.option', $child->value, ' -- ' . $child->text);
			}
		}

		return $options;
	}
}
