<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\Categories;


class CategoriesModel extends ListModel
{
	public    $id          = null;
	public    $list        = null;
	public    $data        = null;
	protected $_total      = null;
	protected $pagination = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'id', 'a.id',
				'published', 'a.published',
        'ordering', 'a.ordering',
        'cattype', 'a.cattype',
        'name', 'a.name',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'string'));

    $this->setState('list.ordering', $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'a.ordering', 'string'));
		$this->setState('list.direction', $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_ordering_dir', 'DESC', 'string'));

    //$listOrdering  = $this->getState('list.ordering', 'a.ordering');
    //$orderDirn     = $this->getState('list.direction', 'ASC');

		parent::populateState('a.ordering', 'asc');
	}
  
	public function getPagination()
	{
		if(!is_null($this->pagination)) {
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		$this->_total = $this->_getListCount($query);

		return $this->_total;
	}

	public function getList()
	{
		if(!is_null($this->list)) {
			return $this->list;
		}

		$countCategory = $this->getRelatedProductsForCategory();
		$countFiles    = $this->getRelatedFilesForCategory();

		$db = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));

		$list = $db->loadObjectList();
		$this->list = [];

		foreach ($list as &$item)
		{
			$item->products = isset($countCategory[$item->id]) ? $countCategory[$item->id] : null;
			$item->files    = isset($countFiles[$item->id]) ? $countFiles[$item->id] : null;

			$this->list[] = $item;

			if(!$item->has_children)
			{
				continue;
			}

			$children = $this->getChildrenById($item->id);

			// Add file and product count to children
			array_walk($children,
				function (&$child) use ($countCategory, $countFiles)
				{
					$child->products = isset($countCategory[$child->id]) ? $countCategory[$child->id] : null;
					$child->files    = isset($countFiles[$child->id]) ? $countFiles[$child->id] : null;
				}
			);

			$this->list = array_merge($this->list, $children);
		}

		return $this->list;
	}

	private function _buildListQuery()
	{
		$published  = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'string');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['a.*', 'child.id as has_children', 'parent.name as parent_name'])
		            ->from($db->quoteName('#__larts_subs_categories', 'a'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_categories', 'child') . ' ON ' . $db->quoteName('child.parentid') . ' = ' . $db->quoteName('a.id'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_categories', 'parent') . ' ON ' . $db->quoteName('parent.id') . ' = ' . $db->quoteName('a.parentid'))
		            ->where($db->quoteName('a.parentid') . ' = 0')
		            ->group($db->quoteName('a.id'));

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$where = [
				$db->quoteName('a.name') . $like_filter,
				$db->quoteName('child.name') . $like_filter,
			];
			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if($published != null) {
			$query->where($db->quoteName('a.published') . ' = ' . $published);
		}

    $listOrdering  = $this->getState('list.ordering', 'a.ordering');
    $orderDirn     = $this->getState('list.direction', 'ASC');
    $query->order($db->escape($listOrdering) . ' ' . $db->escape($orderDirn));
   
		return $query;
	}

	private function getChildrenById($id = 0)
	{
		if(!$id)
		{
			return [];
		}

		$filter_search = Factory::getApplication()->getUserStateFromRequest('categories_filter_search', 'categories_filter_search', '', 'string');
		$filter_search = strtolower(trim($filter_search));

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['a.*', '0 as has_children', 'parent.name as parent_name'])
		            ->from($db->quoteName('#__larts_subs_categories', 'a'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_categories', 'parent') . ' ON ' . $db->quoteName('parent.id') . ' = ' . $db->quoteName('a.parentid'))
		            ->where($db->quoteName('a.parentid') . ' = ' . (int) $id)
		            ->order($db->quoteName('a.ordering') . ' ASC');

		if ($filter_search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $filter_search) . '%');

			$query->where($db->quoteName('a.name') . $like_filter);
		}

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	// Getting attached products to a category.
	private function getRelatedProductsForCategory()
	{
		$list = [];

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['category_id as id ,COUNT(product_id) as total'])
		            ->from($db->quoteName('#__larts_subs_product_categories'))
		            ->group('category_id');

		$db->setQuery($query);

		$results = $db->loadObjectList();

		foreach ($results as $result)
		{
			$list[$result->id] = $result->total;
		}

		return $list;
	}

	// Getting attached products to a category.
	private function getRelatedFilesForCategory()
	{
		$list = [];

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['category_id as id ,COUNT(file_id) as total'])
		            ->from($db->quoteName('#__larts_subs_file_categories'))
		            ->group('category_id');

		$db->setQuery($query);

		$results = $db->loadObjectList();

		foreach ($results as $result)
		{
			$list[$result->id] = $result->total;
		}

		return $list;
	}

	public function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_categories'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);

		return (object) $db->loadObject();
	}

	public function publish(&$pks, $value = 1)
	{
		if (empty($pks)) {
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_categories'))
		            ->set($db->quoteName('published') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

	public function saveOrdering($pks = null, $order = null)
	{
		$row = $this->getTable('categories');

		for ($i = 0; $i < count($pks); $i++)
		{
			$row->load((int) $pks[$i]);

			if ($row->ordering == $order[$i]) {
				continue;
			}

			$row->ordering = $order[$i];

			if(!$row->store()) {
				return false;
			}
		}

		return true;
	}

	public function store($data)
	{
    if(empty($data['hash'])) {
      // create access hash
      $userid = Factory::getApplication()->getIdentity()->id;
      $data['hash'] = md5($userid .'_'. time());
    }
    else {
      // do not store again and again
      unset($data['hash']);
    }

		// Set ordering to the last item if not set
    if($data['ordering'] == 0) {
      $max = Categories::getMaxItemId();
      $data['ordering'] = $max + 1;
    }
    
		$table = $this->getTable();

		if(!$table->bind($data)) {
			return false;
		}

		$table = $this->prepareTable($table);

		if(!$table->check()) {
			return false;
		}

		if(!$table->store()) {
			return false;
		}

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	protected function prepareTable($table)
	{
		if (empty($table->id)) {
			return $table;
		}

    return $table;
	}

	public function remove($cid)
	{
		if (empty($cid)) {
			return true;
		}

		$db   = Factory::getDbo();
		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_categories'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_file_categories'))
		            ->where($db->quoteName('category_id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_product_categories'))
		            ->where($db->quoteName('category_id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.category', 'category', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			return $this->getData();
		}

		return parent::loadFormData();
	}
  
	public function aliasCheck($id = null, $alias = null)
	{
		if(!$alias) {
			return;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) as total'])
		            ->from($db->quoteName('#__larts_subs_categories'))
		            ->where($db->quoteName('alias') . ' = ' . $db->quote($alias))
		            ->where($db->quoteName('published') . ' = 1');

		if ($id) {
			$query->where($db->quoteName('id') . ' != ' . $id);
		}

		$db->setQuery($query);

		return $db->loadObject()->total;
	}
  
}
