<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\VAT\VATClasses;


class CountriesModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'vatrequired',
				'published',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null));
		$this->setState('filter.vatrequired', $this->getUserStateFromRequest($this->context . '.filter.vatrequired', 'filter_vatrequired', null));

		parent::populateState('country', 'asc');
	}

	public function getPagination()
	{

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		return $this->_getListCount($query);
	}

	private function _buildListQuery()
	{
		$vat    = $this->getUserStateFromRequest($this->context . '.filter.vatrequired', 'vatrequired', null, 'int');
		$state  = $this->getUserStateFromRequest($this->context . '.filter.published', 'published', null, 'int');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'search', null, 'string');
		$search = strtolower(trim($search));

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_countries'));

		if (is_numeric($vat))
		{
			$query->where($db->quoteName('requires_vat') . ' = ' . $vat);
		}

		if (is_numeric($state))
		{
			$query->where($db->quoteName('published') . ' = ' . $state);
		}

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$where = [
				$db->quoteName('country') . $like_filter,
				$db->quoteName('country_2_code') . ' = ' . $db->quote($search),
				$db->quoteName('country_3_code') . ' = ' . $db->quote($search),
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		$query->order($db->quoteName('country'));

		return $query;
	}

	public function getList()
	{
		if(!is_null($this->list))
		{
			return $this->list;
		}

		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));

		return $db->loadObjectList();
	}

	public function getData()
	{
		if(!is_null($this->data))
		{
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_countries'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);

		return $db->loadObject();
	}

	public function publish(&$pks, $value = 1)
	{
		if (empty($pks))
		{
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_countries'))
		            ->set($db->quoteName('published') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

	public function store($data)
	{
		$data['country_3_code'] = strtoupper($data['country_3_code']);
		$data['country_2_code'] = strtoupper($data['country_2_code']);
		$data['vatrate_std'] = preg_replace('#[^0-9\.]#', '', str_replace(',', '.', $data['vatrate_std']));

		$row = $this->getTable();

		if(!$row->bind($data))
		{
			return false;
		}

		if(!$row->check())
		{
			return false;
		}

		if(!$row->store())
		{
			return false;
		}

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	public function remove($cid)
	{
		if (empty($cid))
		{
			return true;
		}

		$db   = Factory::getDbo();
		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_countries'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.country', 'country', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit') {
			return $this->getData();
		}

		return parent::loadFormData();
	}

  public function getVATClasses()
  {
    return VATClasses::loadAll();
  }

}


