<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Coupon;


class CouponsModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;
	var $config     = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'coupontype',
				'published',
				'ordering',
				'order_direction',
			];
		}

		parent::__construct($config, $factory);

		$this->config = LartsConfig::get();

		$cid = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		//$this->setState('filter.ordering', $this->getUserStateFromRequest($this->context . 'filter.ordering', 'filter_ordering', 'c.name', 'string'));
		//$this->setState('filter.order_direction', $this->getUserStateFromRequest($this->context . 'filter.order_direction', 'filter_order_direction', 'ASC', 'string'));
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.coupontype', $this->getUserStateFromRequest($this->context . 'filter.coupontype', 'filter_coupontype', null));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null));

		parent::populateState('a.id', 'asc');
	}

	public function getPagination()
	{
		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));
		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		return $this->_getListCount($query);
	}

	private function _buildListQuery()
	{
		$state  = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'int');
		$type   = $this->getUserStateFromRequest($this->context . '.filter.coupontype', 'filter_coupontype', null, 'int');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', '', 'string');
		$search = strtolower(trim($search));

		$db = Factory::getDbo();

		$last_use = $db->getQuery(true)
		               ->select(['date'])
		               ->from('#__larts_subs_cart_items')
		               ->where($db->quoteName('coupon_code') . ' = ' . $db->quoteName('c.coupon_code'))
		               ->order($db->quoteName('date') . ' DESC');

		$query = $db->getQuery(true)
		            ->select(['c.*'])
		            ->select('(' . $last_use . ' LIMIT 1) AS last_use')
		            ->from($db->quoteName('#__larts_subs_coupons', 'c'));

		if (is_numeric($state))
		{
			$query->where($db->quoteName('c.published') . ' = ' . (int) $state);
		}

		if (is_numeric($type))
		{
			$query->where($db->quoteName('c.type') . ' = ' . $type);
		}

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$where = [
				$db->quoteName('c.name') . $like_filter,
				$db->quoteName('c.coupon_code') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		$query->order($db->quoteName('c.id') . ' DESC');

		return $query;
	}

	public function getList()
	{
		if(!is_null($this->list))
		{
			return $this->list;
		}

		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		foreach ($this->list as &$item)
		{
			if (empty($item->product_ids))
			{
				$item->products = [];
				continue;
			}

			$query = $db->getQuery(true)
			            ->select(['name'])
			            ->from($db->quoteName('#__larts_subs_products'))
			            ->where($db->quoteName('id') . ' IN (' . $item->product_ids . ')');

			$db->setQuery($query);
			$item->products = $db->loadColumn();
		}

		return $this->list;
	}

	public function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_coupons'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$this->data = (object) $db->loadObject();

		$this->data->product_ids = isset($this->data->product_ids) ? explode(',', $this->data->product_ids) : [];
		$this->data->user_ids    = isset($this->data->user_ids) ? explode(',', $this->data->user_ids) : [];

		return $this->data;
	}

	public function copyCoupon($id)
	{
		if (empty($id)) {
			return true;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_coupons'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);
		$db->setQuery($query);

		$data = $db->loadAssoc();

		// Unset the ID & used column
		unset($data['id']);
    $data['used'] = 0;
    $data['created_by'] = Factory::getApplication()->getIdentity()->id;

		// Append the word [COPIED] to the name
		$data['name'] = $data['name'] . ' ' . Text::_('COM_LARTSSUBS_COPIED');

		// Explode productids to array
		if(!empty($data['product_ids']))
		{
			$data['product_ids'] = explode(',', $data['product_ids']);
		}

		$this->store($data);

		return true;
	}

	public function publish(&$pks, $value = 1)
	{
		if (empty($pks)) {
			return true;
		}

		$pks = ArrayHelper::toInteger($pks);

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_coupons'))
		            ->set($db->quoteName('published') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . implode(',', $pks) . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

	public function store($data)
	{
		// Make sure the coupon code  only contains numerals and uppercase letters
		$data['coupon_code'] = preg_replace('#[^0-9A-Z]#', '', strtoupper($data['coupon_code']));

		if (is_array($data['product_ids'])) {
			// Assign groups to the products:
			//$data['product_ids'] = ArrayHelper::toInteger($data['product_ids']);

			// Implode cids for more actions (when more selected)
			$data['product_ids'] = implode(',', $data['product_ids']);
		}

		// If full order is enabled
		if ($data['full_order'])
		{
			// Remove product_ids
			$data['product_ids'] = '';
		}

		// convert dates
		$data['valid_from'] = !empty($data['valid_from']) ? (new Date($data['valid_from']))->toSql() : null;
		$data['valid_to'] = !empty($data['valid_to']) ? (new Date($data['valid_to']))->toSql() : null;
		
		Coupon::storeCoupon($data);
	

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	public function remove($cid)
	{
		if (empty($cid))
		{
			return true;
		}

		$cid = ArrayHelper::toInteger($cid);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_coupons'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . implode(',', $cid) . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.coupon', 'coupon', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit') {
      $app = Factory::getApplication();
      $temp = (array) $app->getUserState('com_lartssubscriptions.coupon', []);
      if(!empty($temp)) {
        $this->data = new \stdClass;
        foreach ($temp as $k => $v) {
          if(!empty($v)) {
            $this->data->$k = $v;
          }
        }
        return $this->data;
      }

      if($this->id > 0) {
        return $this->getData();
      }
    }

		return parent::loadFormData();
	}
}

