<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\Base\Country;

class CustomersModel extends ListModel
{
	var $id         = null;
	var $userid     = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;
  var $customerID = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'id', 'u.id',
        'username', 'users.username',
        'lastname', 'u.lastname',
        'email', 'users.email',
				'city', 'u.city',
				'country', 'c.country',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		parent::populateState($ordering, $direction);

		$list = $this->getUserStateFromRequest($this->context . '.list', 'list', [], 'array');
		if(isset($list['ordering'])) {
			$this->setState('list.ordering', $list['ordering']);
		}
		if(isset($list['direction'])) {
			$this->setState('list.direction', $list['direction']);
		}

//		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string'));
//    $this->setState('list.ordering', $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'a.ordering', 'string'));
//		$this->setState('list.direction', $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_ordering_dir', 'DESC', 'string'));

    $this->setState('filter.is_business', $this->getUserStateFromRequest($this->context . '.filter.is_business', 'filter_is_business', null, 'int'));
		$this->setState('filter.country', $this->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null, 'int'));

	}

	function getPagination()
	{
		if(!is_null($this->pagination))
		{
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	function getTotal()
	{
		$query = $this->_getListQuery();

		return $this->_getListCount($query);
	}

	function _getListQuery()
	{
		$db  = Factory::getDbo();

	  $listOrder  = $this->getState('list.ordering', 'u.id');
    $orderDirn  = $this->getState('list.direction', 'DESC');
	  //$listOrder  = 'u.id';
    //$orderDirn  = 'DESC';

		$search      = $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', 0, 'string');
		$is_business = $this->getUserStateFromRequest($this->context . '.filter.is_business', 'filter_is_business', null);
		$country     = $this->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null);

		$query = $db->getQuery(true)
		            ->select(['u.*', 'c.country', 'users.email', 'users.username'])
		            ->from($db->quoteName('#__larts_subs_customers', 'u'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('u.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'users') . ' ON (' . $db->quoteName('u.userid') . ' = ' . $db->quoteName('users.id') . ')');

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);

			$where = [
				$fullname . $like_filter,
				$db->quoteName('u.companyname') . $like_filter,
				$db->quoteName('users.username') . $like_filter,
				$db->quoteName('users.email') . $like_filter,
			];

			if (is_numeric($search)) {
				$where[] = $db->quoteName('u.userid') . ' = ' . (int) $search;
			}

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if (is_numeric($is_business)) {
			$query->where($db->quoteName('u.is_business') . ' = ' . ($is_business != 0 ? (int) $is_business : 0));
		}

		if (!empty($country)) {
			$query->where($db->quoteName('u.country_code') . ' = ' . $db->quote($country));
		}

		$query->where($db->quoteName('u.userid') . ' != 0');

    $query->order($db->quoteName($listOrder) . ' ' . $db->escape($orderDirn));
    //$jquery = $query->__toString();

		return $query;
	}

	function getList()
	{
		if(!is_null($this->list))
		{
			return $this->list;
		}

		$db  = Factory::getDbo();
		$sql = $this->_getListQuery();

		$db->setQuery($sql, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		return $this->list;
	}

  public function lookupCustomerUserId($userid)
	{
    if (empty($userid)) {
      return null;
    }

    $db = Factory::getDbo();
    $query = $db->getQuery(true)
                ->select(['id'])
                ->from($db->quoteName('#__larts_subs_customers'))
                ->where($db->quoteName('userid') . ' = ' . (int) $userid);

    $db->setQuery($query);
    return $db->loadObject();
  }
  
	public function checkOnDoubleAccounts($userid)
	{
    // customer username already in here?
    $cust = $this->lookupCustomerUserId($userid);
    if(!empty($cust)) {
      return true;
    }
    
		return false;
	}

	function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['ju.*', 'rdu.*'])
		            ->from($db->quoteName('#__larts_subs_customers', 'rdu'))
		            ->join('LEFT', $db->quoteName('#__users', 'ju') . ' ON (' . $db->quoteName('ju.id') . ' = ' . $db->quoteName('rdu.userid') . ')')
		            ->where($db->quoteName('rdu.id') . ' = ' . (int) $this->id);
		$db->setQuery($query);
		$this->data = $db->loadObject();

		if (isset($this->data->userid)) {
			$this->customerID = $this->data->userid;
		}
  //		if (isset($this->data->country)) {
  //      $obj = Country::getCountryByCountryCode($this->data->country);
  //      $this->data->country = empty($obj) ? 0 : $obj->id;
  //		}

		return $this->data;
	}

	function store($data)
	{
//    if (!empty($data['country'])) {
//      $c = Country::getCountry($data['country']);
//      $data['country'] = $c->country_2_code;
//		}

    $row = $this->getTable();

		if(!$row->bind($data)) {
			return false;
		}

		if(!$row->check()) {
			return false;
		}

		if(!$row->store()) {
			return false;
		}

		$this->customerID = $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');

		return $this->customerID;
	}

	function getUserId()
	{
		if(!is_null($this->userid)) {
			return $this->userid;
		}

		if(!is_null($this->data) && isset($this->data->userid)) {
			$this->userid = $this->data->userid;
			return $this->userid;
		}

		if (empty($this->id)) {
			return 0;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('id')
		            ->from($db->quoteName('#__larts_subs_customers'))
		            ->where($db->quoteName('userid') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$this->userid = $db->loadResult();

		return $this->userid;
	}

	function remove($cid)
	{
		if (empty($cid))
		{
			return true;
		}

		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		$db = Factory::getDbo();

		## Delete all selected categories
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_customers'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);
		$result = $db->execute();

		if(!$result)
		{
			return false;
		}

		return true;
	}
	
  public function getOrders()
	{
		$userid = $this->getUserId();
		if (empty($userid)) {
			return [];
		}

    // SQL Access2PHP (1899.12.30 00-00-00)
    // SELECT x4oa3_larts_subs_orders.created, x4oa3_larts_subs_cart_items.ordercode, x4oa3_larts_subs_products.name, x4oa3_larts_subs_orders.status
    // FROM (x4oa3_larts_subs_cart_items RIGHT JOIN x4oa3_larts_subs_orders ON x4oa3_larts_subs_cart_items.ordercode = x4oa3_larts_subs_orders.ordercode) LEFT JOIN x4oa3_larts_subs_products ON x4oa3_larts_subs_cart_items.product_id = x4oa3_larts_subs_products.id
    // WHERE (((x4oa3_larts_subs_cart_items.product_id)<>0) AND ((x4oa3_larts_subs_orders.userid)=535))

//    $query = ' SELECT orders.created, x4oa3_larts_subs_cart_items.ordercode, x4oa3_larts_subs_products.name, x4oa3_larts_subs_orders.status'
//    .' FROM (x4oa3_larts_subs_cart_items '
//      . 'RIGHT JOIN x4oa3_larts_subs_orders ON x4oa3_larts_subs_cart_items.ordercode = x4oa3_larts_subs_orders.ordercode) '
//      . 'LEFT JOIN x4oa3_larts_subs_products ON x4oa3_larts_subs_cart_items.product_id = x4oa3_larts_subs_products.id'
//    .' WHERE (((x4oa3_larts_subs_cart_items.product_id)<>0) AND ((x4oa3_larts_subs_orders.userid)=535))';

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['orders.id, orders.created, orders.ordercode, orders.status'])
		            ->from($db->quoteName('#__larts_subs_orders', 'orders'))
		            ->where($db->quoteName('orders.userid') . ' = ' . (int) $userid)
		            ->order($db->quoteName('orders.created') . ' ASC');

		$db->setQuery($query);
    $jquery = $query->__toString();

		return $db->loadObjectList();
  }
  
	public function getProducts()
	{
		$userid = $this->getUserId();
		if (empty($userid)) {
			return [];
		}

    // SQL Access2PHP (1899.12.30 00-00-00)
    // SELECT x4oa3_larts_subs_periods.ordercode, x4oa3_larts_subs_products.name, x4oa3_larts_subs_periods.licence, x4oa3_larts_subs_periods.valid_from, x4oa3_larts_subs_periods.valid_to, x4oa3_larts_subs_products.period_type
    // FROM x4oa3_larts_subs_periods LEFT JOIN x4oa3_larts_subs_products ON x4oa3_larts_subs_periods.product_id = x4oa3_larts_subs_products.id
    // WHERE (((x4oa3_larts_subs_periods.userid)=589))

    //$query = ' SELECT #__larts_subs_periods.ordercode, #__larts_subs_products.name, #__larts_subs_periods.licence, #__larts_subs_periods.valid_from, #__larts_subs_periods.valid_to, #__larts_subs_products.period_type'
    //.' FROM #__larts_subs_periods' 
    //.' LEFT JOIN #__larts_subs_products ON #__larts_subs_periods.product_id = #__larts_subs_products.id'
    //.' WHERE (((#__larts_subs_periods.userid)=589))';

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['periods.id, periods.ordercode, periods.licence, periods.valid_from, periods.valid_to, periods.user_info, periods.internal_info, p.name, p.period_type'])
		            ->from($db->quoteName('#__larts_subs_periods', 'periods'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('periods.product_id') . ' = ' . $db->quoteName('p.id') . ')')
		            ->where($db->quoteName('periods.userid') . ' = ' . (int) $userid)
		            ->order($db->quoteName('periods.id') . ' ASC');

		$db->setQuery($query);
    $jquery = $query->__toString();

		return $db->loadObjectList();
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.customer', 'customer', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			return $this->getData();
		}

		return parent::loadFormData();
	}
}
