<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Language\Text;
use Joomla\Utilities\ArrayHelper;

use LartsSubscriptions\Files;
use LartsSubscriptions\Base\Image;


class FilesModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;
  var $_total = 0;
  var $file_id = 0;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'type',
				'name',
				'alias',
				'downloads',
				'stabillity',
				'ordering_dir',
				'published',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.category', $this->getUserStateFromRequest($this->context . 'filter.category', 'filter_category', null, 'int'));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'int'));

 		$ordering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'f.id', 'string');
		$direction = $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_direction', 'DESC', 'string');

		parent::populateState($ordering, $direction);
	}

	function getPagination()
	{
		if(!is_null($this->pagination))
		{
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	function getTotal()
	{
		$query = $this->_buildListQuery();

		$this->_total = $this->_getListCount($query);

		return $this->_total;
	}

	function getList()
	{
		if(!is_null($this->list)) {
			return $this->list;
		}

		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		// collect all categories of every single file
    foreach ($this->list as &$item)
		{
			$query = $db->getQuery(true)
			            ->select(['name as category_name'])
			            ->from($db->quoteName('#__larts_subs_file_categories', 'f'))
			            ->join('LEFT', $db->quoteName('#__larts_subs_categories', 'c') . ' ON (' . $db->quoteName('c.id') . ' = ' . $db->quoteName('f.category_id') . ')')
			            ->where($db->quoteName('file_id') . ' = ' . (int) $item->id);

			$db->setQuery($query);

			$item->categories = $db->loadColumn();
		}

		return $this->list;
	}

	private function _buildListQuery()
	{
		$app = Factory::getApplication();

		$category   = $this->getUserStateFromRequest($this->context . 'filter.category', 'filter_category', null);
		$search     = $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', 0, 'string');
		$published  = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null);

    $ordering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'f.id', 'string');
		$direction = $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_direction', 'DESC', 'string');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['f.*'])
		            ->from($db->quoteName('#__larts_subs_files', 'f'))
		            ->join('LEFT OUTER', $db->quoteName('#__larts_subs_file_categories', 'c') . ' ON (' . $db->quoteName('f.id') . ' = ' . $db->quoteName('c.file_id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_categories', 'cat') . ' ON (' . $db->quoteName('c.category_id') . ' = ' . $db->quoteName('cat.id') . ')');

		if (is_numeric($category)) {
			$query->where($db->quoteName('c.category_id') . ' = ' . ($category != 0 ? (int) $category : 0));
		}

		if (is_numeric($published)) {
			$query->where($db->quoteName('f.published') . ' = ' . $published);
		}

		if ($search) {
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$where = [
				$db->quoteName('f.filename') . $like_filter,
				$db->quoteName('f.alias') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		$query->order($ordering . ' ' . $direction)
		      ->group($db->quoteName('f.id'));

		return $query;
	}

	// File information is created for the file copy system
	function getFileInformation($file_to_copy)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('id') . ' = ' . (int) $file_to_copy);

		$db->setQuery($query);

		return $db->loadAssoc();
	}

	function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$this->data = (object) $db->loadObject();

		return $this->data;
	}

	function publish(&$pks, $value = 1)
	{
		if (empty($pks))
		{
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_files'))
		            ->set($db->quoteName('published') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

	function store($data, $categories = [], $products = [])
	{
		$jinput = Factory::getApplication()->GetInput();
		// Get the uploaded file from the POST data.
		$file_to_add  = $jinput->files->get('file_to_add', null, 'raw');
		$image = $jinput->files->get('file_image');

    // Make sure the download link is empty if not an url/link
    if($data['source_file_type'] == Files::SOURCE_TYPE_LOCAL) {
			$data['download_link'] = '';
      $data['filename_orig'] = $data['file_local'];
    }
		else if($data['source_file_type'] == Files::SOURCE_TYPE_UPLOAD) {
      if(strlen($file_to_add['name']) > 0) {
        if($file_to_add['error'] != 0) {
          return false;
        }

        // old one will be removed while saving new 
        $data['download_link'] = '';
        $data['filename_orig'] = $file_to_add['name'];
  		}
		}
		else {
			$data['filename_physical'] = '';
      $data['filename_orig'] = '';
		}

    if (!$data['use_own_filename']) {
      $data['use_filename'] = '';
    }
    
    if(empty($data['hash'])) {
      // create access hash
      $userid = Factory::getApplication()->getIdentity()->id;
      $data['hash'] = md5($userid .'_'. time());
    }
    else {
      // do not store again and again
      unset($data['hash']);
    }
    
		$row = $this->getTable();

		if(!$row->bind($data)) {
			return false;
		}

		if(!$row->check()) {
			return false;
		}

		if(!$row->store()) {
			return false;
		}

		$id = $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
		$this->file_id = $id;


		$files = new Files;
   
		// handle file...
		if ($data['source_file_type'] == Files::SOURCE_TYPE_UPLOAD && strlen($file_to_add['name']) > 0) {
			if(!$files->uploadFile($id, $file_to_add, $data['use_filename'], $data['keep_original_filename'])) {
				return false;
			}
    }
		else if ($data['source_file_type'] == Files::SOURCE_TYPE_LOCAL && strlen($data['file_local']) > 0)
		{
			if(!$files->setServerFile($id, $data['file_local'])) {
				return false;
			}
		}
    else if ($data['source_file_type'] == Files::SOURCE_TYPE_EXTERN)
		{
			if(!$files->setDirectLinkDownload($id, $data)) {
				return false;
			}
		}


    // Update related products
    if(!Files::deleteCategories($id)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }
    if(!Files::saveCategories($id, $categories)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }

		return $id;
	}

	public function saveFilenameInDatabase($filename_to_save, $id, $extension)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_files'))
		            ->set([
			            $db->quoteName('filename_physical') . ' = ' . $db->quote($filename_to_save),
			            $db->quoteName('extension') . ' = ' . $db->quote($extension),
		            ])
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}

		return true;
	}

	public function aliasCheck($alias, $file_id = null)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['COUNT("id") AS total'])
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('alias') . ' = ' . $db->quote($alias))
		            ->group($db->quoteName('alias'));

		if ($file_id)
		{
			$query->where($db->quoteName('id') . ' != ' . (int) $file_id);
		}

		$db->setQuery($query);

		$file = $db->loadObject();

		if(!empty($file) && $file->total > 0)
		{
			return false;
		}

		return true;
	}

	function getFileID() {
		return $this->file_id;
	}

	public function remove($cid)
	{
		if (empty($cid))
		{
			return true;
		}

		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select($db->quoteName(['id', 'filename_physical']))
		            ->from($db->quoteName('#__larts_subs_files'))
		            ->where($db->quoteName('id') . ' IN ' . '(' . $cids . ')')
		            ->where($db->quoteName('filename_physical') . ' != ""');

		$db->setQuery($query);
		$filedata = $db->loadObjectList();

		// download path for files.
    $files = new Files;
    $path  = $files->getBasePath();

		foreach ($filedata as $file)
		{
			if (file_exists($path . $file->filename_physical))
			{
				JFile::delete($path . $file->filename_physical);
			}
		}

		// Remove file from the file 2product table:
		if ($this->delQuery('#__larts_subs_file_categories', 'file_id', $cids) === false)
		{
			return false;
		}

		// Remove file from the file table:
		if ($this->delQuery('#__larts_subs_files', 'id', $cids) === false)
		{
			return false;
		}

		return true;
	}

	private function delQuery($table, $condition, $cids)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->delete($db->quoteName($table))
		            ->where($db->quoteName($condition) . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

	function getCategories()
	{
		$model = $this->getInstance('categories', 'LartssubsModel');

		return $model->getList(true);
	}

	public function getProducts()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['id', 'name'])
		            ->from($db->quoteName('#__larts_subs_products'))
		            //->where($db->quoteName('ispublic') . ' = 1')
		            ->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	public function getActive()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_file_categories'))
		            ->where($db->quoteName('file_id') . ' = ' . (int) $this->id);

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	public function saveorder($pks = null, $order = null)
	{
		$row = $this->getTable('files');

		for ($i = 0; $i < count($pks); $i++)
		{
			$row->load((int) $pks[$i]);

			if ($row->ordering == $order[$i])
			{
				continue;
			}

			$row->ordering = $order[$i];

			if(!$row->store())
			{
				return false;
			}
		}

		return true;
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.file', 'file', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$app  = Factory::getApplication();


    $task = $app->getInput()->getString('task');

		if ($task === 'edit')
		{
			$data = $this->getData();

			if (empty($data->id)) {
				return $data;
			}

			$data->period_and_type    = isset($data->period) ? ($data->period . ':' . $data->period_type) : '';
      $data->categories = Files::getCategories($data->id);

      if($data->source_file_type == Files::SOURCE_TYPE_LOCAL) {
        $data->file_local = $data->filename_orig;
      }
      if($data->source_file_type == Files::SOURCE_TYPE_UPLOAD) {
        $data->uploaded_file = $data->filename_orig;
      }
      
      
			return $data;
		}

		return parent::loadFormData();
	}

  

}
