<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Pagination\Pagination;
use Joomla\Utilities\ArrayHelper;


class LogfilesModel extends ListModel
{
	protected $pagination;
	protected $total;
  var $id = 0;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'country',
				'download',
				'transaction',
				'subscription',
			];
		}

		parent::__construct($config, $factory);

		$cid = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.country', $this->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null, 'string'));
		$this->setState('filter.download', $this->getUserStateFromRequest($this->context . '.filter.download', 'filter_download', null, 'int'));
		$this->setState('filter.transaction', $this->getUserStateFromRequest($this->context . '.filter.transaction', 'filter_transaction', null, 'int'));
		$this->setState('filter.subscription', $this->getUserStateFromRequest($this->context . '.filter.subscription', 'filter_subscription', null, 'int'));

		parent::populateState('l.id', 'desc');
	}

	public function getLogs()
	{
		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));

		return $db->loadObjectList();
	}

	public function getPagination()
	{
		if(!empty($this->pagination))
		{
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		$this->total = $this->_getListCount($query);

		return $this->total;
	}

	private function _buildListQuery()
	{
		$db = Factory::getDbo();

		$subscription = $this->getUserStateFromRequest($this->context . '.filter.subscription', 'filter_subscription', null, 'int');
		$request = $this->getUserStateFromRequest($this->context . '.filter.transaction', 'filter_transaction', null, 'int');
		$type    = $this->getUserStateFromRequest($this->context . '.filter.download', 'filter_download', null, 'int');
		$country = $this->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null, 'string');
		$search  = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string');

		$ordering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 'l.id', 'string');
		$direction = $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'desc', 'string');

		$query = $db->getQuery(true)
		            ->select(['l.*', 'u.id as user_id', 'u.firstname', 'u.lastname', 'user.email', 'c.country', 'user.name'])
		            ->from($db->quoteName('#__larts_subs_logging', 'l'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('l.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'user') . ' ON (' . $db->quoteName('l.userid') . ' = ' . $db->quoteName('user.id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('l.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')');

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);

			$where = [
				$fullname . $like_filter,
				$db->quoteName('u.companyname') . $like_filter,
				$db->quoteName('user.username') . $like_filter,
				$db->quoteName('user.email') . $like_filter,
				$db->quoteName('c.country') . $like_filter,
				$db->quoteName('l.ip_address') . $like_filter,
				$db->quoteName('l.action') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if($country) {
			$query->where($db->quoteName('l.country_code') . ' = ' . $db->quote($country));
		}

		if(is_numeric($type)) {
			$query->where($db->quoteName('l.type') . ' = ' . $type);
		}

		if(is_numeric($request)) {
			$query->where($db->quoteName('l.request_type') . ' = ' .$request);
		}

		if(is_numeric($subscription))
		{
			if($subscription == 1) {
				$query->where($db->quoteName('l.userid') . ' = 0');
			}
			else {
				$query->where($db->quoteName('l.userid') . ' > 0');
			}
		}

		$query->where($db->quoteName('l.userid') . ' >= 0');
		$query->order($db->quoteName($ordering) . ' ' . $direction);

		return $query;
	}

	public function remove($cid)
	{
		if (empty($cid)) {
			return true;
		}
		$cids  = ArrayHelper::toInteger($cid);

    $db   = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_logging'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' .implode(',', $cids). ' )');

		$db->setQuery($query);

    return $db->execute();
	}
  
  
  
  
  
  
}


