<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\VAT\VATClasses;
use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\VAT\MOSS;
use LartsSubscriptions\VAT\VATItem;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\Order;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\Cart;

class MossModel extends ListModel
{
	public    $id          = null;
	public    $list        = null;
	public    $data        = null;
	protected $_total      = null;
	protected $_pagination = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'year', 'quater',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
    $y = $this->getUserStateFromRequest($this->context . 'year', 'year', null, 'int');
    $q = $this->getUserStateFromRequest($this->context . 'quater', 'quater', null, 'int');
    
    if(empty($y)) { $y = date('Y'); }
    if(empty($q)) { $q = DateEx::getQuater(Date('Y-m-d')); }
   
		$this->setState('year', $y);
		$this->setState('quater', $q);
    
		parent::populateState('id', 'asc');
	}

	public function getPagination()
	{
		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		$this->_total = $this->_getListCount($query);

    return $this->_total;
	}

  public function getItems()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$y = $this->getState('year');
		$q = $this->getState('quater');
    $dRange = DateEx::getDateRangeForQuarter($q, $y);

    // get all ordercode of all transactions of date range
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('DISTINCT t.ordercode')
		            ->from($db->quoteName('#__larts_subs_transactions', 't'))
		            ->where($db->quoteName('t.status'). '='. Transaction::PROCESSED)
                ->andWhere($db->quoteName('t.deleted'). '= false')    // is not deleted
                ->andWhere('(' .$db->quoteName('t.date'). '>=' .$db->quote($dRange['begin']). 'AND'
                        .$db->quoteName('t.date'). '<=' .$db->quote($dRange['end']). ')')
                ;
		$db->setQuery($query);
		$transactions = $db->loadObjectList();

    $moss = new MOSS;

    foreach ($transactions as $trans) {
      // get items of order
      $order = Order::getOrderByOrdercode($trans->ordercode);
      if(empty($order))
        continue;
      
      $amounts = json_decode($order->amounts);
      
      // check if order is fully paid
      if(($order->status & OrderStatus::PAID) == 0)
        continue;

      foreach ($amounts->vat_bypercent as $vat => $vatval)
      {
        $item = new VATItem;
        $item->country_code = $order->country_code;
        $item->vatRate = $vat;
        $item->refunds = 0;
        if($amounts->gross_price >= 0) {
          $item->sales = $amounts->gross_price;
          $item->vatCharged = $amounts->vat_total;
        }
        else {
          $item->refunds = $amounts->gross_price;
          $item->vatRefunded = $amounts->vat_total;
        }

        $moss->add($item);
      }
      
    } // foreach

    return $moss->getItems();
   
    
	}

  public function getList($get_all = false)
	{
 		if(!is_null($this->list))
		{
			return $this->list;
		}

		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query);

		if(!$get_all) {
			$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		}

		$this->list = $db->loadObjectList();

		return $this->list;
	}

	private function _buildListQuery()
	{
		$state  = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'string');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string');

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->order($db->quoteName('name') . ' ASC');
		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');
			$where = [
				$db->quoteName('name') . $like_filter,
			];
			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if(!empty($state)) {
			$query->where($db->quoteName('published') . ' = ' . $state);
		}

		return $query;
	}

}
