<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\User\User;
use Joomla\CMS\Component\ComponentHelper;

use LartsSubscriptions\Coupon;
use LartsSubscriptions\CheckoutManager;
use LartsSubscriptions\UserRegState;


class OrderModel extends ListModel
{
  var $ordercode = 0;
  var $session= null;
  var $data;
  var $message;
  
	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
    $this->session = Factory::getSession();
    $this->ordercode = intval(Factory::getSession()->get('ordercode'));
    
		parent::__construct($config, $factory);
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.order', 'order', ['load_data' => $loadData]);
	}

 	protected function loadFormData()
	{
    // restore user input
    $app = Factory::getApplication();
    $temp = UserRegState::getStateData();
    $this->data = new \stdClass;
    foreach ($temp as $k => $v) {
      if(!empty($v)) {
        $this->data->$k = $v;
      }
    }
    return $this->data;

		//return parent::loadFormData();
	}
  
	public function loadCustomerData($email)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['id'])
			->from($db->quoteName('#__users'))
			->where($db->quoteName('email') . ' = ' . $db->quote($email));

		$db->setQuery($query);
		$data = $db->loadObject();

		$session = Factory::getSession();
		$session->set('order_userid', $data->id);

		return $data->id;
	}

	public function register($temp, $configuration = [])
	{
		// Initialise the table with JUser.
		$user = new User;
		$data = (array) $this->getUserData();

		// Merge in the registration data.
		foreach ($temp as $k => $v)
		{
			$data[$k] = $v;
		}

		//jimport('joomla.user.helper');
		$data['activation'] = '';
		$data['block']      = 0;

		// Bind the data.
		if(!$user->bind($data))
		{
			$this->setError(Text::sprintf('COM_USERS_REGISTRATION_BIND_FAILED', $user->getError()));

			return false;
		}

		// Load the users plugin group.
		PluginHelper::importPlugin('user');

		// Store the data.
		$user->save();

		// Return the userid.
		return $user->id;
	}

	public function getUserData()
	{
		if(!is_null($this->data))
		{
			return $this->data;
		}

		$this->data = new \stdClass;
		$app        = Factory::getApplication();
		$params     = ComponentHelper::getParams('com_users');

		// Override the base user data with any data in the session.
		$temp = UserRegState::getStateData();
		foreach ($temp as $k => $v)
		{
			$this->data->$k = $v;
		}

		// Get the groups the user should be added to after registration.
		$this->data->groups = isset($this->data->groups) ? array_unique($this->data->groups) : [];

		// Get the default new user group, Registered if not specified.
		$system = $params->get('new_usertype', 2);

		$this->data->groups[] = $system;

		// Unset the passwords.
		unset($this->data->password1);
		unset($this->data->password2);

		// Trigger user data preparation event.
    //PluginManager::execPlugin($group, $plugin, 'onContentPrepareData', ['com_users.registration', $this->data]);
		PluginHelper::importPlugin('user');
		$results = Factory::getApplication()->triggerEvent('onContentPrepareData', ['com_users.registration', $this->data]);

		// Check for errors encountered while preparing the data.
		if (count($results) && in_array(false, $results, true))
		{
			//$this->setError($dispatcher->getError());
			$this->data = false;
		}

		return $this->data;
	}

	public function applyCoupon($coupon_code)
	{
		$ordercode   = intval(Factory::getSession()->get('ordercode'));
		$replace     = [":", "/", "\\", "@", "#", "@", "!", "$", "?"];
		$coupon_code = str_replace($replace, "", $coupon_code);
		$coupon_code = strtoupper($coupon_code);

		$coupon = new Coupon($ordercode);

		if ($check = $coupon->check($coupon_code))
		{
			$db = Factory::getDbo();

			$query = $db->getQuery(true)
				->select('*')
				->from($db->quoteName('#__larts_subs_cart_items'))
				->where($db->quoteName('ordercode') . ' = ' . $ordercode)
				->where($db->quoteName('product_id') . ' > 0');

			$db->setQuery($query);
			$order = $db->loadObjectList();

			$coupon->apply($order, $coupon_code);
		}

		$this->message = $coupon->getMessage();

		return ($check !== false);
	}

	public function getMessage()
	{
		return $this->message;
	}

	public function getOrder($ordercode = 0)
	{
    $ordercode = $ordercode ?: $this->ordercode;
      
		$db = Factory::getDbo();

		// Get the global DB session
		//$session = Factory::getSession();

		$query = $db->getQuery(true);

//		// Check if there is an active coupon
//		$query->select(['COUNT(id) AS total', 'coupon_code'])
//			->from($db->quoteName('#__larts_subs_cart_items'))
//			->where($db->quoteName('ordercode') . ' = ' . (int) $this->ordercode)
//			->where($db->quoteName('coupon_code') . ' != ""')
//			->group($db->quoteName('coupon_code'));
//
//		$db->setQuery($query);
//		$order = $db->loadObject();
//
//		$order_total = isset($order->total) ? $order->total : 0;
//
//		if ($order_total >= 1) {
//			$coupon = new Coupon;
//			$coupon->check($order->coupon_code);
//		}

		$query = $db->getQuery(true)
			->select([
				'o.*',
				'p.name as productname', 'p.price', 'p.period', 'p.period_type', 'p.id AS product',
				//'COUNT( p.id ) AS count_products',
			])
			->from($db->quoteName('#__larts_subs_cart_items', 'o'))
			->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('o.product_id') . ' = ' . $db->quoteName('p.id') . ')')
			->where($db->quoteName('o.ordercode') . ' = ' . (int)$ordercode)
			->order($db->quoteName('o.discount') . ' ASC')
			->group($db->quoteName('o.product_id'));

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	public function store($data)
	{
		$row = $this->getTable('order');

		// Bind the form fields to the web link table
		if(!$row->bind($data)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Make sure the web link table is valid
		if(!$row->check()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Store the web link table to the database
		if(!$row->store()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	public function storeCustomer($data)
	{
    $cm = new CheckoutManager;
    return $cm->RegisterNewUser($data);


    $row = $this->getTable('users');

		if (empty($data['userid']))
		{
			return false;
		}

		// Bind the form fields to the web link table
		if(!$row->bind($data)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Make sure the web link table is valid
		if(!$row->check()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Store the web link table to the database
		if(!$row->store()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		return true;
	}

	public function getUserinfo()
	{
		$session = Factory::getSession();
		$db      = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['u.*', 'uu.email'])
			->from($db->quoteName('#__larts_subs_customers', 'u'))
			->join('LEFT', $db->quoteName('#__users', 'uu') . ' ON (' . $db->quoteName('uu.id') . ' = ' . $db->quoteName('u.userid') . ')')
			->where($db->quoteName('u.userid') . ' = ' . $db->quote($session->get('order_userid')));

		$db->setQuery($query);

		return $db->loadObject();
	}

	public function getProducts()
	{
    $jinput = Factory::getApplication()->GetInput();
    $product_filter = $jinput->get('filter', '');
    
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['id', 'name', 'period_type', 'period'])
			->from($db->quoteName('#__larts_subs_products'))
			->where($db->quoteName('published') . ' = 1')
			->order($db->quoteName('ordering') . ' ASC');
    
    if(!empty($product_filter)) {
			$query->andWhere($db->quoteName('name') . ' LIKE ' .$db->quote('%'.$product_filter.'%'));
    }
      

		$db->setQuery($query);
		$this->data = $db->loadObjectList();

		return $this->data;
	}

	public function removeFromCart($ordercode, $product_id)
	{
		$db = Factory::getDbo();

		// delete product from cart.
		$query = $db->getQuery(true)
			->delete($db->quoteName('#__larts_subs_cart_items'))
			->where([
				$db->quoteName('product_id') . ' = ' . (int) $product_id,
				$db->quoteName('ordercode') . ' = ' . (int) $ordercode,
			]);

		$db->setQuery($query);
		return $db->execute();
	}
}
