<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;

use LartsSubscriptions\OrderStatus;

class OrdersModel extends ListModel
{
	protected $id = 0;
	protected $list = null;
	protected $_pagination;
	protected $_total = 0;

	// LartssubsModelPendings constructor.
	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'id', 'o.id',
				'ordercode', 'o.ordercode',
				'lastname', 'u.lastname',
				'created', 'o.created',
			];
		}

		parent::__construct($config, $factory);

		$app = Factory::getApplication();

//		$limit      = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
//		$limitstart = $app->input->get('limitstart', 0, 'int');
//		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
//
//		$this->setState('limit', $limit);
//		$this->setState('limitstart', $limitstart);

		$array    = $app->input->get('cid', [0], 'array');
		$this->id = (int) $array[0];

  }

  protected function populateState($ordering = 'o.id', $direction = 'asc')
	{
		parent::populateState($ordering, $direction);

		$list = $this->getUserStateFromRequest($this->context . '.list', 'list', [], 'array');
		if(isset($list['ordering'])) {
			$this->setState('list.ordering', $list['ordering']);
		}
		if(isset($list['direction'])) {
			$this->setState('list.direction', $list['direction']);
		}
	}

	// Loading pagination
	public function getPagination()
	{
		if(!is_null($this->pagination)) {
			return $this->pagination;
		}

		//$this->pagination = new Pagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit'));
		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));
		return $this->pagination;
	}

	// Loading total items from the database.
	public function getTotal()
	{
		$query = $this->_buildListQuery();
		//$this->_total = $this->_getListCount($this->getListQuery(), $this->getState('limitstart'), $this->getState('limit'));
		$this->_total = $this->_getListCount($query);
		return $this->_total;
	}

	// Getting all items from the database.
	public function getList()
	{
		if(!is_null($this->list)) {
			return $this->list;
		}

    $db = Factory::getDBO();

		$query = $this->_buildListQuery();
		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
    
		$this->list = $db->loadObjectList();
		return $this->list;
	}

	// Getting all items from the database.
	protected function _buildListQuery()
	{
		$db = Factory::getDbo();

		//$listOrdering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 'o.id', 'string');
		//$orderDirn = $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'desc', 'string');
	  $listOrder  = $this->getState('list.ordering', 'o.id');
    $orderDirn  = $this->getState('list.direction', 'DESC');
    $search     = $this->getUserStateFromRequest($this->context . '.filter.search', 'search', null, 'string');
    $status = OrderStatus::PENDING;
    
		$query = $db->getQuery(true)
		            ->select(['o.*', 'u.id as user_id', 'u.firstname', 'u.lastname', 'user.email'])
		            ->from($db->quoteName('#__larts_subs_orders', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('o.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'user') . ' ON (' . $db->quoteName('o.userid') . ' = ' . $db->quoteName('user.id') . ')')
		            ->where('('.$db->quoteName('o.status') .'& '. $status.') = 0')
            ;

    if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);

			$where = [
				$fullname . $like_filter,
				$db->quoteName('o.ordercode') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		$query->order($db->quoteName($listOrder) . ' ' . $db->escape($orderDirn));
    //$jquery = $query->__toString();
		return $query;
	}

	// Getting a single item from the database.
	public function getData()
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName('id') . ' = ' . $this->id);

		$db->setQuery($query);

		return $db->loadObject();
	}
}
