<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\OrderStatus;

class PendingsModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;

	/**
	 * LartssubsModelPendings constructor.
	 * @throws Exception
	 */
	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'status',
				'ordering',
				'direction',
			];
		}

		parent::__construct($config, $factory);

		$cid = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('list.ordering', $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 'o.created', 'string'));
		$this->setState('list.direction', $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'desc', 'string'));

		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));

		parent::populateState($ordering, $direction);
	}

	public function getPagination()
	{
		if(!is_null($this->pagination))
		{
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_getListQuery();

		return $this->_getListCount($query);
	}

	public function getList()
	{
		$db = Factory::getDBO();

		$query = $this->_getListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		return $this->list;
	}

	protected function _getListQuery()
	{

		$db = Factory::getDbo();

		$ordering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 'o.id', 'string');
		$direction = $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'ASC', 'string');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string');

		$query = $db->getQuery(true)
		            ->select(['o.*', 'u.id as user_id', 'u.firstname', 'u.lastname', 'user.email'])
		            ->from($db->quoteName('#__larts_subs_orders', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('o.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'user') . ' ON (' . $db->quoteName('o.userid') . ' = ' . $db->quoteName('user.id') . ')')
		            ->where('(('.$db->quoteName('o.status') .'&'.OrderStatus::PENDING. ') != 0)')
		            ->andwhere($db->quoteName('o.is_offer') .' = 0')
                 ;

    
    if ($search)
		{
			if (is_numeric($search)) {
  			$query->where($db->quoteName('o.ordercode') . ' = ' . (int) $search);
			}
      
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);

			$where = [
				$fullname . $like_filter,
				$db->quoteName('user.email') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		$query->order($db->quoteName($ordering) . ' ' . $direction);
    
    //$jquery = $query->__toString();
		return $query;
	}

	public function getData()
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_orders'))
		            ->where($db->quoteName('id') . ' = ' . $this->id);

		$db->setQuery($query);

		return $db->loadObject();
	}
}
