<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Language\Associations;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\Image;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\BundledProducts;
use LartsSubscriptions\ThirdParty\LartsPkgManager;
use LartsSubscriptions\Product;

class ProductsModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;
	var $_total     = 0;
  var $ordering = '';

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'id', 'a.id',
        'name', 'a.name',
        'price', 'a.price',
        'created', 'a.created',
        'updated', 'a.updated',
				'published', 'a.published',
        'ordering', 'a.ordering',
        'type', 'a.type',
        'period_type', 'a.period_type',
			];

      $assoc = Associations::isEnabled();
			if ($assoc) {
				$config['filter_fields'][] = 'association';
			}
    }

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->getInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	//  Populating the state of the dropdowns.
	protected function populateState($ordering = null, $direction = null)
	{
    $app = Factory::getApplication();
    $formSubmitted = $app->input->post->get('form_submitted');

    $this->setState('filter.product_type', $this->getUserStateFromRequest($this->context . '.filter.product_type', 'filter_product_type', null, 'int'));
		$this->setState('filter.period_type', $this->getUserStateFromRequest($this->context . '.filter.period_type', 'filter_period_type'));
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search'));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published'));

    $search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
    $this->setState('filter.search', $search);

    $published = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', '');
    $this->setState('filter.published', $published);

    $ordering = $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'a.ordering', 'string');

    $this->setState('list.ordering', $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'a.ordering', 'string'));
		$this->setState('list.direction', $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_ordering_dir', 'DESC', 'string'));

		parent::populateState($ordering, $direction);
  }

	public function getPagination()
	{
		if(!is_null($this->pagination)) {
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	// Getting the total items which are going to be loaded to define the pagination.
	public function getTotal()
	{
		$query = $this->_buildListQuery();
		$this->_total = $this->_getListCount($query);

		return $this->_total;
	}

	// Getting all products for the list in the backend
	public function getList()
	{
		if(!is_null($this->list)) {
			return $this->list;
		}
    
		$db = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		foreach ($this->list as &$item)
		{
			$item->parents  = $item->has_parents ? BundledProducts::getParentsById($item->id, 1) : [];
			$item->children = $item->has_children ? BundledProducts::getChildrenById($item->id, 1) : [];
		}

		return $this->list;
	}

	// The query which is going to be used fo the listing and the pagination.
	private function _buildListQuery()
	{
		$db = Factory::getDbo();

	  $listOrder  = $this->getState('list.ordering', 'a.ordering');
    $orderDirn  = $this->getState('list.direction', 'ASC');
		$published  = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'string');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string');

		$period    = $this->getUserStateFromRequest($this->context . '.filter.period_type', 'filter_period_type', null, 'string');
		$published = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'int');
		$type      = $this->getState('filter.product_type');

    //$this->ordering = $listOrdering;  // save for view use
    
		$total = $db->getQuery(true)
		            ->select(['COUNT(id)'])
                ->from($db->quoteName('#__larts_subs_periods'))
		            ->where($db->quoteName('product_id') . ' = ' . $db->quoteName('a.id'))
		            ->where($db->quoteName('published') . ' = 1');

		$has_parents = $db->getQuery(true)
		                  ->select(['COUNT(product_id)'])
		                  ->from($db->quoteName('#__larts_subs_products_bundled'))
		                  ->where($db->quoteName('bundled_product_id') . ' = ' . $db->quoteName('a.id'));

		$has_children = $db->getQuery(true)
		                   ->select(['COUNT(bundled_product_id)'])
		                   ->from($db->quoteName('#__larts_subs_products_bundled'))
		                   ->where($db->quoteName('product_id') . ' = ' . $db->quoteName('a.id'));

		$query = $db->getQuery(true)
		            ->select('a.*')
		            ->select('(' . $total . ') AS total_sold')
		            ->select('(' . $has_parents . ') AS has_parents')
		            ->select('(' . $has_children . ') AS has_children')
		            ->from($db->quoteName('#__larts_subs_products', 'a'));

		if ($period) {
			$query->where($db->quoteName('a.period_type') . ' = ' . $db->quote($period));
		}

		if ($search) {
			$query->where($db->quoteName('a.name') . ' LIKE ' . $db->quote('%' . $search . '%'));
		}

		if (is_numeric($published)) {
			$query->where($db->quoteName('a.published') . ' = ' . $published);
		}

		if (is_numeric($type)) {
			$query->where($db->quoteName('a.type') . ' = ' . $type);
		}

    $query->order($db->escape($listOrder) . ' ' . $db->escape($orderDirn));

		return $query;
	}

	// Getting the total amount of active clients to show in the list of products.
	public function getActiveClients()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['count(product_id)', 'product_id'])
		            ->from($db->quoteName('#__larts_subs_periods'))
		            ->group($db->quoteName('product_id'));

		$db->setQuery($query);

		return $db->loadAssocList();
	}

	// Getting details of a product to be shown in an edit page.
	public function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

    if($this->id == 0) {
      return null;
    }
      
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('id') . '= ' . (int) $this->id);

		$db->setQuery($query);

		return (object) $db->loadObject();
	}

	public function publish(&$pks, $value = 1)
	{
		if(!count($pks))
		{
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_products'))
		            ->set($db->quoteName('published') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

	public function store($data, $categories = [], $relatedProds = [], $bundledProds = [])
	{
		// Make sure the price is numerical
		$data['price'] = preg_replace('#[^0-9\.]#', '', str_replace(',', '.', $data['price']));
    $data['price'] = empty($data['price']) ? '0' : $data['price'];
		$data['shipping'] = preg_replace('#[^0-9\.]#', '', str_replace(',', '.', $data['shipping']));
    $data['shipping'] = empty($data['shipping']) ? '0' : $data['shipping'];

		// convert dates
		if(isset($data['created'])) {
			unset($data['created']);
		}
		$data['updated'] = (new Date($data['updated']))->toSql();

		// Set ordering to the last item if not set
    if($data['ordering'] == 0) {
      $max = Product::getMaxItemId();
      $data['ordering'] = $max + 1;
    }


		$row = $this->getTable();

		if(!$row->bind($data)) {
			return false;
		}

		if(!$row->check()) {
			return false;
		}

		if(!$row->store()) {
      $err = $row->getError();
			return false;
		}

		$jinput = Factory::getApplication()->GetInput();

		$id       = $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
		$this->id = $id;


    // Update related products
    if(!$this->deleteReleatedProducts($id)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }
    if(!$this->saveReleatedProducts($id, $relatedProds)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }
    
    // Update bundled products
    if(!$this->deleteBundledProducts($id)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }
    if(!$this->saveBundledProducts($id, $bundledProds)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }

    // Update products categories
    if(!$this->deleteProductCategories($id)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }
    if(!$this->saveProductCategories($id, $categories)) {
 			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_ERROR'), 'error');
			return false;
    }

		return $id;
	}

	public function saveOrdering($pks = null, $order = null)
	{
		$row = $this->getTable('products');

		for ($i = 0; $i < count($pks); $i++)
		{
			$row->load((int) $pks[$i]);

			if ($row->ordering == $order[$i]) {
				continue;
			}

			$row->ordering = $order[$i];

			if(!$row->store()) {
				return false;
			}
		}

		return true;
	}

	public function remove($cid)
	{
		if (empty($cid)) {
			return true;
		}

		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		$db = Factory::getDbo();

		// Delete the selected products
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute()) {
			return false;
		}

		// Delete all product childs with the selected products.
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_products_bundled'))
		            ->where($db->quoteName('product_id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);
		if(!$db->execute()) {
			return false;
		}

		// Delete all product categories with the selected products.
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_product_categories'))
		            ->where($db->quoteName('product_id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);
		if(!$db->execute()) {
			return false;
		}

		// Delete all product to user with the selected products.
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_periods'))
		            ->where($db->quoteName('product_id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);
		if(!$db->execute()) {
			return false;
		}

		return true;
	}

	public function getCategories()
	{
    /** @var \Joomla\Component\Categories\Administrator\Model\CategoryModel $categoryModel */
    //$categoryModel = Factory::getApplication()->bootComponent('com_categories')
    //    ->getMVCFactory()->createModel('Category', 'Administrator', ['ignore_request' => true]);

    $model = new CategoriesModel();
		//$model = $this->getInstance('categories', 'LartssubsModel');

		return $model->getList(true);
	}


	public function getProducts()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['id', 'name'])
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query);

		return $db->loadObjectList();
	}

  public function saveProductCategories($product_id, $categories)
  {
		// Add the new categories to the DB.
		foreach ($categories as $cat)
		{
			$item = new \stdClass;
			$item->product_id = $product_id;
			$item->category_id  = $cat;

      if(!Factory::getDbo()->insertObject('#__larts_subs_product_categories', $item)) {
        return false;
      }
		}
    return true;
  }

  public function getProductCategories($product_id, $asIdList)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_product_categories'))
		            ->where($db->quoteName('product_id') . ' = ' . (int) $product_id);

		$db->setQuery($query);

		$result = $db->loadObjectList();
    
    if(!$asIdList) {
  		return $result;
    }
 
    $list = [];
    foreach ($result as $item) {
      $list[] = $item->category_id;
    }

		return $list;
	}

  	function deleteProductCategories($product_id)
  {
  		// Delete all selected categories
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_product_categories'))
		            ->where($db->quoteName('product_id') . ' = ' . (int) $product_id);
		$db->setQuery($query);

		return $db->execute();
  }

  public function saveBundledProducts($product_id, $products)
  {
		// Add the new categories to the DB.
		foreach ($products as $prod)
		{
			$item = new \stdClass;
			$item->product_id = $product_id;
			$item->bundled_product_id  = $prod;

      if(!Factory::getDbo()->insertObject('#__larts_subs_products_bundled', $item)) {
        return false;
      }
		}
    return true;
  }

  public function getBundledProducts($product_id, $asIdList)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_products_bundled'))
		            ->where($db->quoteName('product_id') . ' = ' . (int) $product_id);

		$db->setQuery($query);
		$result = $db->loadObjectList();
    
    if(!$asIdList) {
  		return $result;
    }
 
    $list = [];
    foreach ($result as $item) {
      $list[] = $item->bundled_product_id;
    }

		return $list;
	}

  public function deleteBundledProducts($product_id)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_products_bundled'))
		            ->where($db->quoteName('product_id') . ' = ' . (int)$product_id);
		$db->setQuery($query);
		return $db->execute();
  }

  public function saveReleatedProducts($product_id, $products)
  {
		// Add the new categories to the DB.
		foreach ($products as $prod)
		{
			$item = new \stdClass;
			$item->product_id = $product_id;
			$item->related_product_id = $prod;

      if(!Factory::getDbo()->insertObject('#__larts_subs_products_related', $item)) {
        return false;
      }
		}
    return true;
  }

  public function getReleatedProducts($product_id, $asIdList)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_products_related'))
		            ->where($db->quoteName('product_id') . ' = ' . (int) $product_id);

		$db->setQuery($query);
		$result = $db->loadObjectList();
    
    if(!$asIdList) {
  		return $result;
    }
 
    $list = [];
    foreach ($result as $item) {
      $list[] = $item->related_product_id;
    }

		return $list;
	}

  public function deleteReleatedProducts($product_id)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_products_related'))
		            ->where($db->quoteName('product_id') . ' = ' . (int)$product_id);
		$db->setQuery($query);
		return $db->execute();
  }
  
	public function aliasCheck($productid = null, $alias = null)
	{
		if(!$alias) {
			return;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) as total'])
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('alias') . ' = ' . $db->quote($alias))
		            ->where($db->quoteName('published') . ' = 1');

		if ($productid) {
			$query->where($db->quoteName('id') . ' != ' . $productid);
		}

		$db->setQuery($query);

		return $db->loadObject()->total;
	}

	public function getUserGroups()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['id', 'title'])
		            ->from($db->quoteName('#__usergroups'))
		            ->group($db->quoteName('id'))
		            ->order($db->quoteName('id') . ' ASC');

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	public function getForm($data = [], $loadData = true)
	{
		$form = $this->loadForm('com_lartssubscriptions.product', 'product', ['load_data' => $loadData]);
    
    // remove field if not installed
    if(!LartsPkgManager::IsInstalled()) {
      $element = $form->removeField('pkgmanager_id', '');
    }
    
    return $form;
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			$data = $this->getData();

			if (empty($data->id)) {
				return $data;
			}

			$data->period_and_type    = isset($data->period) ? ($data->period . ':' . $data->period_type) : '';
			$data->product_image_file = $this->getImageName();

      $data->categories = $this->getProductCategories($data->id, true);
      $data->bundled_products = $this->getBundledProducts($data->id, true);
      $data->related_products = $this->getReleatedProducts($data->id, true);

      $type = $data->type;
      switch($type) {
      case Product::FILE:
      case Product::MOVIE_STREAM:
        $data->file_id = $data->source_id;
        break;
      case Product::TICKET:
        $data->template_id = $data->source_id;
        break;
      case Product::CATEGORY:
        $data->category_id = $data->source_id;
        break;
      case Product::GROUP_ACCESS:
        $data->group_id = $data->source_id;
        break;
      }
      
			return $data;
		}

		return parent::loadFormData();
	}

	private function getImageName()
	{
		$data = $this->getData();

		if (empty($data->id)) {
			return '';
		}

		return Image::getUrl($data->image);
	}

	public function getProductInformation($product = null)
	{
		if(!$product)
		{
			return;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('id') . ' = ' . (int) $product);

		$db->setQuery($query);

		return $db->loadAssoc();
	}

	public function ProductCategories($product = null)
	{
		$categories = [];

		if(!$product)
		{
			return;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['category_id'])
		            ->from($db->quoteName('#__larts_subs_product_categories'))
		            ->where($db->quoteName('product_id') . ' = ' . (int) $product);

		$db->setQuery($query);

		foreach ($db->loadAssocList() as $result)
		{
			$categories[] = $result["category_id"];
		}

		return $categories;
	}


}
