<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Factory;
use Joomla\CMS\Pagination\Pagination;
use Joomla\Utilities\ArrayHelper;

use LartsSubscriptions\MassMailTemplates;
use LartsSubscriptions\MassMailQueue;


class QueueModel extends ListModel
{
	var $listTemplates = null;
	var $listMails = null;
	var $paginationMails = null;
	var $data       = null;
	var $id         = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		parent::__construct($config, $factory);

		$cid = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	function store($data)
	{
		$row = $this->getTable();

		## Bind the form fields to the web link table
		if(!$row->bind($data))
		{
			$this->setError($this->_db->getErrorMsg());

			return false;
		}

		## Make sure the web link table is valid
		if(!$row->check())
		{
			$this->setError($this->_db->getErrorMsg());

			return false;
		}

		## Store the web link table to the database
		if(!$row->store())
		{
			$this->setError($this->_db->getErrorMsg());

			return false;
		}

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	function getTotalMails()
	{
		$query = $this->getListQueryMails();
		return $this->_getListCount($query);
	}

	function getPaginationMails()
	{
		if(!is_null($this->paginationMails))
		{
			return $this->paginationMails;
		}

		$this->paginationMails = new Pagination($this->getTotalMails(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->paginationMails;
	}

	function getListMails()
	{
		if(!is_null($this->listMails)) {
			return $this->listMails;
		}

		$db    = Factory::getDbo();
		$query = $this->getListQueryMails();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->listMails = $db->loadObjectList();

		return $this->listMails;
	}

  function getListQueryMails()
	{
		$db  = Factory::getDbo();
		$app = Factory::getApplication();

		$filter_type   = $app->getUserStateFromRequest('queue_filter_type', 'queue_filter_type', '0', 'int');
		$filter_search = $app->getUserStateFromRequest('queue_filter_search', 'queue_filter_search', '', 'string');
		$filter_search = strtolower(trim($filter_search));

		$query = $db->getQuery(true)
			->select(['eq.*', 'u.firstname', 'u.lastname', 'user.email', 'u.id as user_id'])
			->from($db->quoteName('#__larts_subs_email_queue', 'eq'))
			->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('u.userid') . ' = ' . $db->quoteName('eq.userid') . ')')
			->join('LEFT', $db->quoteName('#__users', 'user') . ' ON (' . $db->quoteName('user.id') . ' = ' . $db->quoteName('eq.userid') . ')');

		if ($filter_search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $filter_search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);
			$where    = [
				$fullname . $like_filter,
				$db->quoteName('user.email') . $like_filter,
				$db->quoteName('eq.subject') . $like_filter,
				$db->quoteName('eq.body') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		$now = new Date;
		$now = $now->format('Y-m-d');

    //  '0' => ['value' => '0', 'text' => Text::_('COM_LARTSSUBS_UNSENT_EMAILS')],
    //  '1' => ['value' => '3', 'text' => Text::_('COM_LARTSSUBS_PENDING_EMAILS')],
    //  '2' => ['value' => '4', 'text' => Text::_('COM_LARTSSUBS_DELAYED_SENDING')],
    //  '3' => ['value' => '1', 'text' => Text::_('COM_LARTSSUBS_SENT_EMAILS')],
		switch ($filter_type)
		{
			case 4:   // date
				$query->where($db->quoteName('eq.pending_till') . ' > ' . $db->quote($now));
				break;

			case 1:    // sent
				$query->where($db->quoteName('eq.status') . ' = ' .MassMailQueue::MAIL_SENT);
				break;

			case 3:    // pending
				$query->where($db->quoteName('eq.status') . ' = ' .MassMailQueue::MAIL_UNSENT)
              ->andWhere($db->quoteName('eq.pending_till') . ' > ' . $db->quote($now));
				break;
			default:    // unsent
				$query->where($db->quoteName('eq.status') . ' = ' .MassMailQueue::MAIL_UNSENT)
              ->andWhere($db->quoteName('eq.pending_till') . ' <= ' . $db->quote($now));
				break;
		}

		$query->order($db->quoteName('eq.id') . ' DESC');

		return $query;
	}

  function getListTemplates()
	{
		if(!is_null($this->listTemplates)) {
			return $this->listTemplates;
		}

		$db    = Factory::getDbo();
		$query = $this->getListQueryTemplates();

		$db->setQuery($query, $this->getState('listTemplates.start'), $this->getState('listTemplates.limit'));
		$this->listTemplates = $db->loadObjectList();

		return $this->listTemplates;
	}

  function getListQueryTemplates()
	{
		$db  = Factory::getDbo();
		$query = $db->getQuery(true)
                ->select('*')
                ->from($db->quoteName('#__larts_subs_email_templates'))
                ->order($db->quoteName('template'));

 		return $query;
  }

 	public function publish(&$pks, $value = 1)
	{
		$pks = ArrayHelper::toInteger($pks);
    $bRet = MassMailTemplates::publish($pks, $value);

		return $bRet;
	}

	function getProducts()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select(['id', 'name'])
			->from($db->quoteName('#__larts_subs_products'))
			->where($db->quoteName('ispublic') . ' = 1')
			->where($db->quoteName('published') . ' = 1');

		$db->setQuery($query);

		return $db->loadObjectList();
	}

 	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.massmail', 'massmail', ['load_data' => $loadData]);
	}

 	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task != 'Add' && $this->id > 0) {
			return $this->getData();
		}

    // restore user input
    $app = Factory::getApplication();
    $temp = (array) $app->getUserState('com_lartssubscriptions.queue', []);
    $this->data = new \stdClass;
    foreach ($temp as $k => $v) {
      if(!empty($v)) {
        $this->data->$k = $v;
      }
    }
    return $this->data;

		//return parent::loadFormData();
	}

	function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}
    
    $this->data = MassMailTemplates::getTemplate($this->id);

		return $this->data;
	}


	function remove($cid)
	{
		if (empty($cid)) {
			return true;
		}

		$cid =ArrayHelper::toInteger($cid);
		if(!MassMailQueue::deleteMails($cid)) {
			return false;
		}

		return true;
	}
  
	function removePending()
	{
    return MassMailQueue::deleteByStatus(MassMailQueue::MAIL_UNSENT);
	}

  function removeSent()
	{
    return MassMailQueue::deleteByStatus(MassMailQueue::MAIL_SENT);
	}

	function removeTemplates($cid)
	{
		if (empty($cid)) {
			return true;
		}

		$cid = ArrayHelper::toInteger($cid);

    return MassMailTemplates::deleteTemplates($cid);
	}

  
}


