<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Message;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Periods;
use LartsSubscriptions\ThirdParty\LartsPkgManager;


class SubscriptionsModel extends ListModel
{
	var $id         = null;
	var $customerID = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;
	var $config     = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'per.id', 'per.valid_from', 'per.valid_to', 'per.status',
				'u.lastname', 'jusr.email',
				'prod.name', 'per.ordercode',
			];
		}

		parent::__construct($config, $factory);

		$this->config = LartsConfig::get();

		$cid = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		parent::populateState('a.ordering', 'asc');

		$list = $this->getUserStateFromRequest($this->context . '.list', 'list', [], 'array');
		if(isset($list['ordering'])) {
			$this->setState('list.ordering', $list['ordering']);
		}
		if(isset($list['direction'])) {
			$this->setState('list.direction', $list['direction']);
		}

		//$this->setState('list.ordering', $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 'per.id', 'string'));
		//$this->setState('list.direction', $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'desc', 'string'));

		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.status', $this->getUserStateFromRequest($this->context . '.filter.status', 'filter_status', null));
		$this->setState('filter.product_id', $this->getUserStateFromRequest($this->context . '.filter.product_id', 'filter_product_id', null));

		parent::populateState($ordering, $direction);
	}

	public function getPagination()
	{
		if(!is_null($this->pagination))
		{
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_getListQuery();

		return $this->_getListCount($query);
	}

	public function _getListQuery()
	{
	  $listOrder  = $this->getState('list.ordering', 'u.id');
    $orderDirn  = $this->getState('list.direction', 'DESC');

 		$status     = $this->getState('filter.status', '');
		$search     = $this->getState('filter.search', '');
		$product_id = $this->getState('filter.product_id', '');


    $db = Factory::getDbo();
		$query = $db->getQuery(true)
          ->select('per.id as period_id, per.valid_from, per.valid_to, per.ordercode, per.status, '
                  . 'jusr.id as jusr_id, u.id as user_id, u.firstname, u.lastname, jusr.email, '
                  . 'prod.id as product_id, prod.name as prodname, prod.type as product_type, prod.period_type, prod.product_ref')
          ->from($db->quoteName('#__larts_subs_periods', 'per'))
          ->leftJoin($db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('per.userid') . ' = ' . $db->quoteName('u.userid') . ')')
          ->leftJoin($db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('prod.id') . ')')
          ->leftJoin($db->quoteName('#__users', 'jusr') . ' ON (' . $db->quoteName('u.userid') . ' = ' . $db->quoteName('jusr.id') . ')')
          ->group(['u.firstname', 'u.lastname', 'jusr.email', 'prod.name', 'per.ordercode'])
          //->having('prod.product_ref = 0')
          //->having('(Max(prod.product_ref)) Is Not Null')
            ;

		if (is_numeric($status))
		{
      switch($status) {
      case 0:     // inactive
        $now = (new DateTime('now'))->format('Y-m-d H:i:s');
        $query->where($db->quoteName('per.valid_to') . ' < ' . $db->quote($now));
        break;
      case 1:     // active
        $now = (new DateTime('now'))->format('Y-m-d H:i:s');
        $query->where($db->quoteName('per.valid_to') . ' > ' . $db->quote($now));
        break;
      case 2:   // refunded
        $query->where($db->quoteName('per.status') . ' = ' . Periods::STATE_REFUNDED);
        break;
      case 3:   // cancelled
        $query->where($db->quoteName('per.status') . ' = ' . Periods::STATE_CANCELED);
        break;
      case 4:   // disabled
        $query->where($db->quoteName('per.status') . ' = ' . Periods::STATE_DISABLED);
        break;
      }
		}
		if (is_numeric($product_id))
		{
			$query->where($db->quoteName('product_id') . ' = ' . (int) $product_id);
		}

    if ($search)
		{
			if (is_numeric($search)) {
  			$query->where($db->quoteName('per.id') . ' = ' . (int) $search);
			}
      
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);

			$where = [
				$fullname . $like_filter,
				$db->quoteName('u.companyname') . $like_filter,
				$db->quoteName('jusr.username') . $like_filter,
				$db->quoteName('jusr.email') . $like_filter,
				$db->quoteName('prod.name') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

    $query->order($db->quoteName($listOrder) . ' ' . $db->escape($orderDirn));

    return $query;
    
		return $query;
	}

	function getList()
	{
		if(!is_null($this->list)) {
			return $this->list;
		}

		$db = Factory::getDbo();

		$query = $this->_getListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		return $this->list;
	}

	function getData()
	{
		if(!is_null($this->data))
		{
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('per.*, prod.name as product_name, prod.pkgmanager_id, prod.has_licence')
		            ->from($db->quoteName('#__larts_subs_periods', 'per'))
                ->leftJoin($db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('per.product_id') . ' = ' . $db->quoteName('prod.id') . ')')
		            ->where($db->quoteName('per.id') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$this->data = $db->loadObject();

		if (empty($this->data))
		{
      $this->data = (object)array(
				'id'     => 0,
				'status' => Periods::STATE_ACTIVE,
        'mail_changes' => 0,
				'userid' => Factory::getApplication()->GetInput()->get('uid'),
        'product_id' => 0,
        'has_licence' => 0,
        'pkgmanager_id' => 0,
				'pkg_download_key'  => '',
        'valid_from'  => (new Date())->format('Y-m-d'),
        'valid_to' => '',
      );

      return $this->data;
		}
    else {
      if(!empty($this->data->pkgmanager_id)) {
        if(LartsPkgManager::IsInstalled()) {
          $api = LartsPkgManager::getAPI();
          $this->data->pkg_download_key = $api->getUpdateKey($this->data->userid, $this->data->pkgmanager_id);
        }
      }
    }

		$this->setPaymentData();

		return $this->data;
	}

	public function setPaymentData()
	{
		$data = $this->getPaymentData(isset($this->data->ordercode) ? $this->data->ordercode : 0);

		foreach ($data as $key => $value)
		{
			$this->data->{'payment_' . $key} = $value;
		}
	}

	/**
	 * Returning payment data in the subscriptions (edit) view.
	 */
	public function getPaymentData($ordercode = 0)
	{
		if (empty($ordercode))
		{
			return (object) [];
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            't.type, i.ordercode', 'i.provider', 'i.transaction_id',
			            'i.gross_price', 'i.net_price', 'i.disco_price', 'i.vat_total', 'i.vat_percentage',
			            't.date',
			            'o.coupon_code',
		            ])
		            ->from($db->quoteName('#__larts_subs_invoices', 'i'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_transactions', 't') . ' ON ' . $db->quoteName('t.ordercode') . ' = ' . $db->quoteName('i.ordercode'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_cart_items', 'o') . ' ON ' . $db->quoteName('o.ordercode') . ' = ' . $db->quoteName('i.ordercode'))
		            ->where($db->quoteName('i.ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);

		$results = (object) $db->loadObject();
  	return $results;

//		if(!empty($results->id))
//		{
//			return $results;
//		}
//
//		// If empty, the user may used their own ivoice system, use the transaction table
//		return $this->getPaymentDataFromTransactionTable($ordercode);
	}

	/**
	 * Obtain the payment details from the transaction table.
	 */
	private function getPaymentDataFromTransactionTable($ordercode)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            't.*', 't.type as transaction_type',
			            'o.coupon_code',
		            ])
		            ->from($db->quoteName('#__larts_subs_transactions', 't'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_cart_items', 'o') . ' ON ' . $db->quoteName('o.ordercode') . ' = ' . $db->quoteName('t.ordercode'))
		            ->where($db->quoteName('t.ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);

		$results = (object) $db->loadObject();

		if (empty($results->amounts))
		{
			return (object) [];
		}

		// Decoding the amounts from this table
		$transaction = json_decode($results->amounts);

		// Adding some variables to the object
		$results->gross_price    = $transaction->gross_price;
		$results->net_price      = $transaction->net_price;
		$results->disco_price    = $transaction->disco_price;
		$results->vat_total     = $transaction->vat_total;
		$results->vatrate_std = $transaction->vatrate_std;

		return $results;
	}

	function publish(&$pks, $value = 1)
	{
		if (empty($pks))
		{
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_periods'))
		            ->set($db->quoteName('status') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		if(!$db->execute())
		{
			return false;
		}


		if ($value == 0) {
			PluginManager::execPlugins('lartssubscriptions', 'onAfterRemoveSubscription', $pks);
		}
		else {
			PluginManager::execPlugins('lartssubscriptions', 'onAfterProcessSubscription', $pks);
			PluginManager::execPlugins('lartssubscriptions', 'onAfterPublishSubscription', $pks);
		}

		return true;
	}

	function store($data)
	{
		$row = $this->getTable('periods');

    //		if (empty($data['created'])) {
    //			$date = new Date;
    //			$data['created'] = $date->format('Y-m-d');
    //		}

		if ($data['status'] != Periods::STATE_ACTIVE) {
			$arguments = [$data['id']];
			PluginManager::execPlugins('lartssubscriptions', 'onBeforeRemoveSubscription', $arguments);
		}

		if(!$row->bind($data)) {
			return false;
		}

		if(!$row->check()) {
			return false;
		}

		if(!$row->store()) {
			return false;
		}

		if ($data['status'] == Periods::STATE_ACTIVE)
		{
			// Only add to user groups when activating
			$arguments = [
				'product_id' => $data['product_id'],
				'userid'     => $data['userid'],
			];

			PluginManager::execPlugins('lartssubscriptions', 'onAfterProcessSubscription', $arguments);
		}

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	function remove($ids)
	{
		if (empty($ids)) {
			return true;
		}

		$ids = ArrayHelper::toInteger($ids);

		PluginManager::execPlugins('lartssubscriptions', 'onBeforeChangeSubscription', $ids);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_periods'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . implode(',', $ids) . ' )');

		$db->setQuery($query);

		if(!$db->execute()) {
			return false;
		}
		return true;
	}

	public function sendMessage($product_id, $userid, $action)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['name', 'alias'])
		            ->from($db->quoteName('#__larts_subs_products'))
		            ->where($db->quoteName('id') . ' = ' . $db->quote($product_id));

		$db->setQuery($query);
		$product = $db->loadObject();

		// Variables for the message:
		$variables = [
			'action'        => $action,
			'product'       => $product->name,
			'product_alias' => $product->alias,
			'date'          => 'NOW',
		];

		Message::getInstance('send-subscription-state')
		             ->user($userid)
		             ->variables($variables)
		             ->send();

		return true;
	}

	public function getForm($data = [], $loadData = true)
	{
		$form = $this->loadForm('com_lartssubscriptions.subscription', 'subscription', ['load_data' => $loadData]);
    
    return $form;
  }

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit') {
			return $this->getData();
		}
    
    $data = parent::loadFormData();
    $today = (new Date())->format('Y-m-d');
    $data->valid_from = $today;

		return $data;
	}
}
