<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\Utilities\ArrayHelper;

class TemplatesModel extends ListModel
{
	var $id   = null;
	var $list = null;
	var $data = null;
  var $datatype;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'locked',
				'template',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.template', $this->getUserStateFromRequest($this->context . 'filter.template', 'filter_template', null, 'string'));
		$this->setState('filter.locked', $this->getUserStateFromRequest($this->context . '.filter.locked', 'filter_locked', null, 'int'));

		parent::populateState('type', 'desc');
	}

	public function store($data)
	{
		$data['published'] = 1;

		$row = $this->getTable();

		if(!$row->bind($data))
		{
			return false;
		}

		if(!$row->check())
		{
			return false;
		}

		if(!$row->store())
		{
			return false;
		}

		return $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');
	}

	public function getList()
	{
		if(!is_null($this->list))
		{
			return $this->list;
		}

		$search    = $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null);
		$locked    = $this->getUserStateFromRequest($this->context . 'filter.locked', 'filter_locked', null);
		$template  = $this->getUserStateFromRequest($this->context . '.filter.template', 'filter_template', null);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_templates'))
		            ->order($db->quoteName('type') . ' ASC');

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$where    = [
				$db->quoteName('alias') . ' = ' . $db->quote($search),
				$db->quoteName('description') . $like_filter,
				$db->quoteName('subject') . $like_filter,
			];

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if (!empty($template)) {
			$query->where($db->quoteName('type') . ' = ' . $db->quote($template));
		}

		if(is_numeric($locked)) {
			$query->where($db->quoteName('secured') . ' = ' . $locked);
		}

		$db->setQuery($query);
		$this->list = $db->loadObjectList();

		return $this->list;
	}

	public function getData()
	{
		if(!is_null($this->data))
		{
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_templates'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$this->data = (object) $db->loadObject();

		$this->datatype    = '';
		$this->data->alias = isset($this->data->alias) ? $this->data->alias : '';

		return $this->data;
	}

	public function remove($cid = [])
	{
		if (count($cid) < 1)
		{
			Factory::getApplication()->redirect('index.php?option=com_lartssubscriptions');
		}

		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_templates'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.template', 'template', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			return $this->getData();
		}

		return parent::loadFormData();
	}
}
