<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\Order;

class TransactionsModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;
	var $_total = 0;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
        'transaction_id', 't.transaction_id',
        'ordercode', 't.ordercode',
        'date', 't.date',
        'lastname', 'u.lastname',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.provider', $this->getUserStateFromRequest($this->context . '.filter.provider', 'filter_provider', null, 'string'));

    $ord = $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 'a.ordering', 'string');
    $dir = $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_ordering_dir', 'DESC', 'string');
 
    $this->setState('list.ordering', $this->getUserStateFromRequest($this->context . '.list.ordering', 'filter_ordering', 't.ordercode', 'string'));
		$this->setState('list.direction', $this->getUserStateFromRequest($this->context . '.list.direction', 'filter_ordering_dir', 'DESC', 'string'));

		parent::populateState('t.id', 'desc');
	}

	public function getPagination()
	{
		if(!is_null($this->pagination)) {
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();
		$this->_total = $this->_getListCount($query);

		return $this->_total;
	}

	public function getList()
	{
		if(!is_null($this->list))
		{
			return $this->list;
		}

		$db  = Factory::getDbo();
		$sql = $this->_buildListQuery();

		$db->setQuery($sql, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		return $this->list;
	}

	public function _buildListQuery()
	{
		$db = Factory::getDbo();

	  $listOrder  = $this->getState('list.ordering', 't.id');
    $orderDirn  = $this->getState('list.direction', 'ASC');
		$search    = $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', 0, 'string');
		$provider  = $this->getUserStateFromRequest($this->context . '.filter.provider', 'filter_provider', null);
		//$ordering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 't.id', 'string');
		//$direction = $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'desc', 'string');

		$query = $db->getQuery(true)
		            ->select(['t.*', 'u.id as userid', 'u.firstname', 'u.lastname', 'ju.email'])
		            ->from($db->quoteName('#__larts_subs_transactions', 't'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'ju') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('ju.id') . ')')
                ;

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);
			$where    = [
				$db->quoteName('t.transaction_id') . ' = ' . $db->quote($search),
				$db->quoteName('t.email_paypal') . ' = ' . $db->quote($search),
				$fullname . $like_filter,
				$db->quoteName('u.companyname') . $like_filter,
				$db->quoteName('ju.username') . $like_filter,
				$db->quoteName('ju.email') . $like_filter,
			];

			if (is_numeric($search)) {
				$where[] = $db->quoteName('u.userid') . ' = ' . (int) $search;
			}

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if ($provider) {
			$query->where($db->quoteName('t.type') . ' = ' . $db->quote($provider));
		}

    $query->order($db->escape($listOrder) . ' ' . $db->escape($orderDirn));

		return $query;
	}

	public function getData()
	{
		if(!is_null($this->data))
		{
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['t.*', 'u.id as user_id', 'u.firstname', 'u.lastname', 'user.email'])
		            ->from($db->quoteName('#__larts_subs_transactions', 't'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'user') . ' ON (' . $db->quoteName('t.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->where($db->quoteName('t.id') . ' = ' . (int) $this->id);

		$db->setQuery($query);
		$this->data = (object) $db->loadObject();

		return $this->data;
	}

	public function remove($cid = [])
	{
		if (empty($cid)) {
			Factory::getApplication()->redirect('index.php?option=com_lartssubscriptions');
		}

    foreach($cid as $id) {
      $result = Transaction::lookupId($id, 'ordercode');
      if(!empty($result)) {
        Transaction::markDeleted($id);
        Order::updatePayed($result->ordercode, $result->amount);
      }
    }

    
		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_transactions'))
		            ->where($db->quoteName('deleted') . '=1');
		$db->setQuery($query);
		return $db->execute();
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.transaction', 'transaction', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			return $this->getData();
		}

		return parent::loadFormData();
	}
}
