<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Pagination\Pagination;

use LartsSubscriptions\VAT\VATClasses;


class VatclassesModel extends ListModel
{
	public    $id          = null;
	public    $list        = null;
	public    $data        = null;
	protected $_total      = null;
	protected $_pagination = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
        'ordering', 'ordering',
				'published', 'published',
        'name', 'name',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'string'));

		parent::populateState('a.id', 'asc');
	}

	public function getPagination()
	{
		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		$this->_total = $this->_getListCount($query);

		return $this->_total;
	}

	public function getList($get_all = false)
	{
		if(!is_null($this->list))
		{
			return $this->list;
		}

		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query);

		if(!$get_all) {
			$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		}

		$this->list = $db->loadObjectList();

		return $this->list;
	}

	private function _buildListQuery()
	{
		$state  = $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', null, 'string');
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', null, 'string');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_vat_classes'))
		            ->order($db->quoteName('name') . ' ASC');

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');
			$where = [
				$db->quoteName('name') . $like_filter,
			];
			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if(!empty($state)) {
			$query->where($db->quoteName('published') . ' = ' . $state);
		}

		return $query;
	}

	public function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_vat_classes'))
		            ->where($db->quoteName('id') . ' = ' . (int) $this->id);

		$db->setQuery($query);

		return (object) $db->loadObject();
	}

	public function publish(&$pks, $value = 1)
	{
		if (empty($pks)) {
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_vat_classes'))
		            ->set($db->quoteName('published') . ' = ' . (int) $value)
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');

		$db->setQuery($query);

		return $db->execute();
	}

	public function saveorder($pks = null, $order = null)
	{
		$row = $this->getTable('taxclasses');

		for ($i = 0; $i < count($pks); $i++)
		{
			$row->load((int) $pks[$i]);

			if ($row->ordering == $order[$i]) {
				continue;
			}

			$row->ordering = $order[$i];

			if(!$row->store())
			{
				return false;
			}
		}

		return true;
	}

	public function store($data)
	{
    return VATClasses::add($data);

	}

	protected function prepareTable($table)
	{
		if (empty($table->id)) {
			return $table;
		}

		return $table;
	}

	public function remove($cid)
	{
		if (empty($cid)) {
			return true;
		}

		$db   = Factory::getDbo();
		$cid  = ArrayHelper::toInteger($cid);

    foreach($cid as $id) {
      if(!VATClasses::remove($id))
        return false;
    }

    return true;
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.vatclass', 'vatclass', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			return $this->getData();
		}

		return parent::loadFormData();
	}
}
