<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Countries;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;


class HtmlView extends BaseHtmlView
{
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;
  public $vatclasses;
	public $data;
	public $form;

 	public function __construct($config = array())
	{
    parent::__construct($config);
  }

	public function display($tpl = null)
	{
		if ($this->getLayout() == 'form')
		{
			$this->_displayForm($tpl);

			return;
		}

		$this->config     = LartsConfig::get();

		$this->state      = $this->get('State');
		$this->items      = $this->get('list');
		$this->pagination = $this->get('Pagination');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');
		$this->vatclasses    = $this->get('VATClasses');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }
    
    // Only set the toolbar if not modal
	  if ($this->getLayout() !== 'modal') {
		  $this->addToolBar();
	  }
    
		parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data = $this->get('Data');
		$this->form = $this->get('Form');
		$this->vatclasses    = $this->get('VATClasses');

    $this->addFormToolbar();

    parent::display($tpl);
	}

  private function addToolbar()
  {
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_COUNTRIES'), 'location');
    ToolbarHelper::addNew('countries.add');
    ToolbarHelper::editList('countries.edit');
    ToolbarHelper::deleteList('', 'countries.remove', 'JTOOLBAR_DELETE');
    ToolbarHelper::divider();
    ToolbarHelper::publish('countries.publish', 'JTOOLBAR_PUBLISH', true);
    ToolbarHelper::unpublish('countries.unpublish', 'JTOOLBAR_UNPUBLISH', true);
    ToolbarHelper::divider();
  }
  
  private function addFormToolbar()
	{
    $newCountry = isset($this->data->id) ? $this->data->id : null;

    if(!$newCountry):
      ToolbarHelper::title(Text::_('COM_LARTSSUBS_ADD') . ' ' . Text::_('COM_LARTSSUBS_COUNTRY'), 'location');
    else:
      ToolbarHelper::title(Text::_('JACTION_EDIT') . ' ' . Text::_('COM_LARTSSUBS_COUNTRY') . ': ' . $this->data->country, 'location');
    endif;

    ToolbarHelper::apply('countries.apply');
    ToolbarHelper::save('countries.save');

    (!$this->data) ? ToolbarHelper::cancel('countries.cancel') : ToolbarHelper::cancel('countries.cancel', 'JTOOLBAR_CLOSE');

  }

}
