<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Coupons;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarHelper;

use LartsSubscriptions\Config as LartsConfig;


class HtmlView extends BaseHtmlView
{
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;
	public $data;
	public $form;

  public function display($tpl = null)
	{
		if ($this->getLayout() == 'form')
		{
			$this->_displayForm($tpl);

			return;
		}

		$this->config = LartsConfig::get();

		$this->state         = $this->get('State');
		$this->pagination    = $this->get('Pagination');
		$this->items         = $this->get('List');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

    $this->addToolbar();
    
		parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data = $this->get('Data');
		$this->form = $this->get('Form');

    $this->addFormToolbar();

		parent::display($tpl);
	}

  private function addToolbar()
  {
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_COUPONS'), 'tag');
    ToolbarHelper::addNew('coupons.add');
    ToolbarHelper::custom('coupons.duplicate', 'copy', 'copy', Text::_('COM_LARTSSUBS_COPY'), true);
    ToolbarHelper::editList('coupons.edit');
    ToolbarHelper::deleteList('','coupons.remove', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    ToolbarHelper::publish('coupons.publish', 'JTOOLBAR_PUBLISH', true);
    ToolbarHelper::unpublish('coupons.unpublish', 'JTOOLBAR_UNPUBLISH', true);
  }
  
  private function addFormToolbar()
	{
    $toolbar = Toolbar::getInstance();

    $newCoupon = isset($this->data->id) ? $this->data->id : null;

    if(!$newCoupon) {
      ToolbarHelper::title(Text::_('COM_LARTSSUBS_ADD') . ' ' . Text::_('COM_LARTSSUBS_COUPON'), 'tag');
    }
    else {
      ToolbarHelper::title(Text::_('JACTION_EDIT') . ' ' . Text::_('COM_LARTSSUBS_COUPON') . ': ' . $this->data->name, 'tag');
    }

    ToolbarHelper::apply('coupons.apply');
    ToolbarHelper::save('coupons.save');

    (!$this->data) ? ToolbarHelper::cancel('coupons.cancel') : ToolbarHelper::cancel('coupons.cancel', 'JTOOLBAR_CLOSE');

    $toolbar->divider();
    ToolbarHelper::inlinehelp();

 }

}

