<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Customers;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;

class HtmlView extends BaseHtmlView
{
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;
	public $data;
	public $form;
	public $products;
	public $orders;

	public function __construct(array $config = [])
	{
		parent::__construct($config);

		$this->config = LartsConfig::get();
	}

	public function display($tpl = null)
	{
		if ($this->getLayout() == 'form')
		{
			$this->_displayForm($tpl);

			return;
		}

		$this->state         = $this->get('State');
		$this->items         = $this->get('List');
		$this->pagination    = $this->get('Pagination');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

		$this->addToolbar();

    parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data     = $this->get('data');
		$this->products = $this->get('Products');
		$this->orders   = $this->get('Orders');
		$this->form     = $this->get('Form');

		if(!isset($this->data->id))
		{
			// Remove the user info for new user
			$this->form->removeField('userid');
			//$this->form->removeField('watchful_key');

			// Change the new_user_id field into userid
			$this->form->setFieldAttribute('new_user_id', 'name', 'userid');
		}
		else
		{
			// Edit a user doesnt need a user selector.
			$this->form->removeField('new_user_id');
		}

		$this->addFormToolbar();

    parent::display($tpl);
	}
  
	private function addToolbar()
	{
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_CUSTOMERS'), 'users');
		ToolbarHelper::addNew('customers.add');
		ToolbarHelper::editList('customers.edit');
		ToolbarHelper::deleteList('', 'customers.remove', 'JTOOLBAR_DELETE');
	}

	private function addFormToolbar()
	{
    // Obtain the user information for the toolbar.
    $lastname  = isset($this->data->lastname) ? $this->data->lastname : null;
    $firstname = isset($this->data->firstname) ? $this->data->firstname : null;
    $username  = isset($this->data->username) ? ' [' . $this->data->username . ']' : null;
    $userdata  = $firstname . ' ' . $lastname . ' ' . $username;

    // Setting the toolbars up here..
    $text = ! isset($this->data->id) ? Text::_('COM_LARTSSUBS_ADD') : Text::_('JACTION_EDIT');

    // Adding the toolbars
    ToolbarHelper::title($text . ' ' . Text::_('COM_LARTSSUBS_CUSTOMER') . ': ' . $userdata, 'user');
    ToolbarHelper::apply('customers.apply');
    ToolbarHelper::save('customers.save');

    if(!$this->data) {
      ToolbarHelper::cancel('customers.cancel');
    }
    else {
      ## For existing items the button is renamed `close`
      ToolbarHelper::cancel('customers.cancel', 'JTOOLBAR_CLOSE');
    }
	}

}
