<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Files;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Config as LartsConfig;

class HtmlView extends BaseHtmlView
{
	public $data = [];
	public $form;
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;
	public $filters;

  public function display($tpl = null)
	{
		if ($this->getLayout() == 'form')
		{
			$this->_displayForm($tpl);

			return;
		}

		$db  = Factory::getDbo();
		$app = Factory::getApplication();

		$filters = [];

		$filter_ordering_dir = $app->getUserStateFromRequest('files_filter_ordering_dir', 'files_filter_ordering_dir', 'DESC', 'cmd');
		$filter_ordering     = $app->getUserStateFromRequest('files_filter_ordering', 'files_filter_ordering', 'f.id', 'cmd');
		$filter_category     = $app->getUserStateFromRequest('files_filter_category', 'files_filter_category', '0', 'cmd');
		$filter_published    = $app->getUserStateFromRequest('files_filter_published', 'files_filter_published', '2', 'int');
		$filter_search       = $app->getUserStateFromRequest('files_filter_search', 'files_filter_search', '', 'string');
		$filter_order        = $app->getUserStateFromRequest('files_filter_order', 'files_filter_order', 'ordering', 'cmd');

		$filters['filter_search'] = strtolower(trim($filter_search));
		$filters['order']         = $filter_order;

		$query = $db->getQuery(true);

		$query->select(['id AS id', 'name AS name'])
		      ->from($db->quoteName('#__larts_subs_categories'))
		      ->where($db->quoteName('parentid') . ' = 0')
		      ->order($db->quoteName('ordering') . ' ASC');

		$db->setQuery($query);
		$categories = $db->loadObjectList();

		$query = $db->getQuery(true);

		$query->select(['id AS id', 'name AS name', 'parentid'])
		      ->from($db->quoteName('#__larts_subs_categories'))
		      ->where($db->quoteName('parentid') . ' != 0')
		      ->order($db->quoteName('ordering') . ' ASC');

		$db->setQuery($query);
		$subcategories = $db->loadObjectList();

		## Create dropdown menu
		$options[] = HTMLHelper::_('select.option', '0', Text::_('COM_LARTSSUBS_SELECT_CATEGORY'));

		foreach ($categories as $category)
		{
			$cid       = $category->id;
			$options[] = HTMLHelper::_('select.option', $category->id, $category->name);

			foreach ($subcategories as $subcategory)
			{
				if ($subcategory->parentid == $cid)
				{
					$options[] = HTMLHelper::_('select.option', $subcategory->id, '&nbsp;|- ' . $subcategory->name);
				}
			}
		}

		$filters['filter_category'] = HTMLHelper::_('select.genericlist', $options, 'files_filter_category',
			'class="input" onchange="this.form.submit();"', 'value', 'text', $filter_category);

		$filtering = [
			'0' => ['value' => 'f.id', 'text' => Text::_('COM_LARTSSUBS_FILE_ID_ORDERING')],
			'1' => ['value' => 'f.ordering', 'text' => Text::_('COM_LARTSSUBS_TABLE_ORDERING')],
		];

		$filters['filter_ordering'] = HTMLHelper::_('select.genericlist', $filtering, 'files_filter_ordering',
			'class="input" onchange="this.form.submit();"', 'value', 'text', $filter_ordering);

		$filtering_dir = [
			'0' => ['value' => 'DESC', 'text' => Text::_('JGLOBAL_ORDER_DESCENDING')],
			'1' => ['value' => 'ASC', 'text' => Text::_('JGLOBAL_ORDER_ASCENDING')],
		];

		$filters['filter_ordering_dir'] = HTMLHelper::_('select.genericlist', $filtering_dir, 'files_filter_ordering_dir',
			'class="input" onchange="this.form.submit();"', 'value', 'text', $filter_ordering_dir);

		$state = [
			'2' => ['value' => '2', 'text' => Text::_('COM_LARTSSUBS_SELECT_PUBLISH_STATUS')],
			'0' => ['value' => '0', 'text' => Text::_('JUNPUBLISHED')],
			'1' => ['value' => '1', 'text' => Text::_('JPUBLISHED')],
		];

		$filters['filter_published'] = HTMLHelper::_('select.genericlist', $state, 'files_filter_published',
			'class="input" onchange="this.form.submit();"', 'value', 'text', isset($filter_published) ? $filter_published : 2);

		$this->filters = $filters;

		$this->config = LartsConfig::get();

		$this->state         = $this->get('State');
		$this->items         = $this->get('List');
		$this->pagination    = $this->get('Pagination');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

    $this->addToolbar();
    
		parent::display($tpl);
	}

	/**
	 * Display the form
	 */
	public function _displayForm($tpl = null)
	{
		$this->data       = $this->get('Data');
		$this->form       = $this->get('Form');
		$this->config     = LartsConfig::get();

		$now = new Date('-1 day');
		$now = $now->format('Y-m-d');
    
		if (empty($this->form->getValue('release_date', null))) {
			$this->form->setValue('release_date', '', $now);
		}

		if(!empty($this->data->id) && ! $this->data->id) {
			$this->form->removeField('filename_physical');
		}

    $this->addFormToolbar();

    parent::display($tpl);
	}

  private function addToolbar()
	{
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_FILEMANAGEMENT'), 'file');
		ToolbarHelper::addNew('files.add');
    ToolbarHelper::custom('files.prepopulatefile', 'copy', 'copy', Text::_('COM_LARTSSUBS_COPY_FILE'), true);
		ToolbarHelper::editList('files.edit');
		ToolbarHelper::deleteList('', 'files.remove', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    ToolbarHelper::publish('files.publish', 'JTOOLBAR_PUBLISH', true);
    ToolbarHelper::unpublish('files.unpublish', 'JTOOLBAR_UNPUBLISH', true);
	}

  private function addFormToolbar()
	{
    $toolbar = Toolbar::getInstance();
    $file = isset($this->data->filename) ? ': ' . $this->data->filename : '';
    $type = Text::_($file ? 'JACTION_EDIT' : 'COM_LARTSSUBS_ADD');

    ToolbarHelper::title($type . ' ' . Text::_('COM_LARTSSUBS_FILE') . $file, 'file');
    ToolbarHelper::apply('files.apply');
    ToolbarHelper::save('files.save');

    if(!$this->data) {
     ToolbarHelper::cancel('files.cancel');
    }
    else {
     ToolbarHelper::cancel('files.cancel', 'JTOOLBAR_CLOSE');
    }

    $toolbar->divider();
    ToolbarHelper::inlinehelp();

	}
}
