<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Invoices;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;


class HtmlView extends BaseHtmlView
{
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;
	public $data;
	public $form;
	public $products;

	public function __construct(array $config = [])
	{
		parent::__construct($config);

		$this->config = LartsConfig::get();
	}

	public function display($tpl = null)
	{
		if ($this->getLayout() == 'form')
		{
			$this->_displayForm($tpl);

			return;
		}

		$this->state         = $this->get('State');
		$this->items         = $this->get('list');
		$this->pagination    = $this->get('Pagination');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

    $this->addToolbar();
    
		parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data     = $this->get('data');
		$this->form     = $this->get('Form');
		$this->products = $this->get('products');

    $this->addFormToolbar();

		parent::display($tpl);
	}
  
  private function addToolbar()
  {
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_INVOICES_MANAGEMENT'), 'file-2');
    ToolbarHelper::custom('invoices.export', 'download', '', Text::_('COM_LARTSSUBS_DOWNLAOD_SELECTION_XLS'), false);
    ToolbarHelper::custom('invoices.report', 'health', '', Text::_('COM_LARTSSUBS_FINANCIAL_REPORT_XLS'), false);
		ToolbarHelper::deleteList('', 'invoices.remove', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    ToolbarHelper::custom('invoices.send', 'mail', '', Text::_('COM_LARTSSUBS_INVOICES_SEND'), true);
    //ToolbarHelper::publish('invoices.publish', 'JTOOLBAR_PUBLISH', true);
    //ToolbarHelper::unpublish('invoices.unpublish', 'JTOOLBAR_UNPUBLISH', true);
  }
  
  private function addFormToolbar()
	{
    ToolbarHelper::title(Text::_('JACTION_EDIT') . ' ' . Text::_('COM_LARTSSUBS_INVOICE') . ': ' . $this->data->id, 'signup');
    ToolbarHelper::cancel('invoices.cancel', 'JTOOLBAR_CLOSE');
  }
}
