<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Order;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Base\Country;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Customer;


class HtmlView extends BaseHtmlView
{
	protected $data = [];
  public $config;
  public $products;
  public $product_filter;
  public $cart;
  public $items;
  public $form;

	public function display($tpl = null)
	{
		$this->config   = LartsConfig::get();

    $model = $this->getModel();
    
    if ($this->getLayout() == 'show') {
      // show order.....
			$this->_displayOrder($tpl);
			return;
		}

    // create internal cart view.....
    $jinput = Factory::getApplication()->GetInput();
    
    $filters = [];

    // get selectable products
		$this->products = $this->get('products');
    $this->product_filter = $jinput->get('filter', '');
    
    $this->form = $this->get('Form');
    $userid = $this->form->getValue('userid', null, 0);
    if($userid > 0) {
      $userid = $this->form->setValue('newuser', null, 0);
    }

    // get cart items
    $this->cart = new Cart();
    $this->cart->prepareCartItems();
    $this->items = $this->cart->getProducts();


    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }
    
 		$this->addToolbar();

		parent::display($tpl);
	}

  public function _displayOrder($tpl = null)
  {
 		$this->addFormToolbar();
    
    parent::display($tpl);
  }

  private function addToolbar()
	{
    $ordercode = intval(Factory::getSession()->get('ordercode'));

    ToolbarHelper::title(Text::_('COM_LARTSSUBS_START_NEW_ORDER') . ': ' . $ordercode, 'new');
    ToolbarHelper::custom('order.completeOffer', 'file-2', '', Text::_('COM_LARTSSUBS_COMPLETE_OFFER'), false);
    ToolbarHelper::custom('order.completeOrder', 'signup', '', Text::_('COM_LARTSSUBS_COMPLETE_ORDER'), false);
    ToolbarHelper::custom('order.cancelOrder', 'remove', '', Text::_('COM_LARTSSUBS_CANCEL'), false);
	}

	private function addFormToolbar()
	{
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_ORDER_DETAILS'), 'cube');

    ToolbarHelper::back();
	}
  
}



  
  