<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Products;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Product;
use LartsSubscriptions\UserGroups;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\ThirdParty\LartsPkgManager;


class HtmlView extends BaseHtmlView
{
  public $config;
  protected $state;
  public $items;
  protected $pagination;
  public $filterForm;
  public $activeFilters;
	public $ordering = false;
	public $active;
	public $data;
  public $categories;
  public $products;
  public $cross_sell;
  public $children;
  public $form;
  public $usergroups;
    
	public function __construct(array $config = [])
	{
		parent::__construct($config);

		$this->config = LartsConfig::get();
	}

	function display($tpl = null)
	{
		if ($this->getLayout() == 'form') {
			$this->_displayForm($tpl);
			return;
		}

		$this->state         = $this->get('State');
		$this->pagination    = $this->get('Pagination');
		$this->active        = $this->get('ActiveClients');
		$this->items         = $this->get('List');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    if (!empty($errors = $this->get('Errors'))) {
      Factory::getApplication()->enqueueMessage(implode('<br />', $errors), 'error');
			return false;
		}

    $ordering = $this->getModel()->ordering;
    if($ordering == 'a.ordering') {
      $this->ordering = true;
    }
    
 		$this->addToolbar();

		parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data       = $this->get('Data');
		$this->categories = $this->get('Categories');
		$this->active     = $this->get('Active');
		$this->products   = $this->get('Products');
		$this->cross_sell = $this->get('ActiveProducts');
		$this->children   = $this->get('GroupChildren');
		$this->form       = $this->get('Form');

    $this->template_type = 'ticket';
    $this->ProdListOptions['product_ref']['show'] = 1;
    $this->file_id = 0;
    $this->category_id = 0;
    $this->template_id = 0;
    $this->group_id = 0;

		$this->usergroups = $this->get('UserGroups');

    $this->addFormToolbar();

		parent::display($tpl);
	}

  private function addToolbar()
	{
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_PRODUCTS'), 'stack');
		ToolbarHelper::addNew('products.add');
    ToolbarHelper::custom('products.duplicate', 'copy', 'copy', 'JTOOLBAR_DUPLICATE', true);
		ToolbarHelper::editList('products.edit');
		ToolbarHelper::deleteList('', 'products.remove', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    ToolbarHelper::publish('products.publish', 'JTOOLBAR_PUBLISH', true);
    ToolbarHelper::unpublish('products.unpublish', 'JTOOLBAR_UNPUBLISH', true);
	}

  private function addFormToolbar()
  {
    $toolbar = Toolbar::getInstance();
    $product = isset($this->data->name) ? ': ' . $this->data->name : '';
    $type    = Text::_($product ? 'JACTION_EDIT' : 'COM_LARTSSUBS_ADD');
    
    ToolbarHelper::title($type . ' ' . Text::_('COM_LARTSSUBS_PRODUCT') . $product, 'cube');
    ToolbarHelper::apply('products.apply');
    ToolbarHelper::save('products.save');
    ToolbarHelper::save2new('products.save2new');
    if(!$this->data) {
      ToolbarHelper::cancel('products.cancel');
    }
    else {
      ## For existing items the button is renamed `close`
      ToolbarHelper::cancel('products.cancel', 'JTOOLBAR_CLOSE');
    }

    $toolbar->divider();
    ToolbarHelper::inlinehelp();

  }
  
	function isChecked($key, $value, $active)
	{
		foreach ($active as $item)
		{
			if ($value == $item->{$key})
			{
				return true;
			}
		}

		return false;
	}
}
