<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Queue;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Toolbar\ToolbarHelper;

use LartsSubscriptions\Config as LartsConfig;

class HtmlView extends BaseHtmlView
{
  public $config;
  public $templates;
  public $mails;
  public $pagination;
  public $filters;
  public $form;
	public $data;
	public $products;

	function display($tpl = null)
	{
		// If we want the add/edit form..
		if ($this->getLayout() == 'form') {
			$this->_displayForm($tpl);
			return;
		}

    $this->config = LartsConfig::get();
		$this->templates = $this->get('ListTemplates');
		$this->mails = $this->get('ListMails');
		$this->pagination = $this->get('PaginationMails');
    
		$filters = [];

		$app = Factory::getApplication();

		$filter_type   = $app->getUserStateFromRequest('queue_filter_type', 'queue_filter_type', '0', 'int');
		$filter_search = $app->getUserStateFromRequest('queue_filter_search', 'queue_filter_search', '', 'string');

		$type_select = [
			'0' => ['value' => '0', 'text' => Text::_('COM_LARTSSUBS_UNSENT_EMAILS')],
			'1' => ['value' => '3', 'text' => Text::_('COM_LARTSSUBS_PENDING_EMAILS')],
			'2' => ['value' => '4', 'text' => Text::_('COM_LARTSSUBS_DELAYED_SENDING')],
			'3' => ['value' => '1', 'text' => Text::_('COM_LARTSSUBS_SENT_EMAILS')],
		];

		$filters['filter_type'] = HTMLHelper::_('select.genericlist', $type_select, 'queue_filter_type',
			'class="form-select" onchange="this.form.submit();"', 'value', 'text', $filter_type);

		$filters['filter_search'] = strtolower(trim($filter_search));

		$this->filters = $filters;

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

    $this->addToolbar();

		parent::display($tpl);
	}

	function _displayForm($tpl = null)
	{
		$this->form = $this->get('Form');
		$this->data = $this->get('Data');
		$this->products = $this->get('products');
		$this->config = LartsConfig::get();

    $this->addFromToolbar();

		parent::display($tpl);
	}
  
  public function addToolbar()
  {
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_MAILING_OVERVIEW'), 'drawer-2');
    ToolbarHelper::addNew('queue.add');
    ToolbarHelper::editList('queue.edit');
    ToolbarHelper::custom('queue.sendmails', 'thumbs-up', 'success', Text::_('COM_LARTSSUBS_SEND_PENDINGS'), false);
    ToolbarHelper::custom('queue.removePending', 'trash', '', Text::_('COM_LARTSSUBS_REMOVE_PENDING_MAILS'), false);
    ToolbarHelper::custom('queue.removeSent', 'trash', '', Text::_('COM_LARTSSUBS_REMOVE_SENT_MAILS'), false);
 		ToolbarHelper::deleteList('', 'queue.remove', 'JTOOLBAR_DELETE');
  }
  
  public function addFromToolbar()
  {
    ## Setting the toolbars up here..
    $newMessage = isset($this->data->subject) ? $this->data->subject : null;

    if(!$newMessage) {
      ToolbarHelper::title(Text::_('COM_LARTSSUBS_ADD') . ' ' . Text::_('JGLOBAL_EMAIL'), 'drawer-2');
    }
    else {
      ToolbarHelper::title(Text::_('JACTION_EDIT') . ' ' . Text::_('JGLOBAL_EMAIL') . ': ' . $newMessage, 'drawer-2');
    }

    ToolbarHelper::apply('queue.apply');
    ToolbarHelper::save('queue.save');
    ToolbarHelper::custom('queue.queue', 'envelope-opened', '', Text::_('COM_LARTSSUBS_QUEUE'), false);
 
    (!$this->data) ? ToolbarHelper::cancel('queue.cancel') : ToolbarHelper::cancel('queue.cancel', 'JTOOLBAR_CLOSE');

  }
}
