<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Subscriptions;
defined('_JEXEC') or die;

//use Joomla\CMS\Factory;
//use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Config as LartsConfig;
//use LartsSubscriptions\Base\PluginManager;
//use LartsSubscriptions\ThirdParty\LartsLicManager;


class HtmlView extends BaseHtmlView
{
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;
  public $form;
	public $data;
  public $bReadOnly;

  public function __construct($config = [])
	{
		$this->config = LartsConfig::get();

		parent::__construct($config);
	}

	public function display($tpl = null)
	{
		if ($this->getLayout() == 'form') {
			$this->_displayForm($tpl);
			return;
		}

		$this->state         = $this->get('State');
		$this->items         = $this->get('list');
		$this->pagination    = $this->get('Pagination');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

		$this->addToolbar();

    parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->form = $this->get('Form');
		$this->data = $this->get('Data');

    // prevent modification if this comes from an order
    $this->bReadOnly = isset($this->data->ordercode);
    
    // remove licence field if not needed or licence manager not installed
    $hidelic = false;
    if($this->bReadOnly) {
      $hidelic = !$this->data->has_licence;
    }
    if($hidelic || !$this->config->LicMngrInstalled) {
 			$this->form->removeField('licence');
    }
    // remove package download-key field if pkg-manager not installed
    if($this->data->pkgmanager_id == 0 || !$this->config->PkgMngrInstalled) {
 			$this->form->removeField('pkg_download_key');
    }
    
		$this->addFormToolbar();

    parent::display($tpl);
	}

	private function addToolbar()
	{
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_SUBSCRIPTIONS_MANAGEMENT'), 'list');
		ToolbarHelper::addNew('subscriptions.add');
		ToolbarHelper::editList('subscriptions.edit');
		ToolbarHelper::deleteList('', 'subscriptions.remove', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    ToolbarHelper::publish('subscriptions.publish', 'JTOOLBAR_PUBLISH', true);
    ToolbarHelper::unpublish('subscriptions.unpublish', 'JTOOLBAR_UNPUBLISH', true);
    ToolbarHelper::custom('subscriptions.export', 'download', '', Text::_('COM_LARTSSUBS_EXPORT_TO_EXCEL'), false);
	}

	private function addFormToolbar()
	{
    $id = isset($this->data->id) ? $this->data->id : 0;

    $type = ($id != 0) ? Text::_('JACTION_EDIT') : Text::_('COM_LARTSSUBS_ADD');

    ToolbarHelper::title('' . $type . ' ' . Text::_('COM_LARTSSUBS_SUBSCRIPTION'), 'list');

    ToolbarHelper::apply('subscriptions.apply');
    ToolbarHelper::save('subscriptions.save');

		empty($this->data) ? ToolbarHelper::cancel('subscriptions.cancel') : ToolbarHelper::cancel('subscriptions.cancel', 'JTOOLBAR_CLOSE');
	}
  
}
