<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Tickets;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Toolbar\ToolbarHelper;

use LartsSubscriptions\Config as LartsConfig;

class HtmlView extends BaseHtmlView
{
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
	public $activeFilters = [];
  public $data;
  public $form;

	public function display($tpl = null)
	{
		if ($this->getLayout() == 'form') {
			$this->_displayForm($tpl);

			return;
		}

		$this->config = LartsConfig::get();

		$this->state         = $this->get('State');
		$this->items         = $this->get('List');
		$this->pagination    = $this->get('Pagination');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

		$this->addToolbar();

		parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data   = $this->get('Data');
		$this->form   = $this->get('Form');
		$this->config = LartsConfig::get();

		$this->addFormToolbar();

		parent::display($tpl);
	}

	private function addToolbar()
	{
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_TICKETS'), 'stack');
		//ToolbarHelper::addNew('tickets.add');
		ToolbarHelper::editList('tickets.edit');
		ToolbarHelper::deleteList('', 'tickets.remove', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    //ToolbarHelper::publish('tickets.publish', 'JTOOLBAR_PUBLISH', true);
    //ToolbarHelper::unpublish('tickets.unpublish', 'JTOOLBAR_UNPUBLISH', true);
	}

	private function addFormToolbar()
	{
    $isEmpty = empty((array)$this->data);
		$type = $isEmpty ? Text::_('COM_LARTSSUBS_ADD') : Text::_('JACTION_EDIT');

    ToolbarHelper::title($type . ' ' .Text::_('COM_LARTSSUBS_TICKETS'), 'stack');
    ToolbarHelper::apply('tickets.apply');
    ToolbarHelper::save('tickets.save');

		empty($this->data) ? ToolbarHelper::cancel('tickets.cancel') : ToolbarHelper::cancel('tickets.cancel', 'JTOOLBAR_CLOSE');
	}
}
