<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\View\Vatrates;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Base\Country;


class HtmlView extends BaseHtmlView
{
  public $data;
  public $form;
  public $config;
  public $state;
  public $items;
  public $pagination;
  public $filterForm;
  public $activeFilters;

  public function display($tpl = null)
	{
		if ($this->getLayout() == 'form')
		{
			$this->_displayForm($tpl);
			return;
		}

		$this->config = LartsConfig::get();

		$this->state         = $this->get('State');
		$this->pagination    = $this->get('Pagination');
		$this->items         = $this->get('List');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

    // Check for errors.
    if (!empty($errors = $this->get('Errors'))) {
        throw new GenericDataException(implode("\n", $errors), 500);
    }

    $this->addToolbar();
    
		parent::display($tpl);
	}

	public function _displayForm($tpl = null)
	{
		$this->data = $this->get('Data');
		$this->form = $this->get('Form');
    
    if($this->data != null) {
      $country = Country::getCountryByCountryCode($this->data->country_code);
      $this->data->country_name = $country->country;
    }

    $this->addFormToolbar();

		parent::display($tpl);
	}

  private function addToolbar()
  {
    ToolbarHelper::title(Text::_('COM_LARTSSUBS_VATRATES'), 'tag');
    ToolbarHelper::addNew('vatrates.add');
    ToolbarHelper::editList('vatrates.edit');
    ToolbarHelper::deleteList('vatrates.delete', 'JTOOLBAR_DELETE');
		ToolbarHelper::divider();
    ToolbarHelper::custom('vatrates.updateRates', 'bars', '', Text::_('COM_LARTSSUBS_CHECK_VAT_RATES'), false);
    ToolbarHelper::custom('vatrates.removeEURates', 'bars', '', Text::_('COM_LARTSSUBS_REMOVE_VAT_RATES'), false);
  }
  
  private function addFormToolbar()
	{
    $isNew = isset($this->data->id) ? $this->data->id : null;
    if(!$isNew) {
      ToolbarHelper::title(Text::_('COM_LARTSSUBS_ADD') . ' ' . Text::_('COM_LARTSSUBS_VATRATE'), 'tag');
    }
    else {
      ToolbarHelper::title(Text::_('JACTION_EDIT') . ' ' . Text::_('COM_LARTSSUBS_VATRATE') . ': ' . $this->data->country_name, 'tag');
    }

    ToolbarHelper::apply('vatrates.apply');
    ToolbarHelper::save('vatrates.save');

    (!$this->data) ? ToolbarHelper::cancel('vatrates.cancel') : ToolbarHelper::cancel('vatrates.cancel', 'JTOOLBAR_CLOSE');

 }

}

