<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$ordering  = ($listOrder == 'a.ordering');
$saveOrder = ($listOrder == 'a.ordering' && strtolower($listDirn) == 'asc');

if ($saveOrder && !empty($this->items)) {
    $saveOrderingUrl = 'index.php?option=com_lartssubscriptions&task=categories.saveOrderAjax&tmpl=component&' . Session::getFormToken() . '=1';
    HTMLHelper::_('draggablelist.draggable');
}


?>

<?php echo (new FileLayout('menu'))->render('categories'); ?>


<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=categories' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="categoryList">
        <caption class="visually-hidden">
            <?php echo Text::_('COM_FIELDS_FIELDS_TABLE_CAPTION'); ?>,
            <span id="orderedBy"><?php echo Text::_('JGLOBAL_SORTED_BY'); ?> </span>,
            <span id="filteredBy"><?php echo Text::_('JGLOBAL_FILTERED_BY'); ?></span>
        </caption>
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
                <?php echo HTMLHelper::_('searchtools.sort', '', 'a.ordering', $listDirn, $listOrder, null, 'asc', 'JGRID_HEADING_ORDERING', 'icon-sort'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" class="w-5 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JGRID_HEADING_ID', 'a.id', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" class="text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_TYPE', 'a.cattype', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" class="text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_CATEGORY_NAME', 'a.name', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_IS_PARENT'); ?>
            </th>
            <th scope="col" class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PARENT_CATEGORY'); ?>
            </th>
            <th scope="col" class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PRODUCT_COUNT'); ?>
            </th>
            <th scope="col" class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_FILE_COUNT'); ?>
            </th>
          </tr>
        </thead>
        <tbody <?php if ($saveOrder) : ?> 
              class="js-draggable" data-url="<?php echo $saveOrderingUrl; ?>" data-direction="<?php echo strtolower($listDirn); ?>" data-nested="true"
               <?php endif; ?>
        >
          <?php foreach ($this->items as $i => $category) : ?>
            <?php
            $ordering  = ($listOrder == 'a.ordering');
            $published = HTMLHelper::_('grid.published', $category, $i);
            $checked   = HTMLHelper::_('grid.id', $i, $category->id);
            $link      = 'index.php?option=com_lartssubscriptions&task=categories.edit&cid[]=' . $category->id;
            ?>
            <tr class="row<?php echo $i % 2; ?>" data-draggable-group="<?php echo $category->parentid; ?>" item-id="<?php echo $category->id ?>"
                parents="<?php echo $category->parentid ?: ''; ?>" level="<?php echo $category->parentid ? 2 : 1; ?>">

              <td class="text-center">
                <?php echo $checked; ?>
              </td>

              <td class="text-center">
                  <?php $iconClass = ''; ?>
                  <?php if (!$saveOrder) : ?>
                      <?php $iconClass = ' inactive" title="' . Text::_('JORDERINGDISABLED'); ?>
                  <?php endif; ?>
                  <span class="sortable-handler<?php echo $iconClass; ?>">
                      <span class="icon-ellipsis-v" aria-hidden="true"></span>
                  </span>
                  <?php if ($saveOrder) : ?>
                      <input type="text" class="hidden" name="order[]" size="5" value="<?php echo $category->ordering; ?>">
                  <?php endif; ?>
              </td>

              <td class="text-center">
                <?php
                $layout = new FileLayout('list_button_publish');
                echo $layout->render(['cntrl' => 'categories', 'id' => $i, 'value' => $category->published]);
                ?>
              </td>
              <td class="text-center text-nowrap small">
                <?php echo $category->id; ?>
              </td>
              <td class="text-start">
                <?php echo ($category->cattype == 0) ? Text::_('COM_LARTSSUBS_PRODUCT') : Text::_('COM_LARTSSUBS_SUBSCRIPTION'); ?>
              </td>
              <td class="text-start">
                <?php if ($category->parentid): ?>
                  <span class="icon-arrow-right-3"></span>
                <?php endif; ?>
                <a href="<?php echo $link; ?>"><?php echo $category->name; ?></a>
              </td>
              <td class="text-center">
                <?php if ($category->has_children): ?>
                  <i class="fas fa-check green hasTooltip" title="<?php echo Text::_('JYES'); ?>">
                  </i>
                <?php endif; ?>
              </td>
              <td class="text-center">
                <?php if ($category->parentid): ?>
                  <a href="index.php?option=com_lartssubscriptions&task=categories.edit&cid[]=<?php echo (int) $category->parentid; ?>">
                      <?php echo $category->parent_name; ?>
                      <!-- <span class="badge bg-default" style="margin-bottom:1px;"><?php //echo $category->parent_name; ?></span>-->
                  </a>
                <?php endif; ?>
              </td>
              <td class="text-center">
                <?php echo ! empty($category->products) ? $category->products : null; ?>
              </td>
              <td class="text-center">
                <?php echo ! empty($category->files) ? $category->files : null; ?>
              </td>
            </tr>
          <?php endforeach; ?>
        </tbody>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
