<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::_('script', 'com_lartssubscriptions/modal-VATSel.js', array('version' => 'auto', 'relative' => true));

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('countries');

$VATCols = 3 + sizeof($this->vatclasses);
$vatrate_layout = new FileLayout('vat_rate_edit');

?>

<div id="j-main-container container-popup">
  <form action="index.php?option=com_lartssubscriptions&view=countries" method="POST" name="adminForm" id="adminForm" class="form-inline">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>

		<table class="table table-striped">
			<thead>
        <tr style="background-color: rgb(230,230,230)">
          <th class="text-center" colspan="5" style="border-right: 1px solid black">Country</th>
          <th class="text-center" colspan="<?php echo $VATCols; ?>">VAT</th>
        </tr>

				<tr>
          <th class="w-1 text-text-center">
            <?php echo HTMLHelper::_('grid.checkall'); ?>
          </th>
					<th style="width:50px;" class="text-center text-nowrap">
						<?php echo Text::_('JSTATUS'); ?>
					</th>
					<th class="left">
						<?php echo Text::_('COM_LARTSSUBS_COUNTRY'); ?>
					</th>
					<th style="width:50px;" class="text-center text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_THREE_DIGIT_CODE'); ?>
					</th>
					<th style="width:50px;" class="text-center text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_TWO_DIGIT_CODE'); ?>
					</th>
					<th style="width:50px;" class="text-center text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_REQUIRED'); ?>
					</th>
					<th style="width:50px;" class="text-center text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_FORCE'); ?>
					</th>
					<th style="width:50px;" class="text-start text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_VAT_STANDARD'); ?>
					</th>
          
    			<?php foreach ($this->vatclasses as $vc => $vatclass) : ?>
            <th style="width:80px;" class="text-start text-nowrap">
              <?php echo $vatclass->name; ?>
            </th>
    			<?php endforeach; ?>
          
          
				</tr>
			</thead>
			<?php foreach ($this->items as $i => $row) : ?>
				<?php
				$published = HTMLHelper::_('grid.published', $row, $i);
				$checked   = HTMLHelper::_('grid.id', $i, $row->id);
				$link      = 'index.php?option=com_lartssubscriptions&task=countries.edit&cid[]=' . $row->id;
				?>
				<tr>
					<td class="left">
						<?php echo $checked; ?>
					</td>
					<td class="text-center">
						<?php
						$layout = new FileLayout('list_button_publish');
						echo $layout->render(['cntrl' => 'countries', 'id' => $i, 'value' => $row->published]);
						?>
					</td>
					<td class="left">
						<a href="<?php echo $link; ?>"><?php echo $row->country; ?></a>
					</td>
					<td class="text-center">
						<?php echo $row->country_3_code; ?>
					</td>
					<td class="text-center">
						<?php echo $row->country_2_code; ?>
					</td>
					<td class="text-center">
						<?php
						$layout = new FileLayout('label_yes_no');
						echo $layout->render(['value' => $row->requires_vat]);
						?>
					</td>
					<td class="text-center">
						<?php
						$layout = new FileLayout('label_yes_no');
						echo $layout->render(['value' => $row->force_vat_payment]);
						?>
					</td>
					<td class="left text-nowrap">
						<?php
            //$rate = $row->requires_vat == 1 ? $row->vatrate_std . '%' : ''; 
            $rate = $row->vatrate_std . '%'; 
						echo $vatrate_layout->render(['col' => '0', 'country' => $row->country_2_code, 'rate' => $rate]);
						?>
					</td>
          
    			<?php foreach ($this->vatclasses as $vc => $vatclass) : ?>
            <td style="width:50px;" class="left text-nowrap">
              <?php
              $col = 'rateclass_' .$vatclass->id;
              $rate = '';
              if(isset($row->$col) /* && $row->requires_vat == 1*/) {
                $rate = $row->$col > 0 ? $row->$col . '%' : ''; 
              }

              echo $vatrate_layout->render(['col' => $vc+1, 'country' => $row->country_2_code, 'rate' => $rate]);
              ?>
            </td>
    			<?php endforeach; ?>
          
				</tr>
			<?php endforeach; ?>
		</table>

    <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
  </form>
</div>

<div class="modal fade" id="ajax-modal" tabindex="-1">
    <div class="modal-dialog modal-dialog-text-centered modal-dialog-scrollable modal-xl" role="document">
        <div class="modal-content" id="modal-content">
            <!-- Ajax content will be loaded here -->
        </div>
    </div>
</div>

