<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copytext-end       {copytext-end}
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Price;
use LartsSubscriptions\Base\BSWrapper;


HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('coupons');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=coupons' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

			<table class="table table-striped" id="couponsList">
				<thead>
					<tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
						<th class="w-1 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
						</th>
						<th class="text-start">
							<?php echo Text::_('COM_LARTSSUBS_COUPON_NAME'); ?>
						</th>
						<th class="text-start text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_COUPON_CODE'); ?>
						</th>
						<th class="w-5 text-end text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_DISCOUNT'); ?>
						</th>
						<th class="w-10 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_PRODUCT'); ?>
						</th>
						<th sclass="w-1 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_LIMITATION'); ?>
						</th>
						<th class="w-1 text-center text-nowrap">
							<span class="hasTooltip" title="<?php echo Text::_('COM_LARTSSUBS_COUPON_USAGE_DESC'); ?>">
								<?php echo Text::_('COM_LARTSSUBS_COUPON_USAGE'); ?>
							</span>
						</th>
						<th class="w-10 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_LAST_USE'); ?>
						</th>
						<th class="w-10 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_COUPON_VALID_FROM'); ?>
						</th>
						<th class="w-10 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_COUPON_VALID_TO'); ?>
						</th>
						<th class="w-1 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_COUPON_TYPE'); ?>
						</th>
					</tr>
				</thead>
				<?php foreach ($this->items as $i => $row) : ?>
					<?php
					$checked = HTMLHelper::_('grid.id', $i, $row->id);
					$link    = 'index.php?option=com_lartssubscriptions&task=coupons.edit&cid[]=' . $row->id;
					$expired = false;
					if(!empty($row->valid_to)) {
						$expired = time() > strtotime($row->valid_to);
					}
					?>
					<tr>
						<td class="text-center">
							<?php echo $checked; ?>
						</td>
						<td class="text-center">
							<?php
							$layout = new FileLayout('list_button_publish');
							echo $layout->render(['cntrl' => 'coupons', 'id' => $i, 'value' => $row->published]);
							?>
						</td>
						<td>
							<a href="<?php echo $link; ?>"><?php echo $row->name; ?></a>
						</td>
						<td class="text-start">
							<?php echo $row->coupon_code; ?>
						</td>
						<td class="text-end">
							<?php
                if($row->discount_type == 'percent') {
                  echo BSWrapper::label($row->discount . '%', 'success');
                }
                else {
                  echo BSWrapper::label(Price::_($row->discount), 'info');
                }
							?>
						</td>
						<td class="text-center">
							<?php
              if(!empty($row->products)) {
                if(count($row->products) > 3) {
                  echo '<span class="badge rounded-pill bg-dark">' .Text::_('Selection'). '</span>';
                }
                else {
                  foreach ($row->products as $prod) {
                    echo '<span class="badge rounded-pill bg-info">' .$prod. '</span>';
                  }
                }
              } ?>
						</td>
						<td class="text-center">
							<?php echo ($row->limit > 0) ? $row->limit : ''; ?>
						</td>
						<td class="text-center">
							<?php echo $row->used; ?>
						</td>
						<td class="text-center text-nowrap small">
              <?php 
                if(!empty($row->last_use)) {
                  $text = DateEx::_($row->last_use, Text::_('DATE_FORMAT_LC4'));
                  echo BSWrapper::label($text, 'default');
                }
              ?>
						</td>
						<td class="text-center text-nowrap small">
              <?php 
                if(!empty($row->valid_from)) {
                  $text = DateEx::_($row->valid_from, Text::_('DATE_FORMAT_LC4'));
                  if($expired) {
                    $ltype = 'bg-light text-dark';
                  }
                  else {
                    $ltype = 'bg-success';
                  }
                  echo '<span class="badge ' .$ltype. '">' .$text. '</span>';
                }
              ?>
						</td>
						<td class="text-center text-nowrap small">
              <?php 
                if(!empty($row->valid_to)) {
                  $text = DateEx::_($row->valid_to, Text::_('DATE_FORMAT_LC4'));
                  if($expired) {
                    $ltype = 'bg-light text-dark';
                  }
                  else {
                    $ltype = 'bg-success';
                  }
                  echo '<span class="badge ' .$ltype. '">' .$text. '</span>';
                }
              ?>
						</td>
						<td class="text-center">
							<?php $text = ($row->type == 0) ? Text::_('COM_LARTSSUBS_COUPON_TYPE_MANUAL') : Text::_('COM_LARTSSUBS_COUPON_TYPE_AUTOMATED'); 
                echo BSWrapper::label($text);
              ?>
						</td>
					</tr>
				<?php endforeach; ?>
			</table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>
		</div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
