<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\BSWrapper;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];

?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('customers');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=customers' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="customerList">
        <caption class="visually-hidden">
            <?php echo Text::_('COM_FIELDS_FIELDS_TABLE_CAPTION'); ?>,
            <span id="orderedBy"><?php echo Text::_('JGLOBAL_SORTED_BY'); ?> </span>,
            <span id="filteredBy"><?php echo Text::_('JGLOBAL_FILTERED_BY'); ?></span>
        </caption>
        <thead>
          <tr>
            <th class="w-1 text-center">
               <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th class="w-1 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JGLOBAL_FIELD_ID_LABEL', 'a.userid', $listDirn, $listOrder); ?>
            </th>
            <th class="w-15 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_NAME', 'a.name', $listDirn, $listOrder); ?>
            </th>
            <th class="w-15 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_EMAILADDRESS', 'a.email', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start">
              <?php echo Text::_('COM_LARTSSUBS_ADDRESS'); ?>
            </th>
            <th class="text-start">
              <?php echo Text::_('COM_LARTSSUBS_ZIPCODE'); ?>
            </th>
            <th class="w-10 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_CITY', 'a.email', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_COUNTRY', 'a.country', $listDirn, $listOrder); ?>
            </th>
            <th></th>
          </tr>
        </thead>
        <?php $k = 0; ?>
        <?php foreach ($this->items as $i => $row) : ?>
          <?php
          $published = HTMLHelper::_('grid.published', $row, $i);
          $checked   = HTMLHelper::_('grid.id', $i, $row->id);
          $link      = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . $row->id;

          ?>
          <tr class="<?php echo "row$k"; ?>">
            <td class="text-center">
              <?php echo $checked; ?>
            </td>
            <td class="text-center text-nowrap">
              <?php if ($row->userid > 0): ?>
                <?php echo $row->userid; ?>
              <?php else: ?>
                <?php echo BSWrapper::label('N/A', 'info'); ?>
              <?php endif; ?>
            </td>
            <td>
              <a href="<?php echo $link; ?>"> <?php echo ($row->is_business == 1) ? $row->companyname . ' - ' : ''; ?><?php echo $row->firstname . ' ' . $row->lastname; ?></a><br>
              <small><span class="icon-user"></span> <?php echo $row->username; ?></small>
            </td>
            <td class="text-start small text-nowrap">
              <span class="icon-envelope"></span> <?php echo $row->email; ?></td>
            <td class="text-start">
              <?php echo $row->address1 . ($row->address2 != '' ? ' || ' . $row->address2 : ''); ?>

            </td>
            <td class="text-start small text-nowrap">
              <?php echo $row->zipcode; ?>
            </td>
            <td class="text-start small">
              <?php echo $row->city; ?>
            </td>
            <td class="text-start small">
              <?php echo $row->country; ?><br />
              <small><?php echo ! empty($row->state) ? $row->state : null; ?></small>
            </td>
          </tr>
          <?php $k = 1 - $k; ?>
        <?php endforeach; ?>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
