<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Menu as LartsMenu;
use LartsSubscriptions\Base\BSWrapper;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::_('stylesheet', 'com_lartssubscriptions/panels.css', ['version' => 'auto', 'relative' => true]);

ToolbarHelper::title(Text::_('COM_LARTSSUBS_CP'), 'cog');
ToolbarHelper::preferences('com_lartssubscriptions');

$config = LartsConfig::get();

// Import the plugins:
PluginHelper::importPlugin('lartssubscriptions');
//PluginHelper::importPlugin('lartssubscriptions_payment');
?>

<div class="row larts-edit-row">
	<div class="span7 col-md-7">
		<?php

		$layout = new FileLayout('dashboard_tile');
		foreach (LartsMenu::adminTiles() as $id => $data)
		{
      if(strstr($data['2'], '.') == false) {
				$url = 'index.php?option=com_lartssubscriptions&view=' .$data['2'];
      }
      else {
				$url = 'index.php?option=com_lartssubscriptions&task=' .$data['2'];
      }
			echo $layout->render([
				'url'  => $url,
				'icon' => $data['0'],
				'text' => $data['1'],
			]);
		}

    ?>
		<div class="clearfix"></div>
    
    <?php $layout = new FileLayout('dashboard_dash');
			echo $layout->render(['view' => $this]);
    ?>

	</div>

	<div class="span5 col-md-5">

		<div class="panel panel-default">
			<div class="panel-heading" style="font-weight: bold"><?php echo Text::_('COM_LARTSSUBS_COMPONENT_DATA'); ?></div>
			<div class="panel-body">

				<table class="table">

					<tr>
            <?php $vt = isset($this->extparams->versiontype) ? ' - '.$this->extparams->versiontype : ''; ?>
						<td width="50%"><?php echo Text::_('COM_LARTSSUBS_VERSION'); ?></td>
            <td width="50%"><?php echo $this->extparams->version . strtoupper($vt); ?></td>
					</tr>
					<tr>
						<td width="50%"><?php echo Text::_('COM_LARTSSUBS_SUPPORT'); ?></td>
						<td width="50%"><a href="https://www.logical-arts-extensions.com/index.php/support/contact" target="_blank">Ask Your Question</a></td>
					</tr>
					<tr>
						<td><?php echo Text::_('COM_LARTSSUBS_FAQ'); ?></td>
						<td><a href="https://www.logical-arts-extensions.com/questions.html" target="_blank">Frequently Asked</a></td>
					</tr>
					<tr>
						<td><?php echo Text::_('COM_LARTSSUBS_DEVELOPMENT'); ?></td>
						<td><a href="https://www.logical-arts-extensions.com" target="_blank">Logical-Arts</a></td>
					</tr>
					<tr>
						<td><?php echo Text::_('COM_LARTSSUBS_LICENCE'); ?></td>
						<td><a href="https://www.gnu.org/licenses/old-licenses/gpl-2.0.html" target="_blank">GNU GENERAL PUBLIC LICENSE V2</a></td>
					</tr>
					<tr>
						<td><?php echo Text::_('COM_LARTSSUBS_COPYRIGHTS'); ?></td>
						<td><a href="https://www.logical-arts-extensions.com" target="_blank">&copy; <?php echo date('Y'); ?>Logical-Arts</a></td>
					</tr>

				</table>


			</div>
		</div>

		<div class="panel panel-default">
			<div class="panel-heading" style="font-weight: bold"><?php echo Text::_('COM_LARTSSUBS_STATUS'); ?></div>
			<div class="panel-body">
				<table class="table">

					<tr>
						<td width="50%"><?php echo Text::_('COM_LARTSSUBS_SHOP'); ?></td>
						<td width="50%">
              <?php
                if($config->shop_enabled) {
                  echo BSWrapper::label('Enabled', 'success');
                }
                else {
                  echo BSWrapper::label('Disabled', 'important');
                }
              ?>
            </td>
					</tr>
					<tr>
						<td width="50%"><?php echo Text::_('COM_LARTSSUBS_MAILER'); ?></td>
						<td width="50%">
              <?php
                if($config->mailer_enabled) {
                  echo BSWrapper::label('Enabled', 'success');
                }
                else {
                  echo BSWrapper::label('Disabled', 'important');
                }
              ?>
            </td>
					</tr>

          <tr>
						<td width="50%"><?php echo Text::_('COM_LARTSSUBS_PKGMANAGER'); ?></td>
						<td width="50%">
              <?php
                if($config->PkgMngrInstalled) {
                  echo BSWrapper::label('Installed', 'success');
                }
                else {
                  echo BSWrapper::label('Not Installed', 'default');
                }
              ?>
            </td>
					</tr>
					<tr>
						<td><?php echo Text::_('COM_LARTSSUBS_LICMANAGER'); ?></td>
						<td>
              <?php
                if($config->LicMngrInstalled) {
                  echo BSWrapper::label('Installed', 'success');
                }
                else {
                  echo BSWrapper::label('Not Installed', 'default');
                }
              ?>
            </td>
					</tr>
          
          <?php foreach ($this->payment_plugins as $pay) : ?>
            <tr>
              <td><?php $link = 'index.php?option=com_plugins&task=plugin.edit&extension_id='.$pay->extension_id;
                echo '<a href="'.$link.'">' .Text::_('COM_LARTSSUBS_PAYMENT') .' - ' . ucfirst($pay->element) .'</a>'; ?></td>
              <td>
                <?php
                  if($pay->enabled) { echo BSWrapper::label('Enabled', 'success'); }
                  else { echo BSWrapper::label('Disabled', 'important'); }
                ?>
              </td>
            </tr>
          <?php endforeach; ?>

				</table>

			</div>
		</div>


	</div>

</div>
