<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Helper;
use LartsSubscriptions\Base\BSWrapper;
use LartsSubscriptions\Files;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];

$files = new Files;
$pathDownloads  = $files->getBasePath();

?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('files');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=files' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="filesList">
        <caption class="visually-hidden">
            <?php echo Text::_('COM_FIELDS_FIELDS_TABLE_CAPTION'); ?>,
            <span id="orderedBy"><?php echo Text::_('JGLOBAL_SORTED_BY'); ?> </span>,
            <span id="filteredBy"><?php echo Text::_('JGLOBAL_FILTERED_BY'); ?></span>
        </caption>
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_FILENAME', 'a.filename', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo Text::_('JFIELD_ALIAS_LABEL'); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PHYSICAL_FILENAME', 'a.filename_physical', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo Text::_('JCATEGORY'); ?>
            </th>
            <th class="w-5 text-center"><?php echo Text::_('COM_LARTSSUBS_DOWNLOAD'); ?></th>
            <th class="w-5 text-center" class="text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_DATE_ADDED', 'a.created', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_DOWNLOADS', 'a.downloads', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_VIEWS', 'a.views', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_DOWNLOAD_LOCATION'); ?>
            </th>
          </tr>
        </thead>
        <?php $count = count($this->items); ?>
        <?php foreach ($this->items as $i => $row) : ?>
          <?php

          $published = HTMLHelper::_('grid.published', $row, $i);
          $checked   = HTMLHelper::_('grid.id', $i, $row->id);
          $link      = 'index.php?option=com_lartssubscriptions&&task=files.edit&cid[]=' . $row->id;
          $download  = 'index.php?option=com_lartssubscriptions&task=files.download&file=' . $row->id;
          $filename  = $row->filename_physical;
          if(strlen($filename) > 0 && $row->source_file_type != Files::SOURCE_TYPE_EXTERN) {
            $filename = $pathDownloads. '/' .$row->filename_physical;
          }
          ?>
          <tr>
            <td class="text-center">
              <?php echo $checked; ?>
            </td>
            <td class="text-center">
              <?php
              $layout = new FileLayout('list_button_publish');
              echo $layout->render(['cntrl' => 'files', 'id' => $i, 'value' => $row->published]);
              ?>
            </td>
            <td class="text-start">
              <a href="<?php echo $link; ?>"><?php echo $row->filename; ?></a><br>
              <div class="hidden-desktop">
                <small>
                  <?php echo Text::_('JFIELD_ALIAS_LABEL'); ?>: <?php echo $row->alias; ?>
                  <?php
                  echo empty($row->categories)
                    ? ''
                    : '<br>' . Text::_('JCATEGORIES') . ': '
                    . (count($row->categories) > 3
                      ? BSWrapper::label(Text::_('Selection'), 'primary')
                      : BSWrapper::label($row->categories, 'info')
                    );
                  ?>
                </small>
              </div>
            </td>
            <td class="text-start small">
              <?php echo $row->alias; ?>
            </td>
            <td class="text-start small">
              <?php echo $row->filename_physical; ?>
            </td>
            <td class="text-start small">
              <?php
              echo empty($row->categories)
                ? ''
                : (count($row->categories) > 3
                  ? '<span class="label">' . Text::_('Selection') . '</span>'
                  : '<span class="badge bg-info">'
                  . implode('</span> <span class="badge bg-nfo">', $row->categories)
                  . '</span>'
                );
              ?>
            </td> 
            <td class="text-center">
              <a href="<?php echo $download; ?>" class="btn btn-sm btn-outline-secondary" type="button"><?php echo Text::_('COM_LARTSSUBS_DOWNLOAD_FILE'); ?></a>
            </td>
            <td class="text-center text-nowrap small">
              <?php echo DateEx::_($row->created, Text::_('DATE_FORMAT_LC4')); ?>
            </td>
            <td class="text-center">
              <?php echo $row->downloads; ?>
            </td>
            <td class="text-center">
              <?php echo $row->views; ?>
            </td>
            <td class="text-center text-nowrap">
              <?php
                $srctype = ($row->source_file_type == Files::SOURCE_TYPE_EXTERN) ? Text::_('COM_LARTSSUBS_EXTERNAL') : Text::_('COM_LARTSSUBS_LOCAL');
                $labeltype = 'default';
                if($row->source_file_type != Files::SOURCE_TYPE_EXTERN) {
                  $labeltype = file_exists($filename) ? 'success' : 'important';
                }
                echo BSWrapper::label($srctype, $labeltype);
              ?>
            </td>
          </tr>
        <?php endforeach; ?>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
