<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copytext-end       {copytext-end}
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Price;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\Base\BSWrapper;
use LartsSubscriptions\Invoice;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/refund.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::_('script','com_lartssubscriptions/refund.js', array('version' => 'auto', 'relative' => true));
HTMLHelper::_('script','com_lartssubscriptions/invoices.js', array('version' => 'auto', 'relative' => true));

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('invoices');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=invoices' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

			<table class="table table-striped" id="invoicesList">
        <caption class="visually-hidden">
            <?php echo Text::_('COM_FIELDS_FIELDS_TABLE_CAPTION'); ?>,
            <span id="orderedBy"><?php echo Text::_('JGLOBAL_SORTED_BY'); ?> </span>,
            <span id="filteredBy"><?php echo Text::_('JGLOBAL_FILTERED_BY'); ?></span>
        </caption>
				<thead>
					<tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
						<th class="w-10 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JGLOBAL_FIELD_ID_LABEL', 'a.invoice_number', $listDirn, $listOrder); ?>
						</th>
						<th class="w-10 text-center">
							<?php echo Text::_('COM_LARTSSUBS_DOWNLOAD'); ?>
						</th>
						<th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_NAME', 'a.name', $listDirn, $listOrder); ?>
						</th>
						<th class="w-10 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_INVOICE_DATE', 'a.date', $listDirn, $listOrder); ?>
						</th>
						<th class="w-10 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_ORDER_ID', 'a.orderid', $listDirn, $listOrder); ?>
						</th>
						<th class="w-10 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_PAYMENT_METHOD'); ?>
						</th>
						<th class="w-5 text-end text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_PRICE'); ?>
						</th>
						<th class="w-5 text-end text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_DISCOUNT'); ?>
						</th>
						<th class="w-5 text-end text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_VAT'); ?>
						</th>
						<th class="w-5 text-end text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_VAT'); ?> (%)
						</th>
						<th class="w-5 text-end text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_NET_TOTAL'); ?>
						</th>
						<th class="w-1 text-center text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_TYPE'); ?>
						</th>
						<th class="w-1 text-center text-nowrap">
							<?php echo Text::_('JSTATUS'); ?>
						</th>
					</tr>
				</thead>
				<?php $k = 0; ?>
				<?php foreach ($this->items as $i => $row) : ?>
					<?php
					$published = HTMLHelper::_('grid.published', $row, $i);
					$checked   = HTMLHelper::_('grid.id', $i, $row->id);
					$link      = 'index.php?option=com_lartssubscriptions&task=invoices.edit&cid[]=' . $row->id;
					$dl        = 'index.php?option=com_lartssubscriptions&task=invoices.getinvoice&cid[]=' . $row->id;
					$enable = ($row->invoice_type == Invoice::TYPE_CREDIT) ? 'disabled' : null;
					?>
					<tr class="<?php echo "row$k"; ?>">
						<td class="text-center">
							<?php echo $checked; ?>
						</td>
						<td class="text-center text-nowrap">
							<a href="<?php echo $link; ?>"><?php echo ! empty($row->invoice_number) ? $row->invoice_number : $row->id; ?></a>
						</td>
						<td class="text-center">
							<div class="btn-group">
								<a class="btn btn-micro" href="<?php echo $dl; ?>">
                  <i class="icon-download" title="<?php echo Text::_('COM_LARTSSUBS_DOWNLOAD_INVOICE'); ?>"></i>
                </a>
			
								<?php if ($row->invoice_type == Invoice::TYPE_DEBIT) : ?>
									<a class="btn btn-sm btn-danger refund <?php echo $enable; ?>" href="javascript:void(0);" id="invoice_<?php echo $row->id; ?>" onClick="onClickRefund(<?php echo $row->id; ?>)">
                    <i class="icon-undo" title="<?php echo Text::_('COM_LARTSSUBS_REFUND'); ?>"></i>
                  </a>
								<?php else: ?>
									<a class="btn btn-sm btn-inverse disabled" href="#" id="invoice_<?php echo $row->id; ?>">
                    <i class="icon-undo"></i>
                  </a>
								<?php endif; ?>

							</div>
						</td>
						<td class="text-start">
							<?php if ($row->userid > 0): ?>
								<a href="<?php echo $link; ?>"><?php echo $row->firstname . ' ' . $row->lastname; ?></a><br>
								<small>
									<?php echo $row->address1; ?>
									<?php echo $row->zipcode ? '- ' . $row->zipcode : ''; ?>
									<?php echo $row->city ? '- ' . $row->city : ''; ?>
									<?php echo $row->country_code ? '( ' . $row->country_code . ' )' : ''; ?>
									<div class="visible-phone clearfix">
										<?php echo Text::_('COM_LARTSSUBS_INVOICE_DATE'); ?>: <?php echo DateEx::_($row->invoicedate); ?>
									</div>
								</small>
							<?php else: ?>
                <?php echo BSWrapper::label(Text::_('COM_LARTSSUBS_REQUESTED_ANONYMISATION'), 'info'); ?>
							<?php endif; ?>
						</td>
						<td class="text-center text-nowrap small">
							<?php echo DateEx::_($row->invoicedate); ?>
						</td>
						<td class="text-center text-nowrap small">
              <?php echo BSWrapper::label($row->ordercode); ?>
						</td>
						<td class="text-center text-nowrap small">
              <?php echo BSWrapper::label($row->provider); ?>
						</td>
						<td class="text-end text-nowrap small">
							<?php echo Price::_($row->gross_price); ?>
						</td>
						<td class="text-end text-nowrap small">
							<?php echo $row->disco_price ? Price::_($row->disco_price) : ''; ?>
						</td>
						<td class="text-end text-nowrap small">
							<?php echo $row->vat_total ? Price::_($row->vat_total) : ''; ?>
						</td>
						<td class="text-end text-nowrap small">
							<?php echo $row->vat_percentage ? $row->vat_percentage . '%' : ''; ?>
						</td>
						<td class="text-end text-nowrap small">
							<?php echo Price::_($row->net_price); ?>
						</td>
						<td class="text-center">
              <?php 
                $ltype = $row->invoice_type == Invoice::TYPE_CREDIT ? 'important' : 'success';
                $ltxt = $row->invoice_type == Invoice::TYPE_CREDIT ? 'C' : 'D';
                $ltooltip = $row->invoice_type == Invoice::TYPE_CREDIT ? 'COM_LARTSSUBS_CREDIT_INVOICE' : 'COM_LARTSSUBS_DEBIT_INVOICE';
                $ltooltip = Text::_($ltooltip);
                echo BSWrapper::label($ltxt, $ltype, $ltooltip);
              ?>
              <?php //echo OrderStatus::hasState($row->order_status, OrderStatus::REFUNDED) ? 'C' : 'D' ?>
						</td>
						<td class="text-center">
							<?php 
                $ltype = $row->paid ? 'success' : 'warning';
                $text = Text::_($row->paid ? 'COM_LARTSSUBS_INVOICE_STATE_PAID' : 'COM_LARTSSUBS_INVOICE_STATE_UNPAID');
                echo BSWrapper::label($text, $ltype);
              ?>
						</td>
					</tr>
					<?php $k = 1 - $k; ?>
				<?php endforeach; ?>
			</table>

      <div class="text-text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>

