<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Helper;

$wa = $this->document->getWebAssetManager();
$wa->useScript('keepalive')
   ->useScript('form.validate');

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);
?>

<?php if ($this->data->paid == 3) : ?>
	<div class="alert alert-info">
		<h4><?php echo Text::_('COM_LARTSSUBS_INFORMATION'); ?></h4>
		<?php echo Text::sprintf('COM_LARTSSUBS_REFUNDED_AT_DATE', DateEx::_($this->data->refund_date)); ?>
	</div>
<?php endif; ?>

<?php if ($this->data->invoice_type == 1) : ?>
	<div class="alert alert-info">
		<h4><?php echo Text::_('COM_LARTSSUBS_INFORMATION'); ?></h4>
		<?php echo Text::sprintf('COM_LARTSSUBS_REFUND_FOR_INVOICE', $this->data->refund_invoice); ?>
	</div>
<?php endif; ?>

	<form action="index.php" method="POST" name="adminForm" id="adminForm" class="form-horizontal">

		<div class="row larts-edit-row">
			<div class="span4 col-md-4">
				<fieldset class="form-horizontal-desktop">
					<h4><?php echo Text::_('COM_LARTSSUBS_INVOICE_INFORMATION'); ?></h4>

					<?php echo $this->form->renderFieldset('invoice'); ?>
				</fieldset>
			</div>

			<div class="span4 col-md-4">
				<fieldset class="form-horizontal-desktop">
					<h4><?php echo Text::_('COM_LARTSSUBS_PRICE_INFORMATION'); ?></h4>

					<?php echo $this->form->renderFieldset('price'); ?>
				</fieldset>
			</div>

			<div class="span4 col-md-4">
				<fieldset class="form-horizontal-desktop">
					<h4><?php echo Text::_('COM_LARTSSUBS_CUSTOMER_INFORMATION'); ?></h4>

					<?php echo $this->form->renderFieldset('customer'); ?>

					<?php if(!empty($this->data->companyname)) : ?>
					<fieldset class="form-horizontal-desktop">
						<h4><?php echo Text::_('COM_LARTSSUBS_COMPANY_INFORMATION'); ?></h4>

						<?php echo $this->form->renderFieldset('company'); ?>
						<?php endif; ?>
					</fieldset>
			</div>
		</div>
		<div class="row larts-edit-row">
			<div class="span7 col-md-7">
				<fieldset class="form-horizontal-desktop">
					<h4><?php echo Text::_('COM_LARTSSUBS_PRODUCTS'); ?></h4>

					<table class="table table-striped">
						<thead>
							<tr>
								<th style="width:1%;" class="center text-nowrap">
									<?php echo Text::_('JGLOBAL_FIELD_ID_LABEL'); ?>
								</th>
								<th class="left">
									<?php echo Text::_('COM_LARTSSUBS_PRODUCT'); ?>
								</th>
								<th style="width:15%;" class="center text-nowrap hidden-tablet">
									<?php echo Text::_('COM_LARTSSUBS_PERIOD'); ?>
								</th>
								<th style="width:15%;" class="center text-nowrap">
									<?php echo 'Licence'; ?>
								</th>
								<th style="width:15%;" class="center text-nowrap">
									<?php echo Text::_('COM_LARTSSUBS_EXPIRES'); ?>
								</th>
							</tr>
						</thead>
						<?php $k = 0; ?>
						<?php foreach ($this->products as $i => $row) : ?>
							<?php
							$linksub = 'index.php?option=com_lartssubscriptions&amp;controller=subscriptions&amp;task=edit&amp;cid[]=' . (int) $row->id;
							$period  = $row->count_products * $row->period;
							?>
							<tr class="<?php echo "row$k"; ?>">
								<td class="center text-nowrap">
                  <?php if($row->id == 0) {
                    echo '-';
                  }
                  else {
                    echo '<a href="'.$linksub.'">'.$row->id.'</a>';
                  }
									?>
								</td>
								<td class="left">
                  <?php if($row->id == 0) {
                    echo 'Coupon';
                  }
                  else {
                    echo $row->name;
                    echo '<div class="small hidden-desktop">';
                    echo Text::_('COM_LARTSSUBS_PERIOD') . ' : ';
                    echo Helper::getPeriodText($period, $row->period_type);
                    echo '<br></div>';
                  }
									?>
								</td>
								<td class="center small hidden-tablet">
                  <?php if($row->id == 0) { echo '-'; }
                    else { echo Helper::getPeriodText($period, $row->period_type); }
									?>
								</td>
								<td class="center small hidden-tablet">
                  <?php if($row->id == 0) { echo '-'; }
                    else { echo $row->licence; }
									?>
								</td>
								<td class="center text-nowrap small">
                  <?php if($row->id == 0) { echo '-'; }
                    else { 
                      $valid_to = isset($row->valid_to) ? $row->valid_to : '';
                      echo ( ! in_array($row->period_type, ['lifetime', 'onetime'])) ? DateEx::_($valid_to) : '';
                    }
									?>
								</td>
							</tr>
							<?php $k = 1 - $k; ?>
						<?php endforeach; ?>
					</table>
				</fieldset>
			</div>
		</div>

		<input type="hidden" name="id" value="<?php echo $this->data->id; ?>" />
		<input type="hidden" name="option" value="com_lartssubscriptions" />
		<input type="hidden" name="task" value="" />
		<input type="hidden" name="controller" value="invoices" />
	</form>

<?php
