<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Base\BSWrapper;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('logfiles');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=logfiles' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      
      <table class="table table-striped">
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th class="w-10 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_TYPE', 'a.type', $listDirn, $listOrder); ?>
            </th>
            <th class="w-20 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PERFORMED_ACTION', 'a.request_type', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_NAME', 'a.lastname', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_COUNTRY', 'a.country', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_IP_ADDRESS', 'a.ip_address', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JDATE', 'a.date', $listDirn, $listOrder); ?>
            </th>
          </tr>
        </thead>
        <?php foreach ($this->items as $i => $row) : ?>
          <?php
          $link_user = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . (int) $row->user_id;

          if ($row->user_id == 0 && $row->userid > 0)
          {
            $link_user = 'index.php?option=com_users&task=user.edit&id=' . (int) $row->userid;
          }

          $user_info = ! empty($row->lastname) ? $row->firstname . ' ' . $row->lastname : $row->name;
          $checked   = HTMLHelper::_('grid.id', $i, $row->id);
          ?>
          <tr>
            <td class="text-start">
              <?php echo $checked; ?>
            </td>
            <td class="text-start text-nowrap">
              <?php
              if ($row->type != 1) {
                echo Text::_('COM_LARTSSUBS_INVOICE');
              }
              else if ($row->request_type == 1) {
                echo Text::_('COM_LARTSSUBS_REMOTE') . ' ' . Text::_('COM_LARTSSUBS_DOWNLOAD');
              }
              else if ($row->request_type == 2) {
                echo Text::_('COM_LARTSSUBS_WATCHFUL') . ' ' . Text::_('COM_LARTSSUBS_DOWNLOAD');
              }
              else {
                echo Text::_('COM_LARTSSUBS_WEBSITE') . ' ' . Text::_('COM_LARTSSUBS_DOWNLOAD');
              }
              ?>
              <?php if ($row->user_id > 0): ?>
                <br>
                <?php echo BSWrapper::label(Text::_('COM_LARTSSUBS_SUBSCRIPTION'), 'info'); ?>
              <?php endif; ?>

              <div class="visible-phone">
                <small><?php echo $row->action; ?></small>
              </div>
            </td>
            <td class="text-start">
              <?php echo $row->action; ?>
            </td>
            </td>
            <td class="text-start">
              <?php if ($row->user_id > 0 || $row->userid > 0): ?>
                <a href="<?php echo $link_user; ?>">
                  <?php echo $user_info; ?>
                </a>
                <br>
                <small><?php echo $row->email; ?></small>
                <div class="visible-phone">
                  <small><?php echo $row->country; ?></small>
                </div>
              <?php else: ?>
                <?php echo BSWrapper::label('N/A', 'info'); ?>
              <?php endif; ?>
            </td>
            <td class="text-start text-nowrap">
              <?php echo isset($row->country) ? $row->country : $row->country_code; ?>
            </td>
            <td class="text-start text-nowrap">
              <?php echo $row->ip_address; ?>
            </td>
            <td class="text-center text-nowrap">
              <?php echo DateEx::_($row->date, Text::_('DATE_FORMAT_LC4')); ?>
            </td>
          </tr>
        <?php endforeach; ?>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
