<?php
/**
 * @package         Lartssubscriptions Free
 * @version         21.9.2
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2021 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
//use Joomla\CMS\Router\Route;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;
//use Joomla\CMS\Filter\OutputFilter;

use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\Price;
use LartsSubscriptions\Helper;
//use LartsSubscriptions\Base\BSWrapper;
use LartsSubscriptions\Periods;
//use LartsSubscriptions\ThirdParty\LartsLicManager;
use LartsSubscriptions\Base\PluginManager;


// Get the ordercode if there is one.
$ordercode = intval(Factory::getSession()->get('ordercode'));
$userid = $this->form->getValue('userid', null, 0);

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::stylesheet('com_lartssubscriptions/stylej3.css', ['version' => 'auto', 'relative' => true]);

$amount = new OrderAmounts($ordercode, true);

$grossprice     = $amount->grossprice();
$net_price      = $amount->netprice();
$discount_price = $amount->discount();
$vat_price      = $amount->vatprice();
$vatrates    = $amount->getVatPercentsAppliedStr();

Factory::getDocument()->addScriptDeclaration('
  function onChangeUser()
  {
    form = document.getElementById("adminForm");
    
    input = document.createElement("input");
    input.setAttribute("name", "setuser");
    input.setAttribute("value", "setuser");
    input.setAttribute("type", "hidden");

    form.appendChild(input);

    form.submit();
  }
  function onChangeNewUser()
  {
    userid = document.getElementById("userid");
    userid.setAttribute("value", "");
  }
');

?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('order');
?>

<form action="index.php" method="POST" name="adminForm" id="adminForm" enctype="multipart/form-data">
<div class="row larts-edit-row">


  <?php // Customer column ?>
	<div class="span4 col-md-4 larts-col-manualcart">

      <div class="larts-user-select">
        <fieldset class="form-horizontal">
        <h4><?php echo Text::_('COM_LARTSSUBS_LOAD_CUSTOMER_DATA'); ?></h4>
          <?php echo $this->form->renderFieldset('user_select'); ?>
        </fieldset>
      </div>
			<h4><?php echo Text::_('COM_LARTSSUBS_CUSTOMER_INFORMATION'); ?></h4>
      <fieldset class="form-horizontal">
        <?php echo $this->form->renderFieldset('customer'); ?>
      </fieldset>

	</div>
    
  <?php // Products column ?>
  <div class="span3 col-md-4 larts-col-manualcart">

			<h4>
        <xdiv class="container"><div class="row">
          <div class="col-6">
            <?php echo Text::_('COM_LARTSSUBS_SELECT_PRODUCT'); ?>
          </div>
          <div class="col-6">
            <button class="btn btn-success btn-mini" name="addprod" value="addprod" type="submit"><?php echo Text::_('COM_LARTSSUBS_ADD_PRODUCTS'); ?></button>
          </div>
        </div></xdiv>
			</h4>

      <fieldset class="input-group mb-2">
        <input type="text" name="product_filter" class="form-control" value="<?php echo $this->product_filter; ?>" size="25" style="width: 150px;"/>
        <div class="input-append">
          <button class="btn btn-primary " name="filter" value="filter" type="submit"><?php echo 'search'; ?></button>
        </div>
      </fieldset>
      
      <table class="table table-striped">

				<?php foreach ($this->products as $item) : ?>
					<tr>
						<td width="2%">
							<div align="center">
								<input name="product[]" type="checkbox" value="<?php echo $item->id; ?>" />
							</div>
						</td>
						<td>
							<div align="left"><?php echo $item->name; ?></div>
						</td>
						<td width="25%">
              <div align="left"><?php echo Periods::str($item->period, $item->period_type); ?></div>
						</td>
					</tr>
				<?php endforeach; ?>
			</table>

	</div>

  <?php // Cart column ?>
	<div class="span5 col-md-4 larts-col-manualcart">

    <h4><?php echo Text::_('COM_LARTSSUBS_COUPON_CODE'); ?></h4>
    <fieldset class="input-group mb-2">
      <input type="text" name="coupon_code" class="form-control" id="coupon_code" maxlength="50" />
      <div class="input-append">
        <input name="coupon" value="coupon" type="submit" class="btn btn-outline-secondary" value="<?php echo Text::_('COM_LARTSSUBS_COUPON_SUBMIT'); ?>" />
      </div>
    </fieldset>

 		<div style="clear:both; margin-bottom:40px;"></div>

		<h4><?php echo Text::_('COM_LARTSSUBS_YOUR_CART'); ?></h4>

		<table class="table">
			<thead>
				<th class="w-70"><?php echo Text::_('COM_LARTSSUBS_PRODUCT'); ?></th>
				<th class="w-20"></th>
				<th class="w-10"></th>
			</thead>

			<?php foreach ($this->items as $i => $item) : ?>
				<?php
				$checked   = HTMLHelper::_('grid.id', $i, $item->id);
				$remove    = 'index.php?option=com_lartssubscriptions&task=order.remove&pid=' . $item->product_id;
				$price     = $item->quantity * $item->price;
				$label = ($userid > 0 ? 'success' : 'warning');

				?>
				<?php if ($item->product_id != 0) : ?>

					<tr>
						<td>
              <b><?php echo $item->quantity; ?>&nbsp;x&nbsp;</b>
              <?php //echo BSWrapper::label($item->quantity, 'success'); ?>
							<?php echo $item->product_name; ?>

							<br><small> (<?php echo Helper::getPeriodText($item->period, $item->period_type); ?> <?php echo Text::_('COM_LARTSSUBS_SUBSCRIPTION'); ?>)</small>

							<?php // discount
							if(!empty($item->id) && $item->discount > 0) {
                $txt = Price::getDiscountText($item->discount_group, $item->discount_type, $item->discount_amount, $item->discount);
								echo '<br/><small>' .$txt. '</small>';
							}
							?>
              <?php // licence
              if($item->has_licence) {
                if(empty($userid)) {
                  echo '<br />for new licence';
                }
                else
                {
                  // Trigger plugins
                  $lics = array();
                  $plgReturns = PluginManager::execPlugins('lartssubscriptions', 'onGetUserLicences', ['userid' => $userid], false, false);
                  if(!empty($plgReturns)) {
                    // walk back to front 
                    foreach(array_reverse($plgReturns) as $plgRet) {
                      foreach($plgRet as $lic) {
                        $lics[] = $lic;
                      }
                    }
                  }

                  echo '<div class="larts_cart_licence"><small style="margin-right: 10px;">for licence:</small>';
                  echo '<select class="form-select form-select-sm " onchange="updateProductLicInCart('.$item->product_id.', this.value);">';
                  echo '<option' . (empty($item->licence) ? ' selected="selected"' : ''). ' value="0">New Licence</option>';

                  if(!empty($lics)) {
                    foreach ($lics as $lic) {
                      if($lic->licence_number == $item->licence) {
                        echo '<option selected="selected" value="'.$lic->licence_number.'">'.$lic->licence_number.'</option>';
                      }
                      else {
                        echo '<option value="'.$lic->licence_number.'">'.$lic->licence_number.'</option>';
                      }
                    }
                  }
                  echo '</select>';
                  echo '</div>';

                }
              }
              ?>
						</td>
						<td>
							<div style="text-align:right;"><?php echo Price::_($price); ?></div>
						</td>
						<td>
							<div style="padding-top:-3px; text-align:center;">
								<a href="<?php echo $remove; ?>">
									<i class="fas fa-minus-square darkred" aria-hidden="true"></i>
								</a>
							</div>
						</td>
					</tr>

				<?php else : ?>

					<tr>
						<td colspan="3">
							<?php if(!empty($item->id)) {
                $txt = Price::getDiscountText($item->discount_group, $item->discount_type, $item->discount_amount, $item->discount);
								echo '<br/><small>' .$txt. '</small>';
							}
							?>

						</td>
					</tr>

				<?php endif; ?>

			<?php endforeach; ?>

		</table>

		<table class="table borderless pull-right float-end" style="width:80%;">

			<?php if ($discount_price != 0) : ?>
				<tr>
					<td width="70%">
						<div align="right"><?php echo Text::_('COM_LARTSSUBS_ORDER_SUBTOTAL'); ?></div>
					</td>
					<td width="40%">
						<div style="text-align:right; padding-right:8px;">
							<?php echo Price::_($grossprice); ?>
						</div>
					</td>
					<td width="5%"></td>
				</tr>
				<tr>
					<td width="50%">
						<div align="right"><?php echo Text::_('COM_LARTSSUBS_DISCOUNT'); ?></div>
					</td>
					<td width="40%">
						<div style="text-align:right; padding-right:8px;">
							<?php echo Price::_($discount_price); ?>
						</div>
					</td>
					<td></td>
				</tr>

			<?php endif; ?>

			<tr>
				<td width="70%">
					<div align="right"><?php echo Text::_('COM_LARTSSUBS_ORDER_SUBTOTAL'); ?></div>
				</td>
				<td width="40%">
					<div style="text-align:right; padding-right:8px;">
						<?php echo Price::_($grossprice - $discount_price); ?>
					</div>
				</td>
				<td></td>
			</tr>

      <tr>
        <td>
          <div align="right"><?php echo Text::_('COM_LARTSSUBS_TOTAL_VAT'); ?>
            <small>( <?php echo $vatrates; ?> )</small>
          </div>
          
        </td>
        <td>
          <div style="text-align:right; padding-right:8px;">
            <?php echo Price::_($vat_price); ?>
          </div>
        </td>
        <td>&nbsp;</td>
      </tr>

			<tr>
				<td>
					<div align="right"><?php echo Text::_('COM_LARTSSUBS_ORDER_TOTAL'); ?></div>
				</td>
				<td>
					<div style="text-align:right; padding-right:8px;">
						<?php echo Price::_($net_price); ?>
					</div>
				</td>
				<td>&nbsp;</td>
			</tr>

		</table>

	</div>
    <input type="hidden" name="option" value="com_lartssubscriptions" />
    <input type="hidden" name="task" value="order.onSubmit" />
</div>
</form>

