<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;

use LartsSubscriptions\Price;
use LartsSubscriptions\Transaction;
use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\OrderAmounts;


HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

$model = $this->getModel();
$ordercode = Factory::getApplication()->GetInput()->getInt('ordercode', 0);
$order = $model->getOrder($ordercode);
$amounts = new OrderAmounts($ordercode);

$transObj = new Transaction;
$transactions = $transObj->lookupAll($ordercode);

?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('');
?>

<form action="index.php?option=com_lartssubscriptions" method="POST" name="adminForm" id="adminForm">
	<input type="hidden" name="task" value="" />
</form>

<div class="row larts-edit-row">
  <div class="span4 col-md-4">

  <h4>Order <?php echo $ordercode; ?></h4>

  <table class="table">
    <thead>
      <th style=""><?php echo Text::_('COM_LARTSSUBS_PRODUCT'); ?></th>
      <th style="width:50px">Qantity</th>
      <th style="width:50px">Price</th>
      <th style="width:50px">Total</th>
    </thead>
    <?php foreach ($order as $idx => $item) : ?>
    <?php 
        $prodname = $item->productname;
        $price = $item->price;
        $total = $item->quantity * $item->price;
        if(!empty($item->coupon_code)) {
          $prodname = 'Coupon: ' .$item->coupon_code;
          $price = 0;
          $total = -($item->discount);
        }
     ?>
    <tr>
      <td>
        <div><?php echo $prodname; ?></div>
      </td>
      <td>
        <div style="text-align:center; padding-right:8px;">
          <?php echo $item->quantity; ?>
        </div>
      </td>
      <td>
        <div style="text-align:right; padding-right:8px;">
          <?php echo Price::_($price); ?>
        </div>
      </td>
      <td>
        <div style="text-align:right; padding-right:8px;">
          <?php echo Price::_($total); ?>
        </div>
      </td>
    </tr>
    <?php endforeach; ?>
    
    <?php // Totals ?>
    <tr>
      <td></td>
      <td colspan="2">
        <div style="text-align:left; padding-right:8px;">VAT</div>
      </td>
      <td>
        <div style="text-align:right; padding-right:8px;">
          <?php echo Price::_($amounts->vat_total); ?>
        </div>
      </td>
    </tr>
    <tr>
      <td></td>
      <td colspan="2">
        <div style="text-align:left; padding-right:8px;">Discount</div>
      </td>
      <td>
        <div style="text-align:right; padding-right:8px;">
          <?php echo Price::_($amounts->discount); ?>
        </div>
      </td>
    </tr>
    <tr>
      <td></td>
      <td colspan="2">
        <div style="text-align:left; padding-right:8px;">Total</div>
      </td>
      <td>
        <div style="text-align:right; padding-right:8px;">
          <?php echo Price::_($amounts->netprice); ?>
        </div>
      </td>
    </tr>
    
  </table>
  
  <?php // TRANSACTION ?>
  
  <span><hr/></span>
  <table class="table">
    <thead>
      <th class="left"><?php echo Text::_('COM_LARTSSUBS_TRANSACTION'); ?></th>
      <th class="right" style="width:150px"><?php echo Text::_('JDATE'); ?></th>
      <th class="left" style="width:60px"><?php echo Text::_('COM_LARTSSUBS_AMOUNT'); ?></th>
      <th class="left" style="width:100px"><?php echo Text::_('COM_LARTSSUBS_PAYMENT_TYPE'); ?></th>
    </thead>
    <?php foreach ($transactions as $idx => $trans) : ?>
      <tr>
        <td>
          <div><?php echo $trans->transaction_id; ?></div>
        </td>
        <td>
          <div style="text-align:left; padding-right:8px;">
            <?php echo DateEx::_($trans->date, 'd.m.Y H:i:s'); ?>
          </div>
        </td>
        <td>
          <div style="text-align:right; padding-right:8px;">
            <?php echo Price::_($trans->amount); ?>
          </div>
        </td>
        <td>
          <div style="text-align:right; padding-right:8px;">
            <span class="label"><?php echo $trans->type; ?></span>
          </div>
        </td>
      </tr>
      <?php endforeach; ?>
    </table>

  </div>
</div>