<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copytext-end       {copytext-end}
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\TemporaryTransaction;
use LartsSubscriptions\Price;
use LartsSubscriptions\OrderAmounts;
use LartsSubscriptions\OrderStatus;
use LartsSubscriptions\TransactionType;
use LartsSubscriptions\Base\BSWrapper;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];


$field = Factory::getApplication()->GetInput()->get('field', '');
$order = Factory::getApplication()->GetInput()->get('order', '');

?>

<script language="javascript">
  function submitme(option) {
    a = option.split(",");
    document.adminForm.field.value = a[0];
    document.adminForm.order.value = a[1];
    document.adminForm.submit();
  }
</script>

<?php
$layout = new FileLayout('menu');
echo $layout->render('orders');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=orders' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-success">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('COM_LARTSSUBS_NO_ORDERS_FOUND'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped">
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th class="w-10 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_ORDERCODE', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_NAME', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start d-md-none">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_ORDERCODE', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_IP_ADDRESS', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JDATE', 'o.date', $listDirn, $listOrder); ?>
            </th>
            <th class="text-end text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_AMOUNT'); ?>
            </th>
            <th class="text-end text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_VAT'); ?>
            </th>
            <th class="text-end text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_COUNTRY', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="text-end text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_DISCOUNT'); ?>
            </th>
            <th class="text-end text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PAID', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_STATE', 'o.ordercode', $listDirn, $listOrder); ?>
            </th>
          </tr>

        </thead>
        <?php foreach ($this->items as $i => $row) : ?>

          <?php
          $checked         = HTMLHelper::_('grid.id', $i, $row->id);
          $link_user       = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . (int) $row->user_id;
          $link_ordercode  = 'index.php?option=com_lartssubscriptions&task=order.show&ordercode=' . (int) $row->ordercode;
          $processed_label = OrderStatus::hasState($row->status, OrderStatus::PAID) ? 'success' : 'warning';
          $processed_name  = '';
          if(OrderStatus::hasState($row->status, OrderStatus::PROCESSED)) {
            $processed_name = OrderStatus::Str($row->status & OrderStatus::PROCESSED);
          }
          else {
            $processed_name = OrderStatus::Str($row->status & OrderStatus::PENDING);
          }

          $amounts = new OrderAmounts;
          $amounts ->setJson($row->amounts);
          $params          = ! empty($row->params) ? json_decode($row->params) : [];
          ?>

          <tr id="order<?php echo $row->id; ?>">
            <td class="text-center">
              <?php echo $checked; ?>
            </td>
            <td class="text-start">
              <a href="<?php echo $link_ordercode; ?>">
                <?php echo $row->ordercode; ?>
              </a>
            </td>
            <td class="text-start">
              <a href="<?php echo $link_user; ?>">
                <?php echo $row->firstname . ' ' . $row->lastname; ?>
              </a><br>
              <?php echo $row->email; ?>
            </td>
            <td class="text-start d-md-none">
              <?php echo $row->ordercode; ?>
              <br>
              <?php echo $row->email; ?>
              <br>
              <a href="<?php echo $link_user; ?>">
                <?php echo $row->firstname . ' ' . $row->lastname; ?>
              </a><br>
              <?php echo $row->email; ?>
            </td>
            <td class="text-start text-nowrap small">
              <?php echo $row->ip_address; ?>
            </td>
            <td class="text-start text-nowrap small">
              <?php echo DateEx::_($row->created); ?>
            </td>
            <td class="text-end text-nowrap small">
              <?php echo Price::_($amounts->netprice); ?>
            </td>
            <td class="text-end text-nowrap small">
              <?php echo Price::_($amounts->vat_total); ?>
            </td>
            <td class="text-start text-nowrap small">
              <?php echo $row->country_code; ?>
            </td>
            <td class="text-end text-nowrap small">
              <?php echo Price::_($amounts->discount); ?>
            </td>
            <td class="text-end text-nowrap small">
              <?php echo Price::_($amounts->payed_total); ?>
            </td>
            <td class="text-start">
              <?php echo BSWrapper::label($processed_name, $processed_label); ?>
            </td>
          </tr>

        <?php endforeach; ?>

      </table>

      <div class="text-d-none d-md-table-cell"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
		<input name="boxchecked" type="hidden" value="0" />
    <input type="hidden" name="field" value="<?php echo $field; ?>" />
    <input type="hidden" name="order" value="<?php echo $order; ?>" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>


    
