<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Price;
use LartsSubscriptions\Helper;
use LartsSubscriptions\Product;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$ordering  = ($listOrder == 'a.ordering');
$saveOrder = ($listOrder == 'a.ordering' && strtolower($listDirn) == 'asc');

if ($saveOrder && !empty($this->items)) {
  $saveOrderingUrl = 'index.php?option=com_lartssubscriptions&task=products.saveOrderAjax&tmpl=component&' . Session::getFormToken() . '=1';
  HTMLHelper::_('draggablelist.draggable');
}
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('products');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=products' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="productList">
        <caption class="visually-hidden">
            <?php echo Text::_('COM_FIELDS_FIELDS_TABLE_CAPTION'); ?>,
            <span id="orderedBy"><?php echo Text::_('JGLOBAL_SORTED_BY'); ?> </span>,
            <span id="filteredBy"><?php echo Text::_('JGLOBAL_FILTERED_BY'); ?></span>
        </caption>
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
                <?php echo HTMLHelper::_('searchtools.sort', '', 'a.ordering', $listDirn, $listOrder, null, 'asc', 'JGRID_HEADING_ORDERING', 'icon-sort'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" class="w-5 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JGRID_HEADING_ID', 'a.id', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" style="width:80px;" class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_TYPE', 'a.type', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PRODUCT_NAME', 'a.name', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" style="width:100px;" class="right text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_CREATE_DATE', 'a.created', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" style="width:60px;" class="right text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_LAST_UPDATED', 'a.updated', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" style="width:60px;" class="right text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_SOLD'); ?>
            </th>
            <th scope="col" style="width:60px;" class="right text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PRICE', 'a.price', $listDirn, $listOrder); ?>
            </th>
            <th scope="col" style="width:60px;" class="text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PERIOD'); ?>
            </th>
            <th scope="col" style="width:60px;" class="text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PRODUCT_IS_GROUP'); ?>
            </th>
            <th scope="col" style="width:60px;" class="text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PRODUCT_BUNDLED_BY'); ?>
            </th>
            <th scope="col" style="width:20px;" class="text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_ISPUBLIC'); ?>
            </th>
          </tr>
        </thead>
        <tbody <?php if ($saveOrder) : ?> 
                class="js-draggable" data-url="<?php echo $saveOrderingUrl; ?>" data-direction="<?php echo strtolower($listDirn); ?>"
               <?php endif; ?>
        >
          <?php foreach ($this->items as $i => $row) : ?>
            <?php
             $ordering  = ($listOrder == 'a.ordering');
             $published = HTMLHelper::_('grid.published', $row, $i);
             $checked   = HTMLHelper::_('grid.id', $i, $row->id);
             $link      = 'index.php?option=com_lartssubscriptions&task=products.edit&cid[]=' . $row->id;

            ?>
            <tr class="row<?php echo $i % 2; ?>" data-draggable-group="<?php echo 'none'; ?>" item-id="<?php echo $row->id; ?>">

              <td class="text-center">
                <?php echo $checked; ?>
              </td>
              <td class="text-center">
                  <?php $iconClass = ''; ?>
                  <?php if (!$saveOrder) : ?>
                      <?php $iconClass = ' inactive" title="' . Text::_('JORDERINGDISABLED'); ?>
                  <?php endif; ?>
                  <span class="sortable-handler<?php echo $iconClass; ?>">
                      <span class="icon-ellipsis-v" aria-hidden="true"></span>
                  </span>
                  <?php if ($saveOrder) : ?>
                      <input type="text" class="hidden" name="order[]" size="5" value="<?php echo $row->ordering; ?>">
                  <?php endif; ?>
              </td>
              <td class="text-center">
                <?php
                $layout = new FileLayout('list_button_publish');
                echo $layout->render(['cntrl' => 'products', 'id' => $i, 'value' => $row->published]);
                ?>
              </td>
              <td class="text-center">
                <?php echo $row->id;    // .'/'. $row->ordering; 
                if($row->product_ref > 0) { echo '<br> > ' .$row->product_ref; }
                ?>
              </td>
              <td class="text-start">
                <?php echo Product::GetTypeStr($row->type); ?>
              </td>
              <td class="text-start">
                <a href="<?php echo $link; ?>"><?php echo $row->name; ?></a>
              </td>
              <td class="text-end text-nowrap">
                <?php if(!empty($row->created) && !empty($row->created)): ?>
                  <?php echo DateEx::_($row->created, Text::_('DATE_FORMAT_LC4')); ?>
                <?php endif; ?>
              </td>
              <td class="text-end text-nowrap">
                <?php if(!empty($row->updated) && !empty($row->created)): ?>
                  <?php echo DateEx::_($row->updated, Text::_('DATE_FORMAT_LC4')); ?>
                <?php endif; ?>
              </td>
              <td class="text-end text-nowrap">
                <?php echo $row->total_sold; ?>
              </td>
              <td class="text-end text-nowrap">
                <?php echo Price::_($row->price); ?>
              </td>
              <td class="text-center text-nowrap small">
                <?php echo Helper::getPeriodText($row->period, $row->period_type); ?>
              </td>
              <td class="text-center">
                <?php if ($row->has_children): ?>
                  <i class="fas fa-check green hasTooltip" aria-hidden="true" title="<?php echo Text::_('JYES'); ?>"></i>
                <?php endif; ?>
              </td>
              <td class="text-center">
                <?php foreach ($row->parents as $parent_id => $parent_name): ?>
                  <a href="index.php?option=com_lartssubscriptions&task=products.edit&cid[]=<?php echo (int) $parent_id; ?>">
                    <span class="badge bg-default"><?php echo $parent_name; ?></span>
                  </a>
                <?php endforeach; ?>
              </td>
              <td class="text-center">
                <span class="icon-<?php echo $row->ispublic ? 'publish' : 'unpublish' ?> hasTooltip"
                      title="<?php echo Text::_($row->ispublic ? 'COM_LARTSSUBS_PUBLIC' : 'COM_LARTSSUBS_NOT_PUBLIC'); ?>"></span>
              </td>
            </tr>
          <?php endforeach; ?>
        </tbody>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
