<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Date\Date;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Helper;
use LartsSubscriptions\Base\BSWrapper;
use LartsSubscriptions\Periods;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];

$dToday = new Date();

?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('subscriptions');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=subscriptions' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="customersList">
         <caption class="visually-hidden">
            <?php echo Text::_('COM_FIELDS_FIELDS_TABLE_CAPTION'); ?>,
            <span id="orderedBy"><?php echo Text::_('JGLOBAL_SORTED_BY'); ?> </span>,
            <span id="filteredBy"><?php echo Text::_('JGLOBAL_FILTERED_BY'); ?></span>
        </caption>
       <thead>
          <tr>
           <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo Text::_('JGLOBAL_FIELD_ID_LABEL'); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PRODUCT', 'prodname', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_ORDERCODE', 'per.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="w-15 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_NAME', 'u.lastname', $listDirn, $listOrder); ?>
            </th>
            <th class="w-15 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_EMAIL', 'jusr.email', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_STARTED', 'per.valid_from', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_EXPIRES', 'per.valid_to', $listDirn, $listOrder); ?>
            </th>
            <th class="w-auto text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'per.status', $listDirn, $listOrder); ?>
            </th>
            <th class="w-auto text-center">
              <?php echo Text::_('COM_LARTSSUBS_ADD_NEW'); ?>
            </th>
          </tr>
        </thead>
        <?php $k = 0; ?>
        <?php foreach ($this->items as $i => $row) : ?>
          <?php
          $checked   = HTMLHelper::_('grid.id', $i, $row->period_id);
          $linksub   = 'index.php?option=com_lartssubscriptions&task=subscriptions.edit&cid[]=' . (int) $row->period_id;
          $linkorder = 'index.php?option=com_lartssubscriptions&task=order.show&ordercode=' . (int) $row->ordercode;
          $linkuser  = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . (int) $row->user_id;
          $period    = ''; //$row->ordercount * $row->period;

          ?>
          <tr class="<?php echo "row$k"; ?>">
            <td class="text-center">
              <?php echo $checked; ?>
            </td>
            <td class="text-center text-nowrap">
              <a href="<?php echo $linksub; ?>"><?php echo $row->period_id; ?></a>
            </td>
            <td class="text-start">
              <?php echo $row->prodname; ?>
              <div class="small hidden-desktop">
                <?php echo Text::_('COM_LARTSSUBS_NAME'); ?>: 
                <a href="<?php echo $linkuser; ?>">
                  <?php echo $row->firstname . ' ' . $row->lastname; ?>
                </a><br>
              </div>
            </td>
            <td class="text-start text-nowrap">
              <?php if (!empty($row->ordercode)): ?>
                <a href="<?php echo $linkorder; ?>"><?php echo $row->ordercode; ?></a>
              <?php else: ?>
                <span>manually</span>
              <?php endif; ?>
            </td>
            <td class="text-start">
              <?php if ($row->user_id > 0): ?>
                <a href="<?php echo $linkuser; ?>" title="Joomla-Id: <?php echo $row->jusr_id; ?>">
                  <?php echo $row->firstname . ' ' . $row->lastname; ?>
                </a><br>
                <?php //if ($row->companyname) { echo '<small>' .$row->companyname . '</small><br />'; } ?>
                <small><?php echo $row->email; ?></small>
              <?php else: ?>
                <span class="badge bg-info">N/A</span>
              <?php endif; ?>
            </td>
            <td class="text-start text-nowrap">
              <?php echo $row->email; ?>
            </td>
            <td class="text-center text-nowrap">
              <?php echo DateEx::_($row->valid_from); ?>
            </td>
            <td class="text-center text-nowrap">
              <?php echo DateEx::_($row->valid_to); ?>
              <?php //echo ( ! in_array($row->period_type, ['lifetime', 'onetime',])) ? DateEx::_($row->valid_max, Text::_('DATE_FORMAT_LC4')) : ''; ?>
            </td>
            <td class="text-center text-nowrap">
              <?php 
                if($row->status == Periods::STATE_DISABLED) {
                  $text = Text::_('COM_LARTSSUBS_DISABLED');
                  $type = 'important';
                }
                else if($row->status == Periods::STATE_REFUNDED) {
                  $text = Text::_('COM_LARTSSUBS_REFUNDED');
                  $type = 'important';
                }
                else if($row->status == Periods::STATE_CANCELED) {
                  $text = Text::_('COM_LARTSSUBS_CANCELED');
                  $type = 'important';
                }
                else if($row->status == Periods::STATE_ACTIVE) {
                  if($row->valid_from > $dToday) {
                    $text = Text::_('COM_LARTSSUBS_PENDING');
                    $type = 'warning';
                  }
                  else if($row->valid_to >= $dToday) {
                    $text = Text::_('COM_LARTSSUBS_ACTIVE');
                    $type = 'success';
                  }
                  else {//if($row->valid_to < $dToday) {
                    $text = Text::_('COM_LARTSSUBS_EXPIRED');
                    $type = 'important';
                  }
                }
                echo BSWrapper::label($text, $type);
                ?>
            </td>
            <td class="text-center">
              <?php if($row->period_type != 'onetime') : ?>
                <a href="index.php?option=com_lartssubscriptions&task=subscriptions.edit&uid=<?php echo (int) $row->jusr_id; ?>" 
                   class="btn btn-success btn-sm">
                  <span class="icon-plus"></span>
                  <?php echo Text::_('COM_LARTSSUBS_ADD_SUBSCRIPTION_TO_USER'); ?>
                </a>
              <?php endif; ?>
            </td>
          </tr>
          <?php $k = 1 - $k; ?>
        <?php endforeach; ?>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
