<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Base\BSWrapper;


HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

$searchToolsOptions = [];

?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('templates');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=templates' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>

		<table class="table table-striped">
			<thead>
				<tr>
					<th class="w-1 text-center"></th>
					<th class="w-5 text-start text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_TYPE'); ?>
					</th>
					<th class="w-15 text-start text-nowrap">
						<?php echo Text::_('JFIELD_ALIAS_LABEL'); ?>
					</th>
					<th class="text-start">
						<?php echo Text::_('JGLOBAL_DESCRIPTION'); ?>
					</th>
					<th class="text-start">
						<?php echo Text::_('COM_LARTSSUBS_EMAIL_SUBJECT'); ?>
					</th>
					<th class="w-5 text-center text-nowrap">
						<?php echo Text::_('COM_LARTSSUBS_BCC_ON'); ?>
					</th>
					<th class="w-5 text-center text-nowrap">
						<?php echo Text::_('JGLOBAL_MODIFIED'); ?>
					</th>
				</tr>
			</thead>
			<?php $k = 0; ?>
			<?php foreach ($this->items as $i => $template) : ?>
				<?php
				$checked     = HTMLHelper::_('grid.id', $i, $template->id);
				$link        = Route::_('index.php?option=com_lartssubscriptions&task=templates.edit&cid[]=' . $template->id . '');
				$type        = '';
				$alias       = $template->alias;
				$description = $template->description;

				if (strpos($description, "\n") !== false)
				{
					list($description, $ignore) = explode("\n", $description, 2);
				}
				?>
				<tr class="<?php echo "row$k"; ?>">
					<td class="text-center">
						<?php if ($template->secured) : ?>
							<i class="icon-lock"></i>
						<?php else: ?>
							<?php echo $checked; ?>
						<?php endif; ?>
					</td>
					<td class="text-start text-nowrap">
						<?php
						switch ($template->type) {
            case 'page':  $type = 'success';
              break;
            case 'email': $type = 'info';
              break;
            case 'invoice': $type = 'warning';
              break;
            default: $type = 'default';
              break;
						}
            $text = Text::_('COM_LARTSSUBS_TEMPLATE_TYPE_' . strtoupper($template->type));
            echo BSWrapper::label($text, $type)
						?>
					</td>
					<td class="text-start text-nowrap">
						<a href="<?php echo $link; ?>"><?php echo $alias; ?></a>
					</td>
					<td class="text-start">
						<?php echo $description; ?>
					</td>
					<td class="text-start">
						<?php if ($template->type == 'email'): ?>
							<?php echo $template->subject; ?>
						<?php endif; ?>
					</td>
					<td class="text-center">
						<?php if ($template->type == 'email'): ?>
							<?php
							$layout = new FileLayout('label_yes_no');
							echo $layout->render(['value' => $template->receive_bcc]);
							?>
						<?php endif; ?>
					</td>
					<td class="text-center text-nowrap small">
						<?php echo DateEx::_($template->lastchange, Text::_('DATE_FORMAT_LC4')); ?>
					</td>
				</tr>
				<?php $k = 1 - $k; ?>
			<?php endforeach; ?>
		</table>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
