<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Editor\Editor;

use LartsSubscriptions\Message;

$wa = $this->document->getWebAssetManager();
$wa->useScript('keepalive')
   ->useScript('form.validate');

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);
HTMLHelper::_('script', 'com_lartssubscriptions/template.admin.js', ['version' => 'auto', 'relative' => true]);



$message = new Message;
$variables = $message->getVariables();

?>

<form action="index.php" method="POST" name="adminForm" id="adminForm">

	<?php
	if(!empty($this->data->description))
	{
		$title       = '';
		$description = trim($this->data->description);

		if (strpos($description, "\n") !== false)
		{
			list($title, $description) = explode("\n", $description, 2);
		}

		echo '<div class="alert alert-info">';
		echo $title ? '<h4 class="alert-heading">' . trim($title) . '</h4>' : '';
		echo $description;
		echo '</div>';
	}

	?>
	<div class="row larts-edit-row form-horizontal">
		<div class="span9 col-md-9">

			<div class="row larts-edit-row">
				<div class="span4 col-md-4">
					<fieldset class="form-horizontal-desktop">
						<h3><?php echo Text::_('JDETAILS'); ?></h3>
						<?php echo $this->form->renderFieldset(! empty($this->data->secured) ? 'details_core' : 'details_user'); ?>
					</fieldset>
				</div>

				<div class="span4 col-md-4">
					<fieldset class="form-horizontal-desktop" id="email_settings"
						<?php echo isset($this->data->type) && $this->data->type != 'email' ? 'style="display:none;"' : ''; ?>>
						<h3><?php echo Text::_('COM_LARTSSUBS_EMAIL_SETTINGS'); ?></h3>
						<?php echo $this->form->renderFieldset('email'); ?>
					</fieldset>
				</div>

				<div class="span4 col-md-4">
					<fieldset class="form-horizontal-desktop" id="email_settings"
						<?php echo isset($this->data->type) && $this->data->type != 'email' ? 'style="display:none;"' : ''; ?>>
						<h3><?php echo Text::_('COM_LARTSSUBS_EXTRA_EMAIL_SETTINGS'); ?></h3>
						<?php echo $this->form->renderFieldset('email_bcc'); ?>
					</fieldset>
				</div>
			</div>

			<?php
  		$cfgEditor = Factory::getConfig()->get('editor');
			$editor = Editor::getInstance($cfgEditor);
			echo $editor->display('body',
				isset($this->data->body) ? $this->data->body : '',
				'100%', '500', '100', '40');
			?>
		</div>

		<div class="span3 col-md-3">
			<fieldset class="form-vertical">
				<div class="well">
					<h3 class="well-header"><?php echo Text::_('COM_LARTSSUBS_TEMPLATE_VARIABLES'); ?></h3>

					<p><?php echo Text::_('COM_LARTSSUBS_TEMPLATE_VARIABLES_DESC'); ?></p>

					<div id="variables_subtemplates"
						<?php echo (isset($this->data->type) && $this->data->type == 'template') ? 'style="display:none;"' : '' ?>>

						<h4><?php echo Text::_('COM_LARTSSUBS_TEMPLATE_SUB'); ?></h4>

						{header}<br>
						{footer}
            
					</div>

					<div>
						<?php
						foreach ($variables as $key => $value)
						{
							if ($key == '@end') {
								break;
							}

							if (empty($key)) {
								continue;
							}

							if ($key['0'] == '@')
							{
								echo '</div>';
								echo '<div id="variables_' . ltrim($key, '@') . '">';
								echo '<h4>' . Text::_($value) . '</h4>';
								continue;
							}

							echo '{' . $key . '}<br>';
						}
						?>
					</div>
				</div>
			</fieldset>
		</div>
	</div>

	<input type="hidden" name="id" value="<?php echo isset($this->data->id) ? $this->data->id : ''; ?>" />
	<input type="hidden" name="option" value="com_lartssubscriptions" />
	<input type="hidden" name="task" value="" />
	<input type="hidden" name="controller" value="templates" />
	<?php if(!empty($this->data->secured)): ?>
		<input type="hidden" name="type" value="<?php echo $this->data->type; ?>" />
	<?php endif; ?>
</form>
