<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Base\BSWrapper;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));

?>

<?php echo (new FileLayout('menu'))->render('tickets'); ?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=tickets' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="ticketsList">
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th class="w-1 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_TICKET_NUMBER', 'a.ticket_number', $listDirn, $listOrder); ?>
            </th>
            <th class="w-1 text-start">
            </th>
            <th class="w-5 text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_ORDERCODE', 'a.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="w-15 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_USER', 'a.lastname', $listDirn, $listOrder); ?>
            </th>
            <th class="w-15 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_PRODUCT', 'a.product_name', $listDirn, $listOrder); ?>
            </th>
            <th></th>
          </tr>
        </thead>
        <tbody>
          <?php $k = 0; ?>
          <?php foreach ($this->items as $i => $ticket) : ?>
            <?php
            $published = HTMLHelper::_('grid.published', $ticket, $i);
            $checked = HTMLHelper::_('grid.id', $i, $ticket->id);
            $link = 'index.php?option=com_lartssubscriptions&task=tickets.edit&cid[]=' . $ticket->id;
            $link_user = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . (int) $ticket->userid;
            $link_ordercode = 'index.php?option=com_lartssubscriptions&task=order.show&ordercode=' . (int) $ticket->ordercode;
            $link_product = 'index.php?option=com_lartssubscriptions&task=products.edit&cid[]=' . (int) $ticket->product_id;
            $dl = 'index.php?option=com_lartssubscriptions&task=tickets.getticket&id=' . $ticket->id;
            ?>
            <tr class="<?php echo "row$k"; ?>">
              <td class="text-start">
                <?php echo $checked; ?>
              </td>
              <td class="text-center">
                <?php
                $layout = new FileLayout('list_button_publish');
                echo $layout->render(['cntrl' => 'tickets', 'id' => $i, 'value' => $ticket->published]);
                ?>
              </td>
              <td class="right">
                <a href="<?php echo $link; ?>">
                  <?php echo $ticket->ticket_number; ?>
                </a>
              </td>
              <td class="text-center">
                <div class="btn-group">
                  <a class="btn btn-micro" href="<?php echo $dl; ?>">
                    <i class="icon-download" title="<?php echo Text::_('COM_LARTSSUBS_DOWNLOAD_INVOICE'); ?>"></i>
                  </a>
                </div>
              </td>

              <td class="text-center">
                <a href="<?php echo $link_ordercode; ?>">
                  <?php echo BSWrapper::label($ticket->ordercode); ?>
                </a>
              </td>
              <td class="text-start">
                <a href="<?php echo $link_user; ?>">
                  <?php echo $ticket->firstname . ' ' . $ticket->lastname .' '. BSWrapper::label($ticket->userid); ?>
                </a><br>
                <?php echo $ticket->email; ?>
              </td>
              <td class="text-start">
                <a href="<?php echo $link_product; ?>">
                  <?php echo $ticket->product_name; ?>
                </a>
              </td>
              <td></td>
            </tr>
          <?php endforeach; ?>
        </tbody>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
      