<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copytext-end       {copytext-end}
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Price;
use LartsSubscriptions\Base\BSWrapper;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$searchToolsOptions = [];
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('transactions');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=transactions' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped">
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_NAME', 'u.lastname', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start ">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_TRANSACTION', 't.transaction_token', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_ORDERCODE', 't.ordercode', $listDirn, $listOrder); ?>
            </th>
            <th class="w-10 text-start text-nowrap ">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_TRANSACTION_EMAIL', 't.email', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JDATE', 't.date', $listDirn, $listOrder); ?>
            </th>
            <th class="w-5 text-end text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_AMOUNT'); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PAYMENT_TYPE'); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_PLUGIN'); ?>
            </th>
            <th class="w-5 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_STATUS'); ?>
            </th>
            <th class="w-10 text-center text-nowrap">
              <?php echo Text::_('COM_LARTSSUBS_COMMENT'); ?>
            </th>
          </tr>

        </thead>
        <?php foreach ($this->items as $i => $row) : ?>
          <?php
          $checked          = HTMLHelper::_('grid.id', $i, $row->id);
          $link_transaction = 'index.php?option=com_lartssubscriptions&task=transactions.edit&cid[]=' . $row->id;
          $link_user        = 'index.php?option=com_lartssubscriptions&task=customers.edit&cid[]=' . (int) $row->userid;
          $linkorder = 'index.php?option=com_lartssubscriptions&task=order.show&ordercode=' . (int) $row->ordercode;
          ?>
          <tr>
            <td class="text-center">
              <?php echo $checked; ?>
            </td>
            <td class="text-start ">
              <?php if ($row->userid > 0): ?>
                <a href="<?php echo $link_user; ?>">
                  <?php echo $row->firstname . ' ' . $row->lastname; ?>
                </a>
              <?php else: ?>
                <span class="badge bg-info">N/A</span>
              <?php endif; ?>
            </td>
            <td class="text-start ">
              <a href="<?php echo $link_transaction; ?>">
                <?php echo $row->transaction_token; ?><br>
                <?php echo $row->transaction_id; ?>
              </a>
            </td>
            <td class="text-start">
              <a href="<?php echo $linkorder; ?>"><?php echo $row->ordercode; ?></a>
            </td>
            <td class="text-start text-nowrap ">
              <?php echo $row->email_paypal ?: $row->email; ?>
            </td>
            <td class="text-center text-nowrap small">
              <?php echo DateEx::_($row->date); ?>
            </td>
            <td class="text-end text-nowrap small">
              <?php echo Price::_($row->amount); ?>
            </td>
            <td class="text-center text-nowrap">
              <?php echo BSWrapper::label($row->type, 'default'); ?>
            </td>
            <td class="text-center text-nowrap">
              <?php echo BSWrapper::label($row->plugin, 'default'); ?>
            </td>
            <td class="text-center text-nowrap">
              <?php $txt = $row->status == 0 ? Text::_('COM_LARTSSUBS_PENDING') : Text::_('COM_LARTSSUBS_PROCESSED');
                    $type = $row->status == 0 ? 'warning' : 'success';
                echo BSWrapper::label($txt, $type); ?>
            </td>
            <td class="text-start">
              <?php echo $row->comment; ?>
            </td>
          </tr>
        <?php endforeach; ?>
      </table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
