<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
$ordering  = ($listOrder == 'a.ordering');
$saveOrder = ($listOrder == 'a.ordering' && strtolower($listDirn) == 'asc');

if ($saveOrder && !empty($this->items)) {
  $saveOrderingUrl = 'index.php?option=com_lartssubscriptions&task=vatclasses.saveOrderAjax&tmpl=component&' . Session::getFormToken() . '=1';
  HTMLHelper::_('draggablelist.draggable');
}
?>

<?php echo (new FileLayout('menu'))->render('vatclasses'); ?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=vatclasses' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

      <table class="table table-striped" id="categoryList">
        <thead>
          <tr>
            <th class="w-1 text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
                <?php echo HTMLHelper::_('searchtools.sort', '', 'ordering', $listDirn, $listOrder, null, 'asc', 'JGRID_HEADING_ORDERING', 'icon-sort'); ?>
            </th>
            <th scope="col" class="w-1 text-center">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'published', $listDirn, $listOrder); ?>
            </th>
            <th class="text-start">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_VATCLASS', 'name', $listDirn, $listOrder); ?>
            </th>
          </tr>
        </thead>
        <tbody>
          <?php foreach ($this->items as $i => $item) : ?>
            <?php
            $published = HTMLHelper::_('grid.published', $item, $i);
            $checked   = HTMLHelper::_('grid.id', $i, $item->id);
            $link = 'index.php?option=com_lartssubscriptions&task=vatclasses.edit&cid[]=' . $item->id;
            ?>
            <tr>
               <td class="text-center">
                <?php echo $checked; ?>
              </td>
              <td class="text-center">
                  <?php $iconClass = ''; ?>
                  <?php if (!$saveOrder) : ?>
                      <?php $iconClass = ' inactive" title="' . Text::_('JORDERINGDISABLED'); ?>
                  <?php endif; ?>
                  <span class="sortable-handler<?php echo $iconClass; ?>">
                      <span class="icon-ellipsis-v" aria-hidden="true"></span>
                  </span>
                  <?php if ($saveOrder) : ?>
                      <input type="text" class="hidden" name="order[]" size="5" value="<?php echo $row->ordering; ?>">
                  <?php endif; ?>
              </td>
              <td class="text-center">
                <?php
                $layout = new FileLayout('list_button_publish');
                echo $layout->render(['cntrl' => 'vatclasses', 'id' => $i, 'value' => $item->published]);
                ?>
              </td>
              <td class="text-start">
                <a href="<?php echo $link; ?>"><?php echo $item->name; ?></a>
              </td>
            </tr>
          <?php endforeach; ?>
        </tbody>
      </table>

      <div class="center"><?php echo $this->pagination->getListFooter(); ?></div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
