<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

use LartsSubscriptions\Base\DateEx;
use LartsSubscriptions\Price;
use LartsSubscriptions\VAT\VatRates;

HTMLHelper::_('stylesheet', 'com_lartssubscriptions/adminj4.css', ['version' => 'auto', 'relative' => true]);

try {
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('table.columns')
      ->useScript('multiselect');
}
catch(Exception $e) {}

$searchToolsOptions = [];
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));
  
?>

<?php
$layout = new FileLayout('menu');
echo $layout->render('vatrates');
?>

<form action="<?php echo Route::_('index.php?option=com_lartssubscriptions&view=vatrates' . $this->state->get('filter.context')); ?>" method="post" name="adminForm" id="adminForm">
<div class="row">
  <div class="col-md-12">
    <div id="j-main-container" class="j-main-container">

		<?php echo LayoutHelper::render('joomla.searchtools.default', ['view' => $this, 'options' => $searchToolsOptions]); ?>
    <?php if (empty($this->items)) : ?>
        <div class="alert alert-info">
            <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
            <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
        </div>
    <?php else : ?>

			<table class="table table-striped">
				<thead>
					<tr>
            <th class="w-1 text-text-center">
              <?php echo HTMLHelper::_('grid.checkall'); ?>
            </th>
						<th class="w-1 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
						</th>
						<th class="w-10 text-center" style="width:60px;">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_COUNTRY', 'a.country_code', $listDirn, $listOrder); ?>
						</th>
						<th class="w-10 text-start text-nowrap"">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_VAT_TYPE', 'a.type', $listDirn, $listOrder); ?>
						</th>
						<th class="w-5 text-center text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_RATE', 'a.rate', $listDirn, $listOrder); ?>
						</th>
						<th class="w-20 text-start text-nowrap">
              <?php echo HTMLHelper::_('searchtools.sort', 'COM_LARTSSUBS_CATEGORY', 'a.category', $listDirn, $listOrder); ?>
						</th>
						<th class="text-start text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_CATEGORY_INFO'); ?>
						</th>
						<th class="text-start text-nowrap">
							<?php echo Text::_('COM_LARTSSUBS_RATE_INFOC'); ?>
						</th>
					</tr>
				</thead>
				<?php foreach ($this->items as $i => $row) : ?>
					<?php
					$checked = HTMLHelper::_('grid.id', $i, $row->id);
					$link    = 'index.php?option=com_lartssubscriptions&task=vatrates.edit&cid[]=' . $row->id;
					?>
					<tr>
						<td class="text-center">
							<?php echo $checked; ?>
						</td>
						<td class="text-center">
							<?php
							$layout = new FileLayout('list_button_publish');
							echo $layout->render(['cntrl' => 'vatrates', 'id' => $i, 'value' => $row->published]);
							?>
						</td>
						<td class="text-center">
							<?php echo $row->country_code; ?>
						</td>
						<td class="text-start">
							<?php echo VatRates::getTypeStr($row->type); ?>
						</td>
						<td class="text-center">
							<?php echo $row->rate; ?>
						</td>
						<td class="text-start">
							<?php echo $row->category; ?>
						</td>
						<td class="text-start">
							<?php echo $row->category_desc; ?>
						</td>
						<td class="text-start">
							<?php echo $row->rate_desc; ?>
						</td>
					</tr>
				<?php endforeach; ?>
			</table>

      <div class="text-center"><?php echo $this->pagination->getListFooter(); ?></div>
		</div>

    <?php endif; ?>

    <input name="task" type="hidden" value="" />
    <input name="boxchecked" type="hidden" value="0" />
    <?php echo HTMLHelper::_('form.token'); ?>
    </div>
  </div>
</div>
</form>
